/******************************************************************************
 **
 ** PROJECT      : UEIP
 ** FILE NAME    : ifxmips_ppa_datapath_vr9_e5.c
 ** MODULES      : MII0/1 Acceleration Package (VR9 PPA E5 v1)
 **
 ** DATE         : 19 OCT 2009
 ** AUTHOR       : Xu Liang
 ** DESCRIPTION  : MII0/1 Driver with Acceleration Firmware (D5 v1) + E1
 ** COPYRIGHT    :   Copyright (c) 2006
 **          Infineon Technologies AG
 **          Am Campeon 1-12, 85579 Neubiberg, Germany
 **
 **    This program is free software; you can redistribute it and/or modify
 **    the Free Software Foundation; either version 2 of the License, or
 **    it under the terms of the GNU General Public License as published by
 **    (at your option) any later version.
 **
 ** HISTORY
 ** $Date        $Author         $Comment
 ** 19 OCT 2009  Xu Liang        Initiate Version
 *******************************************************************************/

/*
 * ####################################
 *              Version No.
 * ####################################
 */

#define AVM_DESTINATION_TEST 0

#define VER_FAMILY      0x40        //  bit 0: res
//      1: Danube
//      2: Twinpass
//      3: Amazon-SE
//      4: res
//      5: AR9
//      6: VR9
//      7: AR10
#define VER_DRTYPE      0x03        //  bit 0: Normal Data Path driver
//      1: Indirect-Fast Path driver
//      2: HAL driver
//      3: Hook driver
//      4: Stack/System Adaption Layer driver
//      5: PPA API driver
#define VER_INTERFACE   0x03        //  bit 0: MII 0
//      1: MII 1
//      2: ATM WAN
//      3: PTM WAN
#define VER_ACCMODE     0x01        //  bit 0: Routing
//      1: Bridging
#define VER_MAJOR       0
#define VER_MID         1
#define VER_MINOR       1
//#define RESET_AR10_STYLE

#define DMA_CHAN_NR_DIRECTPATH_CPU_TO_PPE  3
#define DMA_CHAN_NR_DEFAULT_CPU_TX         2
#ifdef CONFIG_IFX_ETHSW_API
// #define AVM_DOESNT_USE_IFX_ETHSW_API 1
#endif

#define PPE_HOLD_TIMEOUT               120



/*
 * ####################################
 *              Head File
 * ####################################
 */

/*
 *  Common Head File
 */
#include <linux/autoconf.h>
#include <linux/vmalloc.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/ctype.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <linux/etherdevice.h>  /*  eth_type_trans  */
#include <linux/ethtool.h>      /*  ethtool_cmd     */
#include <linux/if_ether.h>
#include <linux/if_arp.h>
#include <asm/uaccess.h>
#include <asm/unistd.h>
#include <asm/irq.h>
#include <asm/delay.h>
#include <asm/io.h>
#include <asm/prom.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/xfrm.h>
#include <linux/avm_pa_ifx_multiplexer.h>
#include <net/pkt_sched.h>
/*
 *  Chip Specific Head File
 */
#include <ifx_types.h>
#include <ifx_regs.h>
#include <common_routines.h>
#include <ifx_pmu.h>
#include <ifx_rcu.h>
#include <ifx_gpio.h>
#include <ifx_led.h>
#include <ifx_dma_core.h>
#include <ifx_clk.h>
#include <ifx_eth_framework.h>

#include <switch_api/ifx_ethsw_kernel_api.h>
#include <switch_api/ifx_ethsw_api.h>
#include <net/ifx_ppa_stack_al.h>
#include <net/ifx_ppa_api_directpath.h>
#include <net/ifx_ppa_api.h>
#include <net/ifx_ppa_ppe_hal.h>

#include <avmnet_config.h>
#if defined(CONFIG_IP_MULTICAST_FASTFORWARD)
#include <avmnet_multicast.h>
#endif /*--- #if defined(CONFIG_IP_MULTICAST_FASTFORWARD) ---*/

#include "../common/swi_ifx_common.h"
#include "../common/ifx_ppa_datapath.h"
#include "../7port/swi_7port.h"
#include "../7port/swi_7port_reg.h"
#include "ifxmips_ppa_datapath_fw_vr9_d5.h"
#include "ifxmips_ppa_datapath_fw_vr9_e1.h"

#include "ifxmips_ppa_hal_vr9_e5.h"
#include "../7port/ifxmips_ppa_datapath_7port_common.h"

/*
 *  Bonding Data Structure Header File
 */
#include "vr9_bonding_fw_data_structure.h"
#include "vr9_bonding_addr_def.inc"


/*
 * ####################################
 *   Parameters to Configure PPE
 * ####################################
 */

#if defined(DEBUG_SPACE_BETWEEN_HEADER_AND_DATA)
static unsigned int min_fheader_byteoff = ~0;
#endif

static int ethwan = 0;

static int wanitf = ~0;

static int ipv6_acc_en = 1;

static int wanqos_en = 8;


#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  static int dsl_bonding     = 2;
  static int pci_ppe_addr    = 0x18200000;
  static int bm_addr         = 0x01E00000;
  static int pci_ddr_addr    = 0x18000000;
  static int pci_bm_ppe_addr = 0x19600000;
#else
  static int dsl_bonding     = 0;
#endif

static char *avm_ata_dev = NULL;

static int unknown_session_dump_len = DUMP_SKB_LEN;
module_param(unknown_session_dump_len, int, S_IRUSR | S_IWUSR);

static int queue_gamma_map[4] = { 0xFE, 0x01, 0x00, 0x00 };

#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_SIZE)
static int directpath_tx_queue_size = 1524 * 50;
#endif
#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_PKTS)
static int directpath_tx_queue_pkts = 50;
#endif

#define MODULE_PARM_ARRAY(a, b)   module_param_array(a, int, NULL, 0)
#define MODULE_PARM(a, b)         module_param(a, int, 0)

MODULE_PARM(ethwan, "i");
MODULE_PARM_DESC(ethwan, "WAN mode, 2 - ETH WAN, 1 - ETH0 mixed, 0 - DSL WAN.");
MODULE_PARM(wanitf, "i");
MODULE_PARM_DESC(
		wanitf,
		"WAN interfaces, bit 0 - ETH0, 1 - ETH1, 2 - reserved for CPU0, 3/4/5 - DirectPath, 6/7 - DSL");

module_param(avm_ata_dev, charp, 0);
MODULE_PARM_DESC(avm_ata_dev, "eth0: ethernet_wan with NAT, ath0: WLAN_WAN with NAT");
MODULE_PARM(ipv6_acc_en, "i");
MODULE_PARM_DESC(ipv6_acc_en, "IPv6 support, 1 - enabled, 0 - disabled.");

MODULE_PARM(wanqos_en, "i");
MODULE_PARM_DESC(wanqos_en, "WAN QoS support, 0 - disabled, others - enabled and the number of queues.");


MODULE_PARM(dsl_bonding, "i");
MODULE_PARM_DESC(dsl_bonding, "DSL Bonding Mode, 0 - disable bonding, 1 - on chip bonding, 2 - off chip bonding");
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  MODULE_PARM(pci_ppe_addr, "i");
  MODULE_PARM_DESC(pci_ppe_addr, "Base Address of PPE on System Slave from System Master's point of view.");
  MODULE_PARM(bm_addr, "i");
  MODULE_PARM_DESC(bm_addr, "Base Address of Bonding Master from System Master's point of view.");
  MODULE_PARM(pci_ddr_addr, "i");
  MODULE_PARM_DESC(pci_ddr_addr, "Base Address of DDR from System Slave's point of view.");
  MODULE_PARM(pci_bm_ppe_addr, "i");
  MODULE_PARM_DESC(pci_bm_ppe_addr, "Base Address of PPE on Bonding Slave from Bonding Master's point of view.");
#endif

MODULE_PARM_ARRAY(queue_gamma_map, "4-4i");
MODULE_PARM_DESC(queue_gamma_map,
		"TX QoS queues mapping to 4 TX Gamma interfaces.");

#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_SIZE)
MODULE_PARM(directpath_tx_queue_size, "i");
MODULE_PARM_DESC(directpath_tx_queue_size, "directpath TX queue size in total buffer size");
#endif
#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_PKTS)
MODULE_PARM(directpath_tx_queue_pkts, "i");
MODULE_PARM_DESC(directpath_tx_queue_pkts, "directpath TX queue size in total packet number");
#endif

/*
 * ####################################
 *              Board Type
 * ####################################
 */

#define BOARD_VR9_REFERENCE                     0x01

/*
 * ####################################
 *              Definition
 * ####################################
 */

#define BOARD_CONFIG                            BOARD_VR9_REFERENCE

#define AVM_SETUP_MAC 							1

#define DEBUG_SKB_SWAP                          0

#define ENABLE_P2P_COPY_WITHOUT_DESC            1

#define ENABLE_LED_FRAMEWORK                    0

#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_SIZE)
#define ENABLE_DIRECTPATH_TX_QUEUE            1
#define ENABLE_DIRECTPATH_TX_QUEUE_SIZE       directpath_tx_queue_size
#elif defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_PKTS)
#define ENABLE_DIRECTPATH_TX_QUEUE            1
#define ENABLE_DIRECTPATH_TX_QUEUE_PKTS       directpath_tx_queue_pkts
#endif

#ifdef CONFIG_IFX_PPA_NAPI_ENABLE
#define ENABLE_NAPI                           1
#else
#error driver just working with napi support so far
#endif

#define ENABLE_SWITCH_FLOW_CONTROL              0

#define ENABLE_STATS_ON_VCC_BASIS               1

#define ENABLE_MY_MEMCPY                        0

#define ENABLE_FW_MODULE_TEST                   0

#if defined(CONFIG_AVMNET_DEBUG) 
#define DEBUG_DUMP_INIT                         0

#define DEBUG_FIRMWARE_TABLES_PROC              1

#define DEBUG_MEM_PROC                          1

#define DEBUG_FW_PROC                           1

#define DEBUG_MIRROR_PROC                       0

#define DEBUG_BONDING_PROC                      0

#define DEBUG_REDIRECT_FASTPATH_TO_CPU          0

#else // defined(CONFIG_AVMNET_DEBUG) 

#define DEBUG_DUMP_INIT                         0

#define DEBUG_FIRMWARE_TABLES_PROC              1

#define DEBUG_MEM_PROC                          0

#define DEBUG_FW_PROC                           0

#define DEBUG_MIRROR_PROC                       0

#define DEBUG_BONDING_PROC                      0

#define DEBUG_REDIRECT_FASTPATH_TO_CPU          0

#endif // defined(CONFIG_AVMNET_DEBUG) 

#define PPE_MAILBOX_IGU0_INT                    INT_NUM_IM2_IRL23

#define PPE_MAILBOX_IGU1_INT                    INT_NUM_IM2_IRL24

#if defined(CONFIG_DSL_MEI_CPE_DRV) && !defined(CONFIG_IFXMIPS_DSL_CPE_MEI)
#define CONFIG_IFXMIPS_DSL_CPE_MEI            1
#endif

#define AVM_NET_TRACE_DUMP				  		0

//  specific board related configuration
#if defined (BOARD_CONFIG)
#if BOARD_CONFIG == BOARD_VR9_REFERENCE
#endif
#endif

#if !defined(DISABLE_INLINE) || !DISABLE_INLINE
#define INLINE                                inline
#else
#define INLINE
#endif

/*
 *  Mailbox Signal Bit
 */
#define DMA_TX_CH1_SIG                          (1 << 31)   //  IGU0
#define DMA_TX_CH0_SIG                          (1 << 30)   //  IGU0
#define CPU_TO_WAN_TX_SIG                       (1 << 17)   //  IGU1
#define CPU_TO_WAN_SWAP_SIG                     (1 << 16)   //  IGU1
#define WAN_RX_SIG(i)                           (1 << 0)    //  IGU1, 0: Downstream DSL Traffic
/*
 *  Eth Mode
 */
#define RGMII_MODE                              0
#define MII_MODE                                1
#define REV_MII_MODE                            2
#define RED_MII_MODE_IC                         3   //  Input clock
#define RGMII_MODE_100MB                        4
#define TURBO_REV_MII_MODE                      6	//  Turbo Rev Mii mode
#define RED_MII_MODE_OC                         7   //  Output clock
#define RGMII_MODE_10MB                         8

/*
 *  Bonding Definition
 */
#define L1_PTM_TC_BOND_MODE                     0
#define L2_ETH_TRUNK_MODE                       1
#define LINE_NUMBER                             2
#define BEARER_CHANNEL_PER_LINE                 2
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  #define SAVE_AND_SWITCH_PPE_BASE_TO_SM(_SAVED_ADDR)   \
        unsigned int _SAVED_ADDR;                       \
        _SAVED_ADDR = g_ppe_base_addr;                  \
        g_ppe_base_addr = IFX_PPE_ORG(0);

  #define SAVE_AND_SWITCH_PPE_BASE_TO_BM(_SAVED_ADDR)   \
        unsigned int _SAVED_ADDR;                       \
        _SAVED_ADDR = g_ppe_base_addr;                  \
        g_ppe_base_addr = IFX_PPE_ORG(1);

  #define RESTORE_PPE_BASE(_SAVED_ADDR)                 \
        g_ppe_base_addr = _SAVED_ADDR;
  #define PPE_ID                                (g_ppe_base_addr == IFX_PPE_ORG(0) ? 0 : 1)
#else
  #define SAVE_AND_SWITCH_PPE_BASE_TO_SM(_SAVED_ADDR)
  #define SAVE_AND_SWITCH_PPE_BASE_TO_BM(_SAVED_ADDR)
  #define RESTORE_PPE_BASE(_SAVED_ADDR)
  #define PPE_ID                                0
#endif
#define MAX_DESC_PER_GAMMA_ITF                  128


/*
 *  Constant Definition
 */
#define ETH_WATCHDOG_TIMEOUT                    (10 * HZ)
#define ETOP_MDIO_DELAY                         1
#define IDLE_CYCLE_NUMBER                       30000

/*change DMA packet size for IP tunnelling feature */
#define DMA_PACKET_SIZE                         1600    //  ((1518 + 62 + 8) + 31) & ~31
#define DMA_RX_ALIGNMENT                        128
#define DMA_TX_ALIGNMENT                        32
#define DMA_RX_ALIGNED(x)                          (((unsigned int)(x) + DMA_RX_ALIGNMENT - 1) & ~(DMA_RX_ALIGNMENT - 1))
#define DMA_TX_ALIGNED(x)                          (((unsigned int)(x) + DMA_TX_ALIGNMENT - 1) & ~(DMA_TX_ALIGNMENT - 1))
/*
 *  Ethernet Frame Definitions
 */
#define ETH_CRC_LENGTH                          4
#define ETH_MAX_DATA_LENGTH                     ETH_DATA_LEN
#define ETH_MIN_TX_PACKET_LENGTH                ETH_ZLEN

/*
 *  PDMA Settings
 */
//#define EMA_CMD_BUF_LEN                         0x0010
//#define EMA_CMD_BASE_ADDR                       (0x1710 << 2)
//#define EMA_DATA_BUF_LEN                        0x0040
//#define EMA_DATA_BASE_ADDR                      (0x16d0 << 2)
//#define EMA_WRITE_BURST                         0x02
//#define EMA_READ_BURST                          0x02
/*
 *  Firmware Settings
 */
#define CPU_TO_WAN_TX_DESC_NUM                  64  //  WAN CPU TX
#define WAN_TX_DESC_NUM(i)                      __ETH_WAN_TX_QUEUE_LEN
#define CPU_TO_WAN_SWAP_DESC_NUM                32
#define WAN_TX_DESC_NUM_TOTAL                   512
#define WAN_RX_DESC_NUM                         64
#define DMA_TX_CH1_DESC_NUM                     (g_eth_wan_mode ? (g_eth_wan_mode == 3 /* ETH WAN */ && g_wanqos_en ? 64 : DMA_RX_CH1_DESC_NUM) : WAN_RX_DESC_NUM)
#define DMA_RX_CH1_DESC_NUM                     64
#define DMA_RX_CH2_DESC_NUM                     64
#define DMA_TX_CH0_DESC_NUM                     DMA_RX_CH2_DESC_NUM //  DMA_TX_CH0_DESC_NUM is used to replace DMA_TX_CH2_DESC_NUM

//  Bonding Specific
#define E1_FRAG_TX_DESC_NUM_TOTAL               (16 * 8)
#define E1_FRAG_RX_DESC_NUM                     32
#define B1_RX_LINK_LIST_DESC_NUM                (32 * 8)
#define __US_BOND_PKT_DES_BASE(i)               (0x5C00 + (i) * 0x80)
#define US_BOND_PKT_DES_BASE(i)                 ((volatile struct US_BOND_PKT_DES *)SB_BUFFER_BOND(2, __US_BOND_PKT_DES_BASE(i)))
#define __US_BG_CTXT_BASE(i)                    (__US_BG_CTXT + (i) * 4)
#define US_BG_CTXT_BASE(i)                      ((volatile struct US_BG_CTXT *)SB_BUFFER_BOND(2, __US_BG_CTXT_BASE(i)))
#define __DS_BOND_GIF_LL_DES_BASE(i)            (__DS_BOND_GIF_LL_DESBA + (i) * 2)
#define DS_BOND_GIF_LL_DES_BASE(i)              ((volatile struct DS_BOND_GIF_LL_DES *)SB_BUFFER_BOND(2, __DS_BOND_GIF_LL_DES_BASE(i)))
#define DS_BOND_FREE_LL_CTXT_BASE               ((volatile struct DS_BOND_GIF_LL_CTXT *)SB_BUFFER_BOND(2, __DS_BOND_FREE_LL_CTXT))
#define __DS_BOND_GIF_LL_CTXT_BASE(i)           (__DS_BOND_GIF_LL_CTXT + (i) * 4)
#define DS_BOND_GIF_LL_CTXT_BASE(i)             ((volatile struct DS_BOND_GIF_LL_CTXT *)SB_BUFFER_BOND(2, __DS_BOND_GIF_LL_CTXT_BASE(i)))
#define __DS_BG_CTXT_BASE(i)                    (__DS_BG_CTXT + (i) * 8)
#define DS_BG_CTXT_BASE(i)                      ((volatile struct DS_BG_CTXT *)SB_BUFFER_BOND(2, __DS_BG_CTXT_BASE(i)))
#define __DS_BOND_GIF_MIB_BASE(i)               (__DS_BOND_GIF_MIB + i * 16)
#define DS_BOND_GIF_MIB_BASE(i)                 ((volatile struct DS_BOND_GIF_MIB *)SB_BUFFER_BOND(2, __DS_BOND_GIF_MIB_BASE(i)))
#define __DS_BG_MIB_BASE(i)                     (__DS_BG_MIB + i * 16)
#define DS_BG_MIB_BASE(i)                       ((volatile struct DS_BG_MIB *)SB_BUFFER_BOND(2, __DS_BG_MIB_BASE(i)))
#define US_BOND_GIF_MIB(i)                      SB_BUFFER_BOND(2, __US_E1_FRAG_Q_FRAG_MIB + (i))

#define IFX_PPA_PORT_NUM                        8

/*
 *  Bits Operation
 */
//#define GET_BITS(x, msb, lsb)                   (((x) & ((1 << ((msb) + 1)) - 1)) >> (lsb))
//#define SET_BITS(x, msb, lsb, value)            (((x) & ~(((1 << ((msb) + 1)) - 1) ^ ((1 << (lsb)) - 1))) | (((value) & ((1 << (1 + (msb) - (lsb))) - 1)) << (lsb)))
/*
 *  Internal Tantos Switch Related
 */
//#define MDIO_OP_READ                            (2 << 10)
//#define MDIO_OP_WRITE                           (1 << 10)
//#define TANTOS_CHIP_ID                          0x2599

#define SWITCH_CLASS_VALUE                      8

/*
 *  DMA/EMA Descriptor Base Address
 */
#define CPU_TO_WAN_TX_DESC_BASE                 ((volatile struct tx_descriptor *)SB_BUFFER_BOND(0, 0x3D00))                 /*         64 each queue    */
#define __ETH_WAN_TX_QUEUE_NUM                  g_wanqos_en
#define __ETH_WAN_TX_QUEUE_LEN                  ((WAN_TX_DESC_NUM_TOTAL / __ETH_WAN_TX_QUEUE_NUM) < 256 ? (WAN_TX_DESC_NUM_TOTAL / __ETH_WAN_TX_QUEUE_NUM) : 255)
#define __ETH_WAN_TX_DESC_BASE(i)               (0x5C00 + (i) * 2 * __ETH_WAN_TX_QUEUE_LEN)
#define WAN_TX_DESC_BASE(i)                     ((volatile struct tx_descriptor *)SB_BUFFER_BOND(2, __ETH_WAN_TX_DESC_BASE(i)))  /*  i < __ETH_WAN_TX_QUEUE_NUM, __ETH_WAN_TX_QUEUE_LEN each queue    */
#define CPU_TO_WAN_SWAP_DESC_BASE               ((volatile struct tx_descriptor *)SB_BUFFER_BOND(0, 0x2E80))                 /*         64 each queue    */
#define WAN_RX_DESC_PPE                         0x2600
#define WAN_RX_DESC_BASE                        ((volatile struct tx_descriptor *)SB_BUFFER_BOND(0, WAN_RX_DESC_PPE))        /*         64 each queue    */
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  #define DMA_TX_CH1_WITH_BONDING               (g_dsl_bonding != 2 ? (unsigned int)WAN_RX_DESC_BASE : g_bm_addr)
#else
  #define DMA_TX_CH1_WITH_BONDING               WAN_RX_DESC_BASE
#endif
#define DMA_TX_CH1_DESC_BASE                    (g_eth_wan_mode ? (g_eth_wan_mode == 3 /* ETH WAN */ && g_wanqos_en ? ((volatile struct rx_descriptor *)SB_BUFFER_BOND(0, 0x2600)) : DMA_RX_CH1_DESC_BASE) : (volatile struct rx_descriptor *)DMA_TX_CH1_WITH_BONDING)
#define DMA_RX_CH1_DESC_BASE                    ((volatile struct rx_descriptor *)SB_BUFFER_BOND(2, 0x2580))                 /*         64 each queue    */
#define DMA_RX_CH2_DESC_BASE                    ((volatile struct rx_descriptor *)SB_BUFFER_BOND(0, 0x2F00))                 /*         64 each queue    */
#define DMA_TX_CH0_DESC_BASE                    DMA_RX_CH2_DESC_BASE
//  Bonding Specific
#define E1_FRAG_TX_DESC_NB_PPE(i)               (0x3800 + (i) * 2 * 16) //  i < 8
#define E1_FRAG_TX_DESC_B_PPE(i)                (0x3F60 + (i) * 16)     //  i < 8
#define E1_FRAG_TX_DESC_NB_BASE(i)              ((volatile struct tx_descriptor *)SB_BUFFER_BOND(2, E1_FRAG_TX_DESC_NB_PPE(i)))
#define E1_FRAG_RX_DESC_PPE(i)                  (0x3D00 + (i) * 0x40)   //  i < 2
#define E1_FRAG_RX_DESC_BASE(i)                 ((volatile struct rx_descriptor *)SB_BUFFER_BOND(2, E1_FRAG_RX_DESC_PPE(i)))
#define B1_RX_LINK_LIST_DESC_PPE                0x4800
#define B1_RX_LINK_LIST_DESC_BASE               ((volatile struct tx_descriptor *)SB_BUFFER_BOND(2, B1_RX_LINK_LIST_DESC_PPE))


#if defined(DEBUG_FW_PROC) && DEBUG_FW_PROC
/*
 *  Firmware Proc
 */
//  need check with hejun
#define TIR(i)                                ((volatile u32 *)(0xBE200140) + (i))

#define __CPU_TX_SWAPPER_DROP_PKTS            SB_BUFFER_BOND(2, 0x29A8)
#define __CPU_TX_SWAPPER_DROP_BYTES           SB_BUFFER_BOND(2, 0x29A9)
#define __DSLWAN_FP_SWAPPER_DROP_PKTS         SB_BUFFER_BOND(2, 0x29AA)
#define __DSLWAN_FP_SWAPPER_DROP_BYTES        SB_BUFFER_BOND(2, 0x29AB)

#define __CPU_TXDES_SWAP_RDPTR                SB_BUFFER_BOND(2, 0x29AC)
#define __DSLWAN_FP_RXDES_SWAP_RDPTR          SB_BUFFER_BOND(2, 0x29AD)
#define __DSLWAN_FP_RXDES_DPLUS_WRPTR         SB_BUFFER_BOND(2, 0x29AE)

#define __DSLWAN_TX_PKT_CNT0                  SB_BUFFER_BOND(2, 0x29B0)
#define __DSLWAN_TX_PKT_CNT1                  SB_BUFFER_BOND(2, 0x29B1)
#define __DSLWAN_TX_PKT_CNT2                  SB_BUFFER_BOND(2, 0x29B2)
#define __DSLWAN_TX_PKT_CNT3                  SB_BUFFER_BOND(2, 0x29B3)
#define __DSLWAN_TX_PKT_CNT4                  SB_BUFFER_BOND(2, 0x29B4)
#define __DSLWAN_TX_PKT_CNT5                  SB_BUFFER_BOND(2, 0x29B5)
#define __DSLWAN_TX_PKT_CNT6                  SB_BUFFER_BOND(2, 0x29B6)
#define __DSLWAN_TX_PKT_CNT7                  SB_BUFFER_BOND(2, 0x29B7)

#define __DSLWAN_TXDES_SWAP_PTR0              SB_BUFFER_BOND(2, 0x29B8)
#define __DSLWAN_TXDES_SWAP_PTR1              SB_BUFFER_BOND(2, 0x29B9)
#define __DSLWAN_TXDES_SWAP_PTR2              SB_BUFFER_BOND(2, 0x29BA)
#define __DSLWAN_TXDES_SWAP_PTR3              SB_BUFFER_BOND(2, 0x29BB)
#define __DSLWAN_TXDES_SWAP_PTR4              SB_BUFFER_BOND(2, 0x29BC)
#define __DSLWAN_TXDES_SWAP_PTR5              SB_BUFFER_BOND(2, 0x29BD)
#define __DSLWAN_TXDES_SWAP_PTR6              SB_BUFFER_BOND(2, 0x29BE)
#define __DSLWAN_TXDES_SWAP_PTR7              SB_BUFFER_BOND(2, 0x29BF)

// SB_BUFFER_BOND(2, 0x29C0) - SB_BUFFER_BOND(2, 0x29C7)
#define __VLAN_PRI_TO_QID_MAPPING             SB_BUFFER_BOND(2, 0x29C0)

//
// etx MIB: SB_BUFFER_BOND(2, 0x29C8) - SB_BUFFER_BOND(2, 0x29CF)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT0       SB_BUFFER_BOND(2, 0x29C8)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT1       SB_BUFFER_BOND(2, 0x29C9)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT2       SB_BUFFER_BOND(2, 0x29CA)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT3       SB_BUFFER_BOND(2, 0x29CB)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT4       SB_BUFFER_BOND(2, 0x29CC)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT5       SB_BUFFER_BOND(2, 0x29CD)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT6       SB_BUFFER_BOND(2, 0x29CE)
#define __DSLWAN_TX_THRES_DROP_PKT_CNT7       SB_BUFFER_BOND(2, 0x29CF)

#define __CPU_TO_DSLWAN_TX_PKTS               SB_BUFFER_BOND(2, 0x29D0)
#define __CPU_TO_DSLWAN_TX_BYTES              SB_BUFFER_BOND(2, 0x29D1)

#define ACC_ERX_PID                           SB_BUFFER_BOND(2, 0x2B00)
#define ACC_ERX_PORT_TIMES                    SB_BUFFER_BOND(2, 0x2B01)
#define SLL_ISSUED                            SB_BUFFER_BOND(2, 0x2B02)
#define BMC_ISSUED                            SB_BUFFER_BOND(2, 0x2B03)
#define DPLUS_RX_ON                           SB_BUFFER_BOND(2, 0x5003)
#define ISR_IS                                SB_BUFFER_BOND(2, 0x5004)

#define PRESEARCH_RDPTR                       SB_BUFFER_BOND(2, 0x2B06)

#define SLL_ERX_PID                           SB_BUFFER_BOND(2, 0x2B04)
#define SLL_PKT_CNT                           SB_BUFFER_BOND(2, 0x2B08)
#define SLL_RDPTR                             SB_BUFFER_BOND(2, 0x2B0A)

#define EDIT_PKT_CNT                          SB_BUFFER_BOND(2, 0x2B0C)
#define EDIT_RDPTR                            SB_BUFFER_BOND(2, 0x2B0E)

#define DPLUSRX_PKT_CNT                       SB_BUFFER_BOND(2, 0x2B10)
#define DPLUS_RDPTR                           SB_BUFFER_BOND(2, 0x2B12)

#define SLL_STATE_NULL                        0
#define SLL_STATE_DA                          1
#define SLL_STATE_SA                          2
#define SLL_STATE_DA_BC                       3
#define SLL_STATE_ROUTER                      4

#define PRE_DPLUS_PTR                        SB_BUFFER_BOND(2, 0x5001)
#define DPLUS_PTR                            SB_BUFFER_BOND(2, 0x5002)
#define DPLUS_CNT                            SB_BUFFER_BOND(2, 0x5000)
#endif

#define PPA_VDEV_ID(hw_dev_id) (hw_dev_id +   IFX_VIRT_DEV_OFFSET)
#define HW_DEV_ID(ppa_vdev_id) (ppa_vdev_id - IFX_VIRT_DEV_OFFSET)

/*
 * ####################################
 *              Data Type
 * ####################################
 */

/*
 *  Switch Header, Flag Header & Descriptor
 */
#if ! defined(__BIG_ENDIAN)
#error
#endif

struct rx_descriptor {
  //  0 - 3h
  unsigned int    own                 :1; //  0: PPE, 1: MIPS, this value set is for DSL PKT RX and DSL OAM RX, it's special case.
  unsigned int    c                   :1;
  unsigned int    sop                 :1;
  unsigned int    eop                 :1;
  unsigned int    byteoff             :5;
  unsigned int    gid                 :2;
  unsigned int    qid                 :4;
  unsigned int    err                 :1; //  0: no error, 1: error (RA, IL, CRC, USZ, OVZ, MFL, CPI, CPCS-UU)
  unsigned int    datalen             :16;
  //  4 - 7h
  unsigned int    res3                :3;
  unsigned int    dataptr             :29;
};

//    CPU TX is different compare to QoS TX
struct tx_descriptor {
  //  0 - 3h
  unsigned int    own                 :1; //  0: MIPS, 1: PPE, for CPU to WAN TX, it's inverse, 0: PPE, 1: MIPS
  unsigned int    c                   :1;
  unsigned int    sop                 :1;
  unsigned int    eop                 :1;
  unsigned int    byteoff             :5;
  unsigned int    qid                 :4;
  unsigned int    res1                :2;
  unsigned int    small               :1;
  unsigned int    datalen             :16;
  //  4 - 7h
  unsigned int    dataptr             :32;
};

/*
 * ####################################
 *             Declaration
 * ####################################
 */

/*
 *  Network Operations (PTM)
 */
int ifx_ppa_ptm_init(struct net_device *);
#if defined(SETUP_OWN_NETDEVS)
static struct net_device_stats *ptm_get_stats(struct net_device *);
#endif

int ifx_ppa_ptm_open(struct net_device *);
int ifx_ppa_ptm_stop(struct net_device *);
int ifx_ppa_ptm_hard_start_xmit(struct sk_buff *, struct net_device *);
int ifx_ppa_ptm_hard_start_xmit_b(struct sk_buff *, struct net_device *); //bonding

int ptm_ioctl(struct net_device *, struct ifreq *, int);
void ifx_ppa_ptm_tx_timeout(struct net_device *);

/*
 *  DSL Data LED
 */
static INLINE void adsl_led_flash(void);

/*
 *  Network operations help functions
 */
static INLINE int eth_xmit(struct sk_buff *, unsigned int, int, int, int);

/*
 *  ioctl help functions
 */
static INLINE int ethtool_ioctl(struct net_device *, struct ifreq *);

/*
 *  Buffer manage functions
 */
static INLINE struct sk_buff *alloc_skb_rx(void);
static INLINE struct sk_buff *alloc_skb_tx(int);
static struct sk_buff* skb_break_away_from_protocol(struct sk_buff *);
static INLINE void __get_skb_from_dbg_pool(struct sk_buff *, const char *,
		unsigned int);
#define get_skb_from_dbg_pool(skb)  __get_skb_from_dbg_pool(skb, __FUNCTION__, __LINE__)

static INLINE struct sk_buff *__get_skb_pointer(unsigned int, const char *, unsigned int);
#define get_skb_pointer(dataptr)    __get_skb_pointer(dataptr, __FUNCTION__, __LINE__)
static INLINE void __put_skb_to_dbg_pool(struct sk_buff *, const char *,
		unsigned int);
#define put_skb_to_dbg_pool(skb)    __put_skb_to_dbg_pool(skb, __FUNCTION__, __LINE__)

/*
 *  Mailbox handler
 */
static irqreturn_t mailbox1_irq_handler(int, void *);

/*
 *  Turn On/Off Dma
 */
static INLINE void turn_on_dma_rx(int);
// static INLINE void turn_off_dma_rx(int);

/*
 *  DMA interface functions
 */
static u8 *dma_buffer_alloc(int, int *, void **);
static int dma_buffer_free(u8 *, void *);
#if defined(ENABLE_NAPI) && ENABLE_NAPI
#if 0
static void dma_activate_poll(struct dma_device_info *);
static void dma_inactivate_poll(struct dma_device_info *);
#endif
#endif
static int dma_int_handler(struct dma_device_info *, int, int);
static INLINE int dma_rx_int_handler(struct dma_device_info *, int);
static INLINE int handle_skb_e5(struct sk_buff *skb);
static int process_backlog_e5(struct napi_struct *napi, int quota);

/*
 *  Bonding functions
 */
static void switch_to_eth_trunk(void);
static void switch_to_ptmtc_bonding(void);
static void update_max_delay(int);


/*
 *  Hardware init  & clean-up functions
 */
static INLINE void reset_ppe(void);
static INLINE void init_pmu(void);
static INLINE void start_cpu_port(void);
static INLINE void init_internal_tantos_qos_setting(void);
static INLINE void init_internal_tantos(void);
static INLINE void init_dplus(void);
static INLINE void init_pdma(void);
static INLINE void init_mailbox(void);
static INLINE void clear_share_buffer(void);
static INLINE void clear_cdm(void);
static INLINE void board_init(void);
static INLINE void init_dsl_hw(void);
static INLINE void init_hw(void);
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  static INLINE void init_bonding_hw(void);
#endif

/*
 *  PP32 specific functions
 */
static INLINE int pp32_download_code(int, const u32 *, unsigned int,
		const u32 *, unsigned int);
static INLINE int pp32_start(int, int);
static INLINE void pp32_stop(int);

/*
 *  Init & clean-up functions
 */
static INLINE int init_local_variables(void);
static INLINE void clear_local_variables(void);
static INLINE void init_ptm_tables(void);
static INLINE void init_ptm_bonding_tables(void);
static INLINE void init_qos(void);
static INLINE void init_communication_data_structures(int);
static INLINE int alloc_dma(void);
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
static INLINE int alloc_bonding_dma(void);
#endif

static INLINE void free_dma(void);

/*
 *  local implement memcpy
 */
#if defined(ENABLE_MY_MEMCPY) && ENABLE_MY_MEMCPY
static INLINE void *my_memcpy(unsigned char *, const unsigned char *, unsigned int);
#else
#define my_memcpy             memcpy
#endif

/*
 *  Print Firmware Version ID
 */
static int print_fw_ver(char *, int);
static int print_driver_ver(char *, int, char *, unsigned int, unsigned int,
		unsigned int, unsigned int, unsigned int, unsigned int, unsigned int);

/*
 *  DSL Status Script Notification
 */
static void dsl_file_create(void);
static void dsl_file_delete(void);
static int dsl_proc_read_status(char *, char **, off_t, int, int *, void *);
static int dsl_proc_write_status(struct file *, const char *, unsigned long, void *);

/*
 *  Proc File
 */
static INLINE void proc_file_create(void);
static INLINE void proc_file_delete(void);
static int proc_read_ver(char *, char **, off_t, int, int *, void *);
#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC
static INLINE int proc_buf_copy(char **pbuf, int size, off_t off, int *ppos, const char *str, int len);
#endif
#if defined(SETUP_OWN_NETDEVS)
static int proc_read_mib(char *, char **, off_t, int, int *, void *);
static int proc_write_mib(struct file *, const char *, unsigned long, void *);
#endif
static int proc_read_wanmib(char *, char **, off_t, int, int *, void *);
static int proc_write_wanmib(struct file *, const char *, unsigned long, void *);
#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC
static int proc_read_route(char *, char **, off_t, int, int *, void *);
static int proc_write_route(struct file *, const char *, unsigned long, void *);
static int proc_read_pppoe(char *, char **, off_t, int, int *, void *);
static int proc_read_mtu(char *, char **, off_t, int, int *, void *);
static int proc_read_hit(char *, char **, off_t, int, int *, void *);
static int proc_write_hit(struct file *, const char *, unsigned long, void *);
static int proc_read_mac(char *, char **, off_t, int, int *, void *);
static int proc_read_out_vlan(char *, char **, off_t, int, int *, void *);
static int proc_read_ipv6_ip(char *, char **, off_t, int, int *, void *);
#endif
#if defined(DEBUG_MEM_PROC) && DEBUG_MEM_PROC
static int proc_write_mem(struct file *, const char *, unsigned long, void *);
#endif
static int proc_read_burstlen(char *, char **, off_t, int, int *, void *);
static int proc_write_burstlen(struct file *, const char *, unsigned long,
		void *);
#if defined(DEBUG_FW_PROC) && DEBUG_FW_PROC
static int proc_read_fw(char *, char **, off_t, int, int *, void *);
static int proc_write_fw(struct file *, const char *, unsigned long, void *);
static int proc_write_fwdbg(struct file *, const char *, unsigned long, void *);
static void fw_dbg_start(char *cmdbuf);
static void fwdbg_help(char **tokens, int token_num);
static void dump_ipv4_hdr(struct iphdr * iph);
static void dump_ipv6_hdr(struct ipv6hdr *iph6);
#endif
static INLINE unsigned int sw_get_rmon_counter(int, int);
static INLINE void sw_clr_rmon_counter(int);
static INLINE int read_port_counters(int, char *);
static int port0_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port1_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port2_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port3_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port4_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port5_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port6_counters_proc_read(char *, char **, off_t, int, int *, void *);
static int port_counters_proc_write(struct file *, const char *, unsigned long,
		void *);
static int proc_read_directforwarding(char *, char **, off_t, int, int *,
		void *);
static int proc_write_directforwarding(struct file *, const char *,
		unsigned long, void *);
static int proc_read_clk(char *, char **, off_t, int, int *, void *);
static int proc_write_clk(struct file *, const char *, unsigned long, void *);
static int proc_read_prio(char *, char **, off_t, int, int *, void *);
static int proc_write_prio(struct file *, const char *, unsigned long, void *);
static int proc_read_ewan(char *, char **, off_t, int, int *, void *);
static int proc_write_ewan(struct file *, const char *, unsigned long, void *);
#ifdef CONFIG_IFX_PPA_QOS
static int proc_read_qos(char *, char **, off_t, int, int *, void *);
#endif
static int proc_read_gamma(char *, char **, off_t, int, int *, void *);
#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
static int proc_read_mirror(char *, char **, off_t, int, int *, void *);
static int proc_write_mirror(struct file *, const char *, unsigned long, void *);
#endif
static int proc_read_class(char *, char **, off_t, int, int *, void *);
static int proc_write_class(struct file *, const char *, unsigned long, void *);
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  static int proc_read_dev(char *, char **, off_t, int, int *, void *);
  static int proc_write_dev(struct file *, const char *, unsigned long, void *);
#endif
#if defined(DEBUG_BONDING_PROC) && DEBUG_BONDING_PROC
  static void init_ds_des_array(void);
  static int proc_read_bonding(char *, char **, off_t, int, int *, void *);
  static int proc_write_bonding(struct file *, const char *, unsigned long, void *);
#endif

static int dma_alignment_ptm_good_count = 0;
static int dma_alignment_ptm_bad_count = 0;

static int dma_alignment_eth_good_count = 0;
static int dma_alignment_eth_bad_count = 0;
static int padded_frame_count = 0;

/*
 *  Proc File help functions
 */
/*
 *  Debug functions
 */
#if defined(DEBUG_DUMP_INIT) && DEBUG_DUMP_INIT
static INLINE void dump_init(void);
#else
#define dump_init()
#endif
static int swap_mac(unsigned char *);

/*
 *  Directpath Help Functions
 */
#if defined(ENABLE_DIRECTPATH_TX_QUEUE) && ENABLE_DIRECTPATH_TX_QUEUE
static int send_directpath_tx_queue(void);
#endif

/*
 *  Local MAC Address Tracking Functions
 */
static INLINE void register_netdev_event_handler(void);
static INLINE void unregister_netdev_event_handler(void);
static int netdev_event_handler(struct notifier_block *, unsigned long, void *);

/*
 *  Global Functions
 */
static int dsl_showtime_enter(struct port_cell_info *, void *);
static int dsl_showtime_exit(void);

/*
 *  External Functions
 */
//extern int IFX_VR9_Switch_PCE_Micro_Code_Int(void);
#if defined(CONFIG_IFXMIPS_DSL_CPE_MEI) || defined(CONFIG_IFXMIPS_DSL_CPE_MEI_MODULE)
#if !defined(ENABLE_LED_FRAMEWORK) || !ENABLE_LED_FRAMEWORK
/* das ist bei uns aktuell der fall! */
extern int ifx_mei_atm_led_blink(void) __attribute__ ((weak));
#endif
extern int ifx_mei_atm_showtime_check(int *is_showtime,
		struct port_cell_info *port_cell, void **xdata_addr) __attribute__ ((weak));
#else
#if !defined(ENABLE_LED_FRAMEWORK) || !ENABLE_LED_FRAMEWORK
static inline int ifx_mei_atm_led_blink(void) {return IFX_SUCCESS;}
#endif
static inline int ifx_mei_atm_showtime_check(int *is_showtime, struct port_cell_info *port_cell, void **xdata_addr)
{
	if ( is_showtime != NULL )
	*is_showtime = 0;
	return IFX_SUCCESS;
}
#endif


#if defined( AVM_NET_TRACE_DUMP ) && AVM_NET_TRACE_DUMP && defined(CONFIG_AVM_NET_TRACE)
#include <linux/avm_net_trace.h>
#include <linux/avm_net_trace_ioctl.h>

struct avm_net_trace_device ifx_ant_e5 = {
	.name = "ifx_e5_dma2",
	.minor = AVM_NET_TRACE_DYNAMIC_MINOR,
	.iface = 0,
	.type = 1,
	//.pcap_encap = PCAP_ENCAP_ETHER,
	.pcap_encap = 0,
	.pcap_protocol = 0,
};

#endif
/*
 *  External Variables
 */
#if defined(CONFIG_IFXMIPS_DSL_CPE_MEI) || defined(CONFIG_IFXMIPS_DSL_CPE_MEI_MODULE)
/* das ist bei uns der Fall! */
extern int (*ifx_mei_atm_showtime_enter)(struct port_cell_info *, void *) __attribute__ ((weak));
extern int (*ifx_mei_atm_showtime_exit)(void) __attribute__ ((weak));
#else
int (*ifx_mei_atm_showtime_enter)(struct port_cell_info *, void *) = NULL;
EXPORT_SYMBOL(ifx_mei_atm_showtime_enter);
int (*ifx_mei_atm_showtime_exit)(void) = NULL;
EXPORT_SYMBOL(ifx_mei_atm_showtime_exit);
#endif

#if 0
#include "ifxmips_ppa_proc_vr9_e5.c"
#else
static INLINE void get_ip_port(char **p, int *len, unsigned int *val) {
	int i;
	unsigned int tmp_val[17] = { 0 };
	int is_ipv6 = 0;

	memset(val, 0, sizeof(*val) * 6);

	for (i = 0; i < 4; i++) {
		ifx_ignore_space(p, len);
		if (!*len)
			break;
		val[i] = ifx_get_number(p, len, 0);
	}

	if (**p == '.') {
		is_ipv6 = 1;
		for (i = 0; i < 16 - 4; i++) {
			ifx_ignore_space(p, len);
			if (!*len)
				break;
			tmp_val[i] = ifx_get_number(p, len, 0);
		}
	}

	ifx_ignore_space(p, len);
	if (*len)
		val[4] = ifx_get_number(p, len, 0);

	if (is_ipv6) {
		val[5] = 6;
		for (i = 0; i < 16 - 4; i++)
			val[i + 6] = tmp_val[i];
	} else
		val[5] = 4;
}

static INLINE void get_mac(char **p, int *len, unsigned int *val) {
	int i;

	memset(val, 0, sizeof(*val) * 6);

	for (i = 0; i < 6; i++) {
		ifx_ignore_space(p, len);
		if (!*len)
			break;
		val[i] = ifx_get_number(p, len, 1);
	}
}

#if defined(DEBUG_SPACE_BETWEEN_HEADER_AND_DATA)
static inline void dbg_keep_min(unsigned int curr_val, unsigned int *global_val ){
    if (*global_val > curr_val)
        *global_val = curr_val;
}
#endif

#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC
static INLINE char *get_wanitf(int iswan)
{
	static char itfs[64];
	char *allitfs[] = {
		"ETH0",
		"ETH1",
		"", //reserve for CPU
		"EXT1",
		"EXT2",
		"EXT3",
		"EXT4",
		"PTM0",
		NULL
	};

	int wan_itf = *CFG_WAN_PORTMAP;
	int wan_mixmap = *CFG_MIXED_PORTMAP;
	int i, len = 0;

	int max_exts_idx = 4 + 2;

	memset(itfs, 0, sizeof(itfs));
	if(wan_mixmap != 0 && !iswan) { //mix mode dont have lan
		return itfs;
	}

	for(i = 0; allitfs[i] != NULL; i ++) {
		if(i == 2) continue; //skip CPU port

		if((iswan && (wan_itf & (1 << i))) || (!iswan && !(wan_itf & (1 << i)))) {
			if(i > max_exts_idx) {
				if(!iswan) {
					break; //DSL cannot be lan
				}
				else {
					len += sprintf(itfs + len, "%s ", "DSL");
					break; //DSL only listed once although in A5 we have 2 ports support DSL
				}
			}

			len += sprintf(itfs + len, "%s ", allitfs[i]);

			if(iswan && (wan_mixmap & (1 << i))) {
				len += sprintf(itfs + len, "(Mixed) ");
			}
		}
	}

	return itfs;
}

static INLINE int print_route(char *buf, int i, int f_is_lan, struct rout_forward_compare_tbl *pcompare, struct rout_forward_action_tbl *pwaction)
{
	static const char *dest_list[] = {"ETH0", "ETH1", "CPU0", "EXT_INT1", "EXT_INT2", "EXT_INT3", "EXT_INT4", "PTM"};

	int len = 0;
	u32 bit;
	int j, k;

	len += sprintf(buf + len, "  entry %d\n", i);
	len += sprintf(buf + len, "    compare (0x%08X)\n", (u32)pcompare);
	len += sprintf(buf + len, "      src:  %d.%d.%d.%d:%d\n", pcompare->src_ip >> 24, (pcompare->src_ip >> 16) & 0xFF, (pcompare->src_ip >> 8) & 0xFF, pcompare->src_ip & 0xFF, pcompare->src_port);
	len += sprintf(buf + len, "      dest: %d.%d.%d.%d:%d\n", pcompare->dest_ip >> 24, (pcompare->dest_ip >> 16) & 0xFF, (pcompare->dest_ip >> 8) & 0xFF, pcompare->dest_ip & 0xFF, pcompare->dest_port);
	len += sprintf(buf + len, "    action  (0x%08X)\n", (u32)pwaction);
	len += sprintf(buf + len, "      new %s    %d.%d.%d.%d:%d\n", f_is_lan ? "src :" : "dest:", pwaction->new_ip >> 24, (pwaction->new_ip >> 16) & 0xFF, (pwaction->new_ip >> 8) & 0xFF, pwaction->new_ip & 0xFF, pwaction->new_port);
	len += sprintf(buf + len, "      new MAC :    %02X:%02X:%02X:%02X:%02X:%02X\n", (pwaction->new_dest_mac54 >> 8) & 0xFF, pwaction->new_dest_mac54 & 0xFF, pwaction->new_dest_mac30 >> 24, (pwaction->new_dest_mac30 >> 16) & 0xFF, (pwaction->new_dest_mac30 >> 8) & 0xFF, pwaction->new_dest_mac30 & 0xFF);
	switch ( pwaction->rout_type )
	{
		case 1: len += sprintf(buf + len, "      route type:  IP\n"); break;
		case 2: len += sprintf(buf + len, "      route type:  NAT\n"); break;
		case 3: len += sprintf(buf + len, "      route type:  NAPT\n"); break;
		default: len += sprintf(buf + len, "      route type:  NULL\n");
	}
	if ( pwaction->new_dscp_en )
	len += sprintf(buf + len, "      new DSCP:    %d\n", pwaction->new_dscp);
	else
	len += sprintf(buf + len, "      new DSCP:    original (not modified)\n");
	len += sprintf(buf + len, "      MTU index:   %d\n", pwaction->mtu_ix);
	if ( pwaction->in_vlan_ins )
	len += sprintf(buf + len, "      VLAN insert: enable, VCI 0x%04x\n", pwaction->new_in_vci);
	else
	len += sprintf(buf + len, "      VLAN insert: disable\n");
	len += sprintf(buf + len, "      VLAN remove: %s\n", pwaction->in_vlan_rm ? "enable" : "disable");
	if ( !pwaction->dest_list )
	len += sprintf(buf + len, "      dest list:   none\n");
	else
	{
		len += sprintf(buf + len, "      dest list:   ");
		for ( bit = 1, j = k = 0; bit < 1 << 8; bit <<= 1, j++ )
		if ( (pwaction->dest_list & bit) )
		{
			if ( k )
			len += sprintf(buf + len, ", ");
			len += sprintf(buf + len, dest_list[j]);
			k = 1;
		}
		len += sprintf(buf + len, "\n");
	}
	if ( pwaction->pppoe_mode )
	{
		len += sprintf(buf + len, "      PPPoE mode:  termination\n");
		if ( f_is_lan )
		len += sprintf(buf + len, "      PPPoE index: %d\n", pwaction->pppoe_ix);
	}
	else
	len += sprintf(buf + len, "      PPPoE mode:  transparent\n");
	len += sprintf(buf + len, "      new src MAC index: %d\n", pwaction->new_src_mac_ix);
	if(f_is_lan) {
		len += sprintf(buf + len, "      encap tunnel: %s\n", pwaction->encap_tunnel ? "yes":"no");
		len += sprintf(buf + len, "      tunnel iphdr idx: %d\n", pwaction->tnnl_hdr_idx);
	} else {
		len += sprintf(buf + len, "      decap tunnel: %s\n", pwaction->encap_tunnel ? "yes":"no");
	}
	if ( pwaction->out_vlan_ins )
	len += sprintf(buf + len, "      outer VLAN insert: enable, index %d\n", pwaction->out_vlan_ix);
	else
	len += sprintf(buf + len, "      outer VLAN insert: disable\n");
	len += sprintf(buf + len, "      outer VLAN remove: %s\n", pwaction->out_vlan_rm ? "enable" : "disable");
	len += sprintf(buf + len, "      tcp:         %s\n", pwaction->protocol ? "yes" : "no (UDP)");
	len += sprintf(buf + len, "      entry valid: %s\n", pwaction->entry_vld ? "yes" : "no");
	len += sprintf(buf + len, "      dest qid (dslwan qid): %d\n", pwaction->dest_qid);
	len += sprintf(buf + len, "      accl bytes: %u\n", pwaction->bytes);

	return len;
}

static INLINE int print_mc(char *buf, int i, struct wan_rout_multicast_cmp_tbl *pcompare, struct wan_rout_multicast_act_tbl *paction)
{
	static const char *rout_type[] = {"no IP level editing", "TTL editing", "reserved - 2", "reserved - 3"};
	static const char *dest_list[] = {"ETH0", "ETH1", "CPU0", "EXT_INT1", "EXT_INT2", "EXT_INT3", "res", "ATM"};

	int len = 0;
	u32 bit;
	int j, k;

	len += sprintf(buf + len, "  entry %d - %s\n", i, paction->entry_vld ? "valid" : "invalid");
	len += sprintf(buf + len, "    compare (0x%08X)\n", (u32)pcompare);
	len += sprintf(buf + len, "      wan_dest_ip:  %d.%d.%d.%d\n", (pcompare->wan_dest_ip >> 24) & 0xFF, (pcompare->wan_dest_ip >> 16) & 0xFF, (pcompare->wan_dest_ip >> 8) & 0xFF, pcompare->wan_dest_ip & 0xFF);
	len += sprintf(buf + len, "    action  (0x%08X)\n", (u32)paction);
	len += sprintf(buf + len, "      rout_type:    %s\n", rout_type[paction->rout_type]);
	if ( paction->new_dscp_en )
	len += sprintf(buf + len, "      new DSCP:     %d\n", paction->new_dscp);
	else
	len += sprintf(buf + len, "      new DSCP:     original (not modified)\n");
	if ( paction->in_vlan_ins )
	len += sprintf(buf + len, "      VLAN insert:  enable, VCI 0x%04x\n", paction->new_in_vci);
	else
	len += sprintf(buf + len, "      VLAN insert:  disable\n");
	len += sprintf(buf + len, "      VLAN remove:  %s\n", paction->in_vlan_rm ? "enable" : "disable");
	if ( !paction->dest_list )
	len += sprintf(buf + len, "      dest list:    none\n");
	else
	{
		len += sprintf(buf + len, "      dest list:   ");
		for ( bit = 1, j = k = 0; bit < 1 << 8; bit <<= 1, j++ )
		if ( (paction->dest_list & bit) )
		{
			if ( k )
			len += sprintf(buf + len, ", ");
			len += sprintf(buf + len, dest_list[j]);
			k = 1;
		}
		len += sprintf(buf + len, "\n");
	}
	len += sprintf(buf + len, "      PPPoE mode:   %s\n", paction->pppoe_mode ? "termination" : "transparent");
	if ( paction->new_src_mac_en )
	len += sprintf(buf + len, "      new src MAC index: %d\n", paction->new_src_mac_ix);
	else
	len += sprintf(buf + len, "      new src MAC index: disabled\n");
	if ( paction->out_vlan_ins )
	len += sprintf(buf + len, "      outer VLAN insert: enable, index %d\n", paction->out_vlan_ix);
	else
	len += sprintf(buf + len, "      outer VLAN insert: disable\n");
	len += sprintf(buf + len, "      outer VLAN remove: %s\n", paction->out_vlan_rm ? "enable" : "disable");
	len += sprintf(buf + len, "      dest_qid:          %d\n", paction->dest_qid);

	return len;
}
#endif
#endif

/*
 * ####################################
 *            Local Variable
 * ####################################
 */
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
  extern unsigned int g_slave_ppe_base_addr; // avm: is defined in 7port driver
  unsigned int g_pci_dev_ppe_addr_off;
//  EXPORT_SYMBOL(g_pci_dev_ppe_addr_off);
  extern unsigned int g_ppe_base_addr; //avm: is defined in 7port driver
  static unsigned int g_bm_addr;
  static unsigned int g_pci_host_ddr_addr;
  static unsigned int g_bm_ppe_int_addr;
#endif

static int g_fwcode = FWCODE_ROUTING_ACC_D5;
static struct fw_ver_id g_fw_ver[2] = { { 0 } };

static struct semaphore g_sem; //  lock used by open/close function


static int g_eth_wan_mode = 0;

static unsigned int g_wan_itf = 1 << 7;

static int g_ipv6_acc_en = 1;

static int g_wanqos_en = 0;

static int g_dsl_bonding = 0;

static int g_queue_gamma_map[4];

static int g_cpu_to_wan_tx_desc_pos = 0;
static int g_cpu_to_wan_swap_desc_pos = 0;

static u32 g_mailbox_signal_mask;

static int g_f_dma_opened = 0;

static volatile int g_showtime = 0;
static void *g_xdata_addr = NULL;

static int g_line_showtime[LINE_NUMBER] = {0};
static unsigned int g_datarate_us[LINE_NUMBER * BEARER_CHANNEL_PER_LINE] = {0};
static unsigned int g_datarate_ds[LINE_NUMBER * BEARER_CHANNEL_PER_LINE] = {0};
static unsigned int g_max_delay[LINE_NUMBER] = {0};

static unsigned int g_dsl_ig_fast_brg_bytes     = 0;
static unsigned int g_dsl_ig_fast_rt_ipv4_bytes = 0;
static unsigned int g_dsl_ig_fast_rt_ipv6_bytes = 0;
static unsigned int g_dsl_ig_cpu_bytes          = 0;
static unsigned int g_dsl_eg_fast_pkts          = 0;
#if defined(ENABLE_LED_FRAMEWORK) && ENABLE_LED_FRAMEWORK
static void *g_data_led_trigger = NULL;
#endif

static struct net_device *g_ptm_net_dev[1] = { NULL };

static int g_ptm_prio_queue_map[8];

static int g_eth_prio_queue_map[MAX_ETH_INF][8];
static int g_wan_queue_class_map[8];

#if !defined (AVM_SETUP_MAC)
static u8 g_my_ethaddr[MAX_ADDR_LEN * 2] = {0};
#endif

static struct proc_dir_entry *g_dsl_proc_dir = NULL;

#if defined(SETUP_OWN_NETDEVS)
static unsigned int g_mib_itf = 0x83;
#endif

//	1:switch forward packets between port0/1 and cpu port directly w/o processing
//	0:pkts go through normal path and are processed by switch central function
static int g_ingress_direct_forward_enable = 0;
static int g_egress_direct_forward_enable = 1;

static int g_pmac_ewan = 0;

#if defined(ENABLE_NAPI) && ENABLE_NAPI
static int g_napi_enable = 1;
#endif

static int g_directpath_dma_full = 0;

static struct notifier_block g_netdev_event_handler_nb = { 0 };

static unsigned int last_wrx_total_pdu[2][4] = {{0}};
static unsigned int last_wrx_crc_error_total_pdu[2][4] = {{0}};
static unsigned int last_wrx_cv_cw_cnt[2][4] = {{0}};
static unsigned int last_wrx_bc_overdrop_cnt[2][2] = {{0}};

#if defined(DEBUG_SKB_SWAP) && DEBUG_SKB_SWAP
static struct sk_buff *g_dbg_skb_swap_pool[1536 * 100] = {0};
#endif

#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
static struct net_device *g_mirror_netdev = NULL;
#endif

#if defined(DEBUG_BONDING_PROC) && DEBUG_BONDING_PROC
  static struct rx_descriptor saved_us_des_array[512 + 64];
  static struct rx_descriptor us_des[512 + 64];

  static struct rx_descriptor saved_ds_des_array[64 + 64 + 256];    //  DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2 + B1_RX_LINK_LIST_DESC_NUM];  // DMA_TX_CH1, + E1 rx  + Linklist
  static struct rx_descriptor des[64 + 64 + 256];                   //  DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2 + B1_RX_LINK_LIST_DESC_NUM];  // DMA_TX_CH1, + E1 rx  + Linklist
#endif

/*
 * ####################################
 *           Global Variable
 * ####################################
 */

/*
 *  variable for directpath
 */
#if defined(ENABLE_DIRECTPATH_TX_QUEUE)
// replaced by common ppa_push_through_data:
// struct ppe_directpath_data g_ppe_directpath_data[4];
#error AVM_CONCEPT doesnet support DIRECTPATH_TX_QUEUE so far
#endif

/*
 * ####################################
 *            Local Function
 * ####################################
 */

int e5_in_avm_ata_mode(void) {
	// printk(KERN_ERR "[%s]: %s\n", __FUNCTION__, (g_eth_wan_mode == 3 )?"ATA":"NO ATA");
	return (g_eth_wan_mode == 3);
}

int ifx_ppa_ptm_init(struct net_device *dev) {
	char *env;

	ether_setup(dev); /*  assign some members */

	printk(KERN_ERR "[%s] hard_header_len before: %d\n", __func__, dev->hard_header_len);
	dev->hard_header_len += 4;
	printk(KERN_ERR "[%s] hard_header_len changed: %d\n", __func__, dev->hard_header_len);
	printk(KERN_ERR "CPU_TO_WAN_TX_DESC_BASE[0] =%#x\n", (unsigned int)CPU_TO_WAN_TX_DESC_BASE);

	/*  hook network operations */

	printk(KERN_ERR "[%s] line %d\n", __FUNCTION__, __LINE__);

	env = prom_getenv("macdsl");
	if (env) {
		unsigned int a[6];
		sscanf(env, "%x:%x:%x:%x:%x:%x", &a[0], &a[1], &a[2], &a[3], &a[4],
				&a[5]);
		dev->dev_addr[0] = a[0];
		dev->dev_addr[1] = a[1];
		dev->dev_addr[2] = a[2];
		dev->dev_addr[3] = a[3];
		dev->dev_addr[4] = a[4];
		dev->dev_addr[5] = a[5];
		printk(KERN_ERR "[%s] use urlader mac %2x:%2x:%2x:%2x:%2x:%2x \n",
				__FUNCTION__, (unsigned int) dev->dev_addr[0],
				(unsigned int) dev->dev_addr[1],
				(unsigned int) dev->dev_addr[2],
				(unsigned int) dev->dev_addr[3],
				(unsigned int) dev->dev_addr[4],
				(unsigned int) dev->dev_addr[5]);
	} else {
		dev->dev_addr[0] = 0x00;
		dev->dev_addr[1] = 0x20;
		dev->dev_addr[2] = 0xda;
		dev->dev_addr[3] = 0x86;
		dev->dev_addr[4] = 0x23;
		dev->dev_addr[5] = 0xee;
		printk(KERN_ERR "[%s] use default mac %2x:%2x:%2x:%2x:%2x:%2x \n",
				__FUNCTION__, (unsigned int) dev->dev_addr[0],
				(unsigned int) dev->dev_addr[1],
				(unsigned int) dev->dev_addr[2],
				(unsigned int) dev->dev_addr[3],
				(unsigned int) dev->dev_addr[4],
				(unsigned int) dev->dev_addr[5]);
	}
	printk(KERN_ERR "[%s] line %d\n", __FUNCTION__, __LINE__);
	return 0;
}

#if defined(SETUP_OWN_NETDEVS)
static struct net_device_stats *ptm_get_stats(struct net_device *dev)
{
	int port = 7;
	avmnet_netdev_priv_t *priv = netdev_priv(dev);

	priv->stats.rx_packets = priv->rx_packets
	+ ITF_MIB_TBL(port)->ig_fast_brg_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_udp_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_tcp_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_mc_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv6_udp_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv6_tcp_pkts;
	priv->stats.rx_bytes = priv->rx_bytes
	+ ITF_MIB_TBL(port)->ig_fast_brg_bytes
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_bytes
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv6_bytes;
	priv->stats.rx_errors = 0;
	priv->stats.rx_dropped = priv->rx_dropped
	+ ITF_MIB_TBL(port)->ig_drop_pkts;

	priv->stats.tx_packets = priv->tx_packets + ITF_MIB_TBL(port)->eg_fast_pkts;
	priv->stats.tx_bytes = priv->tx_bytes;
	priv->stats.tx_errors = priv->tx_errors;
	priv->stats.tx_dropped = priv->tx_dropped;

	return &priv->stats;

}
#endif

int ifx_ppa_ptm_open(struct net_device *dev) {
	avmnet_netdev_priv_t *priv = netdev_priv(dev);
	dbg(KERN_ERR "[%s] dev=%p, name=%s, priv=%p\n",
			__FUNCTION__, dev, dev->name, priv);

	if (g_eth_wan_mode)
		return -EIO;

#if defined(CONFIG_IP_MULTICAST_FASTFORWARD)
	avmnet_mcfw_enable(dev);
#endif /*--- #if defined(CONFIG_IP_MULTICAST_FASTFORWARD) ---*/

	down(&g_sem);
	netif_start_queue(dev);

	dbg(KERN_ERR "[%s] napi_enable\n", __FUNCTION__);
	napi_enable(&priv->avm_dev->napi);
	// turn_on_dma_rx(7); //DMA lauft durch
	up(&g_sem);

	return 0;
}

int ifx_ppa_ptm_stop(struct net_device *dev) {
	avmnet_netdev_priv_t *priv = netdev_priv(dev);

	down(&g_sem);
	// turn_off_dma_rx(7); //DMA lauft durch

	netif_stop_queue(dev);

	dbg(KERN_ERR "[%s] napi_disbale\n", __FUNCTION__);
	napi_disable(&priv->avm_dev->napi);
	up(&g_sem);
	// turn_off_dma_rx(7); //DMA lauft durch

#if defined(CONFIG_IP_MULTICAST_FASTFORWARD)
	avmnet_mcfw_disable(dev);
#endif /*--- #if defined(CONFIG_IP_MULTICAST_FASTFORWARD) ---*/

	return 0;
}

int ifx_ppa_ptm_hard_start_xmit(struct sk_buff *skb, struct net_device *dev) {
	unsigned long sys_flag;
	unsigned int f_full;
	int desc_base;
	volatile struct tx_descriptor *desc;
	struct tx_descriptor reg_desc = { 0 };
	struct sk_buff *skb_to_free;
	unsigned int byteoff;
    int cache_wback_len;
	avmnet_netdev_priv_t *priv = netdev_priv(dev);
	avmnet_device_t *avm_dev = priv->avm_dev;
    unsigned int priority;

	if (g_stop_datapath != 0)
		goto PTM_HARD_START_XMIT_FAIL;

	if (!g_showtime) {
		err("not in showtime");
		goto PTM_HARD_START_XMIT_FAIL;
	}

	/*  allocate descriptor */
	desc_base = -1;
	f_full = 1;
	if (CPU_TO_WAN_TX_DESC_BASE[g_cpu_to_wan_tx_desc_pos].own == 0) {
		desc_base = g_cpu_to_wan_tx_desc_pos;
		if (++g_cpu_to_wan_tx_desc_pos == CPU_TO_WAN_TX_DESC_NUM)
			g_cpu_to_wan_tx_desc_pos = 0;
		if (CPU_TO_WAN_TX_DESC_BASE[g_cpu_to_wan_tx_desc_pos].own == 0)
			f_full = 0;
	}
	if (f_full) {
		dev->trans_start = jiffies;
		netif_stop_queue(dev);

		if (priv->lantiq_ppe_hold_timeout == 0){
		    priv->lantiq_ppe_hold_timeout = jiffies + (PPE_HOLD_TIMEOUT * HZ);
		} else if ( time_after(jiffies, priv->lantiq_ppe_hold_timeout )){
    		err("PPE hold timeout reached");
    		BUG();
		}

		local_irq_save(sys_flag);
		err("PPE hold, now=%d, timeout=%d", jiffies,priv->lantiq_ppe_hold_timeout );
		SW_WRITE_REG32_MASK(0, CPU_TO_WAN_TX_SIG, MBOX_IGU1_ISRC);
		g_mailbox_signal_mask |= CPU_TO_WAN_TX_SIG;
		SW_WRITE_REG32(g_mailbox_signal_mask, MBOX_IGU1_IER);
		local_irq_restore(sys_flag);
	} else {
		priv->lantiq_ppe_hold_timeout = 0;
	}
	if (desc_base < 0)
		goto PTM_HARD_START_XMIT_FAIL;
	desc = &CPU_TO_WAN_TX_DESC_BASE[desc_base];

	byteoff = (unsigned int) skb->data & (DMA_TX_ALIGNMENT - 1);
	if (skb_headroom(skb) < sizeof(struct sk_buff *) + byteoff || skb_cloned(skb) || (byteoff > 31)) {

		struct sk_buff *new_skb;

		dma_alignment_ptm_bad_count++;

		ASSERT (byteoff < 32, "byteoff zu gross %d ", byteoff);
		ASSERT(skb_headroom(skb) >= sizeof(struct sk_buff *) + byteoff,
				"   skb_headroom < sizeof(struct sk_buff *) +  byteoff \n");

#if defined(CONFIG_AVMNET_DEBUG) 
		if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_ALIGNMENT )
			printk( KERN_ERR
				"\n"
				"   skb_headroom :   %d \n"
				"   sizeof(struct sk_buff *): %d \n"
				"   byteoff:         %d \n"
				"   hard_header_len: %d\n"
				"   skb->head:       %p\n"
				"   skb->data:       %p\n"
				"   in_dev:          %s\n",
					skb_headroom(skb),
					sizeof(struct sk_buff *),
					byteoff,
					dev->hard_header_len,
					skb->head,
					skb->data,
					skb->input_dev?skb->input_dev->name:"no_input_device");
#endif

		ASSERT(!skb_cloned(skb), "skb is cloned");

		new_skb = alloc_skb_tx(DMA_PACKET_SIZE);
		if (new_skb == NULL) {
			dbg("no memory");
			goto ALLOC_SKB_TX_FAIL;
		}
		skb_put(new_skb, skb->len);
		memcpy(new_skb->data, skb->data, skb->len);
		
		/* AVMGRM: priority wegen QoS auch kopieren */
		new_skb->priority = skb->priority; 
		
		dev_kfree_skb_any(skb);
		skb = new_skb;
		byteoff = (unsigned int) skb->data & (DMA_TX_ALIGNMENT - 1);
		/*  write back to physical memory   */
		dma_cache_wback((unsigned long)skb->data, skb->len);
	} else {
		dma_alignment_ptm_good_count++;
        skb_orphan(skb);
	}

	*(struct sk_buff **) ((unsigned int) skb->data - byteoff
			- sizeof(struct sk_buff *)) = skb;
	/*  write back to physical memory   */
	dma_cache_wback(
			(unsigned long)skb->data - byteoff - sizeof(struct sk_buff *),
			skb->len + byteoff + sizeof(struct sk_buff *));

	/*  free previous skb   */
	skb_to_free = get_skb_pointer(desc->dataptr);
	if (skb_to_free != NULL)
		dev_kfree_skb_any(skb_to_free); // hier stuerzt es ab (Unhandled kernel unaligned access)

	/*  detach from protocol    */
	skb_to_free = skb;
	skb = skb_break_away_from_protocol(skb);
	dev_kfree_skb_any(skb_to_free);

	put_skb_to_dbg_pool(skb);

#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
	if (g_mirror_netdev != NULL) {
		struct sk_buff *new_skb = skb_copy(skb, GFP_ATOMIC);

		if (new_skb != NULL) {
			new_skb->dev = g_mirror_netdev;
			dev_queue_xmit(new_skb);
		}
	}
#endif
	/* AVMGRM: Map priority from TC priority */
    priority = (skb->priority & TC_H_MIN_MASK);
	if (priority >= NUM_ENTITY(g_ptm_prio_queue_map) /* 8 */) 
	   priority = NUM_ENTITY(g_ptm_prio_queue_map)-1;

	/*  update descriptor   */
	reg_desc.small =
			(unsigned int) skb->end - (unsigned int) skb->data < DMA_PACKET_SIZE ?
					1 : 0;
	reg_desc.dataptr = (unsigned int) skb->data
			& (0x1FFFFFFF ^ (DMA_TX_ALIGNMENT - 1));
	reg_desc.datalen = skb->len < ETH_ZLEN ? ETH_ZLEN : skb->len;
    reg_desc.qid = priority;
	reg_desc.byteoff = byteoff;
	reg_desc.own = 1;
	reg_desc.c = 1;
	reg_desc.sop = reg_desc.eop = 1;
	ASSERT (byteoff < 32, "byteoff immernoch zu gross %d", byteoff);

	/*  update MIB  */
	avm_dev->device_stats.tx_packets++;
	avm_dev->device_stats.tx_bytes += reg_desc.datalen;

#if defined(DEBUG_QID)
	{
		static unsigned last_qid = ~0;
		static unsigned qid_count = ~0;
		if (last_qid == reg_desc.qid){
			qid_count++;
		}
		else {
			if (qid_count)
				printk(KERN_ERR "[debug_qid] qid %d was repeated for %d times\n", last_qid, qid_count);
			printk(KERN_ERR "[debug_qid] new qid %d \n", reg_desc.qid);
			qid_count = 0;
			last_qid = reg_desc.qid;
		}


	}
#endif

	dump_skb(skb, DUMP_SKB_LEN, (char *) __func__, 0, reg_desc.qid, 1, 0);

    if ( (cache_wback_len = swap_mac(skb->data)) )
        dma_cache_wback((unsigned long)skb->data, cache_wback_len);


	/*  write discriptor to memory  */
	*((volatile unsigned int *) desc + 1) = *((unsigned int *) &reg_desc + 1);
	wmb();
	*(volatile unsigned int *) desc = *(unsigned int *) &reg_desc;
	// dbg("tx_desc =%#x", (unsigned int)desc);

	dev->trans_start = jiffies;

	adsl_led_flash();

	return NETDEV_TX_OK;

	ALLOC_SKB_TX_FAIL: PTM_HARD_START_XMIT_FAIL: dev_kfree_skb_any(skb);
	if (avm_dev)
		avm_dev->device_stats.tx_dropped++;
	return NETDEV_TX_OK;
}

#if 0
static int ptm_hard_start_xmit_b(struct sk_buff *skb, struct net_device *dev)
{
    swap_mac(skb->data);
    //eth_xmit(skb, 7 /* DSL */, 2, 2, 7 - g_ptm_prio_queue_map[skb->priority >= NUM_ENTITY(g_ptm_prio_queue_map) ? NUM_ENTITY(g_ptm_prio_queue_map) - 1 : skb->priority]);
    eth_xmit(skb, 7 /* DSL */, 2, 2, skb->priority);
    return 0;
}
#endif

int ifx_ppa_ptm_ioctl(struct net_device *dev __attribute__ ((unused)),
		struct ifreq *ifr, int cmd) {
	switch (cmd) {
	case IFX_PTM_MIB_CW_GET:
		((PTM_CW_IF_ENTRY_T *) ifr->ifr_data)->ifRxNoIdleCodewords =
				SW_READ_REG32(DREG_AR_CELL0) + SW_READ_REG32(DREG_AR_CELL1);
		((PTM_CW_IF_ENTRY_T *) ifr->ifr_data)->ifRxIdleCodewords =
				SW_READ_REG32(DREG_AR_IDLE_CNT0)
						+ SW_READ_REG32(DREG_AR_IDLE_CNT1);
		((PTM_CW_IF_ENTRY_T *) ifr->ifr_data)->ifRxCodingViolation =
				SW_READ_REG32(DREG_AR_CVN_CNT0)
						+ SW_READ_REG32(DREG_AR_CVN_CNT1)
						+ SW_READ_REG32(DREG_AR_CVNP_CNT0)
						+ SW_READ_REG32(DREG_AR_CVNP_CNT1);
		((PTM_CW_IF_ENTRY_T *) ifr->ifr_data)->ifTxNoIdleCodewords =
				SW_READ_REG32(DREG_AT_CELL0) + SW_READ_REG32(DREG_AT_CELL1);
		((PTM_CW_IF_ENTRY_T *) ifr->ifr_data)->ifTxIdleCodewords =
				SW_READ_REG32(DREG_AT_IDLE_CNT0)
						+ SW_READ_REG32(DREG_AT_IDLE_CNT1);
		break;
	case IFX_PTM_MIB_FRAME_GET: {
		PTM_FRAME_MIB_T data = { 0 };
		int i;

		data.RxCorrect = SW_READ_REG32(DREG_AR_HEC_CNT0)
				+ SW_READ_REG32(DREG_AR_HEC_CNT1)
				+ SW_READ_REG32(DREG_AR_AIIDLE_CNT0)
				+ SW_READ_REG32(DREG_AR_AIIDLE_CNT1);
		for (i = 0; i < 4; i++)
			data.RxDropped += WAN_RX_MIB_TABLE(i)->wrx_dropdes_pdu;
		for (i = 0; i < 8; i++)
			data.TxSend += WAN_TX_MIB_TABLE(i)->wtx_total_pdu;

		*((PTM_FRAME_MIB_T *) ifr->ifr_data) = data;
	}
		break;
	case IFX_PTM_CFG_GET:
		//  use bear channel 0 preemption gamma interface settings
		((IFX_PTM_CFG_T *) ifr->ifr_data)->RxEthCrcPresent = 1;
		((IFX_PTM_CFG_T *) ifr->ifr_data)->RxEthCrcCheck =
				RX_GAMMA_ITF_CFG(0)->rx_eth_fcs_ver_dis == 0 ? 1 : 0;
		((IFX_PTM_CFG_T *) ifr->ifr_data)->RxTcCrcCheck =
				RX_GAMMA_ITF_CFG(0)->rx_tc_crc_ver_dis == 0 ? 1 : 0;
		;
		((IFX_PTM_CFG_T *) ifr->ifr_data)->RxTcCrcLen =
				RX_GAMMA_ITF_CFG(0)->rx_tc_crc_size == 0 ?
						0 : (RX_GAMMA_ITF_CFG(0)->rx_tc_crc_size * 16);
		((IFX_PTM_CFG_T *) ifr->ifr_data)->TxEthCrcGen =
				TX_GAMMA_ITF_CFG(0)->tx_eth_fcs_gen_dis == 0 ? 1 : 0;
		((IFX_PTM_CFG_T *) ifr->ifr_data)->TxTcCrcGen =
				TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size == 0 ? 0 : 1;
		((IFX_PTM_CFG_T *) ifr->ifr_data)->TxTcCrcLen =
				TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size == 0 ?
						0 : (TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size * 16);
		break;
	case IFX_PTM_CFG_SET: {
		int i;

		for (i = 0; i < 4; i++) {
			RX_GAMMA_ITF_CFG(i)->rx_eth_fcs_ver_dis =
					((IFX_PTM_CFG_T *) ifr->ifr_data)->RxEthCrcCheck ? 0 : 1;

			RX_GAMMA_ITF_CFG(0)->rx_tc_crc_ver_dis =
					((IFX_PTM_CFG_T *) ifr->ifr_data)->RxTcCrcCheck ? 0 : 1;

			switch (((IFX_PTM_CFG_T *) ifr->ifr_data)->RxTcCrcLen) {
			case 16:
				RX_GAMMA_ITF_CFG(0)->rx_tc_crc_size = 1;
				break;
			case 32:
				RX_GAMMA_ITF_CFG(0)->rx_tc_crc_size = 2;
				break;
			default:
				RX_GAMMA_ITF_CFG(0)->rx_tc_crc_size = 0;
			}

			TX_GAMMA_ITF_CFG(0)->tx_eth_fcs_gen_dis =
					((IFX_PTM_CFG_T *) ifr->ifr_data)->TxEthCrcGen ? 0 : 1;

			if (((IFX_PTM_CFG_T *) ifr->ifr_data)->TxTcCrcGen) {
				switch (((IFX_PTM_CFG_T *) ifr->ifr_data)->TxTcCrcLen) {
				case 16:
					TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size = 1;
					break;
				case 32:
					TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size = 2;
					break;
				default:
					TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size = 0;
				}
			} else
				TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size = 0;
		}
	}
		break;
	case IFX_PTM_MAP_PKT_PRIO_TO_Q: {
		struct ppe_prio_q_map cmd;

		if (copy_from_user(&cmd, ifr->ifr_data, sizeof(cmd)))
			return -EFAULT;

		if (cmd.pkt_prio
				< 0 || cmd.pkt_prio >= (int) NUM_ENTITY(g_ptm_prio_queue_map))return -EINVAL;

		if (cmd.qid < 0 || cmd.qid >= g_wanqos_en)
			return -EINVAL;

		g_ptm_prio_queue_map[cmd.pkt_prio] = cmd.qid;
	}
		break;
	default:
		return -EOPNOTSUPP;
	}

	return 0;
}

void ifx_ppa_ptm_tx_timeout(struct net_device *dev) {
	unsigned long sys_flag;

	/*  disable TX irq, release skb when sending new packet */
	local_irq_save(sys_flag);
	g_mailbox_signal_mask &= ~CPU_TO_WAN_TX_SIG;
	SW_WRITE_REG32(g_mailbox_signal_mask, MBOX_IGU1_IER);
	local_irq_restore(sys_flag);

	/*  wake up TX queue    */
	netif_wake_queue(dev);

	return;
}

static INLINE void adsl_led_flash(void) {
#if defined(ENABLE_LED_FRAMEWORK) && ENABLE_LED_FRAMEWORK
	if ( g_data_led_trigger != NULL )
	ifx_led_trigger_activate(g_data_led_trigger);
#else
	if (!IS_ERR(&ifx_mei_atm_led_blink) && &ifx_mei_atm_led_blink)
		ifx_mei_atm_led_blink();
	return;
#endif
}

#if 0
/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int ifx_ppa_eth_init_e5(struct net_device *dev)
{
	dbg(KERN_ERR "[%s] %d\n", __FUNCTION__, __LINE__);
	ether_setup(dev); /*  assign some members */

	ifx_ppa_eth_set_mac_address(dev, NULL);
	dbg(KERN_ERR "[%s] %d\n", __FUNCTION__, __LINE__);

	printk("Activating DMA rx channels\n");
	turn_on_dma_rx(1);

	dbg(KERN_ERR "[%s] %d done\n", __FUNCTION__, __LINE__);
	return 0;
}
#endif

#if defined(SETUP_OWN_NETDEVS)
struct net_device_stats *ifx_ppa_eth_get_stats(struct net_device *dev)
{
	int port;
	avmnet_netdev_priv_t *priv = netdev_priv(dev);

	port = get_eth_port(dev);
	if ( port < 0 )
	return NULL;

	priv->stats.rx_packets = priv->rx_packets
	+ ITF_MIB_TBL(port)->ig_fast_brg_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_udp_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_tcp_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_mc_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv6_udp_pkts
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv6_tcp_pkts;
	priv->stats.rx_bytes = priv->rx_bytes
	+ ITF_MIB_TBL(port)->ig_fast_brg_bytes
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv4_bytes
	+ ITF_MIB_TBL(port)->ig_fast_rt_ipv6_bytes;
	priv->stats.rx_errors = 0;
	priv->stats.rx_dropped = priv->rx_dropped
	+ ITF_MIB_TBL(port)->ig_drop_pkts;

	priv->stats.tx_packets = priv->tx_packets + ITF_MIB_TBL(port)->eg_fast_pkts;
	priv->stats.tx_bytes = priv->tx_bytes;
	priv->stats.tx_errors = priv->tx_errors;
	priv->stats.tx_dropped = priv->tx_dropped;

	return &priv->stats;

}

#endif

#if 0 // wird jetzt von AVMNET uebernommen -> DMA lauft einfach durch
int ifx_ppa_eth_open(struct net_device *dev)
{
	int port;

	avmnet_netdev_priv_t *priv = netdev_priv(dev);

	port = get_eth_port(dev);

	dbg(KERN_ERR "[%s]port=%d dev=%p, dev->name=%s\n", __FUNCTION__, port, dev, dev->name);
	if ( port < 0 )
	return -ENODEV;

	down(&g_sem);

	dbg("%s", dev->name);

	netif_start_queue(dev);

	dbg(KERN_ERR "[%s] napi_enable\n", __FUNCTION__);
	napi_enable(&priv->avm_dev->napi);

	turn_on_dma_rx(port);

	up(&g_sem);

#if 0
	/*--- set_mdio_polling(1); ---*/

	for (i = 0; i < 4; i++) {
		ifx_avm_7port_power_ctrl(i, 1);
	}
#endif

	return 0;
}

int ifx_ppa_eth_stop(struct net_device *dev)
{
	int port;
	/*--- int i; ---*/

	avmnet_netdev_priv_t *priv = netdev_priv(dev);

	port = get_eth_port(dev);

	dbg(KERN_ERR "[%s]port=%d \n", __FUNCTION__, port);
	if ( port < 0 )
	return -ENODEV;

#if 0
	/*--- ifx_avm_7port_power_ctrl(port, 0); ---*/
	for (i = 0; i < 4; i++) {
		ifx_avm_7port_power_ctrl(i, 0);
	}
#endif

	down(&g_sem);

	turn_off_dma_rx(port);

	netif_stop_queue(dev);

	dbg(KERN_ERR "[%s] napi_disable\n", __FUNCTION__);
	napi_disable(&priv->avm_dev->napi);

	up(&g_sem);
	return 0;
}
#endif

int ifx_ppa_eth_hard_start_xmit_e5(struct sk_buff *skb, struct net_device *dev) {
	int port;
	int qid;

	dbg_dma("[%s] s\n", __FUNCTION__);
	port = get_eth_port(dev);
	if (port < 0){
		return -ENODEV;
	}

	if (skb->priority >= NUM_ENTITY(g_eth_prio_queue_map[port]))
		qid = g_eth_prio_queue_map[port][NUM_ENTITY(g_eth_prio_queue_map[port])
				- 1];
	else
		qid = g_eth_prio_queue_map[port][skb->priority];

	eth_xmit(skb, port, DMA_CHAN_NR_DEFAULT_CPU_TX , 2, qid); //  spid - CPU, qid - taken from prio_queue_map
	dbg_dma("[%s] e\n", __FUNCTION__);
	return 0;
}
#if 0
int ifx_ppa_eth_qos_hard_start_xmit(struct sk_buff *skb, struct net_device *dev) {
	int if_id;
	int qid;
	unsigned long sys_flag;
	volatile struct tx_descriptor *desc;
	struct tx_descriptor reg_desc;
	struct sk_buff *skb_to_free;
	int byteoff;
	int len;
	struct sw_eg_pkt_header pkth = { 0 };
	avmnet_netdev_priv_t *priv = netdev_priv(dev);
	avmnet_device_t *avm_dev = priv->avm_dev;

	if (g_stop_datapath != 0)
		goto ETH_XMIT_DROP;

	if_id = get_eth_port(dev);
	if (if_id < 0)
		return -ENODEV;

	if (skb->priority >= NUM_ENTITY(g_eth_prio_queue_map[if_id]))
		qid =
				g_eth_prio_queue_map[if_id][NUM_ENTITY(g_eth_prio_queue_map[if_id])
						- 1];
	else
		qid = g_eth_prio_queue_map[if_id][skb->priority];

	pkth.dpid_en = 1;
	pkth.dpid = 0;
	pkth.spid = 2; //  CPU
	pkth.port_map_en = 1;
	pkth.port_map_sel = 1;
	pkth.port_map = (1 << if_id_to_mac_nr(if_id));
	pkth.lrn_dis = 0;
	pkth.class_en = 1;
    pkth.class    = g_wan_queue_class_map[qid];

	len = skb->len <= ETH_MIN_TX_PACKET_LENGTH ?
			ETH_MIN_TX_PACKET_LENGTH : skb->len;

	dump_skb(skb, DUMP_SKB_LEN, "eth_qos_hard_start_xmit avm", if_id, qid, 1,
			0);

	/*  reserve space to put pointer in skb */
	byteoff = (((unsigned int) skb->data - sizeof(pkth)) & (DMA_TX_ALIGNMENT - 1)) + sizeof(pkth);
	//if ( skb_headroom(skb) < sizeof(struct sk_buff *) + byteoff || skb->end - skb->data < 1564 /* 1518 (max ETH frame) + 22 (reserved for firmware) + 10 (AAL5) + 6 (PPPoE) + 4 x 2 (2 VLAN) */ || skb_shared(skb) || skb_cloned(skb) )

	if ((skb_headroom(skb) < sizeof(struct sk_buff *) + byteoff) || skb_shared(skb) || skb_cloned(skb) || (byteoff > 31)|
	   (unsigned int)skb->end - (unsigned int)skb->data < DMA_PACKET_SIZE  )) {
		struct sk_buff *new_skb;

		ASSERT (byteoff < 32, "byteoff too big: %d", byteoff);

		ASSERT(skb_headroom(skb) >= sizeof(struct sk_buff *) + byteoff, "skb_headroom(skb) < sizeof(struct sk_buff *) + byteoff");
		new_skb = alloc_skb_tx(
				sizeof(pkth) + skb->len < DMA_PACKET_SIZE ?
						DMA_PACKET_SIZE : skb->len); //  may be used by RX fastpath later
		if (!new_skb) {
			err("no memory");
			goto ALLOC_SKB_TX_FAIL;
		}
		skb_put(new_skb, sizeof(pkth) + skb->len);
		my_memcpy(new_skb->data, &pkth, sizeof(pkth));
		my_memcpy(new_skb->data + sizeof(pkth), skb->data, skb->len);
		dev_kfree_skb_any(skb);
		skb = new_skb;
		byteoff = (u32) skb->data & (DMA_TX_ALIGNMENT - 1);
#ifndef CONFIG_MIPS_UNCACHED
		/*  write back to physical memory   */
		dma_cache_wback((u32)skb->data, skb->len);
#endif
	} else {
		dbg("enough headroom: %d, byteoff=%d ", skb_headroom(skb), byteoff);
		skb_push(skb, sizeof(pkth));
		byteoff -= sizeof(pkth);
		my_memcpy(skb->data, &pkth, sizeof(pkth));
		*(struct sk_buff **) ((u32) skb->data - byteoff
				- sizeof(struct sk_buff *)) = skb;
#ifndef CONFIG_MIPS_UNCACHED
		/*  write back to physical memory   */
		dma_cache_wback((u32)skb->data - byteoff - sizeof(struct sk_buff *),
				skb->len + byteoff + sizeof(struct sk_buff *));
#endif
	}

	/*  allocate descriptor */
	local_irq_save(sys_flag);
	desc = (volatile struct tx_descriptor *) (CPU_TO_WAN_TX_DESC_BASE + g_cpu_to_wan_tx_desc_pos);
	if (desc->own) //  PPE hold
	{
		local_irq_restore(sys_flag);
		err("PPE hold");
		goto NO_FREE_DESC;
	}
	if (++g_cpu_to_wan_tx_desc_pos == CPU_TO_WAN_TX_DESC_NUM)
		g_cpu_to_wan_tx_desc_pos = 0;
	local_irq_restore(sys_flag);

	/*  load descriptor from memory */
	reg_desc = *desc;

	/*  free previous skb   */
	ASSERT((reg_desc.dataptr & 31) == 0,
			"reg_desc.dataptr (0x%#x) must be 8 DWORDS aligned",
			reg_desc.dataptr);
	skb_to_free = get_skb_pointer(reg_desc.dataptr);
	dev_kfree_skb_any(skb_to_free);

	/*  detach from protocol    */
	skb_to_free = skb;
	skb = skb_break_away_from_protocol(skb);
	dev_kfree_skb_any(skb_to_free);

	put_skb_to_dbg_pool(skb);

#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
	if (g_mirror_netdev != NULL) {
		struct sk_buff *new_skb = skb_copy(skb, GFP_ATOMIC);

		if (new_skb != NULL) {
			skb_pull(new_skb, sizeof(pkth));
			new_skb->dev = g_mirror_netdev;
			dev_queue_xmit(new_skb);
		}
	}
#endif

	/*  update descriptor   */
	reg_desc.small =
			(unsigned int) skb->end - (unsigned int) skb->data < DMA_PACKET_SIZE ?
					1 : 0;
	reg_desc.dataptr = (u32) skb->data & (0x1FFFFFFF ^ (DMA_TX_ALIGNMENT - 1)); //  byte address
	reg_desc.byteoff = byteoff;
	reg_desc.datalen = len + sizeof(pkth);
	reg_desc.qid = qid;
	reg_desc.own = 1;
	reg_desc.c = 1; //  ?
	reg_desc.sop = reg_desc.eop = 1;
	ASSERT(byteoff < 32, "Byteoff still too big: %d ", byteoff);

	/*  update MIB  */

	dev->trans_start = jiffies;

	avm_dev->device_stats.tx_packets++;
	avm_dev->device_stats.tx_bytes += len;

	/*  write discriptor to memory and write back cache */
	*((volatile u32 *) desc + 1) = *((u32 *) &reg_desc + 1);
	*(volatile u32 *) desc = *(u32 *) &reg_desc;

	return 0;

	NO_FREE_DESC: ALLOC_SKB_TX_FAIL: ETH_XMIT_DROP: dev_kfree_skb_any(skb);
	if (avm_dev)
		avm_dev->device_stats.tx_dropped++;
	return 0;
}
#endif

int ifx_ppa_eth_ioctl_e5(struct net_device *dev, struct ifreq *ifr, int cmd) {
	int port;

	port = get_eth_port(dev);
	if (port < 0)
		return -ENODEV;

	switch (cmd) {
	case SIOCETHTOOL:
		return ethtool_ioctl(dev, ifr);
	case SET_VLAN_COS: {
		struct vlan_cos_req vlan_cos_req;

		if (copy_from_user(&vlan_cos_req, ifr->ifr_data, sizeof(struct vlan_cos_req)))
			return -EFAULT;
	}
		break;
	case SET_DSCP_COS: {
		struct dscp_cos_req dscp_cos_req;

		if (copy_from_user(&dscp_cos_req, ifr->ifr_data, sizeof(struct dscp_cos_req)))
			return -EFAULT;
	}
		break;
	case ETH_MAP_PKT_PRIO_TO_Q: {
		struct ppe_prio_q_map cmd;

		if (copy_from_user(&cmd, ifr->ifr_data, sizeof(cmd)))
			return -EFAULT;

		if (cmd.pkt_prio
				< 0 || cmd.pkt_prio >= (int) NUM_ENTITY(g_eth_prio_queue_map[port]))return -EINVAL;

		if (cmd.qid < 0)
			return -EINVAL;
		if (cmd.qid
				>= ((g_wan_itf & (1 << port)) && g_wanqos_en
						&& (g_eth_wan_mode == 3) ? g_wanqos_en : SWITCH_CLASS_VALUE))
			return -EINVAL;

		g_eth_prio_queue_map[port][cmd.pkt_prio] = cmd.qid;
	}
		break;
	default:
		return -EOPNOTSUPP;
	}

	return 0;
}

void ifx_ppa_eth_tx_timeout_e5(struct net_device *dev) {
	unsigned long sys_flag;
	//int port;
	avmnet_netdev_priv_t *priv = netdev_priv(dev);
	avmnet_device_t *avm_dev = priv->avm_dev;

	avm_dev->device_stats.tx_errors++;

	local_irq_save(sys_flag);
	g_dma_device_ppe->tx_chan[2]->disable_irq(g_dma_device_ppe->tx_chan[2]);
	netif_wake_queue(g_eth_net_dev[0]);
	if (!(g_eth_wan_mode == 3 /* ETH WAN */&& g_wanqos_en))
		netif_wake_queue(g_eth_net_dev[1]);
	local_irq_restore(sys_flag);

	return;
}

static INLINE int eth_xmit(struct sk_buff *skb, unsigned int dpid, int ch,
		int spid __attribute__ ((unused)), int prio __attribute__ ((unused))) {
	struct sw_eg_pkt_header pkth = { 0 };
	int len;
	unsigned int headroom;
	avmnet_netdev_priv_t *priv = NULL;
	avmnet_device_t *avm_dev = NULL;
    struct skb_shared_info *shinfo;

	if (g_stop_datapath != 0)
		goto ETH_XMIT_DROP;


	if (skb->len <= ETH_MIN_TX_PACKET_LENGTH ){
		/*--- 0byte padding behind skb->len, skb->len is not increased ---*/
		if ( skb_pad(skb, ETH_MIN_TX_PACKET_LENGTH - skb->len )){
			skb = NULL;
			goto ETH_XMIT_DROP;
		}
		len = ETH_MIN_TX_PACKET_LENGTH;
		padded_frame_count++;
	} else {
		len = skb->len;
	}

#if defined (CONFIG_AVM_SCATTER_GATHER)
    /* If packet is not checksummed and device does not support
    * checksumming for this protocol, complete checksumming here.
    */
    if (skb->ip_summed == CHECKSUM_PARTIAL) {
        if(unlikely(skb_checksum_help(skb))){
            err(KERN_ERR "skb_checksum fails\n");
            goto ETH_XMIT_DROP;
        }
    }
#endif

#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
	if (g_mirror_netdev != NULL && (g_wan_itf & (1 << dpid))) {
		struct sk_buff *new_skb = skb_copy(skb, GFP_ATOMIC);

		if (new_skb != NULL) {
			new_skb->dev = g_mirror_netdev;
			dev_queue_xmit(new_skb);
		}
	}
#endif


	dump_skb(skb, DUMP_SKB_LEN, "eth_xmit - raw data avm", dpid, ch, 1, 0);

	if ( spid == 2 ){ // don't track directpath traffic here
		if ( (dpid < NUM_ENTITY(g_eth_net_dev)) || ( dpid == 7 )) {
		    struct net_device *dev = NULL;
		    if ( dpid < NUM_ENTITY(g_eth_net_dev) )
		        dev = g_eth_net_dev[dpid];
		    else if (dpid == 7)
		        dev = g_ptm_net_dev[0];

		    if ( dev ){
                priv = netdev_priv(g_eth_net_dev[dpid]);
                g_eth_net_dev[dpid]->trans_start = jiffies;
                avm_dev = priv->avm_dev;
                avm_dev->device_stats.tx_packets++;
                avm_dev->device_stats.tx_bytes += len;
#if defined(CONFIG_IP_MULTICAST_FASTFORWARD)
                if(likely(priv->mcfw_used)){
                    mcfw_portset set;
                    mcfw_portset_reset(&set);
                    mcfw_portset_port_add(&set, 0);
                    (void) mcfw_snoop_send(&priv->mcfw_netdrv, set, skb);
                }
#endif /*--- #if defined(CONFIG_IP_MULTICAST_FASTFORWARD) ---*/
		    }
		}
	}
	//TODO add directpath traffic statistics

	headroom = sizeof(struct sw_eg_pkt_header);

	if (skb_headroom(skb) < headroom) {
		struct sk_buff *new_skb;

		dma_alignment_eth_bad_count++;
		new_skb = alloc_skb_tx(len + sizeof(struct sw_eg_pkt_header));
		if (!new_skb)
			goto ETH_XMIT_DROP;
		else {
			skb_put(new_skb, len + sizeof(struct sw_eg_pkt_header));
			memcpy(new_skb->data + sizeof(struct sw_eg_pkt_header), skb->data, len);
			dev_kfree_skb_any(skb);
			skb = new_skb;
		}
	} else {
		skb_push(skb, sizeof(struct sw_eg_pkt_header));
		dma_alignment_eth_good_count++;

		skb_orphan( skb );
	}

	len += sizeof(struct sw_eg_pkt_header);

	if (spid == 2) {

		// for cpu -> ethernet
		pkth.spid = 2; //  CPU
		pkth.dpid_en = 1;
		pkth.dpid = 1; // 0 or 1 for PORT_MAP_MODE
		pkth.port_map_en = 1;
		pkth.port_map_sel = 1;
		pkth.port_map = (1 << if_id_to_mac_nr(dpid));
		pkth.lrn_dis = 0;
		pkth.class_en = 0;
		pkth.class = 0;
		dbg_dma("port_map: %#x", (unsigned int)pkth.port_map);

	} else {

		// for directpath Loop to CPU (might be interrupted by PCE-Rule -> Ethernet , or PPA -> WAN)
		pkth.spid = spid;
		pkth.dpid_en = 0;
		pkth.dpid = 1;          // don_t care
		pkth.port_map_en = 0;   // don_t care
		pkth.port_map_sel = 0;  // don_t care
		pkth.lrn_dis = 1;
		pkth.class_en = 0;
		pkth.class = prio;
		pkth.port_map = (0x3F);

	}

	memcpy(skb->data, &pkth, sizeof(struct sw_eg_pkt_header));

	put_skb_to_dbg_pool(skb);

	/*--- skb_pad linearizes our skb; so we shouldnt lookup shinfo before this line ---*/
	shinfo = skb_shinfo(skb);
#if defined (CONFIG_AVM_SCATTER_GATHER)
    if(shinfo && shinfo->nr_frags > 0 ) {
        unsigned long lock_flags;

        // this skb is scattered
        dbg_trace_ppa_data("send first fragment (nr_frags = %d)",shinfo->nr_frags );
        lock_flags = acquire_channel_lock(g_dma_device_ppe, ch );

        if(unlikely(send_skb_scattered_first(skb, g_dma_device_ppe , ch))) {
            err("sending first fragment failed");
            free_channel_lock(g_dma_device_ppe, ch, lock_flags);
            goto ETH_XMIT_DROP;
        }
        if(unlikely(send_skb_scattered_successors(skb, g_dma_device_ppe, ch))) {
            err("ah we could just send some of the fragmets, this must not happen!");
            // we cannot do any skb free now :-(
            free_channel_lock(g_dma_device_ppe, ch, lock_flags);
            return 0;
        }
        free_channel_lock(g_dma_device_ppe, ch, lock_flags );
    }
#else
    if(unlikely(shinfo && ( shinfo->nr_frags > 0 ))) {
        err("we don't support Scatter Gather, but we got a scattered SKB");
        goto ETH_XMIT_DROP;
    }
#endif

    else {
    	if (dma_device_write(g_dma_device_ppe, skb->data, len, skb, ch) != len)
		goto ETH_XMIT_DROP;
    }

	return 0;

	ETH_XMIT_DROP: dev_kfree_skb_any(skb);
	if (avm_dev)
		avm_dev->device_stats.tx_dropped++;
	return -1;
}

/*
 *  Description:
 *    Handle ioctl command SIOCETHTOOL.
 *  Input:
 *    dev --- struct net_device *, device responsing to the command.
 *    ifr --- struct ifreq *, interface request structure to pass parameters
 *            or result.
 *  Output:
 *    int --- 0:    Success
 *            else: Error Code (-EFAULT, -EOPNOTSUPP)
 */
static INLINE int ethtool_ioctl(struct net_device *dev __attribute__ ((unused)),
		struct ifreq *ifr) {
	struct ethtool_cmd cmd;

	if (copy_from_user(&cmd, ifr->ifr_data, sizeof(cmd)))
		return -EFAULT;

	switch (cmd.cmd) {
	case ETHTOOL_GSET: /*  get hardware information        */
	{
		memset(&cmd, 0, sizeof(cmd));

		if (copy_to_user(ifr->ifr_data, &cmd, sizeof(cmd)))
			return -EFAULT;
	}
		break;
	case ETHTOOL_SSET: /*  force the speed and duplex mode */
	{
		if (!capable(CAP_NET_ADMIN))
			return -EPERM;

		if (cmd.autoneg == AUTONEG_ENABLE) {
			/*  set property and start autonegotiation                                  */
			/*  have to set mdio advertisement register and restart autonegotiation     */
			/*  which is a very rare case, put it to future development if necessary.   */
		} else {
		}
	}
		break;
	case ETHTOOL_GDRVINFO: /*  get driver information          */
	{
		struct ethtool_drvinfo info;
		char str[32];

		memset(&info, 0, sizeof(info));
		strncpy(info.driver, "Danube Eth Driver (A4)", sizeof(info.driver) - 1);
		sprintf(str, "%d.%d.%d.%d.%d.%d", (int) FW_VER_ID->family,
				(int) FW_VER_ID->fwtype, (int) FW_VER_ID->interface,
				(int) FW_VER_ID->fwmode, (int) FW_VER_ID->major,
				(int) FW_VER_ID->minor);
		strncpy(info.fw_version, str, sizeof(info.fw_version) - 1);
		strncpy(info.bus_info, "N/A", sizeof(info.bus_info) - 1);
		info.regdump_len = 0;
		info.eedump_len = 0;
		info.testinfo_len = 0;
		if (copy_to_user(ifr->ifr_data, &info, sizeof(info)))
			return -EFAULT;
	}
		break;
	case ETHTOOL_NWAY_RST: /*  restart auto negotiation        */
		break;
	default:
		return -EOPNOTSUPP;
	}

	return 0;
}

// ----------------------------  tx_alloc ----------------------------------
static INLINE struct sk_buff *alloc_skb_tx(int len)
{
    struct sk_buff *skb;
    unsigned int align_off;

    skb = dev_alloc_skb(len + DMA_TX_ALIGNMENT);

    if(likely(skb)){
        align_off = DMA_TX_ALIGNMENT - ((u32)skb->data & (DMA_TX_ALIGNMENT - 1));
        skb_reserve(skb, align_off);

        *((u32 *)skb->data - 1) = (u32)skb;
#ifndef CONFIG_MIPS_UNCACHED
        dma_cache_wback((u32)skb->data - sizeof(u32), sizeof(u32));
#endif
    }

    return skb;
}

// ----------------------------  rx_alloc ----------------------------------
static INLINE struct sk_buff *alloc_skb_rx(void)
{
    struct sk_buff *skb;
    unsigned int align_off;

    skb = dev_alloc_skb(DMA_PACKET_SIZE + DMA_RX_ALIGNMENT);
    if(likely(skb)){
        align_off = DMA_RX_ALIGNMENT - ((u32)skb->data & (DMA_RX_ALIGNMENT - 1));
        skb_reserve(skb, align_off);

        *((u32 *)skb->data - 1) = (u32)skb;
#ifndef CONFIG_MIPS_UNCACHED
        dma_cache_wback((u32)skb->data - sizeof(u32), sizeof(u32));
#endif
    }

    return skb;
}

static struct sk_buff* skb_break_away_from_protocol(struct sk_buff *skb) {
	struct sk_buff *new_skb;

	if (skb_shared(skb)) {
		new_skb = skb_clone(skb, GFP_ATOMIC);
		if (new_skb == NULL)
			return NULL;
	} else
		new_skb = skb_get(skb);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,32)
	dst_release(new_skb->dst);
	new_skb->dst = NULL;
#else
	skb_dst_drop(new_skb);
#endif
#ifdef CONFIG_XFRM
	secpath_put(new_skb->sp);
	new_skb->sp = NULL;
#endif
    if ( new_skb->destructor )
    {
        WARN_ON(in_irq());
        new_skb->destructor(new_skb);
        new_skb->destructor = NULL;
    }
#if !defined(CONFIG_IFX_PPA_AVM_USAGE) && (defined(CONFIG_NETFILTER) || LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,32))
	nf_conntrack_put(new_skb->nfct);
	new_skb->nfct = NULL;
#if defined(CONFIG_NF_CONNTRACK) || defined(CONFIG_NF_CONNTRACK_MODULE)
	nf_conntrack_put_reasm(new_skb->nfct_reasm);
	new_skb->nfct_reasm = NULL;
#endif
#ifdef CONFIG_BRIDGE_NETFILTER
	nf_bridge_put(new_skb->nf_bridge);
	new_skb->nf_bridge = NULL;
#endif
#endif

	return new_skb;
}

static INLINE void __get_skb_from_dbg_pool(struct sk_buff *skb __attribute__((unused)),
		const char *func_name __attribute__((unused)), unsigned int line_num __attribute__((unused))) {
#if defined(DEBUG_SKB_SWAP) && DEBUG_SKB_SWAP
	int i;

	for ( i = 0; i < NUM_ENTITY(g_dbg_skb_swap_pool) && g_dbg_skb_swap_pool[i] != skb; i++ );
	if ( i == NUM_ENTITY(g_dbg_skb_swap_pool) || (unsigned int)skb < KSEG0 )
	{
		err("%s:%d: skb (0x%08x) not in g_dbg_skb_swap_pool", func_name, line_num, (unsigned int)skb);
	}
	else
	g_dbg_skb_swap_pool[i] = NULL;
#endif
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/

static INLINE struct sk_buff *__get_skb_pointer(unsigned int dataptr, const char *func_name, unsigned int line_num)
{
	unsigned int skb_dataptr;
	struct sk_buff *skb;

	//  usually, CPE memory is less than 256M bytes
	//  so NULL means invalid pointer
	if ( dataptr == 0 ) {
		dbg("dataptr is 0, it's supposed to be invalid pointer, caller %pF", __builtin_return_address(0));
		return NULL;
	}

	skb_dataptr = (dataptr - 4) | KSEG1;
	skb = *(struct sk_buff **)skb_dataptr;

	__get_skb_from_dbg_pool(skb, func_name, line_num);

	ASSERT((unsigned int)skb >= KSEG0, "%s:%d: invalid skb - skb = %#08x, dataptr = %#08x", func_name, line_num, (unsigned int)skb, dataptr);
	ASSERT( (((unsigned int)skb->data & (0x1FFFFFFF ^ (DMA_RX_ALIGNMENT - 1))) | KSEG1) == (dataptr | KSEG1) ||
			(((unsigned int)skb->data & (0x1FFFFFFF ^ (DMA_TX_ALIGNMENT - 1))) | KSEG1) == (dataptr | KSEG1),
			"%s:%d: invalid skb - skb = %#08x, skb->data = %#08x, dataptr = %#08x",
			func_name, line_num, (unsigned int)skb, (unsigned int)skb->data, dataptr
			);

	return skb;
}

static INLINE void __put_skb_to_dbg_pool(struct sk_buff *skb __attribute__((unused)),
		const char *func_name __attribute__((unused)), unsigned int line_num __attribute__((unused))) {
#if defined(DEBUG_SKB_SWAP) && DEBUG_SKB_SWAP
	unsigned int i;

	for ( i = 0; i < NUM_ENTITY(g_dbg_skb_swap_pool) && g_dbg_skb_swap_pool[i] != NULL; i++ );
	if ( i == NUM_ENTITY(g_dbg_skb_swap_pool) )
	{
		err("%s:%d: g_dbg_skb_swap_pool overrun", func_name, line_num);
	}
	else
	g_dbg_skb_swap_pool[i] = skb;
#endif
}

static irqreturn_t mailbox0_irq_handler(int irq __attribute__((unused)), void *dev_id __attribute__((unused))) {
	u32 mailbox_signal = 0;

	mailbox_signal = *MBOX_IGU0_ISR & *MBOX_IGU0_IER;

#if defined(CONFIG_AVMNET_DEBUG) 
	if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
		printk(KERN_ERR "you've got mail in box0 ;)\n");
#endif

	if (!mailbox_signal)
		return IRQ_HANDLED;
	*MBOX_IGU0_ISRC = mailbox_signal;

	if ((mailbox_signal & DMA_TX_CH0_SIG)) {
		// g_dma_device_ppe->tx_chan[0]->open(g_dma_device_ppe->tx_chan[0]);
		*MBOX_IGU0_IER &= DMA_TX_CH1_SIG;

#if defined(CONFIG_AVMNET_DEBUG) 
		if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
			printk(KERN_ERR "DMA_TX_CH0_SIG\n"); // showtime signal
#endif
	}

	if ((mailbox_signal & DMA_TX_CH1_SIG)) {
		g_dma_device_ppe->tx_chan[1]->open(g_dma_device_ppe->tx_chan[1]);
		*MBOX_IGU0_IER &= DMA_TX_CH0_SIG;

#if defined(CONFIG_AVMNET_DEBUG) 
		if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
			printk(KERN_ERR "DMA_TX_CH1_SIG\n");
#endif
	}

	return IRQ_HANDLED;
}

/*
 *  Description:
 *    Handle IRQ of mailbox and despatch to relative handler.
 *  Input:
 *    irq    --- int, IRQ number
 *    dev_id --- void *, argument passed when registering IRQ handler
 *    regs   --- struct pt_regs *, registers' value before jumping into handler
 *  Output:
 *    none
 */
static irqreturn_t mailbox1_irq_handler(int irq __attribute__ ((unused)), void *dev_id __attribute__ ((unused))) {
	u32 mailbox_signal = 0;

#if defined(CONFIG_AVMNET_DEBUG) 
	if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
		printk(KERN_ERR "you've got mail in box1 ");
#endif

	mailbox_signal = *MBOX_IGU1_ISR & g_mailbox_signal_mask;
	if (!mailbox_signal)
		return IRQ_HANDLED;
	*MBOX_IGU1_ISRC = mailbox_signal;

	if ((mailbox_signal & WAN_RX_SIG(0))) //  PTM Packet
	{
#if defined(CONFIG_AVMNET_DEBUG) 
		if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
			printk(KERN_ERR "WAN_RX_SIG(0)\n"); //showtime signal
#endif

		g_mailbox_signal_mask &= ~WAN_RX_SIG(0);
		g_dma_device_ppe->tx_chan[1]->open(g_dma_device_ppe->tx_chan[1]);
		*MBOX_IGU1_IER = g_mailbox_signal_mask;
	}

	if ((mailbox_signal & CPU_TO_WAN_SWAP_SIG)) {
		struct sk_buff *new_skb;
		volatile struct tx_descriptor *desc = &CPU_TO_WAN_SWAP_DESC_BASE[g_cpu_to_wan_swap_desc_pos];
		struct tx_descriptor reg_desc = { 0 };
		struct sk_buff *skb_to_free;

#if defined(CONFIG_AVMNET_DEBUG) 
		if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
			printk(KERN_ERR "CPU_TO_WAN_SWAP_SIG\n");
#endif
		while (desc->own == 0) {
			new_skb = alloc_skb_rx();
			if (new_skb == NULL)
				break;

#ifndef CONFIG_MIPS_UNCACHED
			/*  invalidate cache    */
			dma_cache_inv((unsigned long)new_skb->data, DMA_PACKET_SIZE);
#endif

			if (++g_cpu_to_wan_swap_desc_pos == CPU_TO_WAN_SWAP_DESC_NUM)
				g_cpu_to_wan_swap_desc_pos = 0;

			/*  free previous skb   */
			skb_to_free = get_skb_pointer(desc->dataptr);
			if (skb_to_free != NULL)
				dev_kfree_skb_any(skb_to_free);
			put_skb_to_dbg_pool(new_skb);

			/*  update descriptor   */
			reg_desc.dataptr = (unsigned int) new_skb->data & (0x1FFFFFFF ^ (DMA_TX_ALIGNMENT - 1));
			reg_desc.own = 1;

			/*  write discriptor to memory  */
			*((volatile unsigned int *) desc + 1) = *((unsigned int *) &reg_desc + 1);
			wmb();
			*(volatile unsigned int *) desc = *(unsigned int *) &reg_desc;

			desc = &CPU_TO_WAN_SWAP_DESC_BASE[g_cpu_to_wan_swap_desc_pos];
		}
	}

	if ((mailbox_signal & CPU_TO_WAN_TX_SIG)) {
#if defined(CONFIG_AVMNET_DEBUG) 
		if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_MAILBOX )
			printk(KERN_ERR "CPU_TO_WAN_TX_SIG\n");
#endif
		g_mailbox_signal_mask &= ~CPU_TO_WAN_TX_SIG;
		SW_WRITE_REG32(g_mailbox_signal_mask, MBOX_IGU1_IER);
		if (g_ptm_net_dev[0]) {
			err(" Mailbox: wake ptm dev\n");
			netif_wake_queue(g_ptm_net_dev[0]);
		}
	}

	return IRQ_HANDLED;
}

static INLINE void turn_on_dma_rx(int mask) {
	unsigned int i;

	if (!g_f_dma_opened) {

		dbg("[%s] turn on all channels\n", __func__);
		ASSERT((u32)g_dma_device_ppe >= 0x80000000, "g_dma_device_ppe = 0x%08X",
				(u32)g_dma_device_ppe);

		for (i = 0; i < g_dma_device_ppe->max_rx_chan_num; i++) {

			ASSERT((u32)g_dma_device_ppe->rx_chan[i] >= 0x80000000, "g_dma_device_ppe->rx_chan[%d] = 0x%08X", i, (u32)g_dma_device_ppe->rx_chan[i]);
			ASSERT(g_dma_device_ppe->rx_chan[i]->control == IFX_DMA_CH_ON, "g_dma_device_ppe->rx_chan[i]->control = %d", g_dma_device_ppe->rx_chan[i]->control);

			if (g_dma_device_ppe->rx_chan[i]->control == IFX_DMA_CH_ON) {
				ASSERT((u32)g_dma_device_ppe->rx_chan[i]->open >= 0x80000000, "g_dma_device_ppe->rx_chan[%d]->open = 0x%08X", i, (u32)g_dma_device_ppe->rx_chan[i]->open);

				//  channel 1, 2 is for fast path
/*---
 * Seems to be a lantiq hack
 *  1) put channel-config to tx-mode
 *  2) then open it (no irq is not enabled but channel is activated)
 *  3) put channnel-config to undefined
 * We use normal open and no_cpu_data flag for setting up directpath channels
 */
#if 0
				if (i == 1 || i == 2)
					g_dma_device_ppe->rx_chan[i]->dir = 1; // 1 : TX-Channel
#endif
				g_dma_device_ppe->rx_chan[i]->open( g_dma_device_ppe->rx_chan[i] );
#if 0
				if (i == 1 || i == 2)
					g_dma_device_ppe->rx_chan[i]->dir = -1; // -1: not defined
#endif
			}
		}
	}
	g_f_dma_opened |= 1 << mask;
	dbg("[%s] g_f_dma_opened = %#x\n", __func__, g_f_dma_opened);
}

#if 0
// dma lauft bei uns dauerhaft
static INLINE void turn_off_dma_rx(int mask) {
	int i;

	g_f_dma_opened &= ~(1 << mask);
	if (!g_f_dma_opened) {
		dbg("[%s] turn off all rx channels\n", __func__);
		for (i = 0; i < g_dma_device_ppe->max_rx_chan_num; i++)
			g_dma_device_ppe->rx_chan[i]->close(g_dma_device_ppe->rx_chan[i]);
	}
	dbg("[%s] g_f_dma_opened = %#x\n", __func__, g_f_dma_opened);
}
#endif

static u8 *dma_buffer_alloc(int len __attribute__ ((unused)), int *byte_offset,
		void **opt) {
	u8 *buf;
	struct sk_buff *skb;

	skb = alloc_skb_rx();
	if (!skb)
		return NULL;

	put_skb_to_dbg_pool(skb);

	buf = (u8 *) skb->data;
	*(u32 *) opt = (u32) skb;
	*byte_offset = 0;
	return buf;
}

static int dma_buffer_free(u8 *dataptr __attribute__((unused)), void *opt) {
	//  ! be careful
	//    this method makes kernel crash when free this DMA device
	//    since the pointers used by fastpast which is supposed not visible to CPU
	//    mix up the pointers so that "opt" is not align with "dataptr".

//    ASSERT(*(void **)((((u32)dataptr | KSEG1) & ~0x0F) - 4) == opt, "address is not match: dataptr = %08X, opt = %08X, *(void **)((((u32)dataptr | KSEG1) & ~0x0F) - 4) = %08X", (u32)dataptr, (u32)opt, *(u32 *)((((u32)dataptr | KSEG1) & ~0x0F) - 4));

	if (opt) {
		get_skb_from_dbg_pool((struct sk_buff *)opt);
		dev_kfree_skb_any((struct sk_buff *) opt);
	}

	return 0;
}

static int dma_int_handler(struct dma_device_info *dma_dev, int status, int dma_chan_no) {
	int ret = 0;
	unsigned long sys_flag;
#if !defined(ENABLE_DIRECTPATH_TX_QUEUE) || !ENABLE_DIRECTPATH_TX_QUEUE
	unsigned int i;
#endif
	dbg_dma("[%s] stat=%d s\n", __FUNCTION__, status);

	switch (status) {
	case RCV_INT:
		ret = dma_rx_int_handler(dma_dev, dma_chan_no);
		break;
	case TX_BUF_FULL_INT:
		if (dma_chan_no == 2) {
			unsigned int dev_nr;
			dbg("eth0/1 TX buffer full!");
			local_irq_save(sys_flag);

            /*
             * TODO vgl. UGW - hier wurden trans_start Zeiten geswapt - warum? -
             * bei mehr als 2 devices ist das nicht mehr trivial -
             */

            for (dev_nr = 0; dev_nr < num_registered_eth_dev; dev_nr++){
                struct net_device *dev = g_eth_net_dev[dev_nr];
                avmnet_netdev_priv_t *priv = netdev_priv(dev);
            	avmnet_device_t *avm_dev = avm_dev = priv->avm_dev;
            	avm_dev->flags |= AVMNET_DEVICE_FLAG_DMA_TX_QUEUE_FULL;
            	netif_stop_queue(dev);
            }

			if (g_dma_device_ppe->tx_chan[2]->control == IFX_DMA_CH_ON)
				g_dma_device_ppe->tx_chan[2]->enable_irq(
						g_dma_device_ppe->tx_chan[2]);
			local_irq_restore(sys_flag);
		} else if (dma_chan_no == DMA_CHAN_NR_DIRECTPATH_CPU_TO_PPE ) {
			dbg("directpath TX (CPU->PPE) buffer full!");
			local_irq_save(sys_flag);
			if (!g_directpath_dma_full) {
				g_directpath_dma_full = 1;
				if (g_dma_device_ppe->tx_chan[3]->control == IFX_DMA_CH_ON)
					g_dma_device_ppe->tx_chan[3]->enable_irq( g_dma_device_ppe->tx_chan[3]);
#if !defined(ENABLE_DIRECTPATH_TX_QUEUE) || !ENABLE_DIRECTPATH_TX_QUEUE
				for (i = 0; i < push_through_devs_supported_by_datapath; i++)
					if ( ppa_virt_rx_devs[i] )
					    avm_pa_rx_channel_suspend( ppa_virt_rx_devs[i]->pid_handle );
#endif
			}
			local_irq_restore(sys_flag);
		} else {
			err( "incorrect DMA TX channel: %d (0 - 1 is reserved for fast path)", dma_chan_no);
		}
		break;
	case TRANSMIT_CPT_INT:
		if (dma_chan_no == 2) {
			unsigned int dev_nr;
			dbg("eth0/1 TX buffer released!");
			local_irq_save(sys_flag);
			g_dma_device_ppe->tx_chan[2]->disable_irq(
					g_dma_device_ppe->tx_chan[2]);
            for (dev_nr = 0; dev_nr < num_registered_eth_dev; dev_nr++){
                struct net_device *dev = g_eth_net_dev[dev_nr];
                avmnet_netdev_priv_t *priv = netdev_priv(dev);
            	avmnet_device_t *avm_dev = avm_dev = priv->avm_dev;
            	avm_dev->flags &= ~AVMNET_DEVICE_FLAG_DMA_TX_QUEUE_FULL;
            	netif_wake_queue( dev );
            }

			local_irq_restore(sys_flag);
		} else if (dma_chan_no == 3) {
			dbg("directpath TX (CPU->PPE) buffer released");
			local_irq_save(sys_flag);
			if (g_directpath_dma_full) {
				g_directpath_dma_full = 0;
				g_dma_device_ppe->tx_chan[3]->disable_irq(
						g_dma_device_ppe->tx_chan[3]);
#if !defined(ENABLE_DIRECTPATH_TX_QUEUE) || !ENABLE_DIRECTPATH_TX_QUEUE
				for (i = 0; i < push_through_devs_supported_by_datapath; i++)
					if ( ppa_virt_rx_devs[i] )
					    avm_pa_rx_channel_resume( ppa_virt_rx_devs[i]->pid_handle );
#endif
			}
			local_irq_restore(sys_flag);
#if defined(ENABLE_DIRECTPATH_TX_QUEUE) && ENABLE_DIRECTPATH_TX_QUEUE
			send_directpath_tx_queue();
#endif
		} else {
			err(
					"incorrect DMA TX channel: %d (0 - 1 is reserved for fast path)",
					dma_chan_no);
		}
		break;
	default:
		err("unkown DMA interrupt event - %d", status);
	}

	dbg_dma("[%s] exit function with %d\n", __FUNCTION__, ret);
	return ret;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if defined(DEBUG_DMA_RX_CHANNEL_COUNTER)
#define MAX_DMA_CHAN 8
int dma_rx_chan_count[MAX_DMA_CHAN];
#endif

static INLINE int dma_rx_int_handler(struct dma_device_info *dma_dev, int dma_chan_no)
{
	struct sk_buff *skb = NULL;
	u8 *buf;
	int len;
    struct softnet_data *queue;
    unsigned long flags;

#if defined(DEBUG_DMA_RX_CHANNEL_COUNTER)
	if ( dma_chan_no < MAX_DMA_CHAN && dma_chan_no >= 0)
		dma_rx_chan_count[dma_chan_no] += 1;
#endif

#if defined(TRACE_RX_PATH)
	static int once = 5;
	dbg(KERN_ERR "[%s] g_napi_enable=%d\n", __FUNCTION__, g_napi_enable);
	if (once) {
		dump_stack();
		once--;
	}
#endif

	len = dma_device_read(dma_dev, &buf, (void **) &skb, dma_chan_no);
	if (len < 0 || !skb ){
		err( "dma_device_read len=%d, skb=%p\n", len, skb);
		return 0;
	}

    get_skb_from_dbg_pool(skb);

    ASSERT( (dma_chan_no == 0)||
    		(dma_chan_no == 3)||
    		(dma_chan_no == 4)||
    		(dma_chan_no == 5)
    		, "rx_chan = %d", dma_chan_no );
    ASSERT( buf != NULL, "skb = %08x, buf = %08x", (unsigned int)skb, (unsigned int)buf);
    ASSERT( (u32)skb >= 0x80000000, "can not restore skb pointer (ch %d) --- skb = 0x%08X", dma_chan_no, (u32)skb);
    ASSERT( skb->data == buf && skb->tail == buf, "skb->data (%08X) or skb->tail (%08X) is corrupted, buf=%08X, rx_ch=%d, dmalen=%d", (u32)skb->data, (u32)skb->tail, (u32)buf, dma_chan_no, len);
    ASSERT( len <= (u32)skb->end - (u32)skb->data, "packet is too large: %d", len);
    ASSERT( *(struct sk_buff **)((((u32)buf | KSEG1) & ~0x1F) - 4) == skb,
    		"address is not match: skb = %08X, *(struct sk_buff **)((((u32)buf | KSEG1) & ~0x0F) - 4) = %08X",
    		(u32)skb, *(u32 *)((((u32)buf | KSEG1) & ~0x1F) - 4));

#if 0
    if( ( dma_chan_no != 5)  || ( skb->data != buf ) || ( skb->tail != buf) ){
    	err( "HACK: rx_chan = %d --> discard it", dma_chan_no );
    	if ( len > 0 && buf ){
#if defined( AVM_NET_TRACE_DUMP ) && AVM_NET_TRACE_DUMP && defined( CONFIG_AVM_NET_TRACE )
    		struct sk_buff *trace_skbuff = alloc_skb( len + 1024, GFP_ATOMIC );
    		if (trace_skbuff){
    			memcpy( trace_skbuff->data, buf , len );
    			trace_skbuff->tail = skb->data + len;
    			trace_skbuff->len = len;
    			avm_net_trace_func( &ifx_ant_e5 , skb, AVM_NET_TRACE_COPY_SKB, AVM_NET_TRACE_DIRECTION_HOST );
    			dev_kfree_skb_any( trace_skbuff );

    		}
#endif
			if (g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_PRINT ) {
				int i;
				for ( i = 1; i <= len ; i++ ) {
					if ( i % 16 == 1 )
						printk("  %4d:", i - 1);
					printk(" %02X", (int)(*((char*)buf + i - 1) & 0xFF));
					if ( i % 16 == 0 )
						printk("\n");
				}
				if ( (i - 1) % 16 != 0 )
					printk("\n");
			}

    	}
    	dev_kfree_skb_any(skb);
    	return 0;
    }
#endif

    if(unlikely(g_stop_datapath)){
        dbg("[%s]: datapath to be stopped, dropping skb.\n", __func__);
        dev_kfree_skb_any(skb);
        return 0;
    }

    if(unlikely((len > (skb->end - skb->data)) || (len < 64))){
        err( "%s[%d]: Packet is too large/small (%d)!!!\n", __func__,__LINE__,len);
        dev_kfree_skb_any(skb);
        return 0;
    }

    if(unlikely(len > (skb->end - skb->tail))){
        err( "%s[%d]: len:%d end:%p tail:%p Err!!!\n", __func__,__LINE__,(len+4), skb->end, skb->tail);
        dev_kfree_skb_any(skb);
        return 0;
    }

    skb_put(skb, len);

    if(buf){
        // dbg("rx len:%d\n",len);
        dump_skb(skb, DUMP_SKB_LEN, "switch_hw_receive raw", 0, dma_chan_no, 0, 0);
    }

    local_irq_save(flags);
    queue = &__get_cpu_var(g_7port_eth_queue);

    if(likely(queue->input_pkt_queue.qlen <= 1000)){
        if(unlikely(queue->input_pkt_queue.qlen == 0)){
            napi_schedule(&queue->backlog);
        }

        __skb_queue_tail(&queue->input_pkt_queue, skb);
    } else {
        dev_kfree_skb_any(skb);
    }

    local_irq_restore(flags);

    return 0;
}

static int process_backlog_e5(struct napi_struct *napi, int quota)
{
    struct sk_buff *skb = NULL;
    struct softnet_data *queue = &__get_cpu_var(g_7port_eth_queue);
    unsigned long start_time = jiffies;
    int work = 0;
    unsigned long flags;

    napi->weight = 64;

    do{
        local_irq_save(flags);
        skb = __skb_dequeue(&queue->input_pkt_queue);

        if (!skb){
            __napi_complete(napi);
            local_irq_restore(flags);
            break;
        }
        local_irq_restore(flags);

        handle_skb_e5(skb);
    }while (++work < quota && jiffies == start_time);

    return work;
}

static INLINE int handle_skb_e5(struct sk_buff *skb)
{
    struct flag_header *fheader;
    struct pmac_header *pheader;
    avmnet_netdev_priv_t *priv;
    avmnet_device_t *avm_dev = NULL;
    unsigned int exp_len, hw_dev_id;

	dump_skb( skb, DUMP_SKB_LEN, "handle_skb_e5 raw data", 0, -1, 0, 0);
	dump_flag_header(g_fwcode, (struct flag_header *) skb->data, __FUNCTION__, 0);

	// fetch flag_header from skb
    fheader = (struct flag_header *) skb->data;

    // fetch pmac_header from skb
    pheader = (struct pmac_header *) (skb->data + fheader->pl_byteoff);

    dump_pmac_header(pheader, __FUNCTION__);

    
    /* 
     * in indirect fast path pheader->len and fheader->len doesn't contain valid packet length, 
     * so we can do PMAC header check only in unaccelerated packets.
     */
    if ( ! fheader->acc_done ) {
       
       /*
        * calculate expected SKB len.
        * Payload offset from fheader + packet length from PMAC header + 4 bytes for frame crc
        */
        exp_len = fheader->pl_byteoff + (pheader->PKT_LEN_HI << 8) + pheader->PKT_LEN_LO + 4u;

        if(exp_len != skb->len ){
            printk(KERN_ERR "[%s] Malformed PMAC header: exp_len=%d, real_len=%d\n", __func__,exp_len,skb->len );
            dump_skb( skb, DUMP_SKB_LEN, "handle_skb_e5", 0, -1, 0, 1);
            dev_kfree_skb_any(skb);
            return 0;
        }
    }

    skb_pull(skb, fheader->pl_byteoff + sizeof(struct pmac_header));

	ASSERT(skb->len >= 60, "packet is too small: %d", skb->len);

#if defined(DEBUG_SPACE_BETWEEN_HEADER_AND_DATA)
	dbg_keep_min(STD_BYTE_OFF + fheader->pl_byteoff - sizeof(struct flag_header), &min_fheader_byteoff);
#endif

	dump_skb(skb, DUMP_SKB_LEN, "handle_skb_e5 packet data", fheader->src_itf, 0, 0, 0);

	//  implement indirect fast path
	if (fheader->acc_done && fheader->dest_list) //  acc_done == 1 && dest_list != 0
	{
		//  Current implementation send packet to highest priority dest only (0 -> 7, 0 is highest)
		//  2 is CPU, so we ignore it here
		if ((fheader->dest_list & (1 << 0))) //  0 - eth0
		{
			eth_xmit(skb, 0, DMA_CHAN_NR_DEFAULT_CPU_TX, 2, fheader->qid);
			return 0;
		} else if ((fheader->dest_list & (1 << 1))) //  1 - eth1
		{
			eth_xmit(skb, 1, DMA_CHAN_NR_DEFAULT_CPU_TX, 2, fheader->qid);
			return 0;
		} else if ((fheader->dest_list & (1 << 7))) //  7 - ptm0
		{
			ifx_ppa_ptm_hard_start_xmit(skb, g_ptm_net_dev[0]);
			return 0;
		} else {

			int dst_id;

			for (dst_id = 3; dst_id < 7; dst_id++) {
				if ((fheader->dest_list & (1 << dst_id))) //  3...6 (einschliesslich 3 u. 6)
				{
					int if_id = HW_DEV_ID(dst_id);

					if ( ifx_lookup_routing_entry && ppa_virt_tx_devs[ if_id ] ){

					    int rout_index;
					    avm_session_handle avm_pa_session;

    					dbg_trace_ppa_data("ACC-Done: ifid=%d, avm_rout_index=%d, is_mc=%d, is_lan=%d, is_coll=%d",
    					        if_id,
    					        fheader->avm_rout_index,
    					        fheader->avm_is_mc,
    					        fheader->avm_is_lan,
    					        fheader->avm_is_coll );
    					ppa_virt_tx_devs[ if_id ]->hw_pkt_tx ++;

    					rout_index = fheader->avm_rout_index;
    					if ( fheader->avm_is_coll && !fheader->avm_is_mc){

    					   /*
    					    * coll_entry numbers start directly after hash_table numbers
    					    * the following should be the same as rout_index |= (1 << 7);
    					    */
    					   rout_index += (MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);

    					}

                    	avm_pa_session = ifx_lookup_routing_entry(rout_index, fheader->avm_is_lan, fheader->avm_is_mc);

                    	if (avm_pa_session != INVALID_SESSION ){
						    avm_pa_tx_channel_accelerated_packet( ppa_virt_tx_devs[ if_id ]->pid_handle, avm_pa_session, skb );
                    	}
                    	else{
        					ppa_virt_tx_devs[ if_id ]->hw_pkt_tx_session_lookup_failed ++;
#if defined(CONFIG_AVMNET_DEBUG) 
        					if ( g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_VDEV ){
                        	    // unsigned int src_ip =  (skb->data[26] << 24) | (skb->data[27] << 16) | (skb->data[28] << 8) | (skb->data[29] << 0);
                                // dump_sessions_with_src_ip( src_ip );
                        	    dump_skb(skb, unknown_session_dump_len, "unknown session", fheader->src_itf, -1, 0, 1);
        					}
#endif
        					dev_kfree_skb_any(skb);
                    	}

						return 0;

					}
				}
			}
			dev_kfree_skb_any(skb);
			return 0;
		}
	}

	switch (fheader->src_itf) {
	case 0: //  incomming LAN IF ( vom Switch klassifizerit )
	case 1: //  incomming WAN IF ( vom Switch klassifizerit )
		avm_dev = mac_nr_to_avm_dev(pheader->SPPID);

		// TKL: FIXME workaround gegen Haenger beim Laden der Wasp-FW, check auf carrier_ok
		if (avm_dev && netif_running(avm_dev->device) && netif_carrier_ok(avm_dev->device)) {
			//  do not need to remove CRC
			//skb->tail -= ETH_CRC_LENGTH;
			//skb->len  -= ETH_CRC_LENGTH;
			unsigned int keep_len = 0;

			dbg_dma("got receive if: avm_name:%s ifname:%s\n",
					avm_dev->device_name, avm_dev->device->name);
			skb->dev = avm_dev->device;
#           if defined(CONFIG_IP_MULTICAST_FASTFORWARD)
            {
                avmnet_netdev_priv_t *priv = netdev_priv(avm_dev->device);

                if(likely(priv->mcfw_used)) {
                    mcfw_snoop_recv(&priv->mcfw_netdrv, 0, skb);
                }
            }
#           endif /*--- #if defined(CONFIG_IP_MULTICAST_FASTFORWARD) ---*/
			skb->protocol = eth_type_trans(skb, avm_dev->device);

			if (skb->len > 0)
				keep_len = skb->len;

			if(likely(avm_pa_dev_receive(AVM_PA_DEVINFO(avm_dev->device), skb) == AVM_PA_RX_ACCELERATED)) {
                avm_dev->device->last_rx = jiffies;
                return 0;
            }

			avm_dev->device_stats.rx_packets++;
			avm_dev->device_stats.rx_bytes += keep_len;
			netif_receive_skb(skb);

			return 0;
		} else {
			if (!avm_dev){
				err( "[%s] no avm dev for mac %d, rx-chan %d\n", __FUNCTION__, pheader->SPPID, 0 );
			}

			dbg("[%s] src_itf %d, mac_port=%d down drop packet\n", __FUNCTION__, fheader->src_itf, pheader->SPPID);
			if (g_dbg_datapath & DBG_ENABLE_MASK_DEBUG_PRINT ) {
				unsigned int i;
				printk("  skb->data = %08X, skb->tail = %08X, skb->len = %d\n", (u32)skb->data, (u32)skb->tail, (int)skb->len);
				for ( i = 1; i <= skb->len ; i++ ) {
					if ( i % 16 == 1 )
						printk("  %4d:", i - 1);
					printk(" %02X", (int)(*((char*)skb->data + i - 1) & 0xFF));
					if ( i % 16 == 0 )
						printk("\n");
				}
				if ( (i - 1) % 16 != 0 )
					printk("\n");
			}
		}
		break;
	case 7: //  PTM
		if (!g_ptm_net_dev[0]) { //in Ethernet WAN mode, ptm device not initialized, but it might get packet for it.
			dbg_dma(KERN_ERR "[%s] ptm netif down drop packet\n", __FUNCTION__);
			break; //just drop this packet
		}
		skb->dev = g_ptm_net_dev[0];
		priv = netdev_priv(skb->dev);
		avm_dev = priv->avm_dev;

		if (netif_running(g_ptm_net_dev[0])) {
			unsigned int keep_len = 0;
			dbg_dma(KERN_ERR "[%s] got a ptm packet", __FUNCTION__);

			skb->protocol = eth_type_trans(skb, g_ptm_net_dev[0]);
			if (skb->len > 0)
				keep_len = skb->len;

            if(likely(avm_pa_dev_receive(AVM_PA_DEVINFO(avm_dev->device), skb) == AVM_PA_RX_ACCELERATED)) {
                avm_dev->device->last_rx = jiffies;
                return 0;
            }
			avm_dev->device_stats.rx_packets++;
			avm_dev->device_stats.rx_bytes += keep_len;
			netif_receive_skb(skb);

			adsl_led_flash();

			dbg_dma(KERN_ERR "[%s] done", __FUNCTION__);
			return 0;
		}
		break;
	default: //  other interface receive (z.b. WLAN )
	    hw_dev_id = HW_DEV_ID(fheader->src_itf);

		if (hw_dev_id < ARRAY_SIZE(ppa_virt_rx_devs) && ppa_virt_rx_devs[hw_dev_id]){
			dbg_trace_ppa_data("PPE_DIRECTPATH_DATA_RX_ENABLE: src_itf=%d", fheader->src_itf );

			//TODO skb->dev muss von AVM_PA gesetzt werden
            ppa_virt_rx_devs[hw_dev_id]->hw_pkt_slow_cnt ++;
            avm_pa_rx_channel_packet_not_accelerated(ppa_virt_rx_devs[hw_dev_id]->pid_handle, skb);
			return 0;
		}else {
			dbg_trace_ppa_data("!PPE_DIRECTPATH_DATA_RX_ENABLE: src_itf=%d", fheader->src_itf );
		}
	}

	dbg(KERN_ERR "[%s] netif down drop packet\n", __FUNCTION__);
	if (avm_dev)
		avm_dev->device_stats.rx_dropped++;
	dev_kfree_skb_any(skb);
	return 0;
}


static void __switch_to_eth_thunk(void)
{
    int i;
    struct rx_gamma_itf_cfg rx_gamma_itf_cfg;

    CFG_STD_DATA_LEN->byte_off  = 0;
    BOND_CONF->max_frag_size    = 1536;
    *PDMA_RX_MAX_LEN_REG        = 0x02040604;

    for ( i = 0; i < 4; i++ )
    {
        rx_gamma_itf_cfg = *RX_GAMMA_ITF_CFG(i);

        rx_gamma_itf_cfg.rx_eth_fcs_ver_dis = 0;
        rx_gamma_itf_cfg.rx_rm_eth_fcs      = 1;
        rx_gamma_itf_cfg.rx_edit_num1       = 4;
        rx_gamma_itf_cfg.rx_edit_type1      = 1;
        rx_gamma_itf_cfg.rx_edit_en1        = 1;

        *RX_GAMMA_ITF_CFG(i) = rx_gamma_itf_cfg;
    }

    // no need to change for tx gif config
    BOND_CONF->bond_mode = L2_ETH_TRUNK_MODE;
}

static void switch_to_eth_trunk(void)
{
    // assume no traffic

    unsigned long sys_flag;
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    unsigned int ppe_base_addr;
#endif

    local_irq_save(sys_flag);
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    ppe_base_addr = g_ppe_base_addr;
    g_ppe_base_addr = IFX_PPE_ORG(0);
#endif

    __switch_to_eth_thunk();

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    // config bonding master
    g_ppe_base_addr = IFX_PPE_ORG(1);
    __switch_to_eth_thunk();
    g_ppe_base_addr = ppe_base_addr;
#endif
    local_irq_restore(sys_flag);
}

static void __switch_to_ptmtc_bonding(void)
{
    int i;
    struct rx_gamma_itf_cfg rx_gamma_itf_cfg;

    CFG_STD_DATA_LEN->byte_off  = 2;
    BOND_CONF->max_frag_size    = 0x200;
    *PDMA_RX_MAX_LEN_REG        = 0x02040204;

    for ( i = 0; i < 4; i++ )
    {
        rx_gamma_itf_cfg = *RX_GAMMA_ITF_CFG(i);

        rx_gamma_itf_cfg.rx_eth_fcs_ver_dis = 1;
        rx_gamma_itf_cfg.rx_rm_eth_fcs      = 0;
        rx_gamma_itf_cfg.rx_edit_num1       = 0;
        rx_gamma_itf_cfg.rx_edit_type1      = 0;
        rx_gamma_itf_cfg.rx_edit_en1        = 0;

        *RX_GAMMA_ITF_CFG(i) = rx_gamma_itf_cfg;
    }

    // no need to change for tx gif config
    BOND_CONF->bond_mode = L1_PTM_TC_BOND_MODE;
}

static void switch_to_ptmtc_bonding(void)
{
    // assume no traffic

    unsigned long sys_flag;
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    unsigned int ppe_base_addr;
#endif

    local_irq_save(sys_flag);
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    ppe_base_addr = g_ppe_base_addr;
    g_ppe_base_addr = IFX_PPE_ORG(0);
#endif

    __switch_to_ptmtc_bonding();

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    // config bonding master
    g_ppe_base_addr = IFX_PPE_ORG(1);
    __switch_to_ptmtc_bonding();
    g_ppe_base_addr = ppe_base_addr;
#endif
    local_irq_restore(sys_flag);
}

static void update_max_delay(int line)
{
    unsigned int user_max_delay;
    unsigned int linerate;
    int i, off;
    unsigned int delay_value;

    if ( g_dsl_bonding && line >= 0 && line < LINE_NUMBER )
    {
        delay_value = 1000;
        user_max_delay = (g_max_delay[line] + 8) / 16;
        for ( i = 0, linerate = 0; i < BEARER_CHANNEL_PER_LINE; i++ )
            linerate += g_datarate_ds[line * BEARER_CHANNEL_PER_LINE + i];
        linerate = (linerate + 50) / 100;
        if ( linerate != 0 )
        {
            //  delay_value = B1_DS_LL_CTXT->max_des_num * 512 * 8 / linerate / (16 / 1000000)
            delay_value = MAX_DESC_PER_GAMMA_ITF * 256 * 10000 / linerate;  //  use default value MAX_DESC_PER_GAMMA_ITF rather than reading max_des_num from each B1_DS_LL_CTXT
        }
        if ( user_max_delay != 0 && user_max_delay < delay_value )
            delay_value = user_max_delay;
        switch ( line )
        {
        case 0:
            off = 0;
            break;
        case 1:
        default:
            off = g_dsl_bonding == 1 ?  /* on-chip */ 2 : /* off-chip */ 4;
        }
        for ( i = 0; i < 2; i++ )
            B1_DS_LL_CTXT(1 + off + i)->max_delay = delay_value;
    }
}


#if defined(RESET_AR10_STYLE)
static int pseudo_dma_int_handler(struct dma_device_info *dma_dev, int status, int chan_no) {
    int ret = 0;
    struct sk_buff *skb = NULL;
    u8 *buf;

    switch ( status )
    {
    case RCV_INT:
        dma_device_read(dma_dev, &buf, (void **)&skb, chan_no);
        if ( skb )
            dev_kfree_skb_any(skb);
    }

    return ret;
}

static INLINE void reset_ppe(void) {
    struct dma_device_info *dma_device = NULL;
    unsigned long org_jiffies;
    unsigned int i;

    //if ( (*DM_RXCFG & 0xFF) == 0x30 )
    //    return;

    dma_device = dma_device_reserve("PPE");
    if ( !dma_device )
    {
        err("Reserve DMA for PPE failed");
        return;
    }

    dma_device->buffer_alloc    = dma_buffer_alloc;
    dma_device->buffer_free     = dma_buffer_free;
    dma_device->intr_handler    = pseudo_dma_int_handler;
    dma_device->tx_burst_len    = 8;
    dma_device->rx_burst_len    = 8;
    dma_device->port_packet_drop_enable = 1;

    for ( i = 0; i < dma_device->max_rx_chan_num; i++ )
    {
        dma_device->rx_chan[i]->packet_size = DMA_PACKET_SIZE;
        dma_device->rx_chan[i]->control     = IFX_DMA_CH_ON;
    }
    for ( i = 0; i < dma_device->max_tx_chan_num; i++ ){
        dma_device->tx_chan[i]->control     = IFX_DMA_CH_ON;
#if defined (CONFIG_AVM_SCATTER_GATHER)
        dma_device->tx_chan[i]->scatter_gather_channel = 1;
#endif
    }

    dma_device_register(dma_device);

    for ( i = 0; i < dma_device->max_rx_chan_num; i++ )
        dma_device->rx_chan[i]->open(dma_device->rx_chan[i]);

    org_jiffies = jiffies;

    while ( ((IFX_REG_R32(DM_RXPGCNT) & 0x0FFF) != 0 || (IFX_REG_R32(DM_RXPKTCNT) & 0x0FFF) != 0 || (IFX_REG_R32(DS_RXPGCNT) & 0x0FFF) != 0) && jiffies - org_jiffies < HZ / 10 + 1 )
        schedule();

    //  reset PPE
	ifx_rcu_rst(IFX_RCU_DOMAIN_DSLDFE, IFX_RCU_MODULE_PPA);
	udelay(1000);
    ifx_rcu_rst(IFX_RCU_DOMAIN_DSLDSP, IFX_RCU_MODULE_PPA);
    udelay(1000);
    ifx_rcu_rst(IFX_RCU_DOMAIN_DSLTC, IFX_RCU_MODULE_PPA);
    udelay(1000);

    // ifx_rcu_rst(IFX_RCU_DOMAIN_PPE, IFX_RCU_MODULE_PPA); //CCB: Dieser Reset koennt zu den 128 kaputten Paketen fuehren
    // udelay(1000);
    *PP32_SRST &= ~0x000303CF;
    udelay(1000);
    *PP32_SRST |= 0x000303CF;
    udelay(1000);

    for ( i = 0; i < dma_device->max_rx_chan_num; i++ )
        dma_device->rx_chan[i]->reset(dma_device->rx_chan[i]);

    dma_device_unregister(dma_device);
    dma_device_release(dma_device);
    dma_device = NULL;
}

#else
static INLINE void reset_ppe(void) {
    //  reset PPE
    ifx_rcu_rst(IFX_RCU_DOMAIN_DSLDSP, IFX_RCU_MODULE_PPA);
    udelay(1000);
    ifx_rcu_rst(IFX_RCU_DOMAIN_DSLTC, IFX_RCU_MODULE_PPA);
    udelay(1000);

    // ifx_rcu_rst(IFX_RCU_DOMAIN_PPE, IFX_RCU_MODULE_PPA); //CCB: Dieser Reset koennte zu den 128 kaputten Paketen fuehren
    // udelay(1000);

    *PP32_SRST &= ~0x000303CF;
    udelay(1000);
    *PP32_SRST |= 0x000303CF;
    udelay(1000);
}
#endif

static INLINE void init_pmu(void) {
	//   5 - DMA, should be enabled in DMA driver
	//   9 - DSL DFE/AFE
	//  15 - AHB
	//  19 - SLL
	//  21 - PPE TC
	//  22 - PPE EMA
	//  23 - PPE DPLUS Master
	//  24 - PPE DPLUS Slave
	//  28 - SWITCH
	//  29 - PPE_TOP

	PPE_TOP_PMU_SETUP(IFX_PMU_ENABLE);
	PPE_SLL01_PMU_SETUP(IFX_PMU_ENABLE);
	PPE_TC_PMU_SETUP(IFX_PMU_ENABLE);
	PPE_EMA_PMU_SETUP(IFX_PMU_ENABLE);
	PPE_TPE_PMU_SETUP(IFX_PMU_ENABLE);
	//  AHB Slave
	DSL_DFE_PMU_SETUP(IFX_PMU_ENABLE);
	PPE_DPLUS_PMU_SETUP(IFX_PMU_ENABLE);
	PPE_DPLUSS_PMU_SETUP(IFX_PMU_ENABLE);
	SWITCH_PMU_SETUP(IFX_PMU_ENABLE);
}

static INLINE void start_cpu_port(void) {
	int i;

	*DS_RXCFG |= 0x00008000; //  enable DSRX
	*DM_RXCFG |= 0x80000000; //  enable DMRX

	for (i = 0; i < 7; i++) {
		IFX_REG_W32_MASK(0, 1, FDMA_PCTRL_REG(i));
		//  start port 0 - 6
		IFX_REG_W32_MASK(0, 1, SDMA_PCTRL_REG(i));
		//  start port 0 - 6
	}
	*PCE_PCTRL_REG(6, 0) |= 0x07; //  clear_dplus
}

#if 0
static INLINE void stop_datapath(void) {
    unsigned long org_jiffies;
    int i;

    g_stop_datapath = 1;    //  going to be unloaded
	wmb();

    //  disable switch external port
    for ( i = 0; i < 6; i++ ) {
        IFX_REG_W32_MASK(1, 0, SDMA_PCTRL_REG(i));  //  stop port 0 - 5
        IFX_REG_W32_MASK(0, 1, FDMA_PCTRL_REG(i));
    }

    //  turn on DMA RX channels
    turn_on_dma_rx(31);

    //  clean ingress datapath
    org_jiffies = jiffies;
    while ( ((IFX_REG_R32(DM_RXPGCNT) & 0x0FFF) != 0 || (IFX_REG_R32(DM_RXPKTCNT) & 0x0FFF) != 0 || (IFX_REG_R32(DS_RXPGCNT) & 0x0FFF) != 0) && jiffies - org_jiffies < HZ / 10 + 1 )
        schedule();
    if ( jiffies - org_jiffies > HZ / 10 )
        err("Can not clear DM_RXPGCNT/DM_RXPKTCNT/DS_RXPGCNT");

    //  turn off DMA RX channels (including loopback RX channels)
    while ( (i = clz(g_f_dma_opened)) >= 0 )
        turn_off_dma_rx(i);

    //  turn off DMA Loopback TX channels
    g_dma_device_ppe->tx_chan[0]->close(g_dma_device_ppe->tx_chan[0]);
    g_dma_device_ppe->tx_chan[1]->close(g_dma_device_ppe->tx_chan[1]);

    *PMAC_HD_CTL_REG = 0x0000074C;  //  recover to default value
    *FDMA_PCTRL_REG(6) &= ~0x02;
}
#endif


static INLINE void init_internal_tantos_qos_setting(void) {
}

static INLINE void init_internal_tantos(void) {
	int i;


	*GLOB_CTRL_REG = 1 << 15; //  enable Switch

	for (i = 0; i < 7; i++)
		sw_clr_rmon_counter(i);

	//*FDMA_PCTRL_REG(6) &= ~0x01;//  disable port (FDMA)
	*FDMA_PCTRL_REG(6) |= 0x02; //  insert special tag

	*PMAC_HD_CTL_REG = 0x01DC; //  PMAC Head

	for (i = 6; i < 12; i++)
		*PCE_PCTRL_REG(i, 0) |= 1 << 11; //  ingress special tag

	*MAC_FLEN_REG = 1518 + 8 + 4 * 2; //  MAC frame + 8-byte special tag + 4-byte VLAN tag * 2
	*MAC_CTRL_REG(6, 2) |= 1 << 3; //  enable jumbo frame

//    for ( i = 0; i < 7; i++ )
//        *PCE_PCTRL_REG(i, 2) |= 3;  //  highest class, mapped to RX channel 0

	*PCE_PMAP_REG(1) = 0x7F; //  monitoring
	//*PCE_PMAP_REG(2) = 0x7F & ~g_pmac_ewan; //  broadcast and unknown multicast
	*PCE_PMAP_REG(2) = 0x7F; //  broadcast and unknown multicast
	//*PCE_PMAP_REG(3) = 0x7F & ~g_pmac_ewan; //  unknown uni-cast
	*PCE_PMAP_REG(3) = 0x7F; //  unknown uni-cast

	*PMAC_EWAN_REG = g_pmac_ewan;

	*PMAC_RX_IPG_REG = 0x8F;

	//IFX_VR9_Switch_PCE_Micro_Code_Int();

	init_internal_tantos_qos_setting();
}

#if defined(RESET_AR10_STYLE)

static INLINE void init_dplus(void) {

	*DM_RXCFG &= ~0x80000000; //  disable DMRX
	*DS_RXCFG &= ~0x00008000; //  disable DSRX

    //  reset D+
// #define RST_DPLUS_BITS  (1 << 4)                //  Slave only
#define RST_DPLUS_BITS  ((1 << 10) | (1 << 4))  //  Master & Slave
    *PP32_SRST &= ~RST_DPLUS_BITS;
    udelay(10);
    *PP32_SRST |= RST_DPLUS_BITS;

    *DM_RXCFG &= ~0x80000000;   //  disable DMRX
    *DS_RXCFG &= ~0x00008000;   //  disable DSRX

	*DM_RXPGCNT = 0x00002000; //  clear page pointer & counter
	*DS_RXPGCNT = 0x00004000; //  clear page pointer & counter

	*DM_RXDB = 0x2550;
	*DM_RXCB = 0x27D0;
	*DM_RXPGCNT = 0x00002000;
	*DM_RXPKTCNT = 0x00002000;
	*DM_RXCFG = 0x00F87014; // seit UGW 5.1.1: hdlen=31
	                        // vorher in UGW 5.1: 0x00587014; hdlen=19
}
#else
static INLINE void init_dplus(void)
{
    *DM_RXCFG &= ~0x80000000;   //  disable DMRX
    *DS_RXCFG &= ~0x00008000;   //  disable DSRX

    *DM_RXPGCNT = 0x00002000;   //  clear page pointer & counter
    *DS_RXPGCNT = 0x00004000;   //  clear page pointer & counter

    *DM_RXDB    = 0x2550;
    *DM_RXCB    = 0x27D0;
    *DM_RXPGCNT = 0x00002000;
    *DM_RXPKTCNT= 0x00002000;
    *DM_RXCFG   = 0x00F87014;

	wmb();
	udelay(100);
}
#endif

static INLINE void init_pdma(void) {
	//*EMA_CMDCFG  = (EMA_CMD_BUF_LEN << 16) | (EMA_CMD_BASE_ADDR >> 2);
	//*EMA_DATACFG = (EMA_DATA_BUF_LEN << 16) | (EMA_DATA_BASE_ADDR >> 2);
	//*EMA_IER     = 0x000000FF;
	//*EMA_CFG     = EMA_READ_BURST | (EMA_WRITE_BURST << 2);

	*PDMA_CFG = 0x00000001;
	*PDMA_RX_CTX_CFG = 0x00082C00;
	*PDMA_TX_CTX_CFG = 0x00081B00;
    *PDMA_RX_MAX_LEN_REG= g_dsl_bonding ? 0x02040204 : 0x02040604;
	*PDMA_RX_DELAY_CFG = 0x000F003F;

	*SAR_MODE_CFG = 0x00000011;
	*SAR_RX_CTX_CFG = 0x00082A00;
	*SAR_TX_CTX_CFG = 0x00082E00;
	*SAR_POLY_CFG_SET0 = 0x00001021;
	*SAR_POLY_CFG_SET1 = 0x1EDC6F41;
	*SAR_POLY_CFG_SET2 = 0x04C11DB7;
	*SAR_CRC_SIZE_CFG = 0x00000F3E;

	*SAR_PDMA_RX_CMDBUF_CFG = 0x01001900;
	*SAR_PDMA_TX_CMDBUF_CFG = 0x01001A00;
}

static INLINE void init_mailbox(void) {
	*MBOX_IGU0_ISRC = 0xFFFFFFFF;
	*MBOX_IGU0_IER = 0x00000000;
	*MBOX_IGU1_ISRC = 0xFFFFFFFF;
	*MBOX_IGU1_IER = 0x00000000; //  Don't need to enable RX interrupt, EMA driver handle RX path.
	dbg("MBOX_IGU1_IER = 0x%08X", *MBOX_IGU1_IER);
}

static INLINE void clear_share_buffer(void) {
	volatile u32 *p;
	unsigned int i;

	p = SB_RAM0_ADDR_BOND(2, 0);
	for (i = 0; i < SB_RAM0_DWLEN + SB_RAM1_DWLEN + SB_RAM2_DWLEN + SB_RAM3_DWLEN; i++)
		SW_WRITE_REG32(0, p++);

	p = SB_RAM6_ADDR_BOND(2, 0);
	for (i = 0; i < SB_RAM6_DWLEN; i++)
		SW_WRITE_REG32(0, p++);
}

static INLINE void clear_cdm(void) {
}

static INLINE void board_init(void) {
}

static INLINE void init_dsl_hw(void) {
	SW_WRITE_REG32(0x00010040, SFSM_CFG0);
	SW_WRITE_REG32(0x00010040, SFSM_CFG1);
	SW_WRITE_REG32(0x00020000, SFSM_PGCNT0);
	SW_WRITE_REG32(0x00020000, SFSM_PGCNT1);
	SW_WRITE_REG32(0x00000000, DREG_AT_IDLE0);
	SW_WRITE_REG32(0x00000000, DREG_AT_IDLE1);
	SW_WRITE_REG32(0x00000000, DREG_AR_IDLE0);
	SW_WRITE_REG32(0x00000000, DREG_AR_IDLE1);
	SW_WRITE_REG32(0x0000080C, DREG_B0_LADR);
	SW_WRITE_REG32(0x0000080C, DREG_B1_LADR);

	SW_WRITE_REG32(0x000001F0, DREG_AR_CFG0);
	SW_WRITE_REG32(0x000001F0, DREG_AR_CFG1);
	SW_WRITE_REG32(0x000001E0, DREG_AT_CFG0);
	SW_WRITE_REG32(0x000001E0, DREG_AT_CFG1);

	/*  clear sync state    */
	//SW_WRITE_REG32(0, SFSM_STATE0);
	//SW_WRITE_REG32(0, SFSM_STATE1);
	SW_WRITE_REG32_MASK(0, 1 << 14, SFSM_CFG0);
	//  enable SFSM storing
	SW_WRITE_REG32_MASK(0, 1 << 14, SFSM_CFG1);

	SW_WRITE_REG32_MASK(0, 1 << 15, SFSM_CFG0);
	//  HW keep the IDLE cells in RTHA buffer
	SW_WRITE_REG32_MASK(0, 1 << 15, SFSM_CFG1);

	SW_WRITE_REG32(0xF0D10000, FFSM_IDLE_HEAD_BC0);
	SW_WRITE_REG32(0xF0D10000, FFSM_IDLE_HEAD_BC1);
	SW_WRITE_REG32(0x00030028, FFSM_CFG0);
	//  Force_idle
	SW_WRITE_REG32(0x00030028, FFSM_CFG1);
}

static INLINE void setup_ppe_conf(void) {
	// disable PPE and MIPS access to DFE memory
	*RCU_PPE_CONF = *RCU_PPE_CONF & ~0x78000000;
}

static INLINE void init_hw(void) {

	setup_ppe_conf(); // ok

	init_pmu();  // ok

#if defined(RESET_AR10_STYLE)
    reset_ppe();
#endif

	init_internal_tantos(); // ok

	init_dplus(); // reset fehlt?!

	init_dsl_hw(); // ok

	init_pdma(); // ok

	init_mailbox(); // ok - mbox2 hinzu

	clear_share_buffer();  //ok

	clear_cdm(); //ok

	board_init(); //ok
}

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
static INLINE void init_bonding_hw(void)
{
    //  off chip bonding (bonding master)
    g_ppe_base_addr = IFX_PPE_ORG(1);

    setup_ppe_conf();

    //  PPE TOP, PPE EMA/PDMA, PPE TC, PPE SLL
    *IFX_PMU_PWDCR &= ~((1 << 29) | (1 << 22) | (1 << 21) | (1 << 19));

    if ( (*IFX_PMU_PWDCR & (1 << 23)) )
        *DM_RXCFG &= ~0x80000000;   //  disable DMRX
    if ( (*IFX_PMU_PWDCR & (1 << 24)) )
        *DS_RXCFG &= ~0x00008000;   //  disable DSRX

    init_dsl_hw();

    init_pdma();

    init_mailbox();

    clear_share_buffer();

    clear_cdm();

    g_ppe_base_addr = IFX_PPE_ORG(0);
}
#endif

/*
 *  Description:
 *    Download PPE firmware binary code.
 *  Input:
 *    src       --- u32 *, binary code buffer
 *    dword_len --- unsigned int, binary code length in DWORD (32-bit)
 *  Output:
 *    int       --- 0:    Success
 *                  else: Error Code
 */
/*
 *  Description:
 *    Download PPE firmware binary code.
 *  Input:
 *    pp32      --- int, which pp32 core
 *    src       --- u32 *, binary code buffer
 *    dword_len --- unsigned int, binary code length in DWORD (32-bit)
 *  Output:
 *    int       --- IFX_SUCCESS:    Success
 *                  else:           Error Code
 */
static INLINE int pp32_download_code(int pp32, const u32 *code_src,
		unsigned int code_dword_len, const u32 *data_src,
		unsigned int data_dword_len) {
	unsigned int clr, set;
	volatile u32 *dest;

	if (code_src == 0 || ((unsigned long) code_src & 0x03) != 0 || data_src == 0
			|| ((unsigned long) data_src & 0x03) != 0)
		return IFX_ERROR;

	clr = pp32 ? 0xF0 : 0x0F;
	if (code_dword_len <= CDM_CODE_MEMORYn_DWLEN(0))
		set = pp32 ? (3 << 6) : (2 << 2);
	else
		set = 0x00;
	SW_WRITE_REG32_MASK(clr, set, CDM_CFG);

	/*  copy code   */
	dest = CDM_CODE_MEMORY(pp32, 0);
	while (code_dword_len-- > 0)
		SW_WRITE_REG32(*code_src++, dest++);

	/*  copy data   */
	dest = CDM_DATA_MEMORY(pp32, 0);
	while (data_dword_len-- > 0)
		SW_WRITE_REG32(*data_src++, dest++);

	return IFX_SUCCESS;
}

/*
 *  Description:
 *    Initialize and start up PP32.
 *  Input:
 *    none
 *  Output:
 *    int  --- IFX_SUCCESS: Success
 *             else:        Error Code
 */
static INLINE int pp32_start(int pp32, int is_e1) {
	unsigned int mask = 1 << (pp32 << 4);
	int ret;

	/*  download firmware   */
	if (is_e1)
		ret = pp32_download_code(
				pp32,
				vr9_e1_binary_code,
				sizeof(vr9_e1_binary_code )
						/ sizeof(*vr9_e1_binary_code),
				vr9_e1_binary_data,
				sizeof(vr9_e1_binary_data)
						/ sizeof(*vr9_e1_binary_data));
	else
		ret = pp32_download_code(pp32, vr9_d5_binary_code,
				sizeof(vr9_d5_binary_code) / sizeof(*vr9_d5_binary_code),
				vr9_d5_binary_data,
				sizeof(vr9_d5_binary_data) / sizeof(*vr9_d5_binary_data));
	if (ret != IFX_SUCCESS)
		return ret;

	/*  run PP32    */
	*PP32_FREEZE &= ~mask;

	/*  idle for a while to let PP32 init itself    */
	//udelay(10);
	mdelay(1);

	/*  capture version number  */
	g_fw_ver[pp32] = *FW_VER_ID;

	return IFX_SUCCESS;
}

/*
 *  Description:
 *    Halt PP32.
 *  Input:
 *    none
 *  Output:
 *    none
 */
static INLINE void pp32_stop(int pp32) {
	unsigned int mask = 1 << (pp32 << 4);

	/*  halt PP32   */
	*PP32_FREEZE |= mask;
}

static INLINE int init_local_variables(void) {
	unsigned int i, j;

	/*  initialize semaphore used by open and close */
	sema_init(&g_sem, 1);

	if (ethwan == 1)
		g_eth_wan_mode = 2;
	else if (ethwan == 2)
		g_eth_wan_mode = 3;

	dbg(KERN_ERR "[%s] g_eth_wan_mode=%d\n", __FUNCTION__, g_eth_wan_mode);
	if (wanitf == ~0 || wanitf == 0) {
		switch (g_eth_wan_mode) {
		case 0: /*  DSL WAN     */
			g_wan_itf = 1 << 7;
			break;
		case 2: /*  Mixed Mode  */
			g_wan_itf = 1 << 0;
			break;
		case 3: /*  ETH WAN     */
			g_wan_itf = 1 << 1;
			break;
		}
	} else {
		g_wan_itf = wanitf;
		switch (g_eth_wan_mode) {
		case 0: /*  DSL WAN     */
			g_wan_itf &= 0xF8;
			g_wan_itf |= 1 << 7;
			break; //  ETH0/1 can not be WAN
		case 2: /*  Mixed Mode  */
			g_wan_itf &= 0x03; //  only ETH0/1 support mixed mode
			if (g_wan_itf == 0x03 || g_wan_itf == 0) { //both ETH0/1 in mixed mode, reset to eth0 mixed mode
				g_wan_itf = 1;
			}
			break;
		case 3: /*  ETH WAN     */
			g_wan_itf &= 0x7B; //  DSL disabled in ETH WAN mode
			if ((g_wan_itf & 0x03) == 0x03) { //both ETH0/1 in WAN mode, remove eth0 wan setting
				g_wan_itf &= ~0x01;
			}
			if ((g_wan_itf & 0x03) == 0) {
				g_wan_itf |= 1 << 1;
			}
			break;
		}

	}

	g_ipv6_acc_en = ipv6_acc_en ? 1 : 0;

	if (g_eth_wan_mode == 0)
		g_wanqos_en = wanqos_en ? wanqos_en : 8;
	else if (g_eth_wan_mode == 3)
		g_wanqos_en = wanqos_en ? wanqos_en : 0;
	if (g_wanqos_en > 8)
		g_wanqos_en = 8;


	g_mailbox_signal_mask = g_wanqos_en ? CPU_TO_WAN_SWAP_SIG : 0;
	if (g_eth_wan_mode == 0)
		g_mailbox_signal_mask |= WAN_RX_SIG(0);

	for (i = 0; i < NUM_ENTITY(g_queue_gamma_map); i++) {
		g_queue_gamma_map[i] = queue_gamma_map[i] & ((1 << g_wanqos_en) - 1);
		for (j = 0; j < i; j++)
			g_queue_gamma_map[i] &= ~g_queue_gamma_map[j];
	}

	dbg(KERN_ERR "[%s] g_wan_itf=%#x\n", __FUNCTION__, g_wan_itf);
	for (i = 0; i < num_registered_eth_dev; i++) {
		unsigned int max_packet_priority = NUM_ENTITY(g_eth_prio_queue_map[i]); //  assume only 8 levels are used in Linux
		int tx_num_q;
		int q_step, q_accum, p_step;

        tx_num_q = (g_wan_itf & (1 << i)) && (g_eth_wan_mode == 3 /* ETH WAN */ && g_wanqos_en) ? __ETH_WAN_TX_QUEUE_NUM : SWITCH_CLASS_VALUE;  //  assume only 8 levels are used in switch

        q_step = tx_num_q - 1;
		p_step = max_packet_priority - 1;
		for (j = 0, q_accum = 0; j < max_packet_priority;
				j++, q_accum += q_step)
			g_eth_prio_queue_map[i][j] = (q_accum + (p_step >> 1)) / p_step;

		if ((g_wan_itf & (1 << i))
				&& (g_eth_wan_mode == 3 /* ETH WAN */&& g_wanqos_en))
			for (j = 0; j < max_packet_priority; j++)
				g_eth_prio_queue_map[i][j] = q_step
						- g_eth_prio_queue_map[i][j];
	}

	if (g_eth_wan_mode == 0 /* DSL WAN */) {
		unsigned int max_packet_priority = NUM_ENTITY(g_ptm_prio_queue_map);
		int tx_num_q;
		int q_step, q_accum, p_step;

		tx_num_q = __ETH_WAN_TX_QUEUE_NUM;
		q_step = tx_num_q - 1;
		p_step = max_packet_priority - 1;
		for (j = 0, q_accum = 0; j < max_packet_priority; j++, q_accum += q_step){
			g_ptm_prio_queue_map[j] = q_step - (q_accum + (p_step >> 1)) / p_step;
			dbg("g_ptm_prio_queue_map[%d] = %d", j,g_ptm_prio_queue_map[j] );
		}
	}

    for ( i = 0; i < NUM_ENTITY(g_wan_queue_class_map); i++ )
        g_wan_queue_class_map[i] = i;
    if ( (g_wan_itf & 0x03) && g_wanqos_en )
    {
        for ( i = 0; i < NUM_ENTITY(g_wan_queue_class_map); i++ )
            g_wan_queue_class_map[i] = NUM_ENTITY(g_wan_queue_class_map) - 1 - i;
    }

	return 0;
}

static INLINE void clear_local_variables(void) {
}

#if defined(ENABLE_FW_MODULE_TEST) && ENABLE_FW_MODULE_TEST

#include "cfg_arrays_d5.h"
#include "cfg_arrays_d5_ipv6.h"

unsigned int ppe_fw_mode = 0; // 0: normal mode, 1: mix mode
unsigned int ppe_wan_hash_algo = 0;// 0: using algo 0, 1: using algo 1
unsigned int acc_proto = 0;// 0: UDP, 1:TCP
unsigned int ipv6_en = 0;// 0: IPv6 disabled, 1: IPv6 enabled

void setup_acc_action_tbl(void)
{
	unsigned int idx;

	if (acc_proto == 0) {
		// clear bit16 of dw3 of each action entry
		unsigned long udp_mask = ~ (1 << 16);

		// setup for Hash entry
		idx = 3;
		if (ipv6_en == 0) {
			while (idx < sizeof(lan_uc_act_tbl_normal_mode_cfg)/sizeof(unsigned long)) {
				lan_uc_act_tbl_normal_mode_cfg[idx] &= udp_mask;
				lan_uc_act_tbl_mixed_mode_cfg[idx] &= udp_mask;

				wan_uc_act_tbl_alo_0_cfg[idx] &= udp_mask;
				wan_uc_act_tbl_alo_1_cfg[idx] &= udp_mask;
				idx += 6;
			}
		} else {
			while (idx < sizeof(lan_uc_act_tbl_normal_mode_cfg_ipv6)/sizeof(unsigned long)) {
				lan_uc_act_tbl_normal_mode_cfg_ipv6[idx] &= udp_mask;
				lan_uc_act_tbl_mixed_mode_cfg_ipv6[idx] &= udp_mask;

				wan_uc_act_tbl_alo_0_cfg_ipv6[idx] &= udp_mask;
				wan_uc_act_tbl_alo_1_cfg_ipv6[idx] &= udp_mask;
				idx += 6;
			}
		}

		// setup for Collsion entry
		idx = 3;
		if (ipv6_en == 0) {
			while (idx < sizeof(lan_uc_col_act_tbl_normal_mode_cfg)/sizeof(unsigned long)) {
				lan_uc_col_act_tbl_normal_mode_cfg[idx] &= udp_mask;
				lan_uc_col_act_tbl_mixed_mode_cfg[idx] &= udp_mask;
				wan_uc_col_act_tbl_cfg[idx] &= udp_mask;
				idx += 6;
			}
		} else {
			while (idx < sizeof(lan_uc_col_act_tbl_normal_mode_cfg_ipv6)/sizeof(unsigned long)) {
				lan_uc_col_act_tbl_normal_mode_cfg_ipv6[idx] &= udp_mask;
				lan_uc_col_act_tbl_mixed_mode_cfg_ipv6[idx] &= udp_mask;
				wan_uc_col_act_tbl_cfg_ipv6[idx] &= udp_mask;
				idx += 6;
			}
		}
	}
	else {
		// set bit16 of dw3 of each action entry
		unsigned long tcp_mask = (1 << 16);

		// setup for Hash entry
		idx = 3;
		if (ipv6_en == 0) {
			while (idx < sizeof(lan_uc_act_tbl_normal_mode_cfg)/sizeof(unsigned long)) {
				lan_uc_act_tbl_normal_mode_cfg[idx] |= tcp_mask;
				lan_uc_act_tbl_mixed_mode_cfg[idx] |= tcp_mask;

				wan_uc_act_tbl_alo_0_cfg[idx] |= tcp_mask;
				wan_uc_act_tbl_alo_1_cfg[idx] |= tcp_mask;
				idx += 6;
			}
		} else {
			while (idx < sizeof(lan_uc_act_tbl_normal_mode_cfg_ipv6)/sizeof(unsigned long)) {
				lan_uc_act_tbl_normal_mode_cfg_ipv6[idx] |= tcp_mask;
				lan_uc_act_tbl_mixed_mode_cfg_ipv6[idx] |= tcp_mask;

				wan_uc_act_tbl_alo_0_cfg_ipv6[idx] |= tcp_mask;
				wan_uc_act_tbl_alo_1_cfg_ipv6[idx] |= tcp_mask;
				idx += 6;
			}
		}

		// setup for Collsion entry
		idx = 3;
		if (ipv6_en == 0) {
			while (idx < sizeof(lan_uc_col_act_tbl_normal_mode_cfg)/sizeof(unsigned long)) {
				lan_uc_col_act_tbl_normal_mode_cfg[idx] |= tcp_mask;
				lan_uc_col_act_tbl_mixed_mode_cfg[idx] |= tcp_mask;
				wan_uc_col_act_tbl_cfg[idx] |= tcp_mask;
				idx += 6;
			}
		} else {
			while (idx < sizeof(lan_uc_col_act_tbl_normal_mode_cfg_ipv6)/sizeof(unsigned long)) {
				lan_uc_col_act_tbl_normal_mode_cfg_ipv6[idx] |= tcp_mask;
				lan_uc_col_act_tbl_mixed_mode_cfg_ipv6[idx] |= tcp_mask;
				wan_uc_col_act_tbl_cfg_ipv6[idx] |= tcp_mask;
				idx += 6;
			}
		}
	}
}

void init_acc_tables(void)
{
	setup_acc_action_tbl();

	// init MAC table
	memcpy((void *)ROUT_MAC_CFG_TBL(0), rout_mac_cfg, sizeof(rout_mac_cfg));

	// PPPoE table
	memcpy((void *)PPPOE_CFG_TBL(0), pppoe_cfg, sizeof(pppoe_cfg));

	// Outer VLAN Config
	memcpy((void *)OUTER_VLAN_TBL(0), outer_vlan_cfg, sizeof(outer_vlan_cfg));

	if (ipv6_en == 0) {
		//Use Original Data Structures

		// lan uc_cmp_tbl_cfg (Hash) and collision
		memcpy((void *)ROUT_LAN_HASH_CMP_TBL(0), lan_uc_cmp_tbl_cfg, sizeof(lan_uc_cmp_tbl_cfg));
		memcpy((void *)ROUT_LAN_COLL_CMP_TBL(0), lan_uc_col_cmp_tbl_cfg, sizeof(lan_uc_col_cmp_tbl_cfg));

		// lan action
		if(ppe_fw_mode == 0) {
			// normal mode
			memcpy((void *)ROUT_LAN_HASH_ACT_TBL(0), lan_uc_act_tbl_normal_mode_cfg, sizeof(lan_uc_act_tbl_normal_mode_cfg));
			memcpy((void *)ROUT_LAN_COLL_ACT_TBL(0), lan_uc_col_act_tbl_normal_mode_cfg, sizeof(lan_uc_col_act_tbl_normal_mode_cfg));
		} else {
			// mix mode
			memcpy((void *)ROUT_LAN_HASH_ACT_TBL(0), lan_uc_act_tbl_mixed_mode_cfg, sizeof(lan_uc_act_tbl_mixed_mode_cfg));
			memcpy((void *)ROUT_LAN_COLL_ACT_TBL(0), lan_uc_col_act_tbl_mixed_mode_cfg, sizeof(lan_uc_col_act_tbl_mixed_mode_cfg));
		}

		// wan hash cmp anc act table
		if ( ppe_wan_hash_algo == 0) {
			// WAN algo 0
			memcpy((void *)ROUT_WAN_HASH_CMP_TBL(0), wan_uc_cmp_tbl_alo_0_cfg, sizeof(wan_uc_cmp_tbl_alo_0_cfg));
			memcpy((void *)ROUT_WAN_HASH_ACT_TBL(0), wan_uc_act_tbl_alo_0_cfg, sizeof(wan_uc_act_tbl_alo_0_cfg));
		} else {
			// WAN algo 1
			memcpy((void *)ROUT_WAN_HASH_CMP_TBL(0), wan_uc_cmp_tbl_alo_1_cfg, sizeof(wan_uc_cmp_tbl_alo_1_cfg));
			memcpy((void *)ROUT_WAN_HASH_ACT_TBL(0), wan_uc_act_tbl_alo_1_cfg, sizeof(wan_uc_act_tbl_alo_1_cfg));
		}

		// wan collision cmp and act table
		memcpy((void *)ROUT_WAN_COLL_CMP_TBL(0), wan_uc_col_cmp_tbl_cfg, sizeof(wan_uc_col_cmp_tbl_cfg));
		memcpy((void *)ROUT_WAN_COLL_ACT_TBL(0), wan_uc_col_act_tbl_cfg, sizeof(wan_uc_col_act_tbl_cfg));

		// wan multicast cmp and act table
		memcpy((void *)ROUT_WAN_MC_CMP_TBL(0), wan_mc_cmp_tbl_cfg, sizeof(wan_mc_cmp_tbl_cfg));
		memcpy((void *)ROUT_WAN_MC_ACT_TBL(0), wan_mc_act_tbl_cfg, sizeof(wan_mc_act_tbl_cfg));
	} else {
		//Use IPv6 Data Structures

		// lan uc_cmp_tbl_cfg (Hash) and collision
		memcpy((void *)ROUT_LAN_HASH_CMP_TBL(0), lan_uc_cmp_tbl_cfg_ipv6, sizeof(lan_uc_cmp_tbl_cfg_ipv6));
		memcpy((void *)ROUT_LAN_COLL_CMP_TBL(0), lan_uc_col_cmp_tbl_cfg_ipv6, sizeof(lan_uc_col_cmp_tbl_cfg_ipv6));

		// lan action
		if(ppe_fw_mode == 0) {
			// normal mode
			memcpy((void *)ROUT_LAN_HASH_ACT_TBL(0), lan_uc_act_tbl_normal_mode_cfg_ipv6, sizeof(lan_uc_act_tbl_normal_mode_cfg_ipv6));
			memcpy((void *)ROUT_LAN_COLL_ACT_TBL(0), lan_uc_col_act_tbl_normal_mode_cfg_ipv6, sizeof(lan_uc_col_act_tbl_normal_mode_cfg_ipv6));
		} else {
			// mix mode
			memcpy((void *)ROUT_LAN_HASH_ACT_TBL(0), lan_uc_act_tbl_mixed_mode_cfg_ipv6, sizeof(lan_uc_act_tbl_mixed_mode_cfg_ipv6));
			memcpy((void *)ROUT_LAN_COLL_ACT_TBL(0), lan_uc_col_act_tbl_mixed_mode_cfg_ipv6, sizeof(lan_uc_col_act_tbl_mixed_mode_cfg_ipv6));
		}

		// wan hash cmp anc act table
		if ( ppe_wan_hash_algo == 0) {
			// WAN algo 0
			memcpy((void *)ROUT_WAN_HASH_CMP_TBL(0), wan_uc_cmp_tbl_alo_0_cfg_ipv6, sizeof(wan_uc_cmp_tbl_alo_0_cfg_ipv6));
			memcpy((void *)ROUT_WAN_HASH_ACT_TBL(0), wan_uc_act_tbl_alo_0_cfg_ipv6, sizeof(wan_uc_act_tbl_alo_0_cfg_ipv6));
		} else {
			// WAN algo 1
			memcpy((void *)ROUT_WAN_HASH_CMP_TBL(0), wan_uc_cmp_tbl_alo_1_cfg_ipv6, sizeof(wan_uc_cmp_tbl_alo_1_cfg_ipv6));
			memcpy((void *)ROUT_WAN_HASH_ACT_TBL(0), wan_uc_act_tbl_alo_1_cfg_ipv6, sizeof(wan_uc_act_tbl_alo_1_cfg_ipv6));
		}

		// wan collision cmp and act table
		memcpy((void *)ROUT_WAN_COLL_CMP_TBL(0), wan_uc_col_cmp_tbl_cfg_ipv6, sizeof(wan_uc_col_cmp_tbl_cfg_ipv6));
		memcpy((void *)ROUT_WAN_COLL_ACT_TBL(0), wan_uc_col_act_tbl_cfg_ipv6, sizeof(wan_uc_col_act_tbl_cfg_ipv6));

		// wan multicast cmp and act table
		memcpy((void *)ROUT_WAN_MC_CMP_TBL(0), wan_mc_cmp_tbl_cfg_ipv6, sizeof(wan_mc_cmp_tbl_cfg_ipv6));
		memcpy((void *)ROUT_WAN_MC_ACT_TBL(0), wan_mc_act_tbl_cfg_ipv6, sizeof(wan_mc_act_tbl_cfg_ipv6));
	}
}

#endif  //  #if defined(ENABLE_FW_MODULE_TEST) && ENABLE_FW_MODULE_TEST


static INLINE void init_ptm_tables(void)
{
    struct cfg_std_data_len cfg_std_data_len = {0};
    struct tx_qos_cfg tx_qos_cfg = {0};
    struct psave_cfg psave_cfg = {0};
    struct eg_bwctrl_cfg eg_bwctrl_cfg = {0};
    struct test_mode test_mode = {0};
    struct rx_bc_cfg rx_bc_cfg = {0};
    struct tx_bc_cfg tx_bc_cfg = {0};
    struct gpio_mode gpio_mode = {0};
    struct gpio_wm_cfg gpio_wm_cfg = {0};
    struct rx_gamma_itf_cfg rx_gamma_itf_cfg = {0};
    struct tx_gamma_itf_cfg tx_gamma_itf_cfg = {0};
    struct wtx_qos_q_desc_cfg wtx_qos_q_desc_cfg = {0};
    unsigned int i;
    int j;

    cfg_std_data_len.byte_off = g_dsl_bonding ? 2 : 0;  //  this field replaces byte_off in rx descriptor of VDSL ingress
    cfg_std_data_len.data_len = DMA_PACKET_SIZE;
    *CFG_STD_DATA_LEN = cfg_std_data_len;

    tx_qos_cfg.time_tick    = cgu_get_pp32_clock() / 62500; //  16 * (cgu_get_pp32_clock() / 1000000)
    tx_qos_cfg.overhd_bytes = 24;   //  8-byte preamble, 12-byte inter-frame gap, E5 specific: 2-byte TC CRC, 1-byte SoF, 1 byte EOF (Cj)
    tx_qos_cfg.eth1_eg_qnum = __ETH_WAN_TX_QUEUE_NUM;
    tx_qos_cfg.eth1_burst_chk = 1;
    tx_qos_cfg.eth1_qss     = 0;
    tx_qos_cfg.shape_en     = 0;    //  disable
    tx_qos_cfg.wfq_en       = 0;    //  strict priority
    *TX_QOS_CFG_DYNAMIC = tx_qos_cfg;

    psave_cfg.start_state   = 0;
    psave_cfg.sleep_en      = 1;    //  enable sleep mode
    *PSAVE_CFG = psave_cfg;

    eg_bwctrl_cfg.fdesc_wm  = 16;
    eg_bwctrl_cfg.class_len = 128;
    *EG_BWCTRL_CFG = eg_bwctrl_cfg;

    test_mode.mib_clear_mode    = 0;
    test_mode.test_mode         = 0;
    *TEST_MODE = test_mode;

    //*GPIO_ADDR = (unsigned int)IFX_GPIO_P0_OUT;
    *GPIO_ADDR = (unsigned int)0x00000000;  //  disabled by default

    gpio_mode.gpio_bit_bc1 = 2;
    gpio_mode.gpio_bit_bc0 = 1;
    gpio_mode.gpio_bc1_en  = 0;
    gpio_mode.gpio_bc0_en  = 0;
    *GPIO_MODE = gpio_mode;

    gpio_wm_cfg.stop_wm_bc1  = 2;
    gpio_wm_cfg.start_wm_bc1 = 4;
    gpio_wm_cfg.stop_wm_bc0  = 2;
    gpio_wm_cfg.start_wm_bc0 = 4;
    *GPIO_WM_CFG = gpio_wm_cfg;

    rx_bc_cfg.local_state   = 0;
    rx_bc_cfg.remote_state  = 0;
    rx_bc_cfg.to_false_th   = 7;
    rx_bc_cfg.to_looking_th = 3;
    *RX_BC_CFG(0) = rx_bc_cfg;
    *RX_BC_CFG(1) = rx_bc_cfg;

    tx_bc_cfg.fill_wm   = 2;
    tx_bc_cfg.uflw_wm   = 2;
    *TX_BC_CFG(0) = tx_bc_cfg;
    *TX_BC_CFG(1) = tx_bc_cfg;

    rx_gamma_itf_cfg.receive_state      = 0;
    rx_gamma_itf_cfg.rx_min_len         = 64;
    rx_gamma_itf_cfg.rx_pad_en          = 1;
    rx_gamma_itf_cfg.rx_eth_fcs_ver_dis = g_dsl_bonding ? 1 : 0;    //  disable Ethernet FCS verification during bonding
    rx_gamma_itf_cfg.rx_rm_eth_fcs      = g_dsl_bonding ? 0 : 1;    //  disable Ethernet FCS verification during bonding
    rx_gamma_itf_cfg.rx_tc_crc_ver_dis  = 0;
    rx_gamma_itf_cfg.rx_tc_crc_size     = 1;
    rx_gamma_itf_cfg.rx_eth_fcs_result  = 0xC704DD7B;
    rx_gamma_itf_cfg.rx_tc_crc_result   = 0x1D0F1D0F;
    rx_gamma_itf_cfg.rx_crc_cfg         = 0x2500;
    rx_gamma_itf_cfg.rx_eth_fcs_init_value  = 0xFFFFFFFF;
    rx_gamma_itf_cfg.rx_tc_crc_init_value   = 0x0000FFFF;
    rx_gamma_itf_cfg.rx_max_len_sel     = 0;
    rx_gamma_itf_cfg.rx_edit_num2       = 0;
    rx_gamma_itf_cfg.rx_edit_pos2       = 0;
    rx_gamma_itf_cfg.rx_edit_type2      = 0;
    rx_gamma_itf_cfg.rx_edit_en2        = 0;
    rx_gamma_itf_cfg.rx_edit_num1       = g_dsl_bonding ? 0 : 4;    //  no PMAC header insertion during bonding
    rx_gamma_itf_cfg.rx_edit_pos1       = 0;
    rx_gamma_itf_cfg.rx_edit_type1      = g_dsl_bonding ? 0 : 1;    //  no PMAC header insertion during bonding
    rx_gamma_itf_cfg.rx_edit_en1        = g_dsl_bonding ? 0 : 1;    //  no PMAC header insertion during bonding
    rx_gamma_itf_cfg.rx_inserted_bytes_1l   = 0x00000007;   //  E5: byte swap of value 0x07000000
    rx_gamma_itf_cfg.rx_inserted_bytes_1h   = 0;
    rx_gamma_itf_cfg.rx_inserted_bytes_2l   = 0;
    rx_gamma_itf_cfg.rx_inserted_bytes_2h   = 0;
    rx_gamma_itf_cfg.rx_len_adj         = -2;               //  E5: remove TC CRC
    for ( i = 0; i < 4; i++ )
        *RX_GAMMA_ITF_CFG(i) = rx_gamma_itf_cfg;

    tx_gamma_itf_cfg.tx_len_adj         = g_dsl_bonding ? 2 : 6;
    tx_gamma_itf_cfg.tx_crc_off_adj     = 6;
    tx_gamma_itf_cfg.tx_min_len         = 0;
    tx_gamma_itf_cfg.tx_eth_fcs_gen_dis = g_dsl_bonding ? 1 : 0;
    tx_gamma_itf_cfg.tx_tc_crc_size     = 1;
    tx_gamma_itf_cfg.tx_crc_cfg         = g_dsl_bonding ? 0x2F02 : 0x2F00;
    tx_gamma_itf_cfg.tx_eth_fcs_init_value  = 0xFFFFFFFF;
    tx_gamma_itf_cfg.tx_tc_crc_init_value   = 0x0000FFFF;
    for ( i = 0; i < NUM_ENTITY(g_queue_gamma_map); i++ ) {
        tx_gamma_itf_cfg.queue_mapping = g_queue_gamma_map[i];
        *TX_GAMMA_ITF_CFG(i) = tx_gamma_itf_cfg;
    }

    for ( j = 0; j < __ETH_WAN_TX_QUEUE_NUM; j++ )
    {
        wtx_qos_q_desc_cfg.length = WAN_TX_DESC_NUM(j);
        wtx_qos_q_desc_cfg.addr   = __ETH_WAN_TX_DESC_BASE(j);
        *WTX_QOS_Q_DESC_CFG(j) = wtx_qos_q_desc_cfg;
    }

    //  default TX queue QoS config is all ZERO

    //  TX Ctrl K Table
    IFX_REG_W32(0x90111293, TX_CTRL_K_TABLE(0));
    IFX_REG_W32(0x14959617, TX_CTRL_K_TABLE(1));
    IFX_REG_W32(0x18999A1B, TX_CTRL_K_TABLE(2));
    IFX_REG_W32(0x9C1D1E9F, TX_CTRL_K_TABLE(3));
    IFX_REG_W32(0xA02122A3, TX_CTRL_K_TABLE(4));
    IFX_REG_W32(0x24A5A627, TX_CTRL_K_TABLE(5));
    IFX_REG_W32(0x28A9AA2B, TX_CTRL_K_TABLE(6));
    IFX_REG_W32(0xAC2D2EAF, TX_CTRL_K_TABLE(7));
    IFX_REG_W32(0x30B1B233, TX_CTRL_K_TABLE(8));
    IFX_REG_W32(0xB43536B7, TX_CTRL_K_TABLE(9));
    IFX_REG_W32(0xB8393ABB, TX_CTRL_K_TABLE(10));
    IFX_REG_W32(0x3CBDBE3F, TX_CTRL_K_TABLE(11));
    IFX_REG_W32(0xC04142C3, TX_CTRL_K_TABLE(12));
    IFX_REG_W32(0x44C5C647, TX_CTRL_K_TABLE(13));
    IFX_REG_W32(0x48C9CA4B, TX_CTRL_K_TABLE(14));
    IFX_REG_W32(0xCC4D4ECF, TX_CTRL_K_TABLE(15));

    //  Bug workaround: US packet go to wrong queue issue for offchip bonding.
    //  For offchip bonding,
    //    SM needs write access to __DPLUS_RX_DSLWAN_FP_QID_BASE (0x5080) on BM.
    //    FPI address of base address of __DPLUS_RX_DSLWAN_FP_QID_BASE (0x5000) will be programmed in __FP_QID_BASE_PDMA_BAR on both SM and BM.
    //    Offset of __DPLUS_RX_DSLWAN_FP_QID_BASE (0x0080) is hardcoded in firmware.
    //    The value on SM must be well calculated as below, the value on BM is a flag to indicate workaround activated.
    //  Calculation:
    //    PCI host outbound memory
    //    PCI Host      PCI Device  Size
    //    0xB8000000    0x1E000000  16MB
    //    0x1E200000 is the location of SB, in order to cross-pci access SB, it is 0xB8200000,
    //    since 0x5000 is in SB_RAM_BLOCK3, FPI address=0xB000, 0xB000 * 4 = 0x2C000,
    //    address = 0xB8200000 + 0x2C000 = 0xB822C000
    if ( g_dsl_bonding == 2 )
        *FP_QID_BASE_PDMA_BAR = (unsigned int)SB_BUFFER_BOND(1, 0x5000);
}

static INLINE void init_ptm_bonding_tables(void)
{
    struct bond_conf bond_conf = {0};
    struct bg_map bg_map = {0};
    struct b1_ds_ll_ctxt b1_ds_ll_ctxt = {0};
    int i;

    if ( g_dsl_bonding == 1 )   //  on chip bonding
    {
        *BM_BS_MBOX_IGU6_ISRS = 0x7220;
        *BM_BM_MBOX_IGU6_ISRS = 0x7220;

        // IGU4: linu up/down status (i.e., in showtime, not in showtime)
        // IGU5: gif on_off status
        *LINK_UP_NOTIFY_ADDR   = 0x7218;    // __MBOX_IGU4_ISRS
        *LINK_DOWN_NOTIFY_ADDR = 0x7219;    //__MBOX_IGU4_ISRC
        // Although only IGU4 is configed, FW will infer gif_on_of_notify_addr
        // based on LINK_UP_NOTIFY_ADDR
        // *GIF_ON_NOTIFY_ADDR = 0x721C     // __MBOX_IGU5_ISRS
        // *GIF_OFF_NOTIFY_ADDR = 0x721D    //__MBOX_IGU5_ISRC


        bond_conf.max_frag_size = 0x200;
        bond_conf.polling_ctrl_cnt = 2;
        bond_conf.dplus_fp_fcs_en  = 0x1;
        bond_conf.bg_num        = 0x2;
        bond_conf.bond_mode     = L1_PTM_TC_BOND_MODE;
        bond_conf.e1_bond_en    = 0x1;
        bond_conf.d5_acc_dis    = 0x0;
        bond_conf.d5_b1_en      = 0x1;
        *BOND_CONF = bond_conf;

        bg_map.map3 = 0x00;
        bg_map.map2 = 0x00;
        bg_map.map1 = 0xFF;
        bg_map.map0 = 0x00;
        *US_BG_QMAP = bg_map;   //  upstream bonding group to queue map

        bg_map.map3 = 0x00;
        bg_map.map2 = 0x00;
        bg_map.map1 = 0x05;
        bg_map.map0 = 0x0A;
        *US_BG_GMAP = bg_map;   //  upstream bonding group to gamma interface map
        *DS_BG_GMAP = bg_map;   //  downstream bonding group to gamma interface map

        b1_ds_ll_ctxt.max_des_num   = MAX_DESC_PER_GAMMA_ITF;
        b1_ds_ll_ctxt.TO_buff_thres = 16; // when free_des_num <= 16, timeout any frag
        b1_ds_ll_ctxt.max_delay = 1000;  //  TODO: fix the number

        *__READ_MIN_CYCLES  = 0xFFFFFFFF;
        *__WRITE_MIN_CYCLES = 0xFFFFFFFF;

        for ( i = 0; i < 9; i++ )
            *B1_DS_LL_CTXT(i) = b1_ds_ll_ctxt;
        B1_DS_LL_CTXT(0)->head_ptr = B1_RX_LINK_LIST_DESC_PPE;
        B1_DS_LL_CTXT(0)->tail_ptr = B1_RX_LINK_LIST_DESC_PPE + (B1_RX_LINK_LIST_DESC_NUM - 1) * 2;
        B1_DS_LL_CTXT(0)->des_num  = B1_RX_LINK_LIST_DESC_NUM;

        for ( i = 0; i < 4; i ++) {
            DS_BG_CTXT_BASE(i)->last_eop = 1;
        }
    }
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    else
    {
        unsigned int up_addr, down_addr;

        /*
         *  off chip bonding (system master)
         */

        bond_conf.polling_ctrl_cnt  = 2;
        bond_conf.e1_bond_en        = 0x1;
        bond_conf.dplus_fp_fcs_en   = 0x1;
        *BOND_CONF = bond_conf;

        bg_map.map3 = 0x00;
        bg_map.map2 = 0x00;
        bg_map.map1 = 0xFF;
        bg_map.map0 = 0x00;
        *US_BG_QMAP = bg_map;

        bg_map.map3 = 0x00;   //  TODO: 0x44 if app1 two bearer channels
        bg_map.map2 = 0x00;   //  TODO: 0x88 if app1 two bearer channels
        bg_map.map1 = 0x11;   //  TODO: 0x55 if app2
        bg_map.map0 = 0x22;   //  TODO: 0xAA if app2
        *US_BG_GMAP = bg_map;
        *DS_BG_GMAP = bg_map;

        // config E1 to use polling mode due to hardware issue
        // echo device > /proc/eth/dev; echo w 2004 2002 > /proc/eth/mem;
        // echo host > /proc/eth/dev; echo w 2022 2002 > /proc/eth/mem; echo w 2002 1 > /proc/eth/mem;
        * US_FRAG_READY_NOTICE_ADDR = 0x2002;
        * SB_BUFFER_BOND(0, 0x2002) = 1;

        // IGU4: linu up/down status (i.e., in showtime, not in showtime)
        // IGU5: gif on_off status
        up_addr   = (unsigned int)SB_BUFFER_BOND(1, 0x7218);
        down_addr = (unsigned int)SB_BUFFER_BOND(1, 0x7219);
        *LINK_UP_NOTIFY_ADDR   = up_addr;   // __MBOX_IGU4_ISRS
        *LINK_DOWN_NOTIFY_ADDR = down_addr; // __MBOX_IGU4_ISRC

        // Although only IGU4 is configed, FW will infer gif_on_of_notify_addr
        // based on LINK_UP_NOTIFY_ADDR
        // *GIF_ON_NOTIFY_ADDR  = 0x721C    // __MBOX_IGU5_ISRS
        // *GIF_OFF_NOTIFY_ADDR = 0x721D    // __MBOX_IGU5_ISRC

        /*
         *  off chip bonding (bonding master)
         */

        g_ppe_base_addr = IFX_PPE_ORG(1);

        // config E1 to use notificaion mode
        *BM_BS_MBOX_IGU6_ISRS = g_bm_ppe_int_addr;
        *BM_BM_MBOX_IGU6_ISRS = 0x7220;

        // config E1 to use polling mode due to hardware issue
        // echo device > /proc/eth/dev; echo w 2004 0x2002 > /proc/eth/mem;
        // echo host > /proc/eth/dev; echo w 2022 2002 > /proc/eth/mem; echo w 2002 1 > /proc/eth/mem;
        * BM_BS_MBOX_IGU6_ISRS = 0x2002;
        * SB_BUFFER_BOND(1, 0x2002) = 1;

        // IGU4: linu up/down status (i.e., in showtime, not in showtime)
        // IGU5: gif on_off status
        *LINK_UP_NOTIFY_ADDR   = 0x7218;    // __MBOX_IGU4_ISRS
        *LINK_DOWN_NOTIFY_ADDR = 0x7219;    // __MBOX_IGU4_ISRC
        // Although only IGU4 is configed, FW will infer gif_on_of_notify_addr
        // based on LINK_UP_NOTIFY_ADDR
        // *GIF_ON_NOTIFY_ADDR  = 0x721C    // __MBOX_IGU5_ISRS
        // *GIF_OFF_NOTIFY_ADDR = 0x721D    // __MBOX_IGU5_ISRC

        // set BM TTHA.pnu = 80 to avoid under flow issue
        // echo device > /proc/eth/dev; echo w 50a 10050 > /proc/eth/mem
        *FFSM_CFG0 = 0x00010050;
        // set US.under_flow_threshold
        // echo device > /proc/eth/dev; echo w 3ec0 360036 > /proc/eth/mem
        *(volatile unsigned int *)TX_BC_CFG(0) = 0x00360036;    //  overwrite setting done in init_ptm_tables

        bond_conf.max_frag_size = 0x200;
        bond_conf.polling_ctrl_cnt = 2;
        bond_conf.dplus_fp_fcs_en  = 0x1;
        bond_conf.bg_num        = 0x2;  //  TODO: 0x4 if two bearer channels are enabled
        bond_conf.bond_mode     = L1_PTM_TC_BOND_MODE;
        bond_conf.e1_bond_en    = 0x1;
        bond_conf.d5_acc_dis    = 0x1;
        bond_conf.d5_b1_en      = 0x1;
        *BOND_CONF = bond_conf;

        bg_map.map3 = 0x00;
        bg_map.map2 = 0x00;
        bg_map.map1 = 0xFF;
        bg_map.map0 = 0x00;
        *US_BG_QMAP = bg_map;

        bg_map.map3 = 0x00;   //  TODO: 0x44 if app1 two bearer channels
        bg_map.map2 = 0x00;   //  TODO: 0x88 if app1 two bearer channels
        bg_map.map1 = 0x11;   //  TODO: 0x55 if app2
        bg_map.map0 = 0x22;   //  TODO: 0xAA if app2
        *US_BG_GMAP = bg_map;
        *DS_BG_GMAP = bg_map;

        b1_ds_ll_ctxt.max_des_num   = MAX_DESC_PER_GAMMA_ITF;
        b1_ds_ll_ctxt.TO_buff_thres = 16; // when free_des_num <= 16, timeout any frag
        b1_ds_ll_ctxt.max_delay = 1000;  //  TODO: fix the number

        *__READ_MIN_CYCLES  = 0xFFFFFFFF;
        *__WRITE_MIN_CYCLES = 0xFFFFFFFF;

        for ( i = 0; i < 9; i++ )
            *B1_DS_LL_CTXT(i) = b1_ds_ll_ctxt;
        B1_DS_LL_CTXT(0)->head_ptr = B1_RX_LINK_LIST_DESC_PPE;
        B1_DS_LL_CTXT(0)->tail_ptr = B1_RX_LINK_LIST_DESC_PPE + (B1_RX_LINK_LIST_DESC_NUM - 1) * 2;
        B1_DS_LL_CTXT(0)->des_num  = B1_RX_LINK_LIST_DESC_NUM;

        for ( i = 0; i < 4; i ++) {
            DS_BG_CTXT_BASE(i)->last_eop = 1;
        }

        g_ppe_base_addr = IFX_PPE_ORG(0);
    }
#endif
}

static INLINE void init_qos(void)
{
    struct wtx_qos_q_desc_cfg wtx_qos_q_desc_cfg = {0};
    struct tx_qos_cfg tx_qos_cfg = {0};
    struct cfg_std_data_len cfg_std_data_len = {0};
    int i;

    for ( i = 0; i < __ETH_WAN_TX_QUEUE_NUM; i++ )
    {
        wtx_qos_q_desc_cfg.length = WAN_TX_DESC_NUM(i);
        wtx_qos_q_desc_cfg.addr   = __ETH_WAN_TX_DESC_BASE(i);
        *WTX_QOS_Q_DESC_CFG(i) = wtx_qos_q_desc_cfg;
    }

    tx_qos_cfg.time_tick    = cgu_get_pp32_clock() / 62500; //  16 * (cgu_get_pp32_clock() / 1000000)
    tx_qos_cfg.eth1_eg_qnum = __ETH_WAN_TX_QUEUE_NUM;
    tx_qos_cfg.eth1_qss     = 1;
    tx_qos_cfg.eth1_burst_chk = 1;
    *TX_QOS_CFG_DYNAMIC = tx_qos_cfg;

    cfg_std_data_len.byte_off = 0;
    cfg_std_data_len.data_len = DMA_PACKET_SIZE;

    *CFG_STD_DATA_LEN = cfg_std_data_len;
}


static INLINE void init_communication_data_structures(
		int fwcode __attribute__ ((unused))) {
	struct eth_ports_cfg eth_ports_cfg = { 0 };
	struct rout_tbl_cfg lan_rout_tbl_cfg = { 0 };
	struct rout_tbl_cfg wan_rout_tbl_cfg = { 0 };
	struct gen_mode_cfg1 gen_mode_cfg1 = { 0 };
    unsigned int i, j;

	*CDM_CFG = CDM_CFG_RAM1_SET(0x00) | CDM_CFG_RAM0_SET(0x00);

    udelay(10);

	*PSEUDO_IPv4_BASE_ADDR = 0xFFFFFF00;

	eth_ports_cfg.wan_vlanid_hi = 0;
	eth_ports_cfg.wan_vlanid_lo = 0;
	eth_ports_cfg.eth0_type = 0; //  lan
	*ETH_PORTS_CFG = eth_ports_cfg;

	lan_rout_tbl_cfg.rout_num = MAX_COLLISION_ROUTING_ENTRIES;
	lan_rout_tbl_cfg.ttl_en = 1;
	lan_rout_tbl_cfg.rout_drop = 0;
	*LAN_ROUT_TBL_CFG = lan_rout_tbl_cfg;

	wan_rout_tbl_cfg.rout_num = MAX_COLLISION_ROUTING_ENTRIES;
	wan_rout_tbl_cfg.wan_rout_mc_num = 64;
	wan_rout_tbl_cfg.ttl_en = 1;
	wan_rout_tbl_cfg.wan_rout_mc_drop = 0;
	wan_rout_tbl_cfg.rout_drop = 0;
	*WAN_ROUT_TBL_CFG = wan_rout_tbl_cfg;

	gen_mode_cfg1.app2_indirect = 1; //  0: means DMA RX ch 3 is dedicated for CPU1 process
	gen_mode_cfg1.us_indirect = 0;
	gen_mode_cfg1.us_early_discard_en = 0;
#if defined(CONFIG_IFX_PPA_MFE)
	gen_mode_cfg1.classification_num = MAX_CLASSIFICATION_ENTRIES;
#else
	gen_mode_cfg1.classification_num = 0;
#endif
	gen_mode_cfg1.ipv6_rout_num = g_ipv6_acc_en ? 2 : 0;
	gen_mode_cfg1.session_ctl_en = 0;
	gen_mode_cfg1.wan_hash_alg = 0;
#if defined(CONFIG_IFX_PPA_MFE)
	gen_mode_cfg1.brg_class_en = 1;
#else
	gen_mode_cfg1.brg_class_en = 0;
#endif
	gen_mode_cfg1.ipv4_mc_acc_mode = 0;
	gen_mode_cfg1.ipv6_acc_en = g_ipv6_acc_en;
	gen_mode_cfg1.wan_acc_en = 1;
	gen_mode_cfg1.lan_acc_en = 1;
	gen_mode_cfg1.sw_iso_mode = 1;
	gen_mode_cfg1.sys_cfg = g_eth_wan_mode; //  3 - ETH1 WAN, 2 - ETH0 mixed, 0 - DSL WAN
											// default - ETH WAN Separate, Ethernet WAN Mode.
											// Ethernet 0 carry LAN traffic, Ethernet 1 carry WAN traffic.
	*GEN_MODE_CFG1 = gen_mode_cfg1;
	GEN_MODE_CFG2->itf_outer_vlan_vld = 0xFF; //  Turn on outer VLAN for all interfaces;

    *CFG_FEATURES = 0;  //  clock gating is disabled

    *CFG_CLASS2QID_MAP(0) = 0;
    *CFG_CLASS2QID_MAP(1) = 0;
    *CFG_QID2CLASS_MAP(0) = 0;
    *CFG_QID2CLASS_MAP(0) = 0;
    if ( (g_wan_itf & 0x80) )
    {
        for ( j = 0; j < NUM_ENTITY(g_eth_prio_queue_map[i]); j++ )
            *CFG_CLASS2QID_MAP(j >> 3) |= g_ptm_prio_queue_map[j] << ((j & 0x07) * 4);
    }
    else
    {
        i = (g_wan_itf & 0x01) ? 0 : 1;
        for ( j = 0; j < NUM_ENTITY(g_eth_prio_queue_map[i]); j++ )
            *CFG_CLASS2QID_MAP(j >> 3) |= g_eth_prio_queue_map[i][j] << ((j & 0x07) * 4);
    }
    for ( j = 0; j < NUM_ENTITY(g_wan_queue_class_map); j++ )
        *CFG_QID2CLASS_MAP(j >> 3) |= g_wan_queue_class_map[j] << ((j & 0x07) * 4);

    *MTU_CFG_TBL(0) = ETH_MAX_DATA_LENGTH;
    for ( i = 1; i < 8; i++ )
        *MTU_CFG_TBL(i) = ETH_MAX_DATA_LENGTH;  //  for ETH

    if ( g_eth_wan_mode == 3 /* ETH WAN */ && g_wanqos_en )
        init_qos();

	*CFG_WAN_PORTMAP = g_wan_itf;
	*CFG_MIXED_PORTMAP = g_eth_wan_mode == 2 /* Mixed Mode */? g_wan_itf : 0;
	*TUNNEL_MAX_ID = -1;

#if defined(ENABLE_FW_MODULE_TEST) && ENABLE_FW_MODULE_TEST
	init_acc_tables();
#endif
}

static INLINE int alloc_dma(void) {
	int ret;
	unsigned int pre_alloc_desc_total_num;
	struct sk_buff **skb_pool;
	struct sk_buff **pskb;
	volatile u32 *p;
	unsigned int i;

#if defined(ENABLE_P2P_COPY_WITHOUT_DESC) && ENABLE_P2P_COPY_WITHOUT_DESC
	pre_alloc_desc_total_num = 0;
#else
	pre_alloc_desc_total_num = DMA_RX_CH2_DESC_NUM;
#endif
    if ( g_eth_wan_mode == 0 && g_dsl_bonding )     //  on chip bonding does not support SWAP queue
    {
        if ( g_dsl_bonding == 1 )
            pre_alloc_desc_total_num += E1_FRAG_RX_DESC_NUM + B1_RX_LINK_LIST_DESC_NUM + WAN_TX_DESC_NUM_TOTAL + DMA_RX_CH1_DESC_NUM + DMA_TX_CH1_DESC_NUM;

    }else if (g_eth_wan_mode == 0 || g_wanqos_en) //  DSL WAN or ETH WAN with FW QoS

        pre_alloc_desc_total_num += CPU_TO_WAN_TX_DESC_NUM
				+ WAN_TX_DESC_NUM_TOTAL + CPU_TO_WAN_SWAP_DESC_NUM
				+ DMA_RX_CH1_DESC_NUM + DMA_TX_CH1_DESC_NUM;

	skb_pool = (struct sk_buff **) kmalloc( pre_alloc_desc_total_num * sizeof(*skb_pool), GFP_KERNEL);
	if (skb_pool == NULL) {
		ret = -ENOMEM;
		printk("[%s] err %d\n", __func__, __LINE__);
		goto ALLOC_SKB_POOL_FAIL;
	}

	for (i = 0; (unsigned int) i < pre_alloc_desc_total_num; i++) {
		skb_pool[i] = alloc_skb_rx();
		if (!skb_pool[i]) {
			ret = -ENOMEM;
			printk("[%s] err %d\n", __func__, __LINE__);
			goto ALLOC_SKB_FAIL;
		}
#ifndef CONFIG_MIPS_UNCACHED
		/*  invalidate cache    */
		dma_cache_inv((unsigned long)skb_pool[i]->data, DMA_PACKET_SIZE);
#endif
#if defined(DEBUG_SKB_SWAP) && DEBUG_SKB_SWAP
		g_dbg_skb_swap_pool[i] = skb_pool[i];
#endif
	}

	g_dma_device_ppe = dma_device_reserve("PPE");
	if (!g_dma_device_ppe) {
		ret = -EIO;
		printk("[%s] err %d\n", __func__, __LINE__);
		goto RESERVE_DMA_FAIL;
	}

	if ( g_dbg_datapath & DBG_ENABLE_MASK_DUMP_INIT )
        dump_dma_device(g_dma_device_ppe, "just reserved");

	g_dma_device_ppe->buffer_alloc = dma_buffer_alloc;
	g_dma_device_ppe->buffer_free = dma_buffer_free;
#if defined(ENABLE_NAPI) && ENABLE_NAPI
	g_dma_device_ppe->activate_poll = g_napi_enable ? dma_activate_poll : NULL;
	g_dma_device_ppe->inactivate_poll = dma_inactivate_poll;
#endif
	g_dma_device_ppe->intr_handler = dma_int_handler;
	//g_dma_device_ppe->max_rx_chan_num = 8;
	//g_dma_device_ppe->max_tx_chan_num = 4;
	g_dma_device_ppe->tx_burst_len = 8;
	g_dma_device_ppe->rx_burst_len = 8;

    g_dma_device_ppe->num_tx_chan  = 4;
    g_dma_device_ppe->num_rx_chan  = 6;

	for (i = 0; i < g_dma_device_ppe->max_rx_chan_num; i++) {
		g_dma_device_ppe->rx_chan[i]->packet_size = DMA_PACKET_SIZE;
		g_dma_device_ppe->rx_chan[i]->control = IFX_DMA_CH_ON;
		//g_dma_device_ppe->rx_chan[i]->channel_packet_drop_enable    = 1;
	}
#if !defined(DEBUG_REDIRECT_FASTPATH_TO_CPU) || !DEBUG_REDIRECT_FASTPATH_TO_CPU
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
	if ( g_dsl_bonding == 2 )
	    g_ppe_base_addr = IFX_PPE_ORG(1);
#endif
	g_dma_device_ppe->rx_chan[1]->desc_base = (int)DMA_RX_CH1_DESC_BASE;
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
	if ( g_dsl_bonding == 2 )
	    g_ppe_base_addr = IFX_PPE_ORG(0);
#endif


	// PPA -> DSL
	g_dma_device_ppe->rx_chan[1]->desc_base = (int) DMA_RX_CH1_DESC_BASE;
	g_dma_device_ppe->rx_chan[1]->desc_len = DMA_RX_CH1_DESC_NUM;
	g_dma_device_ppe->rx_chan[1]->req_irq_to_free = g_dma_device_ppe->rx_chan[1]->irq;
	g_dma_device_ppe->rx_chan[1]->no_cpu_data = 1;

	/* RX2 -> TX0: (PPA -> Switch(port6))*/
	g_dma_device_ppe->rx_chan[2]->desc_base = (int) DMA_RX_CH2_DESC_BASE;
	g_dma_device_ppe->rx_chan[2]->desc_len = DMA_RX_CH2_DESC_NUM;
	g_dma_device_ppe->rx_chan[2]->req_irq_to_free = g_dma_device_ppe->rx_chan[2]->irq;
	g_dma_device_ppe->rx_chan[2]->no_cpu_data = 1;
#if defined(ENABLE_P2P_COPY_WITHOUT_DESC) && ENABLE_P2P_COPY_WITHOUT_DESC
	if (g_eth_wan_mode != 0 /* ETH WAN */&& !g_wanqos_en /* QoS Disabled */) {
		g_dma_device_ppe->rx_chan[1]->loopback_enable = 1;
		g_dma_device_ppe->rx_chan[1]->loopback_channel_number = 1 * 2 + 1; // absolute nummer - entsprict TX1
	}
	/* RX2 -> TX0: (PPA -> Switch(port6))*/
	g_dma_device_ppe->rx_chan[2]->loopback_enable = 1;
	g_dma_device_ppe->rx_chan[2]->loopback_channel_number = 0 * 2 + 1; // absolute nummer - entspricht TX0
#endif
#endif

	for (i = 0; i < g_dma_device_ppe->max_tx_chan_num; i++) {
		g_dma_device_ppe->tx_chan[i]->control = IFX_DMA_CH_ON;
#if defined (CONFIG_AVM_SCATTER_GATHER)
        g_dma_device_ppe->tx_chan[i]->scatter_gather_channel = 1;
#endif
	}
#if !defined(DEBUG_REDIRECT_FASTPATH_TO_CPU) || !DEBUG_REDIRECT_FASTPATH_TO_CPU
	g_dma_device_ppe->tx_chan[0]->desc_base = (int) DMA_TX_CH0_DESC_BASE;
	g_dma_device_ppe->tx_chan[0]->desc_len = DMA_TX_CH0_DESC_NUM;
	g_dma_device_ppe->tx_chan[1]->desc_base = (int) DMA_TX_CH1_DESC_BASE;
	g_dma_device_ppe->tx_chan[1]->desc_len = DMA_TX_CH1_DESC_NUM;
#if !defined(ENABLE_P2P_COPY_WITHOUT_DESC) || !ENABLE_P2P_COPY_WITHOUT_DESC
	g_dma_device_ppe->tx_chan[0]->global_buffer_len = DMA_PACKET_SIZE;
	g_dma_device_ppe->tx_chan[0]->peri_to_peri = 1;
	g_dma_device_ppe->tx_chan[1]->global_buffer_len = DMA_PACKET_SIZE;
	g_dma_device_ppe->tx_chan[1]->peri_to_peri = 1;
#else
	g_dma_device_ppe->tx_chan[0]->loopback_enable = 1;
	if (g_eth_wan_mode == 0 /* DSL WAN */|| g_wanqos_en /* QoS Enabled */) {
		// DSL -> Switch(port11)
		g_dma_device_ppe->tx_chan[1]->global_buffer_len = DMA_PACKET_SIZE;
		g_dma_device_ppe->tx_chan[1]->peri_to_peri = 1;
		g_dma_device_ppe->tx_chan[1]->req_irq_to_free = g_dma_device_ppe->tx_chan[1]->irq;
		g_dma_device_ppe->tx_chan[1]->no_cpu_data = 1;

		// PPA -> Switch(port 6)
		g_dma_device_ppe->tx_chan[0]->req_irq_to_free = g_dma_device_ppe->tx_chan[0]->irq;
		g_dma_device_ppe->tx_chan[0]->no_cpu_data = 1;
	} else
		g_dma_device_ppe->tx_chan[1]->loopback_enable = 1;
#endif
#endif

	g_dma_device_ppe->port_packet_drop_enable = 1;

	if (dma_device_register(g_dma_device_ppe) != IFX_SUCCESS) {
		err("failed in \"dma_device_register\"");
		ret = -EIO;
		printk("[%s] err %d\n", __func__, __LINE__);
		goto DMA_DEVICE_REGISTER_FAIL;
	}

	if ( g_dbg_datapath & DBG_ENABLE_MASK_DUMP_INIT )
        dump_dma_device(g_dma_device_ppe, "registered");

	pskb = skb_pool;

	if (g_eth_wan_mode == 0 || g_wanqos_en) {
	    if ( g_eth_wan_mode == 0 && g_dsl_bonding ) {
	        if ( g_dsl_bonding == 1 ) {
	            p = (volatile u32 *)E1_FRAG_TX_DESC_NB_BASE(0);
	            for ( i = 0; i < E1_FRAG_TX_DESC_NUM_TOTAL; i++ )
	            {
	                *p++ = 0x30000000;
	                *p++ = 0x00000000;
	            }
	            E1_FRAG_TX_DESBA->desba    = E1_FRAG_TX_DESC_NB_PPE(0);
	            E1_FRAG_TX_DESBA->bp_desba = E1_FRAG_TX_DESC_B_PPE(0);

	            p = (volatile u32 *)E1_FRAG_RX_DESC_BASE(0);
	            for ( i = 0; i < E1_FRAG_RX_DESC_NUM; i++ )
	            {
	                ASSERT(((u32)(*pskb)->data & (DMA_ALIGNMENT - 1)) == 0, "E1 Frag RX data pointer (0x%08x) must be 8 DWORDS aligned", (u32)(*pskb)->data);
	                // byte_offset should be set to 2
	                *p++ = 0xB1000000 | DMA_PACKET_SIZE /* 544 */;
	                *p++ = (u32)(*pskb++)->data & 0x1FFFFFE0;
	            }
	            E1_FRAG_RX_DESBA->desba    = E1_FRAG_RX_DESC_PPE(0);
	            E1_FRAG_RX_DESBA->bp_desba = WAN_RX_DESC_PPE;

	            p = (volatile u32 *)E1_FRAG_RX_DESC_BASE(1);
	            for ( i = 0; i < E1_FRAG_RX_DESC_NUM; i++ )
	            {
	                *p++ = 0xB1000000;
	                *p++ = 0x00000000;
	            }

	            p = (volatile u32 *)B1_RX_LINK_LIST_DESC_BASE;
	            for ( i = 0; i < B1_RX_LINK_LIST_DESC_NUM; i++ )
	            {
	                ASSERT(((u32)(*pskb)->data & (DMA_ALIGNMENT - 1)) == 0, "B1 Packet TX data pointer (0x%08x) must be 8 DWORDS aligned", (u32)(*pskb)->data);
	                *p++ = ((B1_RX_LINK_LIST_DESC_PPE + 2 * ((i + 1) & 0xFF)) << 16) | 1664 /* 544 */;
	                *p++ = (u32)(*pskb++)->data & 0x1FFFFFE0;
	            }
	        }
	    }
	    else {

	        p = (volatile u32 *) CPU_TO_WAN_TX_DESC_BASE;
	        for (i = 0; i < CPU_TO_WAN_TX_DESC_NUM; i++) {
	            ASSERT(
	                    ((u32)(*pskb)->data & (DMA_TX_ALIGNMENT - 1)) == 0,
	                    "CPU to WAN TX data pointer (0x%08x) must be 8 DWORDS aligned",
	                    (u32)(*pskb)->data);
	            *p++ = 0x30000000;
	            *p++ = (u32) (*pskb++)->data & 0x1FFFFFE0;
	        }
	    }
	    if ( g_dsl_bonding != 2 ) {
	        p = (volatile u32 *) WAN_TX_DESC_BASE(0);
	        for (i = 0; i < WAN_TX_DESC_NUM_TOTAL; i++) {
	            ASSERT(
	                    ((u32)(*pskb)->data & (DMA_TX_ALIGNMENT - 1)) == 0,
	                    "WAN (PTM/ETH) QoS TX data pointer (0x%08x) must be 8 DWORDS aligned",
	                    (u32)(*pskb)->data);
	            *p++ = 0x30000000;
	            *p++ = (u32) (*pskb++)->data & 0x1FFFFFE0;
	        }

	    }
	    if ( g_dsl_bonding == 0 ) {
	        p = (volatile u32 *) CPU_TO_WAN_SWAP_DESC_BASE;
	        for (i = 0; i < CPU_TO_WAN_SWAP_DESC_NUM; i++) {
	            ASSERT(
	                    ((u32)(*pskb)->data & (DMA_TX_ALIGNMENT - 1)) == 0,
	                    "WAN (PTM) Swap data pointer (0x%08x) must be 8 DWORDS aligned",
	                    (u32)(*pskb)->data);
	            *p++ = 0xB0000000;
	            *p++ = (u32) (*pskb++)->data & 0x1FFFFFE0;
	        }
	    }
	}
#if !defined(DEBUG_REDIRECT_FASTPATH_TO_CPU) || !DEBUG_REDIRECT_FASTPATH_TO_CPU
    if ( g_dsl_bonding != 2 )
    {
        p = (volatile u32 *)DMA_RX_CH1_DESC_BASE;
        for ( i = 0; i < DMA_RX_CH1_DESC_NUM; i++ )
        {
  #if !defined(ENABLE_P2P_COPY_WITHOUT_DESC) || !ENABLE_P2P_COPY_WITHOUT_DESC
            ASSERT(((u32)(*pskb)->data & (DMA_ALIGNMENT - 1)) == 0, "DMA RX channel 1 data pointer (0x%08x) must be 8 DWORDS aligned", (u32)(*pskb)->data);
  #else
            if ( g_eth_wan_mode == 0 || g_wanqos_en ) {
                ASSERT(((u32)(*pskb)->data & (DMA_ALIGNMENT - 1)) == 0, "DMA RX channel 1 data pointer (0x%#x) must be 8 DWORDS aligned", (u32)(*pskb)->data);
            }
  #endif
            *p++ = 0xB0000000 | DMA_PACKET_SIZE;
  #if !defined(ENABLE_P2P_COPY_WITHOUT_DESC) || !ENABLE_P2P_COPY_WITHOUT_DESC
            *p++ = (u32)(*pskb++)->data & 0x1FFFFFE0;
  #else
            if ( g_eth_wan_mode == 0 || g_wanqos_en )
                *p++ = (u32)(*pskb++)->data & 0x1FFFFFE0;
            else
                *p++ = 0x00000000;
  #endif
        }
    }

  #if !defined(ENABLE_P2P_COPY_WITHOUT_DESC) || !ENABLE_P2P_COPY_WITHOUT_DESC
    p = (volatile u32 *)DMA_RX_CH2_DESC_BASE;
    for ( i = 0; i < DMA_RX_CH2_DESC_NUM; i++ )
    {
        ASSERT(((u32)(*pskb)->data & (DMA_ALIGNMENT - 1)) == 0, "DMA RX channel 2 data pointer (0x%08x) must be 8 DWORDS aligned", (u32)(*pskb)->data);
        *p++ = 0xB0000000 | DMA_PACKET_SIZE;
        *p++ = (u32)(*pskb++)->data & 0x1FFFFFE0;
    }
  #endif
#endif

    if ( (g_eth_wan_mode == 0 || g_wanqos_en) && g_dsl_bonding != 2 )
    {
        p = (volatile u32 *)DMA_TX_CH1_DESC_BASE;
        for ( i = 0; i < DMA_TX_CH1_DESC_NUM; i++ )
        {
            ASSERT(((u32)(*pskb)->data & (DMA_ALIGNMENT - 1)) == 0, "WAN (PTM) RX or (ETH) TX data pointer (0x%08x) must be 8 DWORDS aligned", (u32)(*pskb)->data);
            *p++ = 0xB0000000;
            *p++ = (u32)(*pskb++)->data & 0x1FFFFFE0;
        }
    }

    g_f_dma_opened = 0;

    kfree(skb_pool);

    return 0;

DMA_DEVICE_REGISTER_FAIL:
    dma_device_release(g_dma_device_ppe);
    g_dma_device_ppe = NULL;
RESERVE_DMA_FAIL:
    i = pre_alloc_desc_total_num;
ALLOC_SKB_FAIL:
    while ( i-- )
        dev_kfree_skb_any(skb_pool[i]);
    kfree(skb_pool);
ALLOC_SKB_POOL_FAIL:
    return ret;

}

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
static INLINE int alloc_bonding_dma(void)
{
    volatile u32 *p;
    unsigned int pdata = DMA_ALIGNED(CPHYSADDR(g_bm_addr) + DMA_TX_CH1_DESC_NUM * 2 * sizeof(unsigned int));
//    unsigned int pdata = DMA_ALIGNED(CPHYSADDR(g_bm_addr) + (DMA_TX_CH1_DESC_NUM + DMA_RX_CH1_DESC_NUM) * 2 * sizeof(unsigned int));
    unsigned int e1_bar;
    int i;

    e1_bar = (unsigned int)SB_BUFFER_BOND(1, 0x3800);

    E1_FRAG_TX_DESBA->desba    = 0x6000;
    E1_FRAG_TX_DESBA->bp_desba = 0x6760;
    E1_FRAG_RX_DESBA->desba    = 0x6500;
    E1_FRAG_RX_DESBA->bp_desba = 0;         //  offset is ZERO
    *E1_DES_PDMA_BAR = e1_bar;
    *B1_DES_PDMA_BAR = 0;
    *DATAPTR_PDMA_BAR = 0;

    p = (volatile u32 *)DMA_TX_CH1_DESC_BASE;
    for ( i = 0; i < DMA_TX_CH1_DESC_NUM; i++ )
    {
        *p++ = 0xB0000000;
        *p++ = pdata;
        pdata = DMA_ALIGNED(pdata + 1664) ;
    }

    g_ppe_base_addr = IFX_PPE_ORG(1);

    E1_FRAG_TX_DESBA->desba    = E1_FRAG_TX_DESC_NB_PPE(4);
    E1_FRAG_TX_DESBA->bp_desba = E1_FRAG_TX_DESC_B_PPE(4);
    E1_FRAG_RX_DESBA->desba    = E1_FRAG_RX_DESC_PPE(1);
    E1_FRAG_RX_DESBA->bp_desba = 0x6800;    //  offset is ZERO
    *E1_DES_PDMA_BAR = 0;
    *B1_DES_PDMA_BAR = g_pci_host_ddr_addr;
    *DATAPTR_PDMA_BAR = (g_pci_host_ddr_addr & 0xFF000000) | 0x0000FF00;    //  31-16: prefix, 15-0: mask

    p = (volatile u32 *)E1_FRAG_TX_DESC_NB_BASE(0);
    for ( i = 0; i < E1_FRAG_TX_DESC_NUM_TOTAL; i++ )
    {
        *p++ = 0x30000000;
        *p++ = 0x00000000;
    }

    p = (volatile u32 *)E1_FRAG_RX_DESC_BASE(0);
    for ( i = 0; i < E1_FRAG_RX_DESC_NUM * 2; i++ )
    {
        *p++ = 0xB1000000 | 1664 /* 544 */;
        *p++ = pdata;
        pdata = DMA_ALIGNED(pdata + 1664 /* 544 */);
    }

    p = (volatile u32 *)B1_RX_LINK_LIST_DESC_BASE;
    for ( i = 0; i < B1_RX_LINK_LIST_DESC_NUM; i++ )
    {
        *p++ = ((B1_RX_LINK_LIST_DESC_PPE + 2 * ((i + 1) & 0xFF)) << 16) | 1664 /* 544 */;
        *p++ = pdata;
        pdata = DMA_ALIGNED(pdata + 1664 /* 544 */);
    }

    p = (volatile u32 *)WAN_TX_DESC_BASE(0);
    for ( i = 0; i < WAN_TX_DESC_NUM_TOTAL; i++ )
    {
        *p++ = 0x30000000;
        *p++ = pdata;
        pdata = DMA_ALIGNED(pdata + 1664);
    }

    p = (volatile u32 *)DMA_RX_CH1_DESC_BASE;
    for ( i = 0; i < DMA_RX_CH1_DESC_NUM; i++ )
    {
        *p++ = 0xB0000000 | DMA_PACKET_SIZE;
        *p++ = pdata;
        pdata = DMA_ALIGNED(pdata + 1664);
    }
#if defined(ENABLE_MPS_MEMORY) && ENABLE_MPS_MEMORY
    *US_CDMA_RX_DES_PDMA_BAR = DMA_RX_CH1_DESC_MPS_ADDR;
    *US_CDMA_RX_DES_BASE     = 0x6800;
//    *US_CDMA_RX_DES_PDMA_BAR = g_pci_host_ddr_addr;
//    *US_CDMA_RX_DES_BASE     = 0x6800 + DMA_TX_CH1_DESC_NUM * 2;
#else
    *US_CDMA_RX_DES_PDMA_BAR = 0;
    *US_CDMA_RX_DES_BASE     = 0x2580;
#endif

    g_ppe_base_addr = IFX_PPE_ORG(0);

    return 0;
}
#endif


static INLINE void free_dma(void)
{
    volatile struct tx_descriptor *p;
    struct sk_buff *skb_to_free;
    int i;

    if ( g_eth_wan_mode == 0 || g_wanqos_en )
    {
        if ( g_eth_wan_mode == 0 && g_dsl_bonding )
        {
            if ( g_dsl_bonding == 1 )
            {
                p = (volatile struct tx_descriptor *)E1_FRAG_RX_DESC_BASE(0);
                for ( i = 0; i < E1_FRAG_RX_DESC_NUM; i++ )
                {
                    skb_to_free = get_skb_pointer(p->dataptr);
                    dev_kfree_skb_any(skb_to_free);
                    p++;
                }

                p = (volatile struct tx_descriptor *)B1_RX_LINK_LIST_DESC_BASE;
                for ( i = 0; i < B1_RX_LINK_LIST_DESC_NUM; i++ )
                {
                    skb_to_free = get_skb_pointer(p->dataptr);
                    dev_kfree_skb_any(skb_to_free);
                    p++;
                }
            }
        }
        else
        {
            p = (volatile struct tx_descriptor *)CPU_TO_WAN_TX_DESC_BASE;
            for ( i = 0; i < CPU_TO_WAN_TX_DESC_NUM; i++ )
            {
                skb_to_free = get_skb_pointer(p->dataptr);
                dev_kfree_skb_any(skb_to_free);
                p++;
            }
        }

        if ( g_dsl_bonding != 2 )
        {
            p = (volatile struct tx_descriptor *)WAN_TX_DESC_BASE(0);
            for ( i = 0; i < WAN_TX_DESC_NUM_TOTAL; i++ )
            {
                skb_to_free = get_skb_pointer(p->dataptr);
                dev_kfree_skb_any(skb_to_free);
                p++;
            }
        }

        if ( g_dsl_bonding == 0 )
        {
            p = (volatile struct tx_descriptor *)CPU_TO_WAN_SWAP_DESC_BASE;
            for ( i = 0; i < CPU_TO_WAN_SWAP_DESC_NUM; i++ )
            {
                skb_to_free = get_skb_pointer(p->dataptr);
                dev_kfree_skb_any(skb_to_free);
                p++;
            }
        }
    }

#if !defined(DEBUG_REDIRECT_FASTPATH_TO_CPU) || !DEBUG_REDIRECT_FASTPATH_TO_CPU
    if ( g_dsl_bonding != 2 )
    {
        p = (volatile struct tx_descriptor *)DMA_RX_CH1_DESC_BASE;
        for ( i = 0; i < DMA_RX_CH1_DESC_NUM; i++ )
        {
  #if defined(ENABLE_P2P_COPY_WITHOUT_DESC) && ENABLE_P2P_COPY_WITHOUT_DESC
            if ( g_eth_wan_mode != 0 && !g_wanqos_en )
                break;
  #endif
            skb_to_free = get_skb_pointer(p->dataptr);
            dev_kfree_skb_any(skb_to_free);
            p++;
        }
    }
#endif

#if !defined(ENABLE_P2P_COPY_WITHOUT_DESC) || !ENABLE_P2P_COPY_WITHOUT_DESC
    p = (volatile struct tx_descriptor *)DMA_RX_CH2_DESC_BASE;
    for ( i = 0; i < DMA_RX_CH2_DESC_NUM; i++ )
    {
        skb_to_free = get_skb_pointer(p->dataptr);
        dev_kfree_skb_any(skb_to_free);
        p++;
    }
#endif

    if ( (g_eth_wan_mode == 0 || g_wanqos_en) && g_dsl_bonding != 2 )
    {
        p = (volatile struct tx_descriptor *)DMA_TX_CH1_DESC_BASE;
        for ( i = 0; i < DMA_TX_CH1_DESC_NUM; i++ )
        {
            skb_to_free = get_skb_pointer(p->dataptr);
            dev_kfree_skb_any(skb_to_free);
            p++;
        }
    }

	// AVM: we do this in dma_shutdown_gracefully
	dma_device_unregister(g_dma_device_ppe);
	dma_device_release(g_dma_device_ppe);
	g_dma_device_ppe = NULL;

#if defined(DEBUG_SKB_SWAP) && DEBUG_SKB_SWAP
    for ( i = 0; i < NUM_ENTITY(g_dbg_skb_swap_pool); i++ )
        if ( g_dbg_skb_swap_pool[i] != NULL )
        {
            err("skb swap pool is not clean: %d - %08x\n", i, (unsigned int)g_dbg_skb_swap_pool[i]);
        }
#endif
}




#if defined(ENABLE_MY_MEMCPY) && ENABLE_MY_MEMCPY
static INLINE void *my_memcpy(unsigned char *dest, const unsigned char *src, unsigned int count)
{
	char *d = (char *)dest, *s = (char *)src;

	if (count >= 32) {
		int i = 8 - (((unsigned long) d) & 0x7);

		if (i != 8)
		while (i-- && count--) {
			*d++ = *s++;
		}

		if (((((unsigned long) d) & 0x7) == 0) &&
				((((unsigned long) s) & 0x7) == 0)) {
			while (count >= 32) {
				unsigned long long t1, t2, t3, t4;
				t1 = *(unsigned long long *) (s);
				t2 = *(unsigned long long *) (s + 8);
				t3 = *(unsigned long long *) (s + 16);
				t4 = *(unsigned long long *) (s + 24);
				*(unsigned long long *) (d) = t1;
				*(unsigned long long *) (d + 8) = t2;
				*(unsigned long long *) (d + 16) = t3;
				*(unsigned long long *) (d + 24) = t4;
				d += 32;
				s += 32;
				count -= 32;
			}
			while (count >= 8) {
				*(unsigned long long *) d =
				*(unsigned long long *) s;
				d += 8;
				s += 8;
				count -= 8;
			}
		}

		if (((((unsigned long) d) & 0x3) == 0) &&
				((((unsigned long) s) & 0x3) == 0)) {
			while (count >= 4) {
				*(unsigned long *) d = *(unsigned long *) s;
				d += 4;
				s += 4;
				count -= 4;
			}
		}

		if (((((unsigned long) d) & 0x1) == 0) &&
				((((unsigned long) s) & 0x1) == 0)) {
			while (count >= 2) {
				*(unsigned short *) d = *(unsigned short *) s;
				d += 2;
				s += 2;
				count -= 2;
			}
		}
	}

	while (count--) {
		*d++ = *s++;
	}

	return d;
}
#endif
static int print_fw_ver(char *buf, int buf_len)
{
    static char * fw_ver_family_str[] = {
        "reserved - 0",
        "Danube",
        "Twinpass",
        "Amazon-SE",
        "reserved - 4",
        "AR9",
        "GR9",
        "VR9",
        "AR10"
    };
    static char * fw_ver_type_str[] = {
        "reserved - 0",
        "Standard",
        "Acceleration",
        "VDSL2 Bonding"
    };
    static char * fw_ver_interface_str[] = {
        "MII0",
        "MII0 + MII1",
        "MII0 + ATM",
        "MII0 + PTM",
        "MII0/1 + ATM",
        "MII0/1 + PTM",
        "reserved - 6",
        "MII0/1 + PTM Bonding"
    };
    static char * fw_ver_mode_str[] = {
        "reserved - 0",
        "PTM (E1)",
        "reserved - 2",
        "Routing",
        "reserved - 4",
        "Bridging",
        "Bridging + IPv4 Routing",
        "Bridging + IPv4/6 Routing",
        "6rd/DS-lite"
    };

    int len = 0;
    unsigned int i;

    for ( i = 0; i < NUM_ENTITY(g_fw_ver); i++ )
    {
        if ( *(unsigned int *)(&g_fw_ver[i]) == 0 )
            continue;

        len += snprintf(buf + len, buf_len - len, "PPE %d firmware info:\n", i);
        len += snprintf(buf + len, buf_len - len, "  Version ID: %d.%d.%d.%d.%d.%d\n", (int)g_fw_ver[i].family, (int)g_fw_ver[i].fwtype, (int)g_fw_ver[i].interface, (int)g_fw_ver[i].fwmode, (int)g_fw_ver[i].major, (int)g_fw_ver[i].minor);
        if ( g_fw_ver[i].family >= NUM_ENTITY(fw_ver_family_str) )
            len += snprintf(buf + len, buf_len - len, "  Family    : reserved - %d\n", (int)g_fw_ver[i].family);
        else
            len += snprintf(buf + len, buf_len - len, "  Family    : %s\n", fw_ver_family_str[g_fw_ver[i].family]);
        if ( g_fw_ver[i].fwtype >= NUM_ENTITY(fw_ver_type_str) )
            len += snprintf(buf + len, buf_len - len, "  FW Type   : reserved - %d\n", (int)g_fw_ver[i].fwtype);
        else
            len += snprintf(buf + len, buf_len - len, "  FW Type   : %s\n", fw_ver_type_str[g_fw_ver[i].fwtype]);
        if ( g_fw_ver[i].interface >= NUM_ENTITY(fw_ver_interface_str) )
            len += snprintf(buf + len, buf_len - len, "  Interface : reserved - %d\n", (int)g_fw_ver[i].interface);
        else
            len += snprintf(buf + len, buf_len - len, "  Interface : %s\n", fw_ver_interface_str[g_fw_ver[i].interface]);
        if ( g_fw_ver[i].fwmode >= NUM_ENTITY(fw_ver_mode_str) )
            len += snprintf(buf + len, buf_len - len, "  Mode      : reserved - %d\n", (int)g_fw_ver[i].fwmode);
        else
            len += snprintf(buf + len, buf_len - len, "  Mode      : %s\n", fw_ver_mode_str[g_fw_ver[i].fwmode]);
        len += snprintf(buf + len, buf_len - len, "  Release   : %d.%d\n", (int)g_fw_ver[i].major, (int)g_fw_ver[i].minor);
    }

    if ( len == 0 )
    {
        len += snprintf(buf + len, buf_len - len, "PPE firmware info:\n");
        len += snprintf(buf + len, buf_len - len, "  Version ID: %d.%d.%d.%d.%d.%d\n", (int)FW_VER_ID->family, (int)FW_VER_ID->fwtype, (int)FW_VER_ID->interface, (int)FW_VER_ID->fwmode, (int)FW_VER_ID->major, (int)FW_VER_ID->minor);
        if ( FW_VER_ID->family >= NUM_ENTITY(fw_ver_family_str) )
            len += snprintf(buf + len, buf_len - len, "  Family    : reserved - %d\n", (int)FW_VER_ID->family);
        else
            len += snprintf(buf + len, buf_len - len, "  Family    : %s\n", fw_ver_family_str[FW_VER_ID->family]);
        if ( FW_VER_ID->fwtype >= NUM_ENTITY(fw_ver_type_str) )
            len += snprintf(buf + len, buf_len - len, "  FW Type   : reserved - %d\n", (int)FW_VER_ID->fwtype);
        else
            len += snprintf(buf + len, buf_len - len, "  FW Type   : %s\n", fw_ver_type_str[FW_VER_ID->fwtype]);
        if ( FW_VER_ID->interface >= NUM_ENTITY(fw_ver_interface_str) )
            len += snprintf(buf + len, buf_len - len, "  Interface : reserved - %d\n", (int)FW_VER_ID->interface);
        else
            len += snprintf(buf + len, buf_len - len, "  Interface : %s\n", fw_ver_interface_str[FW_VER_ID->interface]);
        if ( FW_VER_ID->fwmode >= NUM_ENTITY(fw_ver_mode_str) )
            len += snprintf(buf + len, buf_len - len, "  Mode      : reserved - %d\n", (int)FW_VER_ID->fwmode);
        else
            len += snprintf(buf + len, buf_len - len, "  Mode      : %s\n", fw_ver_mode_str[FW_VER_ID->fwmode]);
        len += snprintf(buf + len, buf_len - len, "  Release   : %d.%d\n", (int)FW_VER_ID->major, (int)FW_VER_ID->minor);
    }

    return len;
}

static int print_driver_ver(char *buf, int buf_len, char *title,
		unsigned int family, unsigned int type, unsigned int itf,
		unsigned int mode, unsigned int major, unsigned int mid,
		unsigned int minor) {
	static char * dr_ver_family_str[] = { NULL, "Danube", "Twinpass",
			"Amazon-SE", NULL, "AR9", "VR9", "AR10", NULL, };
	static char * dr_ver_type_str[] =
			{ "Normal Data Path", "Indirect-Fast Path", "HAL", "Hook",
					"OS Adatpion Layer", "PPA API" };
	static char * dr_ver_interface_str[] = { "MII0", "MII1", "ATM", "PTM" };
	static char * dr_ver_accmode_str[] = { "Routing", "Bridging", };

	int len = 0;
	unsigned int bit;
	unsigned int i, j;

	len += snprintf(buf + len, buf_len - len, "%s:\n", title);
	len += snprintf(buf + len, buf_len - len,
			"  Version ID: %d.%d.%d.%d.%d.%d.%d\n", family, type, itf, mode,
			major, mid, minor);
	len += snprintf(buf + len, buf_len - len, "  Family    : ");
	for (bit = family, i = j = 0; bit != 0 && i < NUM_ENTITY(dr_ver_family_str);
			bit >>= 1, i++)
		if ((bit & 0x01) && dr_ver_family_str[i] != NULL) {
			if (j)
				len += snprintf(buf + len, buf_len - len, " | %s",
						dr_ver_family_str[i]);
			else
				len += snprintf(buf + len, buf_len - len, dr_ver_family_str[i]);
			j++;
		}
	if (j)
		len += snprintf(buf + len, buf_len - len, "\n");
	else
		len += snprintf(buf + len, buf_len - len, "N/A\n");
	len += snprintf(buf + len, buf_len - len, "  DR Type   : ");
	for (bit = type, i = j = 0; bit != 0 && i < NUM_ENTITY(dr_ver_type_str);
			bit >>= 1, i++)
		if ((bit & 0x01) && dr_ver_type_str[i] != NULL) {
			if (j)
				len += snprintf(buf + len, buf_len - len, " | %s",
						dr_ver_type_str[i]);
			else
				len += snprintf(buf + len, buf_len - len, dr_ver_type_str[i]);
			j++;
		}
	if (j)
		len += snprintf(buf + len, buf_len - len, "\n");
	else
		len += snprintf(buf + len, buf_len - len, "N/A\n");
	len += snprintf(buf + len, buf_len - len, "  Interface : ");
	for (bit = itf, i = j = 0; bit != 0 && i < NUM_ENTITY(dr_ver_interface_str);
			bit >>= 1, i++)
		if ((bit & 0x01) && dr_ver_interface_str[i] != NULL) {
			if (j)
				len += snprintf(buf + len, buf_len - len, " | %s",
						dr_ver_interface_str[i]);
			else
				len += snprintf(buf + len, buf_len - len,
						dr_ver_interface_str[i]);
			j++;
		}
	if (j)
		len += snprintf(buf + len, buf_len - len, "\n");
	else
		len += snprintf(buf + len, buf_len - len, "N/A\n");
	len += snprintf(buf + len, buf_len - len, "  Mode      : ");
	for (bit = mode, i = j = 0; bit != 0 && i < NUM_ENTITY(dr_ver_accmode_str);
			bit >>= 1, i++)
		if ((bit & 0x01) && dr_ver_accmode_str[i] != NULL) {
			if (j)
				len += snprintf(buf + len, buf_len - len, " | %s",
						dr_ver_accmode_str[i]);
			else
				len += snprintf(buf + len, buf_len - len,
						dr_ver_accmode_str[i]);
			j++;
		}
	if (j)
		len += snprintf(buf + len, buf_len - len, "\n");
	else
		len += snprintf(buf + len, buf_len - len, "N/A\n");
	len += snprintf(buf + len, buf_len - len, "  Release   : %d.%d.%d\n", major,
			mid, minor);

	return len;
}


#if (defined(DEBUG_FW_PROC) && DEBUG_FW_PROC) || (defined(DEBUG_BONDING_PROC) && DEBUG_BONDING_PROC)

static int INLINE is_char_in_set(char ch, char * set)
{

    while ( *set )
    {
        if ( ch == *set )
            return 1;
        set++;
    }

    return 0;
}

static int tokenize(char * in, char * sep, char **tokens, int max_token_num, int * finished)
{
    int token_num = 0;

    * finished = 0;

    while ( *in && token_num < max_token_num )
    {
        // escape all seperators
        while ( *in && is_char_in_set(*in, sep) )
            in++;

        // break if no more chars left
        if ( !*in )
            break;

        // found a new token, remember start position
        tokens[token_num++] = in;

        // search end of token
        in++;
        while ( *in && !is_char_in_set(*in, sep) )
            in++;

        if ( !*in )
            // break if no more chars left
            break;
        else
        {
            // tokenize
            *in = '\0';
            in++;
        }
    }

    if ( !*in )
        * finished = 1;

    return token_num;
}

int str2uint(char * p, unsigned int* res, int is_hex)
{
    int i = 0;
    unsigned int uint = 0;

    // [0x][0-9A-Fa-f]+
        if( p[0] == '0' && (  p[1]  == 'x' ||  p[1] == 'X')) {
            is_hex = 1;
            p += 2;
        }

    // check syntax
    while(p[i]) {
        if ( p[i] >= '0' && p[i] <= '9' ) {

            if (is_hex) {
                uint = (uint << 4) + p[i] - '0';
            }
            else {
                uint = uint * 10 + p[i] - '0';
            }

        }
        else {

            if (! is_hex)
                return 0;

            if ( p[i] >= 'a' && p[i] <= 'f' )
                uint = (uint << 4) + p[i] - 'a' + 10;
            else if ( p[i] >= 'A' && p[i] <= 'F' )
                uint = (uint << 4) + p[i] - 'A' + 10;
            else
                return 0;
        }

        ++i;
    }

    * res = uint;
    return 1;
}

#endif

static void dsl_file_create(void)
{
    struct proc_dir_entry *res;

    g_dsl_proc_dir = proc_mkdir("dsl_tc", NULL);

    res = create_proc_entry("status",
                            0,
                            g_dsl_proc_dir);
    if ( res )
    {
        res->read_proc  = dsl_proc_read_status;
        res->write_proc = dsl_proc_write_status;
    }
}

static void dsl_file_delete(void)
{
    remove_proc_entry("status",
                      g_dsl_proc_dir);

    remove_proc_entry("dsl_tc", NULL);
}

static int dsl_proc_read_status(char *page, char **start __attribute__((unused)), off_t off __attribute__((unused)), int count __attribute__((unused)), int *eof, void *data __attribute__((unused)))
{
    int len = 0;
    char *dsl_bonding_str[] = {"non-bonding", "onchip bonding", "offchip bonding"};
    unsigned int i;

    len += sprintf(page + len, "DSL Bonding Capability: %s\n", dsl_bonding_str[g_dsl_bonding]);
    if ( g_dsl_bonding)
    {
        len += sprintf(page + len, BOND_CONF->bond_mode == L1_PTM_TC_BOND_MODE ? "Bonding Mode: PTM TC\n" : "Bonding Mode: ETH Trunk\n");
    }
    len += sprintf(page + len, "Upstream Data Rate:  ");
    for ( i = 0; i < NUM_ENTITY(g_datarate_us); i++ )
        len += sprintf(page + len, "\t%d", g_datarate_us[i]);
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "Downstream Data Rate:");
    for ( i = 0; i < NUM_ENTITY(g_datarate_ds); i++ )
        len += sprintf(page + len, "\t%d", g_datarate_ds[i]);
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "User Specified Max_Delay:");
    for ( i = 0; i < NUM_ENTITY(g_max_delay); i++ )
        len += sprintf(page + len, "\t%u", g_max_delay[i]);
    len += sprintf(page + len, "\n");

    *eof = 1;

    return len;
}

static int dsl_proc_write_status(struct file *file __attribute__((unused)), const char *buf, unsigned long count, void *data __attribute__((unused)))
{
    char *p1, *p2;
    int len;
    int colon;
    char local_buf[1024];

    char *env_var_str[] = {"help", "DSL_LINE_NUMBER", "DSL_TC_LAYER_STATUS", "DSL_EFM_TC_CONFIG_US", "DSL_BONDING_STATUS", "DSL_INTERFACE_STATUS", "DSL_DATARATE_US_BC", "DSL_DATARATE_DS_BC", "MAX_DELAY"};
    int line = -1, down = 0, upstr = 0, downstr = 0;
    int datarate_us[BEARER_CHANNEL_PER_LINE] = {0}, datarate_ds[BEARER_CHANNEL_PER_LINE] = {0};
    unsigned int num, bc;
    unsigned int i;

    len = sizeof(local_buf) < count ? sizeof(local_buf) - 1 : count;
    len = len - copy_from_user(local_buf, buf, len);
    local_buf[len] = 0;

    p1 = local_buf;
    p2 = NULL;
    colon = 1;
    while ( ifx_get_token(&p1, &p2, &len, &colon) )
    {
        for ( i = 0; i < NUM_ENTITY(env_var_str) && ifx_strincmp(p1, env_var_str[i], strlen(env_var_str[i])) != 0; i++ );
        switch ( i )
        {
        case 0: //  help
            printk("Usage:\n");
            printk("  echo <command> [parameters] > /proc/dsl_tc/status\n");
            printk("  commands:\n");
            printk("    DSL_LINE_NUMBER      - parameter: line number\n");
            printk("    DSL_TC_LAYER_STATUS  - parameter: ATM, EFM\n");
            printk("    DSL_EFM_TC_CONFIG_US - parameter: normal, preemption\n");
            printk("    DSL_BONDING_STATUS   - parameter: inactive, active\n");
            printk("    DSL_INTERFACE_STATUS - parameter: down, ready, handshake, training, up\n");
            printk("    DSL_DATARATE_US_BC?  - ?: bearer channel number 0/1, parameter: data rate in bps\n");
            printk("    DSL_DATARATE_DS_BC?  - ?: bearer channel number 0/1, parameter: data rate in bps\n");
            printk("    MAX_DELAY            - parameter: user specified max delay in micorsecond\n");
            break;
        case 1: //  DSL_LINE_NUMBER
            ifx_ignore_space(&p2, &len);
            num = ifx_get_number(&p2, &len, 0);
            if ( num < LINE_NUMBER )
                line = num;
            break;
        case 2: //  DSL_TC_LAYER_STATUS
            //  TODO
            break;
        case 3: //  DSL_EFM_TC_CONFIG_US
            //  TODO
            break;
        case 4: //  DSL_BONDING_STATUS
            if ( g_dsl_bonding )
            {
                p1 = p2;
                colon = 1;
                if ( ifx_get_token(&p1, &p2, &len, &colon) )
                {
                    if ( ifx_stricmp(p1, "inactive") == 0 )
                        switch_to_eth_trunk();
                    else if ( ifx_stricmp(p1, "active") == 0 )
                        switch_to_ptmtc_bonding();
                }
            }
            break;
        case 5: //  DSL_INTERFACE_STATUS
            if ( line >= 0 )
            {
                p1 = p2;
                colon = 1;
                if ( ifx_get_token(&p1, &p2, &len, &colon) && ifx_stricmp(p1, "down") == 0 && g_line_showtime[line] != 0 )
                {
                    g_line_showtime[line] = 0;
                    down = 1;
                }
            }
            break;
        case 6: //  DSL_DATARATE_US_BC
            bc = (unsigned int)(p1[strlen(env_var_str[i])] - '0');
            if ( line >= 0 && bc < BEARER_CHANNEL_PER_LINE )
            {
                ifx_ignore_space(&p2, &len);
                num = ifx_get_number(&p2, &len, 0);
                if ( num != 0 )
                {
                    datarate_us[bc] = num;
                    upstr = 1;
                }
            }
            break;
        case 7: //  DSL_DATARATE_DS_BC
            bc = (unsigned int)(p1[strlen(env_var_str[i])] - '0');
            if ( line >= 0 && bc < BEARER_CHANNEL_PER_LINE )
            {
                ifx_ignore_space(&p2, &len);
                num = ifx_get_number(&p2, &len, 0);
                if ( num != 0 )
                {
                    datarate_ds[bc] = num;
                    downstr = 1;
                }
            }
            break;
        case 8: //  MAX_DELAY
            if ( line >= 0 )
            {
                ifx_ignore_space(&p2, &len);
                num = ifx_get_number(&p2, &len, 0);
                g_max_delay[line] = num;
                if ( g_dsl_bonding )
                    update_max_delay(line);
            }
            break;
        }

        p1 = p2;
        colon = 1;
    }

    if ( line >= 0 )
    {
        if ( down )
        {
            for ( i = 0, num = 0; i < LINE_NUMBER; num += g_line_showtime[i], i++ );
            if ( num == 0 )
            {
                dsl_showtime_exit();
                for ( i = 0; i < NUM_ENTITY(g_datarate_us); i++ )
                    g_datarate_us[i] = 0;
                for ( i = 0; i < NUM_ENTITY(g_datarate_ds); i++ )
                    g_datarate_ds[i] = 0;
            }
        }
        else
        {
            if ( upstr )
            {
                for ( i = 0; i < BEARER_CHANNEL_PER_LINE; i++ )
                    if ( datarate_us[i] != 0 )
                        g_datarate_us[line * BEARER_CHANNEL_PER_LINE + i] = datarate_us[i];
                g_line_showtime[line] = 1;
                g_showtime = 1; //  equivalent to dsl_showtime_enter
            }
            if ( downstr )
            {
                for ( i = 0; i < BEARER_CHANNEL_PER_LINE; i++ )
                    if ( datarate_ds[i] != 0 )
                        g_datarate_ds[line * BEARER_CHANNEL_PER_LINE + i] = datarate_ds[i];
                if ( g_dsl_bonding )
                    update_max_delay(line);
            }
        }
    }

    return count;
}

static int print_dsl_dma_desc(char *buf, volatile u32 *desc_base, int desc_pos) {
    int len = 0;
    u32 *p;

    p = (u32*)desc_base;
    if ( desc_pos == 0 ) {
        len += sprintf(buf + len,"DSL Tx descriptor list:\n" );

        len += sprintf(buf + len," no  address        data pointer command bits (Own, Complete, SoP, EoP, Offset) \n");
        len += sprintf(buf + len,"---------------------------------------------------------------------------------\n");
    }
    len += sprintf(buf + len,"%3d  ",desc_pos);
    len += sprintf(buf + len,"0x%08x     ", (u32)(p + (desc_pos * 2)));
    len += sprintf(buf + len,"%08x     ", *(p + (desc_pos * 2 + 1)));
    len += sprintf(buf + len,"%08x     ", *(p + (desc_pos * 2)));

    if(*(p + (desc_pos * 2)) & 0x80000000)
        len += sprintf(buf + len, "D ");
    else
    	len += sprintf(buf + len, "C ");
    if(*(p + (desc_pos * 2)) & 0x40000000)
        len += sprintf(buf + len, "C ");
    else
        len += sprintf(buf + len, "c ");
    if(*(p + (desc_pos * 2)) & 0x20000000)
        len += sprintf(buf + len, "S ");
    else
        len += sprintf(buf + len, "s ");
    if(*(p + (desc_pos * 2)) & 0x10000000)
        len += sprintf(buf + len, "E ");
    else
        len += sprintf(buf + len, "e ");
    len += sprintf(buf + len,"%02x ", (*(p + (desc_pos * 2)) & 0x0F800000) >> 23);
    if( g_cpu_to_wan_tx_desc_pos  == desc_pos )
        len += sprintf(buf + len,"<- CURR");
    len += sprintf(buf + len,"\n");

    return len;
}

static int proc_read_dsl_tx_dma_descr(IFX_char_t *page, IFX_char_t **start, off_t off __attribute__((unused)), int count, int *eof, void *data __attribute__((unused)))
{
    int len = 0;
    int len_max = off + count;
    char *pstr;
    char *str;
    int llen;
    int j;

    pstr = *start = page;

    str = vmalloc(1024);
    if(!str) {
        return -ENOMEM;
    }

    __sync();

	for (j = 0; j < CPU_TO_WAN_TX_DESC_NUM; j++) {
		llen = print_dsl_dma_desc(str,  (volatile u32 *)CPU_TO_WAN_TX_DESC_BASE, j);
		if ( len <= off && len + llen > off ) {
			memcpy(pstr, str + off - len, len + llen - off);
			pstr += len + llen - off;
		}
		else if ( len > off ) {
			memcpy(pstr, str, llen);
			pstr += llen;
		}
		len += llen;
		if ( len >= len_max )
			goto PROC_READ_DESC_LIST_OVERRUN_END;
	}

    *eof = 1;

    vfree(str);

    return len - off;

PROC_READ_DESC_LIST_OVERRUN_END:
    vfree(str);

    return len - llen - off;
}


static INLINE void proc_file_create(void) {
	struct proc_dir_entry *res;

	// g_eth_proc_dir = proc_mkdir("eth", NULL);  wird im common treiber angelegt;

	create_proc_read_entry("ver", 0, g_eth_proc_dir, proc_read_ver, NULL);

#if defined(SETUP_OWN_NETDEVS)
	res = create_proc_entry("mib",
			0,
			g_eth_proc_dir);
	if ( res )
	{
		res->read_proc = proc_read_mib;
		res->write_proc = proc_write_mib;
	}
#endif

	res = create_proc_entry("wanmib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_wanmib;
		res->write_proc = proc_write_wanmib;
	}

	res = create_proc_entry("dsl_tx_dma_descr", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_dsl_tx_dma_descr;
	}


#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC
	res = create_proc_entry("route",
			0,
			g_eth_proc_dir);
	if ( res )
	{
		res->read_proc = proc_read_route;
		res->write_proc = proc_write_route;
	}

	create_proc_read_entry("pppoe",
			0,
			g_eth_proc_dir,
			proc_read_pppoe,
			NULL);

	create_proc_read_entry("mtu",
			0,
			g_eth_proc_dir,
			proc_read_mtu,
			NULL);

	res = create_proc_entry("hit",
			0,
			g_eth_proc_dir);
	if ( res )
	{
		res->read_proc = proc_read_hit;
		res->write_proc = proc_write_hit;
	}

	create_proc_read_entry("mac",
			0,
			g_eth_proc_dir,
			proc_read_mac,
			NULL);

	create_proc_read_entry("out_vlan",
			0,
			g_eth_proc_dir,
			proc_read_out_vlan,
			NULL);

	if ( g_ipv6_acc_en )
	{
		create_proc_read_entry("ipv6_ip",
				0,
				g_eth_proc_dir,
				proc_read_ipv6_ip,
				NULL);
	}

#endif

#if 0 && defined(ENABLE_DBG_PROC) && ENABLE_DBG_PROC //wird im common Treiber angelegt
	res = create_proc_entry("dbg",
			0,
			g_eth_proc_dir);
	if ( res )
	{
		res->read_proc = proc_read_dbg;
		res->write_proc = proc_write_dbg;
	}
#endif

#if defined(DEBUG_MEM_PROC) && DEBUG_MEM_PROC
	res = create_proc_entry("mem", 0, g_eth_proc_dir);
	if (res)
		res->write_proc = proc_write_mem;
#endif

	res = create_proc_entry("burstlen", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_burstlen;
		res->write_proc = proc_write_burstlen;
	}

#if defined(DEBUG_FW_PROC) && DEBUG_FW_PROC
	res = create_proc_entry("fw", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_fw;
		res->write_proc = proc_write_fw;
	}

	res = create_proc_read_entry("tfwdbg", 0, g_eth_proc_dir, NULL, NULL);
	if (res)
		res->write_proc = proc_write_fwdbg;

#endif

	res = create_proc_entry("port0mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port0_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("port1mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port1_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("port2mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port2_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("port3mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port3_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("port4mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port4_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("port5mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port5_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("port6mib", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = port6_counters_proc_read;
		res->write_proc = port_counters_proc_write;
	}

	res = create_proc_entry("direct_forwarding", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_directforwarding;
		res->write_proc = proc_write_directforwarding;
	}

	res = create_proc_entry("clk", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_clk;
		res->write_proc = proc_write_clk;
	}

	res = create_proc_entry("prio", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_prio;
		res->write_proc = proc_write_prio;
	}

	res = create_proc_entry("ewan", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_ewan;
		res->write_proc = proc_write_ewan;
	}

#ifdef CONFIG_IFX_PPA_QOS
	if (g_eth_wan_mode == 0 || g_wanqos_en) {
		create_proc_read_entry("qos", 0, g_eth_proc_dir, proc_read_qos, NULL);
	}
#endif

	if (g_eth_wan_mode == 0) {
		create_proc_read_entry("gamma", 0, g_eth_proc_dir, proc_read_gamma,
				NULL);
	}


#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
	res = create_proc_entry("mirror", 0, g_eth_proc_dir);
	if (res) {
		res->read_proc = proc_read_mirror;
		res->write_proc = proc_write_mirror;
	}
#endif

    res = create_proc_entry("class",
                            0,
                            g_eth_proc_dir);
    if ( res )
    {
        res->read_proc  = proc_read_class;
        res->write_proc = proc_write_class;
    }
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_dsl_bonding == 2 )
    {
        res = create_proc_entry("dev",
                                0,
                                g_eth_proc_dir);
        if ( res )
        {
            res->read_proc  = proc_read_dev;
            res->write_proc = proc_write_dev;
        }
    }
#endif

#if defined(DEBUG_BONDING_PROC) && DEBUG_BONDING_PROC
    res = create_proc_entry("bonding",
                           0,
                           g_eth_proc_dir);
    if ( res )
    {
        init_ds_des_array();
        res->read_proc  = proc_read_bonding;
        res->write_proc = proc_write_bonding;
    }
#endif
}

static INLINE void proc_file_delete(void) {

#if defined(DEBUG_BONDING_PROC) && DEBUG_BONDING_PROC
    remove_proc_entry("bonding",
                      g_eth_proc_dir);
#endif

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_dsl_bonding == 2 )
    {
        remove_proc_entry("dev",
                          g_eth_proc_dir);
    }
#endif

    remove_proc_entry("class",
                      g_eth_proc_dir);


#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC
	remove_proc_entry("mirror", g_eth_proc_dir);
#endif

	if (g_eth_wan_mode == 0) {
		remove_proc_entry("gamma", g_eth_proc_dir);
	}

#ifdef CONFIG_IFX_PPA_QOS
	if (g_eth_wan_mode == 0 || g_wanqos_en) {
		remove_proc_entry("qos", g_eth_proc_dir);
	}
#endif

	remove_proc_entry("ewan", g_eth_proc_dir);

	remove_proc_entry("prio", g_eth_proc_dir);

#if defined(ENABLE_NAPI) && ENABLE_NAPI
	remove_proc_entry("napi", g_eth_proc_dir);
#endif

	remove_proc_entry("clk", g_eth_proc_dir);

	remove_proc_entry("direct_forwarding", g_eth_proc_dir);

	remove_proc_entry("port6mib", g_eth_proc_dir);

	remove_proc_entry("port5mib", g_eth_proc_dir);

	remove_proc_entry("port4mib", g_eth_proc_dir);

	remove_proc_entry("port3mib", g_eth_proc_dir);

	remove_proc_entry("port2mib", g_eth_proc_dir);

	remove_proc_entry("port1mib", g_eth_proc_dir);

	remove_proc_entry("port0mib", g_eth_proc_dir);

#if defined(DEBUG_FW_PROC) && DEBUG_FW_PROC
	remove_proc_entry("tfwdbg", g_eth_proc_dir);

	remove_proc_entry("fw", g_eth_proc_dir);
#endif

	remove_proc_entry("burstlen", g_eth_proc_dir);

#if defined(DEBUG_MEM_PROC) && DEBUG_MEM_PROC
	remove_proc_entry("mem", g_eth_proc_dir);
#endif

#if 0 && defined(ENABLE_DBG_PROC) && ENABLE_DBG_PROC //wird im common Treiber angelegt
	remove_proc_entry("dbg",
			g_eth_proc_dir);
#endif

#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC

	if ( g_ipv6_acc_en )
	{
		remove_proc_entry("ipv6_ip",
				g_eth_proc_dir);
	}

	remove_proc_entry("out_vlan",
			g_eth_proc_dir);

	remove_proc_entry("mac",
			g_eth_proc_dir);

	remove_proc_entry("hit",
			g_eth_proc_dir);

	remove_proc_entry("mtu",
			g_eth_proc_dir);

	remove_proc_entry("pppoe",
			g_eth_proc_dir);

	remove_proc_entry("genconf",
			g_eth_proc_dir);

	remove_proc_entry("mc",
			g_eth_proc_dir);

#endif
	remove_proc_entry("route",
			g_eth_proc_dir);


	remove_proc_entry("dsl_tx_dma_descr", g_eth_proc_dir);

	remove_proc_entry("wanmib", g_eth_proc_dir);

	remove_proc_entry("mib", g_eth_proc_dir);

	remove_proc_entry("ver", g_eth_proc_dir);

	// remove_proc_entry("eth", NULL); wird im common treiber angelegt;
}

static int proc_read_ver(char *page, char **start __attribute__ ((unused)),
		off_t off __attribute__((unused)), int count, int *eof, void *data __attribute__ ((unused))) {
	int len = 0;

	len += print_driver_ver(page + len, count - len, "PPE datapath driver info",
			VER_FAMILY, VER_DRTYPE, VER_INTERFACE, VER_ACCMODE, VER_MAJOR,
			VER_MID, VER_MINOR);
	len += print_fw_ver(page + len, count - len);

	*eof = 1;

	return len;
}

#if defined(SETUP_OWN_NETDEVS)
static int proc_read_mib(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	int showptm = 0;
	avmnet_netdev_priv_t *priv0, *priv1, *priv2 = NULL;
	char *itf_name[] = {"eth0", "eth1", "cpu", "ext_int1", "ext_int2", "ext_int3", "ext_int4", "ptm0"};
	char *row_name[] = {"ig_fast_brg_pkts", "ig_fast_brg_bytes", "ig_fast_rt_ipv4_udp_pkts", "ig_fast_rt_ipv4_tcp_pkts", "ig_fast_rt_ipv4_mc_pkts", "ig_fast_rt_ipv4_bytes", "ig_fast_rt_ipv6_udp_pkts", "ig_fast_rt_ipv6_tcp_pkts", NULL, "ig_fast_rt_ipv6_bytes", NULL, "ig_cpu_pkts", "ig_cpu_bytes", "ig_drop_pkts", "ig_drop_bytes", "eg_fast_pkts"};
	int i, j, k, h;

	ifx_ppa_eth_get_stats(g_eth_net_dev[0]);
	ifx_ppa_eth_get_stats(g_eth_net_dev[1]);
	priv0 = ifx_eth_fw_netdev_priv(g_eth_net_dev[0]);
	priv1 = ifx_eth_fw_netdev_priv(g_eth_net_dev[1]);

	if ( g_ptm_net_dev[0] )
	{
		ptm_get_stats(g_ptm_net_dev[0]);
		priv2 = ifx_eth_fw_netdev_priv(g_ptm_net_dev[0]);
		showptm = 1;
	}

	len += sprintf(page + len, "Ethernet\n");

	len += sprintf(page + len, "  Firmware  (");
	for ( i = h = 0; i < NUM_ENTITY(itf_name); i++ )
	if ( (g_mib_itf & (1 << i)) )
	len += sprintf(page + len, h++ == 0 ? "%s" : ", %s", itf_name[i]);
	len += sprintf(page + len, ")\n");
	for ( i = k = 0; i < NUM_ENTITY(row_name); i++ )
	if ( row_name[i] != NULL && k < (j = strlen(row_name[i])) )
	k = j;
	k += 6;
	for ( i = 0; i < NUM_ENTITY(row_name); i++ )
	{
		if ( row_name[i] == NULL )
		continue;

		len += (j = sprintf(page + len, "    %s: ", row_name[i]));
		for ( j = k - j; j > 0; j-- )
		page[len++] = ' ';
		for ( j = h = 0; j < 8; j++ )
		if ( (g_mib_itf & (1 << j)) )
		len += sprintf(page + len, h++ == 0 ? "%10u" : ", %10u", ((volatile unsigned int *)ITF_MIB_TBL(j))[i]);
		len += sprintf(page + len, "\n");
	}

	len += sprintf(page + len, "  Driver  (eth0, eth1, ptm0)\n");
	len += sprintf(page + len, "    rx_packets: %10u, %10u, %10u\n", priv0->rx_packets, priv1->rx_packets, showptm ? priv2->rx_packets : 0);
	len += sprintf(page + len, "    rx_bytes:   %10u, %10u, %10u\n", priv0->rx_bytes, priv1->rx_bytes, showptm ? priv2->rx_bytes : 0);
	len += sprintf(page + len, "    rx_dropped: %10u, %10u, %10u\n", priv0->rx_dropped, priv1->rx_dropped, showptm ? priv2->rx_dropped : 0);
	len += sprintf(page + len, "    tx_packets: %10u, %10u, %10u\n", priv0->tx_packets, priv1->tx_packets, showptm ? priv2->tx_packets : 0);
	len += sprintf(page + len, "    tx_bytes:   %10u, %10u, %10u\n", priv0->tx_bytes, priv1->tx_bytes, showptm ? priv2->tx_bytes : 0);
	len += sprintf(page + len, "    tx_errors:  %10u, %10u, %10u\n", priv0->tx_errors, priv1->tx_errors, showptm ? priv2->tx_errors : 0);
	len += sprintf(page + len, "    tx_dropped: %10u, %10u, %10u\n", priv0->tx_dropped, priv1->tx_dropped, showptm ? priv2->tx_dropped : 0);

	*eof = 1;

	ASSERT(len <= 4096, "proc read buffer overflow");
	return len;
}

static int proc_write_mib(struct file *file, const char *buf, unsigned long count, void *data)
{
	char *p1, *p2;
	int len;
	int colon;

	avmnet_netdev_priv_t *priv;
	unsigned int mib_itf = 0;
	char *itf_name[] = {"eth0", "eth1", "cpu", "ext_int1", "ext_int2", "ext_int3", "ext_int4", "ptm_vr9"};
	int i;

	len = sizeof(common_proc_read_buff) < count ? sizeof(common_proc_read_buff) - 1 : count;
	len = len - copy_from_user(common_proc_read_buff, buf, len);
	common_proc_read_buff[len] = 0;

	p1 = common_proc_read_buff;
	p2 = NULL;
	colon = 1;
	while ( ifx_get_token(&p1, &p2, &len, &colon) )
	{
		if ( ifx_stricmp(p1, "on") == 0 || ifx_stricmp(p1, "enable") == 0 )
		mib_itf = 0x40000000;
		else if ( ifx_stricmp(p1, "off") == 0 || ifx_stricmp(p1, "disable") == 0 )
		mib_itf = 0x20000000;
		else if ( ifx_stricmp(p1, "clear") == 0 || ifx_stricmp(p1, "clean") == 0 )
		mib_itf = 0x80000000;
		else if ( (mib_itf & 0xE0000000) )
		{
			if ( ifx_stricmp(p1, "all") == 0 )
			mib_itf |= 0xFF;
			else if ( strlen(p1) == 1 && *p1 >= '0' && *p1 <= '7' )
			mib_itf |= 1 << (*p1 - '0');
			else
			{
				for ( i = 0; i < NUM_ENTITY(itf_name); i++ )
				if ( ifx_stricmp(p1, itf_name[i]) == 0 )
				{
					mib_itf |= 1 << i;
					break;
				}
			}
		}

		p1 = p2;
		colon = 1;
	}

	if ( (mib_itf & 0x40000000) )
	g_mib_itf |= mib_itf & 0xFF;
	else if ( (mib_itf & 0x20000000) )
	g_mib_itf &= ~(mib_itf & 0xFF);
	else if ( (mib_itf & 0x80000000) )
	{
		if ( mib_itf == 0x80000000 )
		mib_itf = 0x800000FF;
		if ( (mib_itf & 0x01) )
		{
			ifx_ppa_eth_get_stats(g_eth_net_dev[0]);
			priv = ifx_eth_fw_netdev_priv(g_eth_net_dev[0]);
			memset(priv, 0, (u32)&priv->tx_dropped - (u32)priv + sizeof(u32));
		}
		if ( (mib_itf & 0x02) )
		{
			ifx_ppa_eth_get_stats(g_eth_net_dev[1]);
			priv = ifx_eth_fw_netdev_priv(g_eth_net_dev[1]);
			memset(priv, 0, (u32)&priv->tx_dropped - (u32)priv + sizeof(u32));
		}
		if ( (mib_itf & 0x80) && (g_ptm_net_dev[0] != NULL))
		{
			ptm_get_stats(g_ptm_net_dev[0]);
			priv = ifx_eth_fw_netdev_priv(g_ptm_net_dev[0]);
			memset(priv, 0, (u32)&priv->tx_dropped - (u32)priv + sizeof(u32));
		}
		for ( i = 0; i < 8; i++ )
		if ( (mib_itf & (1 << i)) )
		{
			void *ptmp = (void *)ITF_MIB_TBL(i);
			memset(ptmp, 0, sizeof(struct itf_mib));
		}
		if ( (mib_itf & g_wan_itf) )
		{
		    for ( i = 0; i < 8; i++ )
		    {
		        void *ptmp = (void *)WAN_TX_MIB_TABLE(i);
		        memset(ptmp, 0, sizeof(struct wan_tx_mib_table));
		    }
		}
	}
	else
	    printk("echo <on|off|clear> <all|eth0|eth1|ptm_vr9|cpu|ext_int?> > /proc/eth/mib\n");

	return count;
}
#endif

static int proc_read_wanmib(char *page, char **start __attribute__ ((unused)),
        off_t off __attribute__((unused)), int count __attribute__ ((unused)), int *eof,
        void *data __attribute__ ((unused))) {


    int len = 0;
    volatile unsigned int *wrx_total_pdu[4]         = {DREG_AR_AIIDLE_CNT0,  DREG_AR_HEC_CNT0,    DREG_AR_AIIDLE_CNT1, DREG_AR_HEC_CNT1};
    volatile unsigned int *wrx_crc_err_pdu[4]       = {GIF0_RX_CRC_ERR_CNT,  GIF1_RX_CRC_ERR_CNT, GIF2_RX_CRC_ERR_CNT, GIF3_RX_CRC_ERR_CNT};
    volatile unsigned int *wrx_cv_cw_cnt[4]         = {GIF0_RX_CV_CNT,       GIF1_RX_CV_CNT,      GIF2_RX_CV_CNT,      GIF3_RX_CV_CNT};
    volatile unsigned int *wrx_bc_overdrop_cnt[2]   = {DREG_B0_OVERDROP_CNT, DREG_B1_OVERDROP_CNT};
    int i;

    len += sprintf(page + len, "RX (Bearer Channels):\n");
    len += sprintf(page + len, "   wrx_bc_overdrop:");
    for ( i = 0; i < 2; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ",             ");
        len += sprintf(page + len, "%10u", IFX_REG_R32(wrx_bc_overdrop_cnt[i]) - last_wrx_bc_overdrop_cnt[PPE_ID][i]);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "   wrx_bc_user_cw: ");
    for ( i = 0; i < 2; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ",             ");
        len += sprintf(page + len, "%10u", IFX_REG_R32(RECEIVE_NON_IDLE_CELL_CNT(i)));
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "   wrx_bc_idle_cw: ");
    for ( i = 0; i < 2; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ",             ");
        len += sprintf(page + len, "%10u", IFX_REG_R32(RECEIVE_IDLE_CELL_CNT(i)));
    }
    len += sprintf(page + len, "\n");

    len += sprintf(page + len, "RX (Gamma Interfaces):\n");
    len += sprintf(page + len, "  wrx_total_pdu:   ");
    for ( i = 0; i < 4; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", IFX_REG_R32(wrx_total_pdu[i]) - last_wrx_total_pdu[PPE_ID][i]);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wrx_dropdes_pdu: ");
    for ( i = 0; i < 4; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_RX_MIB_TABLE(i)->wrx_dropdes_pdu);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wrx_crc_err_pdu: ");
    for ( i = 0; i < 4; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", IFX_REG_R32(wrx_cv_cw_cnt[i]) - last_wrx_cv_cw_cnt[PPE_ID][i]);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wrx_violated_cw: ");
    for ( i = 0; i < 4; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", IFX_REG_R32(wrx_crc_err_pdu[i]) - last_wrx_crc_error_total_pdu[PPE_ID][i]);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wrx_total_bytes: ");
    for ( i = 0; i < 4; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_RX_MIB_TABLE(i)->wrx_total_bytes);
    }
    len += sprintf(page + len, "\n");

    len += sprintf(page + len, "TX (Bearer Channels):\n");
    len += sprintf(page + len, "  total_tx_cw:     %10u,             %10u\n", IFX_REG_R32(TRANSMIT_CELL_CNT(0)), IFX_REG_R32(TRANSMIT_CELL_CNT(1)));

    len += sprintf(page + len, "TX (QoS Queues):\n");
    len += sprintf(page + len, "  wrx_total_pdu:           ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wrx_total_pdu);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wrx_total_bytes:         ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wrx_total_bytes);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wtx_total_pdu:           ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wtx_total_pdu);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wtx_total_bytes:         ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wtx_total_bytes);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wtx_cpu_drop_small_pdu:  ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wtx_cpu_drop_small_pdu);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wtx_cpu_drop_pdu:        ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wtx_cpu_drop_pdu);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wtx_fast_drop_small_pdu: ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wtx_fast_drop_small_pdu);
    }
    len += sprintf(page + len, "\n");
    len += sprintf(page + len, "  wtx_fast_drop_pdu:       ");
    for ( i = 0; i < 8; i++ )
    {
        if ( i != 0 )
            len += sprintf(page + len, ", ");
        len += sprintf(page + len, "%10u", WAN_TX_MIB_TABLE(i)->wtx_fast_drop_pdu);
    }
    len += sprintf(page + len, "\n");


    len += sprintf(page + len, "dma_alignment_ptm_good_count: %d\n", dma_alignment_ptm_good_count );
    len += sprintf(page + len, "dma_alignment_ptm_bad_count: %d\n", dma_alignment_ptm_bad_count   );
    len += sprintf(page + len, "dma_alignment_eth_good_count: %d\n", dma_alignment_eth_good_count );
    len += sprintf(page + len, "dma_alignment_eth_bad_count: %d\n", dma_alignment_eth_bad_count   );
    len += sprintf(page + len, "padded_frame_count: %d\n", padded_frame_count);

#if defined(DEBUG_SPACE_BETWEEN_HEADER_AND_DATA)
    len += sprintf(page + len, "min_fheader_byteoff: %d\n", min_fheader_byteoff);
#endif

    *eof = 1;

    ASSERT(len <= 4096, "proc read buffer overflow");
    return len;
}

static int proc_write_wanmib(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count __attribute__ ((unused)),
		void *data __attribute__ ((unused))) {
	char *p;
	int len, rlen;

    /*--- volatile unsigned int *wrx_total_pdu[4]         = {DREG_AR_AIIDLE_CNT0,  DREG_AR_HEC_CNT0,    DREG_AR_AIIDLE_CNT1, DREG_AR_HEC_CNT1}; ---*/
    /*--- volatile unsigned int *wrx_crc_err_pdu[4]       = {GIF0_RX_CRC_ERR_CNT,  GIF1_RX_CRC_ERR_CNT, GIF2_RX_CRC_ERR_CNT, GIF3_RX_CRC_ERR_CNT}; ---*/
    /*--- volatile unsigned int *wrx_cv_cw_cnt[4]         = {GIF0_RX_CV_CNT,       GIF1_RX_CV_CNT,      GIF2_RX_CV_CNT,      GIF3_RX_CV_CNT}; ---*/
    /*--- volatile unsigned int *wrx_bc_overdrop_cnt[2]   = {DREG_B0_OVERDROP_CNT, DREG_B1_OVERDROP_CNT}; ---*/
	/*--- int i; ---*/

	len = count < sizeof(common_proc_read_buff) ?
			count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p)
		return count;

	if (ifx_stricmp(p, "clear") == 0 || ifx_stricmp(p, "clean") == 0) {
		// die Schleife wirft leider die Warnung: 'Null-Argument, wo Nicht-Null erwartet (Argument 1)'
#if 0
	    for ( i = 0; i < 4; i++ ) {
	        last_wrx_total_pdu[PPE_ID][i]           = IFX_REG_R32(wrx_total_pdu[i]);
	        last_wrx_crc_error_total_pdu[PPE_ID][i] = IFX_REG_R32(wrx_crc_err_pdu[i]);
	        last_wrx_cv_cw_cnt[PPE_ID][i]           = IFX_REG_R32(wrx_cv_cw_cnt[i]);
	        if ( i < 2 )
	        {
	            last_wrx_bc_overdrop_cnt[PPE_ID][i] = IFX_REG_R32(wrx_bc_overdrop_cnt[i]);
	            *RECEIVE_NON_IDLE_CELL_CNT(i)       = 0;
	            *RECEIVE_IDLE_CELL_CNT(i)           = 0;
	            *TRANSMIT_CELL_CNT(i)               = 0;
	        }
	        ppa_memset((void* )WAN_RX_MIB_TABLE(i), 0, sizeof(*WAN_RX_MIB_TABLE(i)));
	    }
	    for ( i = 0; i < 8; i++ )
	        ppa_memset((void*)WAN_TX_MIB_TABLE(i), 0, sizeof(*WAN_TX_MIB_TABLE(i)));

#else
		memset((void* )WAN_RX_MIB_TABLE(0), 0, sizeof(*WAN_RX_MIB_TABLE(0)));
		memset((void* )WAN_RX_MIB_TABLE(1), 0, sizeof(*WAN_RX_MIB_TABLE(1)));
		memset((void* )WAN_RX_MIB_TABLE(2), 0, sizeof(*WAN_RX_MIB_TABLE(2)));
		memset((void* )WAN_RX_MIB_TABLE(3), 0, sizeof(*WAN_RX_MIB_TABLE(3)));

		memset((void*)WAN_TX_MIB_TABLE(0), 0, sizeof(*WAN_TX_MIB_TABLE(0)));
		memset((void*)WAN_TX_MIB_TABLE(1), 0, sizeof(*WAN_TX_MIB_TABLE(1)));
		memset((void*)WAN_TX_MIB_TABLE(2), 0, sizeof(*WAN_TX_MIB_TABLE(2)));
		memset((void*)WAN_TX_MIB_TABLE(3), 0, sizeof(*WAN_TX_MIB_TABLE(3)));
		memset((void*)WAN_TX_MIB_TABLE(4), 0, sizeof(*WAN_TX_MIB_TABLE(4)));
		memset((void*)WAN_TX_MIB_TABLE(5), 0, sizeof(*WAN_TX_MIB_TABLE(5)));
		memset((void*)WAN_TX_MIB_TABLE(6), 0, sizeof(*WAN_TX_MIB_TABLE(6)));
		memset((void*)WAN_TX_MIB_TABLE(7), 0, sizeof(*WAN_TX_MIB_TABLE(7)));
#endif
	}

	return count;
}

#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC
static int proc_read_route(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
	struct rout_forward_compare_tbl *pcompare;
	struct rout_forward_action_tbl *paction;
	int len = 0;
	char *pstr;
	char str[2048];
	int llen;
	int i;

	pstr = *start = page;

	__sync();

	llen = sprintf(str, "Wan Routing Table\n");
	proc_buf_copy(&pstr, count, off, &len, str, llen);

	pcompare = (struct rout_forward_compare_tbl *)ROUT_WAN_HASH_CMP_TBL(0);
	paction = (struct rout_forward_action_tbl *)ROUT_WAN_HASH_ACT_TBL(0);
	for ( i = 0; i < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK; i++ )
	{
		if ( *(u32*)pcompare && *((u32*)pcompare + 1) )
		{
			llen = print_route(str, i, 0, pcompare, paction);
			if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
			goto PROC_READ_ROUTE_OVERRUN_END;
		}

		pcompare++;
		paction++;
	}

	pcompare = (struct rout_forward_compare_tbl *)ROUT_WAN_COLL_CMP_TBL(0);
	paction = (struct rout_forward_action_tbl *)ROUT_WAN_COLL_ACT_TBL(0);
	for ( i = 0; i < WAN_ROUT_TBL_CFG->rout_num; i++ )
	{
		if ( *(u32*)pcompare && *((u32*)pcompare + 1) )
		{
			llen = print_route(str, i + MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK, 0, pcompare, paction);
			if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
			goto PROC_READ_ROUTE_OVERRUN_END;
		}

		pcompare++;
		paction++;
	}

	llen = sprintf(str, "Lan Routing Table\n");
	if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
	goto PROC_READ_ROUTE_OVERRUN_END;

	pcompare = (struct rout_forward_compare_tbl *)ROUT_LAN_HASH_CMP_TBL(0);
	paction = (struct rout_forward_action_tbl *)ROUT_LAN_HASH_ACT_TBL(0);
	for ( i = 0; i < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK; i++ )
	{
		if ( *(u32*)pcompare && *((u32*)pcompare + 1) )
		{
			llen = print_route(str, i, 1, pcompare, paction);
			if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
			goto PROC_READ_ROUTE_OVERRUN_END;
		}

		pcompare++;
		paction++;
	}

	pcompare = (struct rout_forward_compare_tbl *)ROUT_LAN_COLL_CMP_TBL(0);
	paction = (struct rout_forward_action_tbl *)ROUT_LAN_COLL_ACT_TBL(0);
	for ( i = 0; i < LAN_ROUT_TBL_CFG->rout_num; i++ )
	{
		if ( *(u32*)pcompare && *((u32*)pcompare + 1) )
		{
			llen = print_route(str, i + MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK, 1, pcompare, paction);
			if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
			goto PROC_READ_ROUTE_OVERRUN_END;
		}

		pcompare++;
		paction++;
	}

	*eof = 1;

	PROC_READ_ROUTE_OVERRUN_END:
	return len - off;
}

static int proc_write_route(struct file *file __attribute__((unused)), const char *buf, unsigned long count, void *data __attribute__((unused))) {
	static const char *command[] = {
		"add", //  0
		"del",//  1
		"LAN",//  2
		"WAN",//  3
		"new",//  4
		"src",//  5
		"dest",//  6
		"MAC",//  7
		"route",//  8
		"type",//  9
		"DSCP",//  10
		"MTU",//  11
		"index",//  12
		"VLAN",//  13
		"insert",//  14
		"remove",//  15
		"list",//  16
		"PPPoE",//  17
		"mode",//  18
		"ch",//  19
		"id",//  20
		"delete",//  21
		"disable",//  22
		"enable",//  23
		"transparent",//  24
		"termination",//  25
		"NULL",//  26
		"IPv4",//  27
		"NAT",//  28
		"NAPT",//  29
		"entry",//  30
		"tcp",//  31
		"help",//  32
		"vci",//  33
		"yes",//  34
		"no",//  35
		"qid",//  36
		"outer",//  37
		"IP",//  38
		"IPv6",//  39
	};

	static const char *dest_list[] = {"ETH0", "ETH1", "CPU0", "EXT_INT1", "EXT_INT2", "EXT_INT3", "EXT_INT4", "PTM"};
	static const int dest_list_strlen[] = {4, 4, 4, 8, 8, 8, 8, 3};

	int state; //  1: new,
	int prev_cmd;
	int operation;//  1: add, 2: delete
	int type;//  1: LAN, 2: WAN, 0: auto detect
	int entry;
	struct rout_forward_compare_tbl compare_tbl;
	struct rout_forward_action_tbl action_tbl;
	uint32_t ipv6_src_ip[4] = {0}, ipv6_dst_ip[4] = {0};
	int is_ipv6 = 0;
	unsigned int val[20];
	/*--- char common_proc_read_buff[1024]; ---*/
	int len;
	char *p1, *p2;
	int colon;
	unsigned int i, j;
	u32 mask;
	u32 bit;
	u32 *pu1, *pu2, *pu3;

	len = sizeof(common_proc_read_buff) < count ? sizeof(common_proc_read_buff) - 1 : count;
	len = len - copy_from_user(common_proc_read_buff, buf, len);
	common_proc_read_buff[len] = 0;

	state = 0;
	prev_cmd = 0;
	operation = 0;
	type = 0;
	entry = -1;

	memset(&compare_tbl, 0, sizeof(compare_tbl));
	memset(&action_tbl, 0, sizeof(action_tbl));

	p1 = common_proc_read_buff;
	colon = 1;
	while ( ifx_get_token(&p1, &p2, &len, &colon) )
	{
		for ( i = 0; i < sizeof(command) / sizeof(*command); i++ ) 
        {
            if ( ifx_stricmp(p1, command[i]) == 0 )
            {
                switch ( i )
                {
                    case 0:
                    if ( !state && !operation )
                    {
                        operation = 1;
    //                      printk("add\n");
                    }
                    state = prev_cmd = 0;
                    break;
                    case 1:
                    case 21:
                    if ( !state && !operation )
                    {
                        operation = 2;
    //                      printk("delete\n");
                    }
                    state = prev_cmd = 0;
                    break;
                    case 2:
                    if ( !state && !type )
                    {
                        type = 1;
    //                      printk("lan\n");
                    }
                    state = prev_cmd = 0;
                    break;
                    case 3:
                    if ( !state && !type )
                    {
                        type = 2;
    //                      printk("wan\n");
                    }
                    state = prev_cmd = 0;
                    break;
                    case 4:
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 4;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 5:
                    if ( state == 1 )
                    {
                        if ( prev_cmd == 4 )
                        {
                            //  new src
                            if ( !type )
                            type = 1;

                            //  check for "new src mac index"
                            ifx_ignore_space(&p2, &len);
                            if ( ifx_strincmp(p2, "mac ", 4) == 0 )
                            {
                                state = 2;
                                prev_cmd = 5;
                                break;
                            }
                            else
                            {
                                get_ip_port(&p2, &len, val);
    //                              printk("new src: %d.%d.%d.%d:%d\n", val[0], val[1], val[2], val[3], val[4]);
                                action_tbl.new_ip = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                                action_tbl.new_port = val[4];
                            }
                        }
                        else
                        state = 0;
                    }
                    if ( state == 0 )
                    {
                        //  src
                        get_ip_port(&p2, &len, val);
    //                      printk("src: %d.%d.%d.%d:%d\n", val[0], val[1], val[2], val[3], val[4]);
                        if ( val[5] == 4 )
                        {
                            if ( is_ipv6 )
                            ipv6_src_ip[0] = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                            else
                            compare_tbl.src_ip = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                        }
                        else
                        {
                            is_ipv6 = 1;
                            ipv6_src_ip[0] = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                            ipv6_src_ip[1] = (val[6] << 24) | (val[7] << 16) | (val[8] << 8) | val[9];
                            ipv6_src_ip[2] = (val[10] << 24) | (val[11] << 16) | (val[12] << 8) | val[13];
                            ipv6_src_ip[3] = (val[14] << 24) | (val[15] << 16) | (val[16] << 8) | val[17];
                        }
                        compare_tbl.src_port = val[4];
                    }
                    state = prev_cmd = 0;
                    break;
                    case 6:
                    if ( state == 1 )
                    {
                        if ( prev_cmd == 4 )
                        {
                            //  new dest
                            if ( !type )
                            type = 2;

                            get_ip_port(&p2, &len, val);
    //                          printk("new dest: %d.%d.%d.%d:%d\n", val[0], val[1], val[2], val[3], val[4]);
                            action_tbl.new_ip = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                            action_tbl.new_port = val[4];
                        }
                        else
                        state = 0;
                    }
                    if ( state == 0 )
                    {
                        if ( !colon )
                        {
                            int llen;

                            llen = len;
                            p1 = p2;
                            while ( llen && *p1 <= ' ' )
                            {
                                llen--;
                                p1++;
                            }
                            if ( llen && (*p1 == ':' || (*p1 >= '0' && *p1 <= '9')) )
                            colon = 1;
                        }
                        if ( colon )
                        {
                            //  dest
                            get_ip_port(&p2, &len, val);
    //                          printk("dest: %d.%d.%d.%d:%d\n", val[0], val[1], val[2], val[3], val[4]);
                            if ( val[5] == 4 )
                            {
                                if ( is_ipv6 )
                                ipv6_dst_ip[0] = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                                else
                                compare_tbl.dest_ip = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                            }
                            else
                            {
                                is_ipv6 = 1;
                                ipv6_dst_ip[0] = (val[0] << 24) | (val[1] << 16) | (val[2] << 8) | val[3];
                                ipv6_dst_ip[1] = (val[6] << 24) | (val[7] << 16) | (val[8] << 8) | val[9];
                                ipv6_dst_ip[2] = (val[10] << 24) | (val[11] << 16) | (val[12] << 8) | val[13];
                                ipv6_dst_ip[3] = (val[14] << 24) | (val[15] << 16) | (val[16] << 8) | val[17];
                            }
                            compare_tbl.dest_port = val[4];
                        }
                        else
                        {
                            state = 1;
                            prev_cmd = 6;
                            break;
                        }
                    }
                    state = prev_cmd = 0;
                    break;
                    case 7:
                    if ( state == 1 && prev_cmd == 4 )
                    {
                        //  new MAC
                        get_mac(&p2, &len, val);
    //                      printk("new MAC: %02X.%02X.%02X.%02X:%02X:%02X\n", val[0], val[1], val[2], val[3], val[4], val[5]);
                        action_tbl.new_dest_mac54 = (val[0] << 8) | val[1];
                        action_tbl.new_dest_mac30 = (val[2] << 24) | (val[3] << 16) | (val[4] << 8) | val[5];
                    }
                    else if ( state == 2 && prev_cmd == 5 )
                    {
                        state = 3;
                        prev_cmd = 7;
                        break;
                    }
                    state = prev_cmd = 0;
                    break;
                    case 8:
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 8;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 9:
                    if ( state == 1 && prev_cmd == 8 )
                    {
                        state = 2;
                        prev_cmd = 9;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 10:
                    if ( state == 1 && prev_cmd == 4 )
                    {
                        ifx_ignore_space(&p2, &len);
                        if ( len && *p2 >= '0' && *p2 <= '9' )
                        {
                            //  new DSCP
                            val[0] = ifx_get_number(&p2, &len, 0);
    //                          printk("new DSCP: %d\n", val[0]);
                            if ( !action_tbl.new_dscp_en )
                            {
                                action_tbl.new_dscp_en = 1;
                                action_tbl.new_dscp = val[0];
                            }
                        }
                        else if ( (len == 8 || (len > 8 && (p2[8] <= ' ' || p2[8] == ','))) && ifx_strincmp(p2, "original", 8) == 0 )
                        {
                            p2 += 8;
                            len -= 8;
                            //  new DSCP original
    //                          printk("new DSCP: original\n");
                            //  the reset value is 0, so don't need to do anything
                        }
                    }
                    state = prev_cmd = 0;
                    break;
                    case 11:
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 11;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 12:
                    if ( state == 1 )
                    {
                        if ( prev_cmd == 11 )
                        {
                            //  MTU index
                            ifx_ignore_space(&p2, &len);
                            val[0] = ifx_get_number(&p2, &len, 0);
    //                          printk("MTU index: %d\n", val[0]);
                            action_tbl.mtu_ix = val[0];
                        }
                        else if ( prev_cmd == 13 )
                        {
                            //  VLAN insert enable
                            //  VLAN index
                            ifx_ignore_space(&p2, &len);
                            val[0] = ifx_get_number(&p2, &len, 0);
    //                          printk("VLAN insert: enable, index %d\n", val[0]);
                            if ( !action_tbl.in_vlan_ins )
                            {
                                action_tbl.in_vlan_ins = 1;
                                action_tbl.new_in_vci = val[0];
                            }
                        }
                        else if ( prev_cmd == 17 )
                        {
                            //  PPPoE index
                            ifx_ignore_space(&p2, &len);
                            val[0] = ifx_get_number(&p2, &len, 0);
    //                          printk("PPPoE index: %d\n", val[0]);
                            action_tbl.pppoe_ix = val[0];
                        }
                    }
                    else if ( state == 3 && prev_cmd == 7 )
                    {
                        //  new src mac index
                        ifx_ignore_space(&p2, &len);
                        val[0] = ifx_get_number(&p2, &len, 0);
                        action_tbl.new_src_mac_ix = val[0];
                    }
                    else if ( state == 2 && prev_cmd == 13 )
                    {
                        //  outer VLAN enable
                        //  outer VLAN index
                        ifx_ignore_space(&p2, &len);
                        val[0] = ifx_get_number(&p2, &len, 0);
    //                      printk("outer VLAN insert: enable, index %d\n", val[0]);
                        if ( !action_tbl.out_vlan_ins )
                        {
                            action_tbl.out_vlan_ins = 1;
                            action_tbl.out_vlan_ix = val[0];
                        }
                    }
                    state = prev_cmd = 0;
                    break;
                    case 13:
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 13;
                    }
                    else if ( state == 1 && prev_cmd == 37 )
                    {
                        state = 2;
                        prev_cmd = 13;
                        printk("outer vlan\n");
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 14:
                    if ( (state == 1 || state == 2) && prev_cmd == 13 )
                    {
                        state++;
                        prev_cmd = 14;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 15:
                    if ( (state == 1 || state == 2) && prev_cmd == 13 )
                    {
                        state++;
                        prev_cmd = 15;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 16:
                    if ( state == 1 && prev_cmd == 6 )
                    {
                        mask = 0;
                        do
                        {
                            ifx_ignore_space(&p2, &len);
                            if ( !len )
                            break;
                            for ( j = 0, bit = 1; j < sizeof(dest_list) / sizeof(*dest_list); j++, bit <<= 1 )
                            if ( (len == dest_list_strlen[j] || (len > dest_list_strlen[j] && (p2[dest_list_strlen[j]] <= ' ' || p2[dest_list_strlen[j]] == ','))) && ifx_strincmp(p2, dest_list[j], dest_list_strlen[j]) == 0 )
                            {
                                p2 += dest_list_strlen[j];
                                len -= dest_list_strlen[j];
                                mask |= bit;
                                break;
                            }
                        }while ( j < sizeof(dest_list) / sizeof(*dest_list) );
    //                      if ( mask )
    //                      {
    //                          //  dest list
    //                          printk("dest list:");
    //                          for ( j = 0, bit = 1; j < sizeof(dest_list) / sizeof(*dest_list); j++, bit <<= 1 )
    //                              if ( (mask & bit) )
    //                              {
    //                                  printk(" %s", dest_list[j]);
    //                              }
    //                          printk("\n");
    //                      }
    //                      else
    //                          printk("dest list: none\n");
                        action_tbl.dest_list = mask;
                    }
                    state = prev_cmd = 0;
                    break;
                    case 17:
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 17;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 18:
                    if ( state == 1 && prev_cmd == 17 )
                    {
                        state = 2;
                        prev_cmd = 18;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 19:
                    if ( state == 1 && prev_cmd == 6 )
                    {
                        state = 2;
                        prev_cmd = 19;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 20:
                    if ( state == 2 && prev_cmd == 19 )
                    {
                        //  dest ch id
                        ifx_ignore_space(&p2, &len);
                        val[0] = ifx_get_number(&p2, &len, 0);
    //                      printk("dest ch id: %d\n", val[0]);
                        //action_tbl.dest_chid = val[0];
                    }
                    state = prev_cmd = 0;
                    break;
                    case 22:
                    case 23:
                    if ( state == 2 )
                    {
                        if ( prev_cmd == 14 )
                        {
                            //  VLAN insert
    //                          printk("VLAN insert: %s (%d)", command[i], i - 22);
                            if ( (i - 22) )
                            {
                                ifx_ignore_space(&p2, &len);
                                if ( len > 5 && (p2[5] <= ' ' || p2[5] == ':') && ifx_strincmp(p2, "index", 5) == 0 )
                                {
                                    p2 += 6;
                                    len -= 6;
                                    //  VLAN index
                                    ifx_ignore_space(&p2, &len);
                                    val[0] = ifx_get_number(&p2, &len, 0);
    //                                  printk(", index %d", val[0]);
                                    if ( !action_tbl.in_vlan_ins )
                                    {
                                        action_tbl.in_vlan_ins = 1;
                                        //action_tbl.vlan_ix = val[0];
                                        action_tbl.new_in_vci = val[0];
                                    }
                                }
                                else if ( len > 3 && (p2[3] <= ' ' || p2[3] == ':') && ifx_strincmp(p2, "vci", 3) == 0 )
                                {
                                    p2 += 4;
                                    len -= 4;
                                    //  New VCI
                                    ifx_ignore_space(&p2, &len);
                                    val[0] = ifx_get_number(&p2, &len, 1);
    //                                  printk(", vci 0x%04X", val[0]);
                                    if ( !action_tbl.in_vlan_ins )
                                    {
                                        action_tbl.in_vlan_ins = 1;
                                        action_tbl.new_in_vci = val[0];
                                    }
                                }
                            }
                            else
                            {
                                action_tbl.in_vlan_ins = 0;
                                action_tbl.new_in_vci = 0;
                            }
    //                          printk("\n");
                        }
                        else if ( prev_cmd == 15 )
                        {
                            //  VLAN remove
    //                          printk("VLAN remove: %s (%d)\n", command[i], i - 22);
                            action_tbl.in_vlan_rm = i - 22;
                        }
                    }
                    else if ( state == 3 )
                    {
                        if ( prev_cmd == 14 )
                        {
                            //  outer vlan insert
    //                          printk("outer VLAN insert: %s (%d)", command[i], i - 22);
                            if ( (i - 22) )
                            {
                                ifx_ignore_space(&p2, &len);
                                if ( len > 5 && (p2[5] <= ' ' || p2[5] == ':') && ifx_strincmp(p2, "index", 5) == 0 )
                                {
                                    p2 += 6;
                                    len -= 6;
                                    //  outer VLAN index
                                    ifx_ignore_space(&p2, &len);
                                    val[0] = ifx_get_number(&p2, &len, 0);
    //                                  printk(", index %d", val[0]);
                                    if ( !action_tbl.out_vlan_ins )
                                    {
                                        action_tbl.out_vlan_ins = 1;
                                        action_tbl.out_vlan_ix = val[0];
                                    }
                                }
                            }
                            else
                            {
                                action_tbl.out_vlan_ins = 0;
                                action_tbl.out_vlan_ix = 0;
                            }
    //                          printk("\n");
                        }
                        else if ( prev_cmd == 15 )
                        {
                            //  outer VLAN remove
    //                          printk("outer VLAN remove: %s (%d)\n", command[i], i - 22);
                            action_tbl.out_vlan_rm = i - 22;
                        }
                    }
                    state = prev_cmd = 0;
                    break;
                    case 24:
                    case 25:
                    if ( state == 2 && prev_cmd == 18 )
                    {
                        //  PPPoE mode
    //                      printk("PPPoE mode: %s (%d)\n", command[i], i - 24);
                        action_tbl.pppoe_mode = i - 24;
                    }
                    state = prev_cmd = 0;
                    break;
                    case 38:
                    case 39:
                    i = 27;
                    case 26:
                    case 27:
                    case 28:
                    case 29:
                    if ( state == 2 && prev_cmd == 9 )
                    {
                        //  route type
    //                      printk("route type: %s (%d)\n", command[i], i - 26);
                        action_tbl.rout_type = i - 26;
                    }
                    state = prev_cmd = 0;
                    break;
                    case 30:
                    if ( !state )
                    {
                        if ( entry < 0 )
                        {
                            ifx_ignore_space(&p2, &len);
                            if ( len && *p2 >= '0' && *p2 <= '9' )
                            {
                                entry = ifx_get_number(&p2, &len, 0);
                                //  entry
    //                              printk("entry: %d\n", entry);
                            }
                        }
                    }
                    state = prev_cmd = 0;
                    break;
                    case 31: // if this flag is not presented, it's UDP by default
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 31;
                        action_tbl.protocol = 1;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    case 32:
                    printk("add\n");
                    printk("  LAN/WAN entry ???\n");
                    printk("    compare\n");
                    printk("      src:  ???.???.???.???:????\n");
                    printk("      dest: ???.???.???.???:????\n");
                    printk("    action\n");
                    printk("      new src/dest:      ???.???.???.???:????\n");
                    printk("      new MAC:           ??:??:??:??:??:?? (HEX)\n");
                    printk("      route type:        NULL/IP/NAT/NAPT\n");
                    printk("      new DSCP:          original/??\n");
                    printk("      MTU index:         ??\n");
                    printk("      VLAN insert:       disable/enable, VCI ???? (HEX)\n");
                    printk("      VLAN remove:       disable/enable\n");
                    printk("      dest list:         ETH0/ETH1/CPU0/EXT_INT?\n");
                    printk("      PPPoE mode:        transparent/termination\n");
                    printk("      PPPoE index:       ??\n");
                    printk("      new src MAC index: ??\n");
                    printk("      outer VLAN insert: disable/enable, index ??\n");
                    printk("      outer VLAN remove: disable/enable\n");
                    printk("      tcp:               yes/no\n");
                    printk("      dest qid:          ??\n");
                    printk("\n");
                    printk("delete\n");
                    printk("  LAN/WAN entry ???\n");
                    printk("    compare\n");
                    printk("      src:  ???.???.???.???:????\n");
                    printk("      dest: ???.???.???.???:????\n");
                    printk("\n");
                    state = prev_cmd = 0;
                    break;
                    case 33:
                    if ( state == 1 && prev_cmd == 13 )
                    {
                        //  vlan vci
                        ifx_ignore_space(&p2, &len);
                        val[0] = ifx_get_number(&p2, &len, 1);
                        if ( !action_tbl.in_vlan_ins )
                        {
                            action_tbl.in_vlan_ins = 1;
                            action_tbl.new_in_vci = val[0];
                        }
                    }
                    state = prev_cmd = 0;
                    break;
                    case 34:
                    if ( state == 1 && prev_cmd == 31 )
                    //  tcp yes
                    action_tbl.protocol = 1;
                    state = prev_cmd = 0;
                    break;
                    case 35:
                    if ( state == 1 && prev_cmd == 31 )
                    //  tcp no
                    action_tbl.protocol = 0;
                    state = prev_cmd = 0;
                    break;
                    case 36:
                    if ( state == 1 && prev_cmd == 6 )
                    {
                        //  dest qid
                        ifx_ignore_space(&p2, &len);
                        val[0] = ifx_get_number(&p2, &len, 0);
                        action_tbl.dest_qid = val[0];
                    }
                    state = prev_cmd = 0;
                    break;
                    case 37:
                    if ( !state )
                    {
                        state = 1;
                        prev_cmd = 37;
                    }
                    else
                    state = prev_cmd = 0;
                    break;
                    default:
                    state = prev_cmd = 0;
                }

                break;
            }
        }

		if ( i == sizeof(command) / sizeof(*command) )
		state = prev_cmd = 0;

		p1 = p2;
		colon = 1;
	}

	if ( operation == 2 )
	{
		u32 is_lan = 0;

		//  delete
		pu1 = (u32*)&compare_tbl;
		pu2 = NULL;
		pu3 = NULL;
		if ( entry < 0 )
		{
			//  search the entry number
			if ( *pu1 && pu1[1] )
			{
				if ( (!type || type == 1) )
				{
					//  LAN
					for ( entry = 0; entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK; entry++ )
					{
						pu2 = (u32*)ROUT_LAN_HASH_CMP_TBL(entry);
						if ( *pu2 == *pu1 && pu2[1] == pu1[1] && pu2[2] == pu1[2] )
						{
							pu3 = (u32*)ROUT_LAN_HASH_ACT_TBL(entry);
							break;
						}
					}
					if ( entry == MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK )
					{
						for ( entry = 0; entry < LAN_ROUT_TBL_CFG->rout_num; entry++ )
						{
							pu2 = (u32*)ROUT_LAN_COLL_CMP_TBL(entry);
							if ( *pu2 == *pu1 && pu2[1] == pu1[1] && pu2[2] == pu1[2] )
							{
								pu3 = (u32*)ROUT_LAN_COLL_ACT_TBL(entry);
								break;
							}
						}
						if ( entry == LAN_ROUT_TBL_CFG->rout_num )
						pu2 = NULL;
						else
						entry += MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK;
					}
					if ( pu3 != NULL )
					is_lan = 1;
				}
				if ( (!type && !pu2) || type == 2 )
				{
					//  WAN
					for ( entry = 0; entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK; entry++ )
					{
						pu2 = (u32*)ROUT_WAN_HASH_CMP_TBL(entry);
						if ( *pu2 == *pu1 && pu2[1] == pu1[1] && pu2[2] == pu1[2] )
						{
							pu3 = (u32*)ROUT_WAN_HASH_ACT_TBL(entry);
							break;
						}
					}
					if ( entry == MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK )
					{
						for ( entry = 0; entry < WAN_ROUT_TBL_CFG->rout_num; entry++ )
						{
							pu2 = (u32*)ROUT_WAN_COLL_CMP_TBL(entry);
							if ( *pu2 == *pu1 && pu2[1] == pu1[1] && pu2[2] == pu1[2] )
							{
								pu3 = (u32*)ROUT_WAN_COLL_ACT_TBL(entry);
								break;
							}
						}
						if ( entry == WAN_ROUT_TBL_CFG->rout_num )
						pu2 = NULL;
						else
						entry += MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK;
					}
					if ( pu3 != NULL )
					is_lan = 0;
				}
			}
		}
		else
		{
			if ( *pu1 && pu1[1] )
			{
				pu3 = NULL;
				//  check compare
				if ( !type || type == 1 )
				{
					//  LAN
					if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK )
					{
						pu2 = (u32*)ROUT_LAN_HASH_CMP_TBL(entry);
						if ( *pu2 != *pu1 || pu2[1] != pu1[1] || pu2[2] != pu1[2] )
						pu2 = NULL;
						else
						pu3 = (u32*)ROUT_LAN_HASH_ACT_TBL(entry);
					}
					else if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK + LAN_ROUT_TBL_CFG->rout_num )
					{
						pu2 = (u32*)ROUT_LAN_COLL_CMP_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
						if ( *pu2 != *pu1 || pu2[1] != pu1[1] || pu2[2] != pu1[2] )
						pu2 = NULL;
						else
						pu3 = (u32*)ROUT_LAN_COLL_ACT_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
					}
					if ( pu3 != NULL )
					is_lan = 1;
				}
				if ( !type || type == 2 )
				{
					//  WAN
					if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK )
					{
						pu2 = (u32*)ROUT_WAN_HASH_CMP_TBL(entry);
						if ( *pu2 != *pu1 || pu2[1] != pu1[1] || pu2[2] != pu1[2] )
						pu2 = NULL;
						else
						pu3 = (u32*)ROUT_WAN_HASH_ACT_TBL(entry);
					}
					else if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK + WAN_ROUT_TBL_CFG->rout_num )
					{
						pu2 = (u32*)ROUT_WAN_COLL_CMP_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
						if ( *pu2 != *pu1 || pu2[1] != pu1[1] || pu2[2] != pu1[2] )
						pu2 = NULL;
						else
						pu3 = (u32*)ROUT_WAN_COLL_ACT_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
					}
					if ( pu3 != NULL )
					is_lan = 0;
				}
			}
			else if ( !*pu1 && !pu1[1] )
			{
				if ( type == 1 )
				{
					if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK )
					{
						pu2 = (u32*)ROUT_LAN_HASH_CMP_TBL(entry);
						pu3 = (u32*)ROUT_LAN_HASH_ACT_TBL(entry);
					}
					else if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK + LAN_ROUT_TBL_CFG->rout_num )
					{
						pu2 = (u32*)ROUT_LAN_COLL_CMP_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
						pu3 = (u32*)ROUT_LAN_COLL_ACT_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
					}
					if ( pu3 != NULL )
					is_lan = 1;
				}
				else if ( type == 2 )
				{
					if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK )
					{
						pu2 = (u32*)ROUT_WAN_HASH_CMP_TBL(entry);
						pu3 = (u32*)ROUT_WAN_HASH_ACT_TBL(entry);
					}
					else if ( entry < MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK + WAN_ROUT_TBL_CFG->rout_num )
					{
						pu2 = (u32*)ROUT_WAN_COLL_CMP_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
						pu3 = (u32*)ROUT_WAN_COLL_ACT_TBL(entry - MAX_ROUTING_ENTRIES_PER_HASH_BLOCK * MAX_HASH_BLOCK);
					}
					if ( pu3 != NULL )
					is_lan = 0;
				}
			}
		}

		if ( pu2 && pu3 )
		{
			is_lan <<= 31;

			if ( ifx_ppa_drv_hal_generic_hook == NULL )
			{
				printk("HAL is not loaded - verify only, no firmware table operation\n");
				printk("del_routing_entry(%d - %08x)\n", entry, (u32)entry | is_lan);
			}
			else
			{
				PPE_ROUTING_INFO route = {0};
				int ret;

				route.entry = (u32)entry | is_lan;
				ret = ifx_ppa_drv_hal_generic_hook(PPA_GENERIC_HAL_DEL_ROUTE_ENTRY, &route, 0);
				printk("%s entry %d deleted %s\n", is_lan ? "LAN" : "WAN", entry, ret == IFX_SUCCESS ? "successfully" : "fail");
			}
		}
	}
	else if ( operation == 1 && type && ((!is_ipv6 && *(u32*)&compare_tbl && *((u32*)&compare_tbl + 1)) || (is_ipv6 && *((u32*)&compare_tbl + 2))) )
	{
		if ( ifx_ppa_drv_hal_generic_hook == NULL )
		{
			char str_to_print[1024];

			printk("HAL is not loaded - verify only, no firmware table operation\n");

			if ( is_ipv6 )
			{
				printk("ipv6 src ip:  %d.%d.%d.%d", ipv6_src_ip[0] >> 24, (ipv6_src_ip[0] >> 16) & 0xFF, (ipv6_src_ip[0] >> 8) & 0xFF, ipv6_src_ip[0] & 0xFF);
				printk(".%d.%d.%d.%d", ipv6_src_ip[1] >> 24, (ipv6_src_ip[1] >> 16) & 0xFF, (ipv6_src_ip[1] >> 8) & 0xFF, ipv6_src_ip[1] & 0xFF);
				printk(".%d.%d.%d.%d", ipv6_src_ip[2] >> 24, (ipv6_src_ip[2] >> 16) & 0xFF, (ipv6_src_ip[2] >> 8) & 0xFF, ipv6_src_ip[2] & 0xFF);
				printk(".%d.%d.%d.%d", ipv6_src_ip[3] >> 24, (ipv6_src_ip[3] >> 16) & 0xFF, (ipv6_src_ip[3] >> 8) & 0xFF, ipv6_src_ip[3] & 0xFF);
				printk("\n");
				printk("ipv6 dest ip: %d.%d.%d.%d", ipv6_dst_ip[0] >> 24, (ipv6_dst_ip[0] >> 16) & 0xFF, (ipv6_dst_ip[0] >> 8) & 0xFF, ipv6_dst_ip[0] & 0xFF);
				printk(".%d.%d.%d.%d", ipv6_dst_ip[1] >> 24, (ipv6_dst_ip[1] >> 16) & 0xFF, (ipv6_dst_ip[1] >> 8) & 0xFF, ipv6_dst_ip[1] & 0xFF);
				printk(".%d.%d.%d.%d", ipv6_dst_ip[2] >> 24, (ipv6_dst_ip[2] >> 16) & 0xFF, (ipv6_dst_ip[2] >> 8) & 0xFF, ipv6_dst_ip[2] & 0xFF);
				printk(".%d.%d.%d.%d", ipv6_dst_ip[3] >> 24, (ipv6_dst_ip[3] >> 16) & 0xFF, (ipv6_dst_ip[3] >> 8) & 0xFF, ipv6_dst_ip[3] & 0xFF);
				printk("\n");
			}
			print_route(str_to_print, 0, type == 1 ? 1 : 0, &compare_tbl, &action_tbl);
			printk(str_to_print);
		}
		else
		{
			PPE_ROUTING_INFO route = {0};
			u8 test_mac[PPA_ETH_ALEN] = {0};

			test_mac[0] = (action_tbl.new_dest_mac54 >> 8) & 0xFF;
			test_mac[1] = action_tbl.new_dest_mac54 & 0xFF;
			test_mac[2] = (action_tbl.new_dest_mac30 >> 24) & 0xFF;
			test_mac[3] = (action_tbl.new_dest_mac30 >> 16) & 0xFF;
			test_mac[4] = (action_tbl.new_dest_mac30 >> 8) & 0xFF;
			test_mac[5] = action_tbl.new_dest_mac30 & 0xFF;

			route.f_is_lan = type == 1 ? 1 : 0;
			if ( is_ipv6 )
			{
				route.src_ip.f_ipv6 = 1;
				memcpy(route.src_ip.ip.ip6, ipv6_src_ip, 16);
				route.dst_ip.f_ipv6 = 1;
				memcpy(route.dst_ip.ip.ip6, ipv6_dst_ip, 16);
			}
			else
			{
				route.src_ip.f_ipv6 = 0;
				route.src_ip.ip.ip = compare_tbl.src_ip;
				route.dst_ip.f_ipv6 = 0;
				route.dst_ip.ip.ip = compare_tbl.dest_ip;
				route.new_ip.f_ipv6 = 0;
				route.new_ip.ip.ip = action_tbl.new_ip;
			}
			route.src_port = compare_tbl.src_port;
			route.dst_port = compare_tbl.dest_port;
			route.new_port = action_tbl.new_port;
			route.f_is_tcp = action_tbl.protocol;
			route.route_type = action_tbl.rout_type;
			memcpy(route.new_dst_mac, test_mac, 6);
			route.src_mac.mac_ix = action_tbl.new_src_mac_ix;
			route.mtu_info.mtu_ix = action_tbl.mtu_ix;
			route.pppoe_mode = action_tbl.pppoe_mode;
			route.pppoe_info.pppoe_ix = action_tbl.pppoe_ix;
			route.f_new_dscp_enable = action_tbl.new_dscp_en;
			route.new_dscp = action_tbl.new_dscp;
			route.f_vlan_ins_enable = action_tbl.in_vlan_ins;
			route.new_vci = action_tbl.new_in_vci;
			route.f_vlan_rm_enable = action_tbl.in_vlan_rm;
			route.f_out_vlan_ins_enable = action_tbl.out_vlan_ins;
			route.out_vlan_info.vlan_entry = action_tbl.out_vlan_ix;
			route.f_out_vlan_rm_enable = action_tbl.out_vlan_rm;
			route.dslwan_qid = action_tbl.dest_qid;
			route.dest_list = action_tbl.dest_list;

			if ( ifx_ppa_drv_hal_generic_hook(PPA_GENERIC_HAL_ADD_ROUTE_ENTRY, &route, 0) == IFX_SUCCESS )
			printk("%s entry add successfully: entry = %d\n", (route.entry & (1 << 31)) ? "LAN" : "WAN", route.entry & ~(1 << 31));
			else
			printk("%s entry add fail\n", type == 1 ? "LAN" : "WAN");
		}
	}
	else
	printk("No operation: operation = %d, type = %d, is_ipv6 = %d\n", operation, type, is_ipv6);

	return count;
}

#if 0
static int proc_read_mc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    volatile struct wan_rout_multicast_cmp_tbl *pcompare;
    volatile struct wan_rout_multicast_act_tbl *paction;
    int len = 0;
    char *pstr;
    char str[2048];
    int llen;
    int i;

    pstr = *start = page;

    __sync();

    llen = sprintf(str, "Multicast Table\n");
    proc_buf_copy(&pstr, count, off, &len, str, llen);

    pcompare = ROUT_WAN_MC_CMP_TBL(0);
    paction = ROUT_WAN_MC_ACT_TBL(0);
    for ( i = 0; i < MAX_WAN_MC_ENTRIES; i++ )
    {
        if ( pcompare->wan_dest_ip )
        {
            llen = print_mc(str, i, (struct wan_rout_multicast_cmp_tbl *)pcompare, (struct wan_rout_multicast_act_tbl *)paction);
            if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
                goto PROC_READ_MC_OVERRUN_END;
        }

        pcompare++;
        paction++;
    }

    *eof = 1;

PROC_READ_MC_OVERRUN_END:
    return len - off;
}
#endif

#if 0
static int proc_read_genconf(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char *str;
    int llen = 0;
    int i;
    unsigned long bit;

    str = *start = page;

    __sync();


    llen += sprintf(str + llen, "ETH_PORTS_CFG    (0x%08X): WAN VLAN ID Hi - %d, WAN VLAN ID Lo - %d\n", (u32)ETH_PORTS_CFG, ETH_PORTS_CFG->wan_vlanid_hi, ETH_PORTS_CFG->wan_vlanid_lo);

    llen += sprintf(str + llen, "LAN_ROUT_TBL_CFG (0x%08X): num - %d, TTL check - %s, no hit drop - %s\n",
                                                            (u32)LAN_ROUT_TBL_CFG,
                                                            LAN_ROUT_TBL_CFG->rout_num,
                                                            LAN_ROUT_TBL_CFG->ttl_en ? "on" : "off",
                                                            LAN_ROUT_TBL_CFG->rout_drop  ? "on" : "off"
                                                            );
    llen += sprintf(str + llen, "                               IP checksum - %s, TCP/UDP checksum - %s, checksum error drop - %s\n",
                                                            LAN_ROUT_TBL_CFG->ip_ver_en ? "on" : "off",
                                                            LAN_ROUT_TBL_CFG->tcpdup_ver_en ? "on" : "off",
                                                            LAN_ROUT_TBL_CFG->iptcpudperr_drop ? "on" : "off"
                                                            );

    llen += sprintf(str + llen, "WAN_ROUT_TBL_CFG (0x%08X): num - %d, TTL check - %s, no hit drop - %s\n",
                                                            (u32)WAN_ROUT_TBL_CFG,
                                                            WAN_ROUT_TBL_CFG->rout_num,
                                                            WAN_ROUT_TBL_CFG->ttl_en ? "on" : "off",
                                                            WAN_ROUT_TBL_CFG->rout_drop  ? "on" : "off"
                                                            );
    llen += sprintf(str + llen, "                               MC num - %d, MC drop - %s\n",
                                                            WAN_ROUT_TBL_CFG->wan_rout_mc_num,
                                                            WAN_ROUT_TBL_CFG->wan_rout_mc_drop ? "on" : "off"
                                                            );
    llen += sprintf(str + llen, "                               IP checksum - %s, TCP/UDP checksum - %s, checksum error drop - %s\n",
                                                            WAN_ROUT_TBL_CFG->ip_ver_en ? "on" : "off",
                                                            WAN_ROUT_TBL_CFG->tcpdup_ver_en ? "on" : "off",
                                                            WAN_ROUT_TBL_CFG->iptcpudperr_drop ? "on" : "off"
                                                            );

    llen += sprintf(str + llen, "GEN_MODE_CFG1    (0x%08X): App2 - %s, U/S - %s, U/S early discard - %s\n"
                                "                               classification table entry - %d, IPv6 route entry - %d\n"
                                "                               session based rate control - %s, IPv4 WAN ingress hash alg - %s\n"
                                "                               multiple field based classification and VLAN assignment for bridging - %s\n"
                                "                               D/S IPv4 multicast acceleration - %s, IPv6 acceleration - %s\n"
                                "                               WAN acceleration - %s, LAN acceleration - %s, switch isolation - %s\n",
                                                            (u32)GEN_MODE_CFG1,
                                                            GEN_MODE_CFG1->app2_indirect ? "indirect" : "direct",
                                                            GEN_MODE_CFG1->us_indirect ? "indirect" : "direct",
                                                            GEN_MODE_CFG1->us_early_discard_en ? "enable" : "disable",
                                                            GEN_MODE_CFG1->classification_num,
                                                            GEN_MODE_CFG1->ipv6_rout_num,
                                                            GEN_MODE_CFG1->session_ctl_en ? "enable" : "disable",
                                                            GEN_MODE_CFG1->wan_hash_alg ? "src IP" : "dest port",
                                                            GEN_MODE_CFG1->brg_class_en ? "enable" : "disable",
                                                            GEN_MODE_CFG1->ipv4_mc_acc_mode ? "IP/port pairs" : "dst IP only",
                                                            GEN_MODE_CFG1->ipv6_acc_en ? "enable" : "disable",
                                                            GEN_MODE_CFG1->wan_acc_en ? "enable" : "disable",
                                                            GEN_MODE_CFG1->lan_acc_en ? "enable" : "disable",
                                                            GEN_MODE_CFG1->sw_iso_mode ? "isolated" : "not isolated"
                                                            );
    llen += sprintf(str + llen, "WAN Interfaces:                %s \n", get_wanitf(1));
    llen += sprintf(str + llen, "LAN Interfaces:                %s \n", get_wanitf(0));
    llen += sprintf(str + llen, "GEN_MODE_CFG2    (0x%08X):", (u32)GEN_MODE_CFG2);
    for ( i = 0, bit = 1; i < 8; i++, bit <<= 1 )
    {
        llen += sprintf(str + llen, " %d - %s", i, (GEN_MODE_CFG2->itf_outer_vlan_vld & bit) ? "on " : "off");
        if ( i == 3 )
            llen += sprintf(str + llen, "\n                              ");
    }
    llen += sprintf(str + llen, "\n");

    llen += sprintf(str + llen, "TX_QOS_CFG       (0x%08X): time_tick - %d, overhd_bytes - %d, eth1_eg_qnum - %d\n"
                                "                               eth1_burst_chk - %s, eth1 - %s, rate shaping - %s, WFQ - %s\n",
                                                            (u32)TX_QOS_CFG_DYNAMIC,
                                                            TX_QOS_CFG_DYNAMIC->time_tick,
                                                            TX_QOS_CFG_DYNAMIC->overhd_bytes,
                                                            TX_QOS_CFG_DYNAMIC->eth1_eg_qnum,
                                                            TX_QOS_CFG_DYNAMIC->eth1_burst_chk ? "on" : "off",
                                                            TX_QOS_CFG_DYNAMIC->eth1_qss ? "FW QoS" : "HW QoS",
                                                            TX_QOS_CFG_DYNAMIC->shape_en ? "on" : "off",
                                                            TX_QOS_CFG_DYNAMIC->wfq_en ? "on" : "off");

    llen += sprintf(str + llen, "KEY_SEL_n        (0x%08X): %08x %08x %08x %08x\n",
                                                            (u32)KEY_SEL_n(0),
                                                            *KEY_SEL_n(0), *KEY_SEL_n(1), *KEY_SEL_n(2), *KEY_SEL_n(3));

    *eof = 1;

    return llen - off;
}
#endif

#if 0
static int proc_write_genconf(struct file *file, const char *buf, unsigned long count, void *data)
{
    char str[2048];
    char *p;
    int len, rlen;

    int f_wan_hi = 0;

    len = count < sizeof(str) ? count : sizeof(str) - 1;
    rlen = len - copy_from_user(str, buf, len);
    while ( rlen && str[rlen - 1] <= ' ' )
        rlen--;
    str[rlen] = 0;
    for ( p = str; *p && *p <= ' '; p++, rlen-- );
    if ( !*p )
    {
        return 0;
    }

    if ( ifx_strincmp(p, "wan hi ", 7) == 0 )
    {
        p += 7;
        rlen -= 7;
        f_wan_hi = 1;
    }
    else if ( ifx_strincmp(p, "wan high ", 9) == 0 )
    {
        p += 9;
        rlen -= 9;
        f_wan_hi = 1;
    }
    else if ( ifx_strincmp(p, "wan lo ", 7) == 0 )
    {
        p += 7;
        rlen -= 7;
        f_wan_hi = -1;
    }
    else if ( ifx_strincmp(p, "wan low ", 8) == 0 )
    {
        p += 8;
        rlen -= 8;
        f_wan_hi = -1;
    }
    else if ( ifx_strincmp(p, "eth0 type ", 10) == 0 )
    {
        p += 10;
        rlen -= 10;
        if ( ifx_stricmp(p, "lan") == 0 )
            ETH_PORTS_CFG->eth0_type = 0;
        else if ( ifx_stricmp(p, "wan") == 0 )
            ETH_PORTS_CFG->eth0_type = 1;
        else if ( ifx_stricmp(p, "mix") == 0 )
            ETH_PORTS_CFG->eth0_type = 2;
    }

    if ( f_wan_hi )
    {
        int num;

        num = ifx_get_number(&p, &rlen, 0);
        if ( f_wan_hi > 0 )
            ETH_PORTS_CFG->wan_vlanid_hi = num;
        else
            ETH_PORTS_CFG->wan_vlanid_lo = num;
    }

    return count;
}
#endif


static int proc_read_pppoe(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
	int len = 0;
	char *pstr;
	char str[2048];
	int llen;
	int i;

	pstr = *start = page;

	__sync();

	llen = sprintf(pstr, "PPPoE Table (0x%08X) - Session ID:\n", (u32)PPPOE_CFG_TBL(0));
	pstr += llen;
	len += llen;

	for ( i = 0; i < 8; i++ )
	{
		llen = sprintf(str, "  %d: %d\n", i, *PPPOE_CFG_TBL(i));
		if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
		goto PROC_READ_PPPOE_OVERRUN_END;
	}

	*eof = 1;

	PROC_READ_PPPOE_OVERRUN_END:
	return len - off;
}

static int proc_read_mtu(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
	int len = 0;
	char *pstr;
	char str[2048];
	int llen;
	int i;

	pstr = *start = page;

	__sync();

	llen = sprintf(pstr, "MTU Table (0x%08X):\n", (u32)MTU_CFG_TBL(0));
	pstr += llen;
	len += llen;

	for ( i = 0; i < 8; i++ )
	{
		llen = sprintf(str, "  %d: %d\n", i, *MTU_CFG_TBL(i));
		if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
		goto PROC_READ_MTU_OVERRUN_END;
	}

	*eof = 1;

	PROC_READ_MTU_OVERRUN_END:
	return len - off;
}

static int proc_read_hit(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
	int len = 0;
	char *pstr;
	char str[2048];
	int llen;
	int i;
	int n;
	unsigned long bit;

	pstr = *start = page;

	__sync();

    llen = sprintf(pstr, "Unicast Routing Hit Table (LAN) (0x%08X):\n", (u32)ROUT_LAN_HASH_HIT_STAT_TBL(0)); //TODO nicht vollstaendig!!
	pstr += llen;
	len += llen;

	llen = sprintf(pstr, "             1 2 3 4 5 6 7 8 9 10\n");
	pstr += llen;
	len += llen;

	n = 1;
	for ( i = 0; i < 8; i++ )
	for ( bit = 0x80000000; bit; bit >>= 1 )
	{
		if ( n % 10 == 1 )
		llen = sprintf(str, "  %3d - %3d:", n, n + 9);

		llen += sprintf(str + llen, " %d", (*ROUT_LAN_HASH_HIT_STAT_TBL(i) & bit) ? 1 : 0);

		if ( n++ % 10 == 0 )
		{
			llen += sprintf(str + llen, "\n");

			if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
			goto PROC_READ_HIT_OVERRUN_END;
		}
	}

	if ( n % 10 != 0 )
	{
		llen += sprintf(str + llen, "\n");

		if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
		goto PROC_READ_HIT_OVERRUN_END;
	}

	llen = sprintf(str, "Multicast Routing Hit Table (0x%08X):\n", (u32)ROUT_WAN_MC_HIT_STAT_TBL(0));
	llen += sprintf(str + llen, "             1 2 3 4 5 6 7 8 9 10\n");
	if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
	goto PROC_READ_HIT_OVERRUN_END;

	n = 1;
	for ( i = 0; i < 1; i++ )
	for ( bit = 0x80000000; bit; bit >>= 1 )
	{
		if ( n % 10 == 1 )
		llen = sprintf(str, "  %3d - %3d:", n, n + 9);

		llen += sprintf(str + llen, " %d", (*ROUT_WAN_MC_HIT_STAT_TBL(i) & bit) ? 1 : 0);

		if ( n++ % 10 == 0 )
		{
			llen += sprintf(str + llen, "\n");

			if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
			goto PROC_READ_HIT_OVERRUN_END;
		}
	}

	if ( n % 10 != 0 )
	{
		llen += sprintf(str + llen, "\n");

		if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
		goto PROC_READ_HIT_OVERRUN_END;
	}

	*eof = 1;

	PROC_READ_HIT_OVERRUN_END:
	return len - off;
}

static int proc_write_hit(struct file *file __attribute__((unused)), const char *buf, unsigned long count, void *data __attribute__((unused)))
{
	char str[2048];
	char *p;
	int len, rlen;

	len = count < sizeof(str) ? count : sizeof(str) - 1;
	rlen = len - copy_from_user(str, buf, len);
	while ( rlen && str[rlen - 1] <= ' ' )
	rlen--;
	str[rlen] = 0;
	for ( p = str; *p && *p <= ' '; p++, rlen-- );
	if ( !*p )
	{
		return 0;
	}

	if ( ifx_stricmp(p, "clear") == 0 || ifx_stricmp(p, "clean") == 0 )
	{
		ppa_memset((void*)ROUT_LAN_HASH_HIT_STAT_TBL(0), 0, 64);
		ppa_memset((void*)ROUT_LAN_COLL_HIT_STAT_TBL(0), 0, 2);//TODO nicht vollstaendig!!
		ppa_memset((void*)ROUT_WAN_HASH_HIT_STAT_TBL(0), 0, 64);
		ppa_memset((void*)ROUT_WAN_COLL_HIT_STAT_TBL(0), 0, 2);//TODO nicht vollstaendig!!
		ppa_memset((void*)ROUT_WAN_MC_HIT_STAT_TBL(0), 0, 2);
	}

	return count;
}

static int proc_read_mac(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
	int len = 0;
	char *pstr;
	char str[2048];
	int llen;
	int i;
	unsigned int mac52, mac10;

	pstr = *start = page;

	__sync();

	llen = sprintf(pstr, "MAC Table:\n");
	pstr += llen;
	len += llen;

	llen = sprintf(pstr, "  ROUT_MAC_CFG_TBL (0x%08X)\n", (u32)ROUT_MAC_CFG_TBL(0));
	pstr += llen;
	len += llen;

	for ( i = 0; i < 16; i++ )
	{
		mac52 = *ROUT_MAC_CFG_TBL(i);
		mac10 = *(ROUT_MAC_CFG_TBL(i) + 1);

		llen = sprintf(str, "    %2d: %02X:%02X:%02X:%02X:%02X:%02X\n", i + 1, mac52 >> 24, (mac52 >> 16) & 0xFF, (mac52 >> 8) & 0xFF, mac52 & 0xFF, mac10 >> 24, (mac10 >> 16) & 0xFF);
		if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
		goto PROC_READ_MAC_OVERRUN_END;
	}

	*eof = 1;

	PROC_READ_MAC_OVERRUN_END:
	return len - off;
}

static int proc_read_out_vlan(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
	int len = 0;
	char *pstr;
	char str[2048];
	int llen;
	int i;

	pstr = *start = page;

	__sync();

	llen = sprintf(pstr, "Outer VLAN Table (0x%08X):\n", (u32)OUTER_VLAN_TBL(0));
	pstr += llen;
	len += llen;

	for ( i = 0; i < 32; i++ )
	{
		llen = sprintf(str, "  %d: 0x%08X\n", i, *OUTER_VLAN_TBL(i));
		if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
		goto PROC_READ_MAC_OVERRUN_END;
	}

	*eof = 1;

	PROC_READ_MAC_OVERRUN_END:
	return len - off;
}

static int proc_read_ipv6_ip(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused)))
{
    int len = 0;
    char *pstr;
    char str[2048];
    int llen;

    int i, j, x;
    volatile char *p;

    pstr = *start = page;

    __sync();

    llen = sprintf(str, "IPv6 IP Table:\n");
    proc_buf_copy(&pstr, count, off, &len, str, llen);

    for ( x = 0; x < MAX_IPV6_IP_ENTRIES_BLOCK; x++ ) {
        for ( i = 0; i < MAX_IPV6_IP_ENTRIES_PER_BLOCK; i++ ) {
            if ( IPv6_IP_IDX_TBL(x, i)[0] != 0 || IPv6_IP_IDX_TBL(x, i)[1] != 0 || IPv6_IP_IDX_TBL(x, i)[2] != 0 || IPv6_IP_IDX_TBL(x, i)[3] != 0 )
            {
                p = (volatile char *)IPv6_IP_IDX_TBL(x, i);
                llen = sprintf(str, "%3d - %u", x * MAX_IPV6_IP_ENTRIES_PER_BLOCK + i, (unsigned int)p[0]);
                for ( j = 1; j < 16; j++ )
                    llen += sprintf(str + llen, ".%u", (unsigned int)p[j]);
                llen += sprintf(str + llen, "\n");

                if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
                    goto PROC_READ_IPv6_IP_OVERRUN_END;
            }
        }
    }

    *eof = 1;

PROC_READ_IPv6_IP_OVERRUN_END:
    return len - off;
}

#endif

#define PP32_REG_ADDR_BEGIN     0x0
#define PP32_REG_ADDR_END       0x1FFF
#define PP32_SB_ADDR_BEGIN      0x2000
#define PP32_SB_ADDR_END        0xFFFF

static inline unsigned long sb_addr_to_fpi_addr_convert(unsigned long sb_addr) {
	if (sb_addr <= PP32_SB_ADDR_END) {
		return (unsigned long) SB_BUFFER_BOND(2, sb_addr);
	} else {
		return sb_addr;
	}
}

#if defined(DEBUG_MEM_PROC) && DEBUG_MEM_PROC

/*
 * Syntax (Switch read/write):
 *  r sw hex_addr
 *  w sw hex_addr hex_data
 *
 * Syntax (PP32 read/write):
 *  r hex_addr
 *  w hex_addr hex_data
 */

static int proc_write_mem(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count,
		void *data __attribute__ ((unused))) {
	char *p1, *p2;
	int len;
	int colon;
	volatile unsigned long *p;
	unsigned long dword;

	/*--- char common_proc_read_buff[1024]; ---*/
	int i, n, l;

	len = sizeof(common_proc_read_buff) < count ?
			sizeof(common_proc_read_buff) - 1 : count;
	len = len - copy_from_user(common_proc_read_buff, buf, len);
	common_proc_read_buff[len] = 0;

	p1 = common_proc_read_buff;
	p2 = NULL;
	colon = 1;
	while (ifx_get_token(&p1, &p2, &len, &colon)) {
		if (ifx_stricmp(p1, "w") == 0 || ifx_stricmp(p1, "write") == 0
				|| ifx_stricmp(p1, "r") == 0 || ifx_stricmp(p1, "read") == 0)
			break;

		p1 = p2;
		colon = 1;
	}

	if (*p1 == 'w') {
		ifx_ignore_space(&p2, &len);
		if (p2[0] == 's' && p2[1] == 'w' && (p2[2] == ' ' || p2[2] == '\t')) {
			unsigned long temp;

			p2 += 3;
			len -= 3;
			ifx_ignore_space(&p2, &len);
			temp = ifx_get_number(&p2, &len, 1);
			p = (unsigned long *) VR9_SWIP_MACRO_REG(temp);
		} else {
			p = (unsigned long *) ifx_get_number(&p2, &len, 1);
			p = (unsigned long *) sb_addr_to_fpi_addr_convert(
					(unsigned long) p);
		}

		if ((u32) p >= KSEG0)
			while (1) {
				ifx_ignore_space(&p2, &len);
				if (!len
						|| !((*p2 >= '0' && *p2 <= '9')
								|| (*p2 >= 'a' && *p2 <= 'f')
								|| (*p2 >= 'A' && *p2 <= 'F')))
					break;

				*p++ = (u32) ifx_get_number(&p2, &len, 1);
			}
	} else if (*p1 == 'r') {
		ifx_ignore_space(&p2, &len);
		if (p2[0] == 's' && p2[1] == 'w' && (p2[2] == ' ' || p2[2] == '\t')) {
			unsigned long temp;

			p2 += 3;
			len -= 3;
			ifx_ignore_space(&p2, &len);
			temp = ifx_get_number(&p2, &len, 1);
			p = (unsigned long *) VR9_SWIP_MACRO_REG(temp);
		} else {
			p = (unsigned long *) ifx_get_number(&p2, &len, 1);
			p = (unsigned long *) sb_addr_to_fpi_addr_convert(
					(unsigned long) p);
		}

		if ((u32) p >= KSEG0) {
			ifx_ignore_space(&p2, &len);
			n = (int) ifx_get_number(&p2, &len, 0);
			if (n) {
				char str[32] = { 0 };
				char *pch = str;
				int k;
				char c;

				n += (l = ((int) p >> 2) & 0x03);
				p = (unsigned long *) ((u32) p & ~0x0F);
				for (i = 0; i < n; i++) {
					if ((i & 0x03) == 0) {
						printk("%08X:", (u32) p);
						pch = str;
					}
					if (i < l) {
						printk("         ");
						sprintf(pch, "    ");
					} else {
						dword = *p;
						printk(" %08X", (u32) dword);
						for (k = 0; k < 4; k++) {
							c = ((char*) &dword)[k];
							pch[k] = c < ' ' ? '.' : c;
						}
					}
					p++;
					pch += 4;
					if ((i & 0x03) == 0x03) {
						pch[0] = 0;
						printk(" ; %s\n", str);
					}
				}
				if ((n & 0x03) != 0x00) {
					for (k = 4 - (n & 0x03); k > 0; k--)
						printk("         ");
					pch[0] = 0;
					printk(" ; %s\n", str);
				}
			}
		}
	}

	return count;
}
#endif

static int proc_read_burstlen(char *page, char **start __attribute__ ((unused)),
		off_t off __attribute__((unused)), int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;
	u32 dma_ps;
	u32 dma_pctrl;
	unsigned long sys_flag;

	local_irq_save(sys_flag);
	dma_ps = *(volatile u32 *) 0xBE104140;
	*(volatile u32 *) 0xBE104140 = 0;
	dma_pctrl = *(volatile u32 *) 0xBE104144;
	*(volatile u32 *) 0xBE104140 = dma_ps;
	local_irq_restore(sys_flag);

	len += sprintf(page + len, "DMA-PPE burst length: Rx %d, Tx %d\n",
			1 << ((dma_pctrl >> 2) & 0x03), 1 << ((dma_pctrl >> 4) & 0x03));

	*eof = 1;

	return len;
}

static int proc_write_burstlen(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count,
		void *data __attribute__ ((unused))) {
	int len;
	char *p1, *p2;
	int colon = 0;
	int mask = 0x3C; //  rx: 0x0C, tx: 0x30, all: 0x3C
	int burstlen = 0;
	int burstlen_mask = 0;
	int f_help = 0;
	u32 dma_ps;
	unsigned long sys_flag;

	len = sizeof(common_proc_read_buff) < count ?
			sizeof(common_proc_read_buff) - 1 : count;
	len = len - copy_from_user(common_proc_read_buff, buf, len);
	common_proc_read_buff[len] = 0;

	p1 = common_proc_read_buff;
	while (ifx_get_token(&p1, &p2, &len, &colon)) {
		if (ifx_stricmp(p1, "rx") == 0)
			burstlen_mask |= (mask = 0x0C);
		else if (ifx_stricmp(p1, "tx") == 0)
			burstlen_mask |= (mask = 0x30);
		else if (ifx_stricmp(p1, "all") == 0)
			burstlen_mask |= (mask = 0x3C);
		else if (strcmp(p1, "2") == 0)
			burstlen = 0x14 & mask;
		else if (strcmp(p1, "4") == 0)
			burstlen = 0x28 & mask;
		else if (strcmp(p1, "8") == 0)
			burstlen = 0x3C & mask;
		else {
			f_help = 1;
			break;
		}

		p1 = p2;
	}

	if (!burstlen_mask && burstlen)
		burstlen_mask = 0x3C;

	if (!burstlen_mask || !burstlen)
		f_help = 1;

	if (!f_help) {
		local_irq_save(sys_flag);
		dma_ps = *(volatile u32 *) 0xBE104140;
		*(volatile u32 *) 0xBE104140 = 0;
		*(volatile u32 *) 0xBE104144 = (*(volatile u32 *) 0xBE104144
				& ~burstlen_mask) | burstlen;
		*(volatile u32 *) 0xBE104140 = dma_ps;
		local_irq_restore(sys_flag);
	} else {
		printk("echo [rx/tx/all] <2/4/8> > /proc/eth/burstlen\n");
	}

	return count;
}

/*------------------------------------------------------------------------------------------*\
 * Complett neu seit UGW 5.1.1 begin
 \*------------------------------------------------------------------------------------------*/

#if defined(DEBUG_FW_PROC) && DEBUG_FW_PROC
static int proc_read_fw(char *page, char **start __attribute__((unused)), off_t off __attribute__((unused)), int count __attribute__((unused)),
		int *eof, void *data __attribute__((unused))) {
	int len = 0;

	len += sprintf(page + len, "Firmware\n");

	len += sprintf(page + len, "  pre_dplus_ptr    = %04X\n", *PRE_DPLUS_PTR);

	len += sprintf(page + len, "  pre_dplus_cnt    = %04X\n",
			(*DM_RXPKTCNT) & 0xff);

	len += sprintf(page + len, "  dplus_ptr        = %04X\n", *DPLUS_PTR);

	len += sprintf(page + len, "  dplus_cnt        = %04X\n", *DPLUS_CNT);

	len += sprintf(page + len, "  DPLUS_RX_ON      = %d\n", *DPLUS_RX_ON);

	len += sprintf(page + len, "  ISR_IS           = %08X\n ", *ISR_IS);

    {
        unsigned char * ls_txt[] = {
            "Looking",
            "Freewheel Sync False",
            "Synced",
            "Freewheel Sync True"
        };
        unsigned char * rs_txt[] = {
            "Out-of-Sync",
            "Synced"
        };

        len += sprintf(page + len, "\nE1 firmware:\n");

        len += sprintf(page + len, "SFSM Status:  bc0                         bc1  \n\n");
        len += sprintf(page + len, "Local State = %d [%-22s], %d [%-22s]\n"
                                         "RemoteState = %d [%-22s], %d [%-22s]\n",
                RX_BC_CFG(0)->local_state, ls_txt[RX_BC_CFG(0)->local_state],
                RX_BC_CFG(1)->local_state, ls_txt[RX_BC_CFG(1)->local_state],
                RX_BC_CFG(0)->remote_state, rs_txt[RX_BC_CFG(0)->remote_state],
                RX_BC_CFG(1)->remote_state, rs_txt[RX_BC_CFG(1)->remote_state]);
        len += sprintf(page + len, "  dbase     = 0x%04x  ( 0x%08x )    , 0x%04x  ( 0x%08x )\n",
                SFSM_DBA(0)->dbase, (unsigned int )SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000),
                SFSM_DBA(1)->dbase, (unsigned int )SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000)  );
        len += sprintf(page + len, "  cbase     = 0x%04x  ( 0x%08x )    , 0x%04x  ( 0x%08x )\n",
                SFSM_CBA(0)->cbase, (unsigned int )SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000),
                SFSM_CBA(1)->cbase, (unsigned int )SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000));
        len += sprintf(page + len, "  sen       = %-26d, %d\n", SFSM_CFG(0)->sen, SFSM_CFG(1)->sen );
        len += sprintf(page + len, "  idlekeep  = %-26d, %d\n", SFSM_CFG(0)->idlekeep, SFSM_CFG(1)->idlekeep );
        len += sprintf(page + len, "  pnum      = %-26d, %d\n", SFSM_CFG(0)->pnum, SFSM_CFG(1)->pnum );
        len += sprintf(page + len, "  pptr      = %-26d, %d\n", SFSM_PGCNT(0)->pptr, SFSM_PGCNT(1)->pptr);
        len += sprintf(page + len, "  upage     = %-26d, %d\n", SFSM_PGCNT(0)->upage, SFSM_PGCNT(1)->upage);
        len += sprintf(page + len, "  rdptr     = %-26d, %d\n", RX_BC_CFG(0)->rx_cw_rdptr, RX_BC_CFG(1)->rx_cw_rdptr );
        len += sprintf(page + len, "\n");


        len += sprintf(page + len, "FFSM Status:  bc0                         bc1  \n\n");
        len += sprintf(page + len, "  dbase     = 0x%04x  ( 0x%08x )    , 0x%04x  ( 0x%08x )\n",
                FFSM_DBA(0)->dbase, (unsigned int )SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000),
                FFSM_DBA(1)->dbase, (unsigned int )SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000) );
        len += sprintf(page + len, "  pnum      = %-26d, %d\n", FFSM_CFG(0)->pnum, FFSM_CFG(1)->pnum );
        len += sprintf(page + len, "  vpage     = %-26d, %d\n", FFSM_PGCNT(0)->vpage, FFSM_PGCNT(1)->vpage );
        len += sprintf(page + len, "  ival      = %-26d, %d\n", FFSM_PGCNT(0)->ival, FFSM_PGCNT(1)->ival );
        len += sprintf(page + len, "  fill_wm   = %-26d, %d\n", TX_BC_CFG(0)->fill_wm, TX_BC_CFG(1)->fill_wm );
        len += sprintf(page + len, "  uflw_wm   = %-26d, %d\n", TX_BC_CFG(0)->uflw_wm, TX_BC_CFG(1)->uflw_wm );
        len += sprintf(page + len, "  hold_pgs  = %-26d, %d\n", TX_BC_CFG(0)->holding_pages, TX_BC_CFG(1)->holding_pages );
        len += sprintf(page + len, "  rdy_pgs   = %-26d, %d\n", TX_BC_CFG(0)->ready_pages, TX_BC_CFG(1)->ready_pages );
        len += sprintf(page + len, "  pend_pgs  = %-26d, %d\n", TX_BC_CFG(0)->pending_pages, TX_BC_CFG(1)->pending_pages );
        len += sprintf(page + len, "  tc_wrptr  = %-26d, %d\n", TX_BC_CFG(0)->cw_wrptr, TX_BC_CFG(1)->cw_wrptr );

        len += sprintf(page + len, "\n");
    }
	*eof = 1;

	return len;
}

static int proc_write_fw(struct file *file __attribute__ ((unused)), const char *buf, unsigned long count, void *data __attribute__((unused))) {
	char *p;
	int len, rlen;

	len = count < sizeof(common_proc_read_buff) ? count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p) {
		return 0;
	}

	if (ifx_stricmp(p, "clear") == 0 || ifx_stricmp(p, "clean") == 0) {
		*__DSLWAN_TX_THRES_DROP_PKT_CNT0 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT1 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT2 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT3 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT4 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT5 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT6 = 0;
		*__DSLWAN_TX_THRES_DROP_PKT_CNT7 = 0;

		*__CPU_TO_DSLWAN_TX_PKTS = 0;
		*__CPU_TO_DSLWAN_TX_BYTES = 0;

		*__CPU_TX_SWAPPER_DROP_PKTS = 0;
		*__CPU_TX_SWAPPER_DROP_BYTES = 0;
		*__DSLWAN_FP_SWAPPER_DROP_PKTS = 0;
		*__DSLWAN_FP_SWAPPER_DROP_PKTS = 0;
	}

	return count;
}

static void fwdbg_read_ethbuf(char **tokens, int token_num) {
	unsigned int start_pg = 0;
	unsigned int print_pg_num;
	unsigned int num;
	unsigned int pg_size;
	volatile unsigned int *dbase, *cbase;
	unsigned int pnum, i;
	unsigned int *cw;

	dbase =
			(volatile unsigned int *) SB_BUFFER_BOND( 2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );
	cbase =
			(volatile unsigned int *) SB_BUFFER_BOND( 2, ( ( struct dmrx_cba * )DM_RXCB)->cbase + 0x2000);
	pnum = ((struct dmrx_cfg *) DM_RXCFG)->pnum;

	i = 0;
	start_pg = i < token_num ? simple_strtoul(tokens[i++], NULL, 0) : start_pg;
	print_pg_num = i < token_num ? simple_strtoul(tokens[i++], NULL, 0) : pnum;

	start_pg %= pnum;
	print_pg_num = print_pg_num > pnum ? pnum : print_pg_num;

#if 0
	for(i = 0; i < token_num - 1; i ++) {
		if(ifx_strincmp(tokens[i], "start_pg", strlen(tokens[i])) == 0) {
			start_pg = simple_strtoul(tokens[i + 1], NULL, 0);
			start_pg = start_pg % pnum;
			i ++;

		} else if(ifx_strincmp(tokens[i], "pg_num", strlen(tokens[i])) == 0) {
			print_pg_num = simple_strtoul(tokens[i + 1], NULL, 0);
			print_pg_num = print_pg_num <= pnum ? print_pg_num : pnum;
			i ++;
		}
	}
#endif

	pg_size = (((struct dmrx_cfg *) DM_RXCFG)->psize + 1) * 32 / 4;

	printk("Share buffer data/ctrl buffer:\n\n");
	for (i = start_pg, num = 0; num < print_pg_num; num++, i = (i + 1) % pnum) {

		struct ctrl_dmrx_2_fw * p_ctrl_dmrx;
		struct ctrl_fw_2_dsrx * p_ctrl_dsrx;
		int j = 0;

		cw = (unsigned int *) dbase + i * pg_size;

		p_ctrl_dmrx = (struct ctrl_dmrx_2_fw *) (&cbase[i]);
		p_ctrl_dsrx = (struct ctrl_fw_2_dsrx *) (&cbase[i]);

		for (j = 0; j < pg_size; j = j + 4) {

			if (j == 0) {

				printk("Pg_id %2d: %08x %08x %08x %08x ", i, cw[0], cw[1],
						cw[2], cw[3]);
				printk("pg_val: %x, byte_off: %x, cos: %x \n",
						p_ctrl_dmrx->pg_val, p_ctrl_dmrx->byte_off,
						p_ctrl_dmrx->cos);
			}

			else if (j == 4) {
				printk("          %08x %08x %08x %08x ", cw[j], cw[j + 1],
						cw[j + 2], cw[j + 3]);
				printk("bytes_cnt: %x, eof: %x \n", p_ctrl_dmrx->bytes_cnt,
						p_ctrl_dmrx->eof);
			}

			else if (j == 12) {
				printk("          %08x %08x %08x %08x ", cw[j], cw[j + 1],
						cw[j + 2], cw[j + 3]);
                printk("pg_val: %x, byte_off: %x, acc_sel: %x\n", ((unsigned int)p_ctrl_dsrx->pkt_status << 4) | p_ctrl_dsrx->flag_len, p_ctrl_dsrx->byte_off,p_ctrl_dsrx->acc_sel );
			}

			else if (j == 16) {
				printk("          %08x %08x %08x %08x ", cw[j], cw[j + 1],
						cw[j + 2], cw[j + 3]);
				printk("cos: %x, bytes_cnt: %x, eof: %x\n", p_ctrl_dsrx->cos,
						p_ctrl_dsrx->bytes_cnt, p_ctrl_dsrx->eof);
			}

			else {
				printk("          %08x %08x %08x %08x \n", cw[j], cw[j + 1],
						cw[j + 2], cw[j + 3]);
			}

		}

		printk("\n");

	}

	return;
}

#if 0
static void fwdbg_read_cb(char **tokens, int token_num)
{
	unsigned int start_pg;
	unsigned int print_pg_num;
	unsigned int num;
	unsigned int i, is_rx = 1;
	unsigned int chan_no = 0;
	volatile unsigned int *dbase0;
	unsigned int pnum0;
	unsigned int * cw;

	for(i = 0; i < token_num; i ++) {
		if(ifx_strincmp(tokens[i], "tx", strlen(tokens[i])) == 0) {
			is_rx = 0;
		} else if(ifx_strincmp(tokens[i], "chan_no", strlen(tokens[i])) == 0 && i < token_num - 1) {
			chan_no = simple_strtoul(tokens[i + 1], NULL, 0);
			if(chan_no >= 1) { //bare channel only 0 or 1
				chan_no = 1;
			}
			i ++;

		} else if(ifx_strincmp(tokens[i], "start_pg", strlen(tokens[i])) == 0 && i < token_num - 1) {
			start_pg = simple_strtoul(tokens[i + 1], NULL, 0);
			i ++;

		} else if(ifx_strincmp(tokens[i], "pg_num", strlen(tokens[i])) == 0 && i < token_num - 1) {
			print_pg_num = simple_strtoul(tokens[i + 1], NULL, 0);
			i ++;

		}
	}

	dbase0 = (volatile unsigned int *)SB_BUFFER_BOND( FFSM_DBA(0)->dbase + 0x2000);
	pnum0 = FFSM_CFG(0)->pnum;

}
#endif

static void fwdbg_clear_ethbuf(char **tokens, int token_num) {
	uint32_t i = 0;
	uint32_t pnum, pg_num, pg_size, start_pg = 0;
	volatile unsigned int *dbase, *cbase;

	dbase =
			(volatile unsigned int *) SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );
	cbase =
			(volatile unsigned int *) SB_BUFFER_BOND(2, ( ( struct dmrx_cba * )DM_RXCB)->cbase + 0x2000);
	pnum = ((struct dmrx_cfg *) DM_RXCFG)->pnum;
	pg_num = pnum;
	pg_size = (((struct dmrx_cfg *) DM_RXCFG)->psize + 1) * 32 / 4;

	start_pg = i < token_num ? simple_strtoul(tokens[i++], NULL, 0) : start_pg;
	pg_num = i < token_num ? simple_strtoul(tokens[i++], NULL, 0) : pg_num;
	start_pg %= pnum;
	pg_num = pg_num > pnum ? pnum : pg_num;

	dbase = dbase + start_pg * pg_size;
	cbase = cbase + start_pg * pg_size;

	ppa_memset((void *) dbase, 0, pg_num * pg_size * sizeof(uint32_t));
	ppa_memset((void *) cbase, 0, pg_num * sizeof(uint32_t));

	return;

}

static void fwdbg_dump_sthdr(char **tokens, int token_num) {
	//uint32_t i = 0;
	uint32_t pnum, pg_size, pg_idx;
	volatile unsigned int *dbase;
	struct flag_header *pfhdr;
	unsigned char *paddr;

	dbase =
			(volatile unsigned int *) SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );

	if (token_num < 1) {
		fw_dbg_start("help dump_st_hdr");
		return;
	}

	pnum = ((struct dmrx_cfg *) DM_RXCFG)->pnum;
	pg_size = (((struct dmrx_cfg *) DM_RXCFG)->psize + 1) * 32 / 4;
	pg_idx = simple_strtoul(tokens[0], NULL, 0);
	if (pg_idx >= pnum) {
		printk(
				"Error : page NO %d is equal or bigger than total page number: %d\n",
				pg_idx, pnum);
		return;
	}

	pfhdr = (struct flag_header *) (dbase + pg_idx * pg_size);
	dump_flag_header(g_fwcode, pfhdr, __FUNCTION__, 0);

	if (token_num > 1 && ifx_strincmp(tokens[1], "iphdr", strlen(tokens[1])) == 0) {
		if (pfhdr->is_inner_ipv4) {
			printk("First header (IPv4)\n");
			paddr = (unsigned char *) pfhdr;
			paddr += 8 + pfhdr->pl_byteoff + pfhdr->ip_inner_offset;
			dump_ipv4_hdr((struct iphdr *) paddr);

		} else if (pfhdr->is_inner_ipv6) {
			printk("First header (IPv6)\n");
			paddr = (unsigned char *) pfhdr;
			paddr += 8 + pfhdr->pl_byteoff + pfhdr->ip_inner_offset;
			dump_ipv6_hdr((struct ipv6hdr *) paddr);
		}

		printk("------------------------------------\n");
		if (pfhdr->is_outer_ipv4) {
			printk("Second header (IPv4)\n");
			paddr = (unsigned char *) pfhdr;
			paddr += 8 + pfhdr->pl_byteoff + pfhdr->ip_outer_offset;
			dump_ipv4_hdr((struct iphdr *) paddr);
		} else if (pfhdr->is_outer_ipv6) {
			printk("Second header (IPv6)\n");
			paddr = (unsigned char *) pfhdr;
			paddr += 8 + pfhdr->pl_byteoff + pfhdr->ip_outer_offset;
			dump_ipv6_hdr((struct ipv6hdr *) paddr);
		}

		printk("------------------------------------\n");
	}
	return;

}

//echo dump_rg [dmrx | dsrx | SLL]
static void fwdbg_dump_rg(char **tokens, int token_num) {
	uint32_t i;

	if (token_num < 1) {
		fw_dbg_start("help dump_rg");
		return;
	}

	if (ifx_strincmp(tokens[0], "dmrx", strlen(tokens[0])) == 0) {

		printk("DMRX_DBA(0x612): 0x%x\n", ((struct dmrx_dba *) DM_RXDB)->dbase);
		printk("DMRX_CBA(0x613): 0x%x\n", ((struct dmrx_cba *) DM_RXCB)->cbase);
		printk("DMRX_CFG(0x614): \n");
		printk("\tSEN: \t0x%x\n", ((struct dmrx_cfg *) DM_RXCFG)->sen);
		printk("\tTRLPG: \t0x%x\n", ((struct dmrx_cfg *) DM_RXCFG)->trlpg);
		printk("\tHDLEN:\t0x%x\n", ((struct dmrx_cfg *) DM_RXCFG)->hdlen);
		printk("\tENDIAN:\t0x%x\n", ((struct dmrx_cfg *) DM_RXCFG)->endian);
		printk("\tPSIZE:\t0x%x\n", ((struct dmrx_cfg *) DM_RXCFG)->psize);
		printk("\tPNUM:\t0x%x\n", ((struct dmrx_cfg *) DM_RXCFG)->pnum);
		printk("DMRX_PGCNT(0x615): \n");
		printk("\tPGPTR: \t0x%x\n", ((struct dmrx_pgcnt *) DM_RXPGCNT)->pgptr);
		printk("\tDSRC: \t0x%x\n", ((struct dmrx_pgcnt *) DM_RXPGCNT)->dsrc);
		printk("\tDVAL: \t0x%x\n", ((struct dmrx_pgcnt *) DM_RXPGCNT)->dval);
		printk("\tDCMD: \t0x%x\n", ((struct dmrx_pgcnt *) DM_RXPGCNT)->dcmd);
		printk("\tUPAGE: \t0x%x\n", ((struct dmrx_pgcnt *) DM_RXPGCNT)->upage);
		printk("DMRX_PKTCNT(0x616): \n");
		printk("\tDSRC: \t0x%x\n", ((struct dmrx_pktcnt *) DM_RXPKTCNT)->dsrc);
		printk("\tDCMD: \t0x%x\n", ((struct dmrx_pktcnt *) DM_RXPKTCNT)->dcmd);
		printk("\tUPKT: \t0x%x\n", ((struct dmrx_pktcnt *) DM_RXPKTCNT)->upkt);

	} else if (ifx_strincmp(tokens[0], "dsrx", strlen(tokens[0])) == 0) {
		printk("DSRX_DB(0x710): 0x%x\n", ((struct dsrx_dba *) DS_RXDB)->dbase);
		printk("DSRX_DB(0x711): 0x%x\n", ((struct dsrx_cba *) DS_RXCB)->cbase);
		printk("DSRX_CFG(0x712): \n");
		printk("\tDREN: \t0x%x\n", ((struct dsrx_cfg *) DS_RXCFG)->dren);
		printk("\tENDIAN:\t0x%x\n", ((struct dsrx_cfg *) DS_RXCFG)->endian);
		printk("\tPSIZE:\t0x%x\n", ((struct dsrx_cfg *) DS_RXCFG)->psize);
		printk("\tPNUM:\t0x%x\n", ((struct dsrx_cfg *) DS_RXCFG)->pnum);
		printk("DSRX_PGCNT(0x713): \n");
		printk("\tPGPTR: \t0x%x\n", ((struct dsrx_pgcnt*) DS_RXPGCNT)->pgptr);
		printk("\tISRC: \t0x%x\n", ((struct dsrx_pgcnt*) DS_RXPGCNT)->isrc);
		printk("\tIVAL: \t0x%x\n", ((struct dsrx_pgcnt*) DS_RXPGCNT)->ival);
		printk("\tICMD: \t0x%x\n", ((struct dsrx_pgcnt*) DS_RXPGCNT)->icmd);
		printk("\tVPAGE: \t0x%x\n", ((struct dsrx_pgcnt*) DS_RXPGCNT)->upage);

	} else if (ifx_strincmp(tokens[0], "sll", strlen(tokens[0])) == 0) {
		unsigned int keynum = ((struct sll_cmd1 *) SLL_CMD1)->ksize;

		printk("SLL_CMD0(0x901):\n");
		printk("\tCMD: \t0x%x\n", ((struct sll_cmd0*) SLL_CMD0)->cmd);
		printk("\tENUM: \t0x%x\n", ((struct sll_cmd0*) SLL_CMD0)->eynum);
		printk("\tEYBASE: \t0x%x\n", ((struct sll_cmd0*) SLL_CMD0)->eybase);
		printk("Key size: 0x%x\n", keynum);
		for (i = 0; i < keynum; i++) {
			printk("Key[%d]:    0x%x\n", i, *(uint32_t*) SLL_KEY(i));
		}
		printk("SLL_RESULT(0x920) \n");
		printk("\tVLD: \t0x%x\n", ((struct sll_result*) SLL_RESULT)->vld);
		printk("\tFOUND: \t0x%x\n", ((struct sll_result*) SLL_RESULT)->fo);
		printk("\tINDEX: \t0x%x\n", ((struct sll_result*) SLL_RESULT)->index);
	} else {
		fw_dbg_start("help dump_rg");
	}

	return;
}

static void fwdbg_route(char **tokens, int token_num) {
	unsigned long addr, *paddr;
	int i;
	volatile struct rout_forward_action_tbl *action_tbl = NULL;
	int val[16], len;

	if (!token_num) {
		goto print_help;
	}

	if (ifx_strincmp(tokens[0], "help", strlen(tokens[0])) == 0) {
		goto print_help;;
	} else {
		addr = simple_strtoul(tokens[0], NULL, 16);
		if (!addr) {
			goto print_help;
		}
		paddr = (unsigned long *) sb_addr_to_fpi_addr_convert(addr);
		action_tbl = (struct rout_forward_action_tbl*) paddr;
	}

	for (i = 1; i < token_num - 1; i += 2) {
		if (ifx_strincmp(tokens[i], "newMAC", strlen(tokens[i])) == 0) {
			len = strlen(tokens[i + 1]);
			get_mac(&tokens[i + 1], &len, val);
//                      printk("new MAC: %02X.%02X.%02X.%02X:%02X:%02X\n", val[0], val[1], val[2], val[3], val[4], val[5]);
			action_tbl->new_dest_mac54 = (val[0] << 8) | val[1];
			action_tbl->new_dest_mac30 = (val[2] << 24) | (val[3] << 16)
					| (val[4] << 8) | val[5];

		} else if (ifx_strincmp(tokens[i], "newip", strlen(tokens[i])) == 0) {
			len = strlen(tokens[i + 1]);
			get_ip_port(&tokens[i + 1], &len, val);
//                          printk("new dest: %d.%d.%d.%d:%d\n", val[0], val[1], val[2], val[3], val[4]);
			action_tbl->new_ip = (val[0] << 24) | (val[1] << 16) | (val[2] << 8)
					| val[3];
			action_tbl->new_port = val[4];
		} else if (ifx_strincmp(tokens[i], "routetype", strlen(tokens[i])) == 0) {
			if (ifx_strincmp(tokens[i + 1], "NULL", strlen(tokens[i + 1])) == 0) {
				action_tbl->rout_type = 0;

			} else if (ifx_strincmp(tokens[i + 1], "IP", strlen(tokens[i + 1])) == 0
					|| ifx_strincmp(tokens[i + 1], "ROUTE", strlen(tokens[i + 1]))
							== 0) {
				action_tbl->rout_type = 1;

			}
			if (ifx_strincmp(tokens[i + 1], "NAT", strlen(tokens[i + 1])) == 0) {
				action_tbl->rout_type = 2;
			}
			if (ifx_strincmp(tokens[i + 1], "NAPT", strlen(tokens[i + 1])) == 0) {
				action_tbl->rout_type = 3;
			}
		} else if (ifx_strincmp(tokens[i], "newDSCP", strlen(tokens[i])) == 0) {
			action_tbl->new_dscp = simple_strtoul(tokens[i + 1], NULL, 0);
			//action_tbl->new_dscp_en = 1;

		} else if (ifx_strincmp(tokens[i], "MTUidx", strlen(tokens[i])) == 0) {
			action_tbl->mtu_ix = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "VLANins", strlen(tokens[i])) == 0) {
			action_tbl->in_vlan_ins = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "VLANrm", strlen(tokens[i])) == 0) {
			action_tbl->in_vlan_rm = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "newDSCPen", strlen(tokens[i])) == 0) {
			action_tbl->new_dscp_en = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "entry_vld", strlen(tokens[i])) == 0) {
			action_tbl->entry_vld = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "TCP", strlen(tokens[i])) == 0) {
			action_tbl->protocol = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "destlist", strlen(tokens[i])) == 0) {
			if (ifx_strincmp(tokens[i + 1], "ETH0", strlen(tokens[i + 1])) == 0) {
				action_tbl->dest_list |= 1 << 0;
			} else if (ifx_strincmp(tokens[i + 1], "ETH1", strlen(tokens[i + 1]))
					== 0) {
				action_tbl->dest_list |= 1 << 1;
			} else if (ifx_strincmp(tokens[i + 1], "CPU0", strlen(tokens[i + 1]))
					== 0) {
				action_tbl->dest_list |= 1 << 2;
			} else if (ifx_strincmp(tokens[i + 1], "EXT_INT1",
					strlen(tokens[i + 1])) == 0) {
				action_tbl->dest_list |= 1 << 3;
			} else if (ifx_strincmp(tokens[i + 1], "EXT_INT2",
					strlen(tokens[i + 1])) == 0) {
				action_tbl->dest_list |= 1 << 4;
			} else if (ifx_strincmp(tokens[i + 1], "EXT_INT3",
					strlen(tokens[i + 1])) == 0) {
				action_tbl->dest_list |= 1 << 5;
			} else if (ifx_strincmp(tokens[i + 1], "DSL_WAN", strlen(tokens[i + 1]))
					== 0) {
				action_tbl->dest_list |= 1 << 7;
			}
		} else if (ifx_strincmp(tokens[i], "PPPoEmode", strlen(tokens[i])) == 0) {
			action_tbl->pppoe_mode = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "PPPoEidx", strlen(tokens[i])) == 0) {
			action_tbl->pppoe_ix = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "newMACidx", strlen(tokens[i])) == 0) {
			action_tbl->new_src_mac_ix = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "VCI", strlen(tokens[i])) == 0) {
			action_tbl->new_in_vci = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "outvlanins", strlen(tokens[i])) == 0) {
			action_tbl->out_vlan_ins = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "outvlanrm", strlen(tokens[i])) == 0) {
			action_tbl->out_vlan_rm = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "outvlanidx", strlen(tokens[i])) == 0) {
			action_tbl->out_vlan_ix = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "mpoatype", strlen(tokens[i])) == 0) {
			action_tbl->mpoa_type = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "destqid", strlen(tokens[i])) == 0) {
			action_tbl->dest_qid = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "tunnelidx", strlen(tokens[i])) == 0) {
			action_tbl->tnnl_hdr_idx = simple_strtoul(tokens[i + 1], NULL, 0);

		} else if (ifx_strincmp(tokens[i], "tunnelen", strlen(tokens[i])) == 0) {
			action_tbl->encap_tunnel = simple_strtoul(tokens[i + 1], NULL, 0);

		}
	}

	return;

	print_help:

	printk("route address [name value] ... \n");
	printk("      newIP(PORT)       ??:??:??:??:????\n");
	printk("      newMAC:           ??:??:??:??:??:?? (HEX)\n");
	printk("      routetype:        NULL/IP/NAT/NAPT\n");
	printk("      newDSCP:          ??\n");
	printk("      newDSCPen:        0|1(disable|enable)\n");
	printk("      MTUidx:           ??\n");
	printk("      VLANins:          0|1(disable/enable)\n");
	printk("      VCI:              ???? (HEX)\n");
	printk("      VLANrm:           0|1(disable/enable)\n");
	printk("      outvlanins:       0|1(disable/enable)\n");
	printk("      outvlanidx:       ??\n");
	printk("      outvlanrm:        0|1(disable/enable)\n");
	printk("      destlist:         ETH0/ETH1/CPU0/EXT_INT/DSL_WAN?\n");
	printk("      PPPoEmode:        0|1(transparent/termination)\n");
	printk("      PPPoEidx:         ??\n");
	printk("      newMACidx:        ??\n");
	printk("      tcp:              0|1(no(UDP)/yes)\n");
	printk("      destqid:          ??\n");
	printk("      entryvld:         0|1(invalid/valid)\n");
	printk("      mpoatype:         ??\n");
	printk("      tunnelidx         ??\n");
	printk("      tunnelen          0|1(disable | enable)\n");

	return;
}

static void dump_ipv4_hdr(struct iphdr * iph) {
	ASSERT(iph != NULL, "ipv4 hdr point is NULL");

	printk("Version         :%d\n", iph->version);
	printk("Header len      :%d\n", iph->ihl);
	printk("TOS             :%d\n", iph->tos);
	printk("Total len       :%d\n", iph->tot_len);
	printk("Id              :%d\n", iph->id);
	printk("Flags              \n");
	printk("  Don't Fragment:%d\n", iph->frag_off & (1 << 14));
	printk("  More Fragments:%d\n", iph->frag_off & (1 << 13));
	printk("Fragment offset :%d\n", iph->frag_off & ~0x0700);
	printk("TTL             :%d\n", iph->ttl);
	printk("Protocol        :%d\n", iph->protocol);
	printk("Header checksum :%d\n", iph->check);
	printk("Src IP address  :%d.%d.%d.%d\n", iph->saddr >> 24,
			iph->saddr >> 16 & 0xFF, iph->saddr >> 8 & 0xFF, iph->saddr & 0xFF);
	printk("Dst IP address  :%d.%d.%d.%d\n", iph->daddr >> 24,
			iph->daddr >> 16 & 0xFF, iph->daddr >> 8 & 0xFF, iph->daddr & 0xFF);

	return;
}

static void dump_ipv6_hdr(struct ipv6hdr *iph6) {
	int i;
	ASSERT(iph6 != NULL, "ipv6 hdr point is NULL");

	printk("Version         :%d\n", iph6->version);
	printk("Traffic clase   :%d\n",
			iph6->priority << 4 | (iph6->flow_lbl[0] & 0xFF00));
	printk(
			"Flow label      :%d\n",
			(iph6->flow_lbl[0] & 0xFF) << 16 | (iph6->flow_lbl[1] << 8)
					| iph6->flow_lbl[2]);
	printk("Payload Len     :%d\n", iph6->payload_len);
	printk("Next Header     :%d\n", iph6->nexthdr);
	printk("Hop limit       :%d\n", iph6->hop_limit);
	printk("Src Address     \n");
	for (i = 0; i < 16; i++) {
		printk("%s%x", i == 0 ? "    " : i % 4 == 0 ? "." : "",
				(unsigned int) iph6->saddr.s6_addr + i);
	}
	printk("\n");
	printk("Dst Address     \n");
	for (i = 0; i < 16; i++) {
		printk("%s%x", i == 0 ? "    " : i % 4 == 0 ? "." : "",
				(unsigned int) iph6->daddr.s6_addr + i);
	}
	printk("\n");

	return;
}

static void fwdbg_dump_iphdr(char **tokens, int token_num) {
	unsigned long addr;
	unsigned long *paddr;

	if (token_num < 2) {
		goto print_help;
	}

	addr = simple_strtoul(tokens[1], NULL, 16);
	if (!addr) {
		goto print_help;
	}
	paddr = (unsigned long *) sb_addr_to_fpi_addr_convert(addr);

	if (ifx_strincmp(tokens[0], "ipv4", strlen(tokens[0])) == 0) {
		dump_ipv4_hdr((struct iphdr *) paddr);
	} else if (ifx_strincmp(tokens[0], "ipv6", strlen(tokens[0])) == 0) {
		dump_ipv6_hdr((struct ipv6hdr *) paddr);
	} else {
		goto print_help;
	}

	return;
	print_help: fw_dbg_start("help dump_ip_hdr");
	return;
}

static void fwdbg_help(char **tokens, int token_num) {
	const char *proc_file = "/proc/eth/tfwdbg";
	const char *cmds[] = { "help", "route", "read_ethbuf", "read_cb",
			"clear_cb", "clear_ethbuf", "dump_st_hdr", "dump_ip_hdr", "dump_rg",
			NULL, };

	int i;

	if (!token_num) { //print commands only
		for (i = 0; cmds[i] != NULL; i++) {
			printk(cmds[i]);
			printk("\t");
			if (i % 3 == 0)
				printk("\n");
		}

		printk("\n\n");
		printk("echo help cmd > %s for details\n", proc_file);
		return;
	}

	if (ifx_strincmp(tokens[0], "read_ethbuf", strlen(tokens[0])) == 0) {
		printk("echo read_ethbuf [start_pg] [print_pg_num] > %s \n", proc_file);
		printk("   :clear eth rx data and control buffer  \n");

	} else if (ifx_strincmp(tokens[0], "read_cb", strlen(tokens[0])) == 0) {
		printk(
				"echo read_rx_cb [rx|tx] [bc0|bc1] [start_pg] [print_pg_num] > %s \n",
				proc_file);
		printk(
				"   :read rx or tx (default is both rx & tx, bare channel 0 & 1) cell/codeword buffer for bc0 or bc1  \n");

	} else if (ifx_strincmp(tokens[0], "clear_cb", strlen(tokens[0])) == 0) {
		printk(
				"echo clear_cb [rx|tx] [bc0|bc1] [start_pg] [clear_pg_num] > %s \n",
				proc_file);
		printk(
				"   :clear rx or tx (or both) cell/codeword buffer for bc0 or bc1  \n");

	} else if (ifx_strincmp(tokens[0], "clear_ethbuf", strlen(tokens[0])) == 0) {
		printk("echo clear_ethbuf [start_pg] [pg_num] > %s \n", proc_file);
		printk("   :clear eth rx data and control buffer  \n");

	} else if (ifx_strincmp(tokens[0], "dump_st_hdr", strlen(tokens[0])) == 0) {
		printk("echo dump_st_hdr start_pg(Hex) [iphdr]> %s \n", proc_file);
		printk(
				"   :dump status header of the packet in the provided page and dump ipv4/ipv6 header indicated by flag header  \n");

	} else if (ifx_strincmp(tokens[0], "dump_ip_hdr", strlen(tokens[0])) == 0) {
		printk(
				"echo dump_ip_hdr [ipv4 | ipv6] [share_buffer_address] [idx] > %s \n",
				proc_file);
		printk("   :dump ipv4 or ipv6 header in the given share buffer  \n");

	} else if (ifx_strincmp(tokens[0], "dump_rg", strlen(tokens[0])) == 0) {
		printk("echo dump_rg [dmrx | dsrx | sll] %s \n", proc_file);
		printk("   :dump given registers  \n");

	} else if (ifx_strincmp(tokens[0], "route", strlen(tokens[0])) == 0) {
		printk("echo route address [name value] %s \n", proc_file);
		printk("   :change route entry's contents \n");
		printk("   :echo route help for details \n");
	} else {
		fwdbg_help((char **) NULL, 0);
	}

}

static void fw_dbg_start(char *cmdbuf) {
	char * tokens[32];
	int finished, token_num;
	int i;

	fw_dbg_t cmds[] = { { "help", fwdbg_help }, { "read_ethbuf",
			fwdbg_read_ethbuf }, { "clear_ethbuf", fwdbg_clear_ethbuf }, {
			"dump_st_hdr", fwdbg_dump_sthdr }, { "dump_rg", fwdbg_dump_rg }, {
			"route", fwdbg_route }, { "dump_ip_hdr", fwdbg_dump_iphdr }, { NULL,
			NULL } };

	token_num = tokenize(cmdbuf, " \t", tokens, 32, &finished);
	if (token_num <= 0) {
		return;
	}

	for (i = 0; cmds[i].cmd != NULL; i++) {
		if (ifx_strincmp(tokens[0], cmds[i].cmd, strlen(tokens[0])) == 0) {
			cmds[i].pfunc(&tokens[1], token_num - 1);
			break;
		}
	}

	if (cmds[i].cmd == NULL) {
		fw_dbg_start("help");
	}

	return;
}
static int proc_write_fwdbg(struct file *file __attribute__((unused)), const char *buf, unsigned long count, void *data __attribute__((unused))) {
	// char common_proc_read_buff[2048];
	char *p;
	int len, rlen;

	len = count < sizeof(common_proc_read_buff) ? count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p) {
		// MOD_DEC_USE_COUNT;
		return 0;
	}

	ASSERT(p < common_proc_read_buff + 2048, "data point out of range");
	fw_dbg_start(p);

#if 0
	if ( ifx_strincmp(tokens[0], "help", 4) == 0) {

		printk("echo read_tx_cb[0|1] [start_pg] [print_pg_num] > /proc/eth/tdbg \n");
		printk("   :read tx cell/codeword buffer for bc0 or bc1  \n");

		printk("echo us_bg_ctxt > /proc/eth/tdbg \n");
		printk("   : US - display us_bg_ctxt\n");

		printk("echo ds_bg_ctxt > /proc/eth/tdbg \n");
		printk("   : DS - display ds_bg_ctxt\n");

		printk("echo ds_bg_mib > /proc/eth/tdbg \n");
		printk("   : DS - display ds_bg_mib\n");

		printk("echo ds_link_list > /proc/eth/tdbg \n");
		printk("   : DS - check link list\n");

		printk("echo rx_bc_cfg > /proc/eth/tdbg \n");
		printk("   : read RX bear channel config\n");

		printk("echo tx_bc_cfg > /proc/eth/tdbg \n");
		printk("   : read TX bear channel config\n");

		printk("echo rx_gif_cfg [0|1|2|3] > /proc/eth/tdbg \n");
		printk("   : read RX gamma interface config\n");

		printk("echo tx_bc_cfg [0|1|2|3] > /proc/eth/tdbg \n");
		printk("   : read TX gamma interface config\n");

		printk("echo idlekeep > /proc/eth/tdbg \n");
		printk("   : read idle keep of BC0 and BC1\n");

		printk("echo set idlekeep[0|1] > /proc/eth/tdbg \n");
		printk("   : set idle keep of BC0 or BC1\n");

		printk("echo clear idlekeep[0|1] > /proc/eth/tdbg \n");
		printk("   : clear idle keep of BC0 or BC1\n");

	}
	else if ( ifx_strincmp(tokens[0], "read_ethbuf", 10) == 0 ) {

		unsigned int start_pg;
		unsigned int print_pg_num;
		unsigned int num;
		unsigned int pg_size;
		volatile unsigned int *dbase, *cbase;
		unsigned int pnum, i;
		unsigned int *cw;

		dbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );
		cbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_cba * )DM_RXCB)->cbase + 0x2000);

		pnum = ( ( struct dmrx_cfg * )DM_RXCFG)->pnum;
		pg_size = ( ( ( struct dmrx_cfg * )DM_RXCFG)->psize + 1) * 32 / 4;

		start_pg = 0;
		print_pg_num = pnum;

		if (token_num >= 2)
		start_pg = simple_strtoul(tokens[1], NULL, 0);
		if (token_num >= 3)
		print_pg_num = simple_strtoul(tokens[2], NULL, 0);

		start_pg = start_pg % pnum;
		if(print_pg_num > pnum)
		print_pg_num = pnum;

		printk("PTM TX BC 0 CELL data/ctrl buffer:\n\n");
		for(i = start_pg, num = 0; num < print_pg_num; num ++, i = (i + 1) % pnum ) {

			struct ctrl_dmrx_2_fw * p_ctrl_dmrx;
			struct ctrl_fw_2_dsrx * p_ctrl_dsrx;
			int j = 0;

			cw = dbase + i * pg_size;

			p_ctrl_dmrx = (struct ctrl_dmrx_2_fw *) ( &cbase[i]);
			p_ctrl_dsrx = (struct ctrl_fw_2_dsrx *) ( &cbase[i]);

			for(j =0; j < pg_size; j=j+4) {

				if(j==0) {

					printk("Pg_id %2d: %08x %08x %08x %08x ", i, cw[0], cw[1], cw[2], cw[3]);
					printk("pg_val: %x, byte_off: %x, cos: %x \n", p_ctrl_dmrx->pg_val, p_ctrl_dmrx->byte_off, p_ctrl_dmrx->cos);
				}

				else if(j==4) {
					printk("          %08x %08x %08x %08x ", cw[j], cw[j+1], cw[j+2], cw[j+3]);
					printk("bytes_cnt: %x, eof: %x \n", p_ctrl_dmrx->bytes_cnt, p_ctrl_dmrx->eof);
				}

				else if(j==12) {
					printk("          %08x %08x %08x %08x ", cw[j], cw[j+1], cw[j+2], cw[j+3]);
					printk("pkt_status: %x, flag_len: %x, byte_off: %x, acc_sel: %x\n", p_ctrl_dsrx->pkt_status, p_ctrl_dsrx->flag_len, p_ctrl_dsrx->byte_off,p_ctrl_dsrx->acc_sel );
				}

				else if(j==16) {
					printk("          %08x %08x %08x %08x ", cw[j], cw[j+1], cw[j+2], cw[j+3]);
					printk("fcs_en: %x, cos: %x, bytes_cnt: %x, eof: %x\n", p_ctrl_dsrx->fcs_en, p_ctrl_dsrx->cos, p_ctrl_dsrx->bytes_cnt,p_ctrl_dsrx->eof );
				}

				else {
					printk("          %08x %08x %08x %08x \n", cw[j], cw[j+1], cw[j+2], cw[j+3]);
				}

			}

			printk("\n");

		}

	}

	else if ( ifx_strincmp(tokens[0], "read_tx_cb", 10) == 0 ) {

		unsigned int start_pg;
		unsigned int print_pg_num;
		unsigned int num;

		if(tokens[0][10] == '0' || tokens[0][10] == '\0') {

			volatile unsigned int *dbase0;
			unsigned int pnum0, i;
			unsigned int * cw;

			dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000);
			pnum0 = FFSM_CFG(0)->pnum;

			start_pg = 0;
			print_pg_num = pnum0;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			print_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum0;
			if(print_pg_num > pnum0)
			print_pg_num = pnum0;

			printk("PTM TX BC 0 CELL data/ctrl buffer:\n\n");
			for(i = start_pg, num = 0; num < print_pg_num; num ++, i = (i + 1) % pnum0 ) {

				cw = dbase0 + i * cw_pg_size;
				printk("CW %2d: %08x \n" , i, cw[0]);
				printk("       %08x %08x %08x %08x\n", cw[1], cw[2], cw[3], cw[4]);
				printk("       %08x %08x %08x %08x\n", cw[5], cw[6], cw[7], cw[8]);
				printk("       %08x %08x %08x %08x\n", cw[9], cw[10], cw[11], cw[12]);
				printk("       %08x %08x %08x %08x\n", cw[13], cw[14], cw[15], cw[16]);
			}

		}

		if (tokens[0][10] == '1' || tokens[0][10] == '\0') {

			volatile unsigned int *dbase1;
			unsigned int pnum1, i;
			unsigned int * cw;

			dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000);
			pnum1 = FFSM_CFG(1)->pnum;

			start_pg = 0;
			print_pg_num = pnum1;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			print_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum1;
			if(print_pg_num > pnum1)
			print_pg_num = pnum1;

			printk("PTM TX BC 1 CELL data/ctrl buffer:\n\n");
			for(i = start_pg, num = 0; num < print_pg_num; num ++, i = (i + 1) % pnum1 ) {

				cw = dbase1 + i * cw_pg_size;
				printk("CW %2d: %08x \n" , i, cw[0]);
				printk("       %08x %08x %08x %08x\n", cw[1], cw[2], cw[3], cw[4]);
				printk("       %08x %08x %08x %08x\n", cw[5], cw[6], cw[7], cw[8]);
				printk("       %08x %08x %08x %08x\n", cw[9], cw[10], cw[11], cw[12]);
				printk("       %08x %08x %08x %08x\n\n", cw[13], cw[14], cw[15], cw[16]);
			}

		}

	}

	else if ( ifx_strincmp(tokens[0], "read_rx_cb", 10) == 0 ) {

		unsigned int start_pg;
		unsigned int print_pg_num;
		unsigned int num;

		unsigned char * cwid_txt[8] = {
			"All_data",
			"End_of_frame",
			"Start_while_tx",
			"All_idle",
			"Start_while_idle",
			"All_idle_nosync",
			"Error",
			"Res"
		};

		if(tokens[0][10] == '0' || tokens[0][10] == '\0') {

			volatile unsigned int *dbase0, *cbase0;
			unsigned int pnum0, i;
			unsigned int * cw;

			dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000);
			cbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000);

			pnum0 = SFSM_CFG(0)->pnum;

			start_pg = 0;
			print_pg_num = pnum0;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			print_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum0;
			if(print_pg_num > pnum0)
			print_pg_num = pnum0;

			printk("PTM RX BC 0 CELL data/ctrl buffer (pnum0 = %d):\n\n", pnum0);
			for(i = start_pg, num = 0; num < print_pg_num; num ++, i = (i + 1) % pnum0 ) {

				struct PTM_CW_CTRL * p_ctrl;

				cw = dbase0 + i * cw_pg_size;
				p_ctrl = (struct PTM_CW_CTRL *) ( &cbase0[i]);

				printk("CW %2d: %08x                            ", i, cw[0]);
				printk("cwid=%d [%-16s], cwer=%d, preempt=%d, short=%d\n",
						p_ctrl->cwid, cwid_txt[p_ctrl->cwid], p_ctrl->cwer, p_ctrl->preempt, p_ctrl->shrt);

				printk("       %08x %08x %08x %08x ", cw[1], cw[2], cw[3], cw[4]);
				printk("state=%d, bad=%d, ber=%-3d, spos=%-3d, ffbn=%-3d\n",
						p_ctrl->state, p_ctrl->bad, p_ctrl->ber, p_ctrl->spos, p_ctrl->ffbn);

				printk("       %08x %08x %08x %08x\n", cw[5], cw[6], cw[7], cw[8]);
				printk("       %08x %08x %08x %08x\n", cw[9], cw[10], cw[11], cw[12]);
				printk("       %08x %08x %08x %08x\n", cw[13], cw[14], cw[15], cw[16]);

			}

		}

		if (tokens[0][10] == '1' || tokens[0][10] == '\0') {

			volatile unsigned int *dbase1, *cbase1;
			unsigned int pnum1, i;
			unsigned int * cw;

			dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000);
			cbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000);

			pnum1 = SFSM_CFG(1)->pnum;

			start_pg = 0;
			print_pg_num = pnum1;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			print_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum1;
			if(print_pg_num > pnum1)
			print_pg_num = pnum1;

			printk("PTM RX BC 1 CELL data/ctrl buffer:\n\n");
			for(i = start_pg, num = 0; num < print_pg_num; num ++, i = (i + 1) % pnum1 ) {

				struct PTM_CW_CTRL * p_ctrl;

				cw = dbase1 + i * cw_pg_size;
				p_ctrl = (struct PTM_CW_CTRL *) ( &cbase1[i]);

				printk("CW %2d: %08x                            ", i, cw[0]);
				printk("cwid=%d [%16s], cwer=%d, preempt=%d, short=%d\n",
						p_ctrl->cwid, cwid_txt[p_ctrl->cwid], p_ctrl->cwer, p_ctrl->preempt, p_ctrl->shrt);

				printk("       %08x %08x %08x %08x ", cw[1], cw[2], cw[3], cw[4]);
				printk("state=%d, bad=%d, ber=%-3d, spos=%-3d, ffbn=%-3d\n",
						p_ctrl->state, p_ctrl->bad, p_ctrl->ber, p_ctrl->spos, p_ctrl->ffbn);

				printk("       %08x %08x %08x %08x\n", cw[5], cw[6], cw[7], cw[8]);
				printk("       %08x %08x %08x %08x\n", cw[9], cw[10], cw[11], cw[12]);
				printk("       %08x %08x %08x %08x\n", cw[13], cw[14], cw[15], cw[16]);

			}

		}
	}
	else if ( ifx_strincmp(tokens[0], "clear_tx_cb", 11) == 0 ) {

		unsigned int start_pg;
		unsigned int clear_pg_num;
		unsigned int num;

		if(tokens[0][11] == '0' || tokens[0][11] == '\0') {
			volatile unsigned int *dbase0;
			unsigned int pnum0;

			dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000);
			pnum0 = FFSM_CFG(0)->pnum;

			start_pg = 0;
			clear_pg_num = pnum0;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum0;
			if(clear_pg_num > pnum0)
			clear_pg_num = pnum0;

			dbase0 = dbase0 + start_pg * cw_pg_size;

			memset(dbase0, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

		}

		if (tokens[0][11] == '1' || tokens[0][11] == '\0') {

			volatile unsigned int *dbase1;
			unsigned int pnum1;

			dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000);
			pnum1 = FFSM_CFG(1)->pnum;

			start_pg = 0;
			clear_pg_num = pnum1;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum1;
			if(clear_pg_num > pnum1)
			clear_pg_num = pnum1;

			dbase1 = dbase1 + start_pg * cw_pg_size;

			memset(dbase1, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

		}
	}

	else if ( ifx_strincmp(tokens[0], "clear_rx_cb", 11) == 0 ) {

		unsigned int start_pg;
		unsigned int clear_pg_num;

		if(tokens[0][11] == '0' || tokens[0][11] == '\0') {

			volatile unsigned int *dbase0, *cbase0;
			unsigned int pnum0;

			dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000);
			cbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000);

			pnum0 = SFSM_CFG(0)->pnum;

			start_pg = 0;
			clear_pg_num = pnum0;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum0;
			if(clear_pg_num > pnum0)
			clear_pg_num = pnum0;

			dbase0 = dbase0 + start_pg * cw_pg_size;
			cbase0 = &cbase0[start_pg];

			memset(dbase0, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);
			memset(cbase0, 0, sizeof(unsigned int ) * clear_pg_num);

		}

		if (tokens[0][11] == '1' || tokens[0][11] == '\0') {

			volatile unsigned int *dbase1, *cbase1;
			unsigned int pnum1;

			dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000);
			cbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000);
			pnum1 = SFSM_CFG(1)->pnum;

			start_pg = 0;
			clear_pg_num = pnum1;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % pnum1;
			if(clear_pg_num > pnum1)
			clear_pg_num = pnum1;

			dbase1 = dbase1 + start_pg * cw_pg_size;
			cbase1 = &cbase1[start_pg];

			memset(dbase1, 0, cw_pg_size * sizeof(unsigned int )* clear_pg_num);
			memset(cbase1, 0, sizeof(unsigned int ) * clear_pg_num);
		}
	}

	else if ( ifx_strincmp(tokens[0], "clear_cb", 8) == 0 ) {

		unsigned int start_pg;
		unsigned int clear_pg_num;

		if(tokens[0][8] == '0' || tokens[0][8] == '\0') {

			//Clear tx_cb0
			volatile unsigned int *tx_dbase0;
			unsigned int tx_pnum0;
			volatile unsigned int *rx_dbase0, *rx_cbase0;
			unsigned int rx_pnum0;

			tx_dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000);
			tx_pnum0 = FFSM_CFG(0)->pnum;

			start_pg = 0;
			clear_pg_num = tx_pnum0;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % tx_pnum0;

			if(clear_pg_num > tx_pnum0)
			clear_pg_num = tx_pnum0;

			tx_dbase0 = tx_dbase0 + start_pg * cw_pg_size;

			memset(tx_dbase0, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

			//Clear rx_cb0

			rx_dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000);
			rx_cbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000);

			rx_pnum0 = SFSM_CFG(0)->pnum;

			start_pg = 0;
			clear_pg_num = rx_pnum0;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % rx_pnum0;
			if(clear_pg_num > rx_pnum0)
			clear_pg_num = rx_pnum0;

			rx_dbase0 = rx_dbase0 + start_pg * cw_pg_size;
			rx_cbase0 = &rx_cbase0[start_pg];

			memset(rx_dbase0, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);
			memset(rx_cbase0, 0, sizeof(unsigned int ) * clear_pg_num);

		}

		if(tokens[0][8] == '1' || tokens[0][8] == '\0') {

			//Clear tx_cb1
			volatile unsigned int *tx_dbase1;
			unsigned int tx_pnum1;
			volatile unsigned int *rx_dbase1, *rx_cbase1;
			unsigned int rx_pnum1;

			tx_dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000);
			tx_pnum1 = FFSM_CFG(1)->pnum;

			start_pg = 0;
			clear_pg_num = tx_pnum1;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % tx_pnum1;

			if(clear_pg_num > tx_pnum1)
			clear_pg_num = tx_pnum1;

			tx_dbase1 = tx_dbase1 + start_pg * cw_pg_size;

			memset(tx_dbase1, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

			rx_dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000);
			rx_cbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000);
			rx_pnum1 = SFSM_CFG(1)->pnum;

			start_pg = 0;
			clear_pg_num = rx_pnum1;

			if (token_num >= 2)
			start_pg = simple_strtoul(tokens[1], NULL, 0);
			if (token_num >= 3)
			clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

			start_pg = start_pg % rx_pnum1;
			if(clear_pg_num > rx_pnum1)
			clear_pg_num = rx_pnum1;

			rx_dbase1 = rx_dbase1 + start_pg * cw_pg_size;
			rx_cbase1 = &rx_cbase1[start_pg];

			memset(rx_dbase1, 0, cw_pg_size * sizeof(unsigned int ) * clear_pg_num);
			memset(rx_cbase1, 0, sizeof(unsigned int )* clear_pg_num);

		}

	}

	else if ( ifx_strincmp(tokens[0], "clear_eth_dbuf", 14) == 0) {
		volatile u32 *p_dbase;
		unsigned int pg_size = 0;
		unsigned int pg_num = 0;
		unsigned int start_pg;
		unsigned int clear_pg_num;

		pg_num = ( ( struct dmrx_cfg * )DM_RXCFG)->pnum;
		pg_size = ( ( ( struct dmrx_cfg * )DM_RXCFG)->psize + 1) * 32 / 4;

		p_dbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );

		start_pg = 0;
		clear_pg_num = pg_num;

		if (token_num >= 2)
		start_pg = simple_strtoul(tokens[1], NULL, 0);
		if (token_num >= 3)
		clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

		start_pg = start_pg % pg_num;
		if(clear_pg_num > pg_num)
		clear_pg_num = pg_num;

		p_dbase = p_dbase + start_pg * pg_size;

		memset(p_dbase, 0, pg_size * sizeof(unsigned int) * clear_pg_num);

	}
	else if ( ifx_strincmp(tokens[0], "clear_eth_cbuf", 14) == 0) {
		volatile u32 *p_cbase;
		unsigned int pg_num = 0;
		unsigned int start_pg;
		unsigned int clear_pg_num;

		pg_num = ( ( struct dmrx_cfg * )DM_RXCFG )->pnum;
		p_cbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_cba * )DM_RXCB)->cbase + 0x2000);

		start_pg = 0;
		clear_pg_num = pg_num;

		if (token_num >= 2)
		start_pg = simple_strtoul(tokens[1], NULL, 0);
		if (token_num >= 3)
		clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

		start_pg = start_pg % pg_num;
		if(clear_pg_num > pg_num)
		clear_pg_num = pg_num;

		p_cbase = &p_cbase[start_pg];

		memset(p_cbase, 0, sizeof(unsigned int) * clear_pg_num );
	}

	else if ( ifx_strincmp(tokens[0], "clear_ethbuf", 12) == 0)
	{
		volatile u32 *p_dbase, *p_cbase;
		unsigned int pg_size = 0;
		unsigned int pg_num = 0;
		unsigned int start_pg;
		unsigned int clear_pg_num;

		//Clear ethernet data buffer
		pg_num = ( ( struct dmrx_cfg * )DM_RXCFG)->pnum;
		pg_size = ( ( ( struct dmrx_cfg * )DM_RXCFG)->psize + 1) * 32 / 4;

		p_dbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase +0x2000);

		start_pg = 0;
		clear_pg_num = pg_num;

		if (token_num >= 2)
		start_pg = simple_strtoul(tokens[1], NULL, 0);
		if (token_num >= 3)
		clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

		start_pg = start_pg % pg_num;
		if(clear_pg_num > pg_num)
		clear_pg_num = pg_num;

		p_dbase = p_dbase + start_pg * pg_size;

		memset(p_dbase, 0, pg_size * sizeof(unsigned int) * clear_pg_num);

		//Clear ethernet control buffer
		pg_num = ( ( struct dmrx_cfg * )DM_RXCFG )->pnum;
		p_cbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_cba * )DM_RXCB)->cbase + 0x2000 );

		start_pg = 0;
		clear_pg_num = pg_num;

		if (token_num >= 2)
		start_pg = simple_strtoul(tokens[1], NULL, 0);
		if (token_num >= 3)
		clear_pg_num = simple_strtoul(tokens[2], NULL, 0);

		start_pg = start_pg % pg_num;
		if(clear_pg_num > pg_num)
		clear_pg_num = pg_num;

		p_cbase = &p_cbase[start_pg];

		memset(p_cbase, 0, sizeof(unsigned int) * clear_pg_num );

	}

	else if ( ifx_strincmp(tokens[0], "us_bg_ctxt", 10) == 0 ) {
		int k = 0;
		int pkt_status = 0;
		int des_addr = 0;
		int des_index = 0;
		int q_no = 0;

		for(; k<4; k++) {
			printk( "US_BG_CTXT[%d]\n", k);
			printk( "         sid: 0x%04x     sop: %x      eop: %x  pkt_status: %x    des_addr: 0x%04x \n",
					US_BG_CTXT_BASE(k)->sid, US_BG_CTXT_BASE(k)->sop, US_BG_CTXT_BASE(k)->eop,
					US_BG_CTXT_BASE(k)->pkt_status, US_BG_CTXT_BASE(k)->des_addr);
			printk( "    data_ptr: 0x%08x qid: %x   gif_id: %x     rem_len: %-4u \n\n",
					US_BG_CTXT_BASE(k)->data_ptr, US_BG_CTXT_BASE(k)->qid, US_BG_CTXT_BASE(k)->gif_id,
					US_BG_CTXT_BASE(k)->rem_len);

			pkt_status = US_BG_CTXT_BASE(k)->pkt_status;

			if(pkt_status == 1) {
				des_addr = US_BG_CTXT_BASE(k)->des_addr;
				des_index = des_addr - 0x5c00;

				while((des_index-0x80)>=0)
				{
					des_index-=0x80;
					q_no ++;
				}

				printk("    ==>US_BOND_PKT_DES_BASE[%d]\n", q_no);
				printk("              own: %x         c: %x        sop: %x           eop: %x           byte_off: %d \n",
						US_BOND_PKT_DES_BASE(q_no)->own,US_BOND_PKT_DES_BASE(q_no)->c,
						US_BOND_PKT_DES_BASE(q_no)->sop,US_BOND_PKT_DES_BASE(q_no)->eop,
						US_BOND_PKT_DES_BASE(q_no)->byte_off);
				printk("        last_frag: %x  frag_num: %-2d  data_len: %-4d   data_ptr: 0x%08x\n\n",
						US_BOND_PKT_DES_BASE(q_no)->last_frag,US_BOND_PKT_DES_BASE(q_no)->frag_num,
						US_BOND_PKT_DES_BASE(q_no)->data_len,US_BOND_PKT_DES_BASE(q_no)->data_ptr);

			}

		}
	}
	else if ( ifx_strincmp(tokens[0], "ds_bg_ctxt", 10) == 0 ) {
		int k = 0;
		for(; k<4; k++) {
			printk( "DS_BG_CTXT[%d]\n", k);
			printk( "       link_state_chg: %u        expected_sid: 0x%04x     last_sop: %u      last_eop: %u\t  bg_pkt_state: %u\n",
					DS_BG_CTXT_BASE(k)->link_state_chg, DS_BG_CTXT_BASE(k)->expected_sid, DS_BG_CTXT_BASE(k)->last_sop,
					DS_BG_CTXT_BASE(k)->last_eop, DS_BG_CTXT_BASE(k)->bg_pkt_state);
			printk( "        overfrag_flag: %u       oversize_flag: %u       no_eop_flag: %u   no_sop_flag: %u\t   no_err_flag: %u \n",
					DS_BG_CTXT_BASE(k)->overfrag_flag, DS_BG_CTXT_BASE(k)->oversize_flag, DS_BG_CTXT_BASE(k)->no_eop_flag,
					DS_BG_CTXT_BASE(k)->no_sop_flag, DS_BG_CTXT_BASE(k)->no_err_flag);
			printk( "    curr_pkt_frag_cnt: %-3d curr_pkt_byte_cnt: %-4u \n\n",
					DS_BG_CTXT_BASE(k)->curr_pkt_frag_cnt, DS_BG_CTXT_BASE(k)->curr_pkt_byte_cnt);
		}
	}
	else if ( ifx_strincmp(tokens[0], "ds_bond_gif_mib", 15) == 0 ) {
		int k = 0;
		for(; k<8; k++) {
			printk( "DS_BOND_GIF_MIB[%d]\n", k);
			printk( "       total_rx_frag_cnt: %-10u total_rx_byte_cnt: %-10u overflow_frag_cnt: %-10u overflow_byte_cnt: %-10u\n",
					DS_BOND_GIF_MIB_BASE(k)->total_rx_frag_cnt, DS_BOND_GIF_MIB_BASE(k)->total_rx_byte_cnt,
					DS_BOND_GIF_MIB_BASE(k)->overflow_frag_cnt, DS_BOND_GIF_MIB_BASE(k)->overflow_byte_cnt);
			printk( "   out_of_range_frag_cnt: %-10u  missing_frag_cnt: %-10u  timeout_frag_cnt: %-10u\n\n",
					DS_BOND_GIF_MIB_BASE(k)->out_of_range_frag_cnt, DS_BOND_GIF_MIB_BASE(k)->missing_frag_cnt,
					DS_BOND_GIF_MIB_BASE(k)->timeout_frag_cnt);
		}
	}
	else if ( ifx_strincmp(tokens[0], "ds_bg_mib", 9) == 0 ) {
		int k = 0;
		for(; k<4; k++) {
			printk( "DS_BG_MIB[%d]\n", k);
			printk( "     conform_pkt_cnt: %-10u  conform_frag_cnt: %-10u conform_byte_cnt: %-10u    no_sop_pkt_cnt: %-10u   no_sop_frag_cnt: %-10u\n",
					DS_BG_MIB_BASE(k)->conform_pkt_cnt, DS_BG_MIB_BASE(k)->conform_frag_cnt,
					DS_BG_MIB_BASE(k)->conform_byte_cnt, DS_BG_MIB_BASE(k)->no_sop_pkt_cnt, DS_BG_MIB_BASE(k)->no_sop_frag_cnt);
			printk( "     no_sop_byte_cnt: %-10u    no_eop_pkt_cnt: %-10u  no_eop_frag_cnt: %-10u   no_eop_byte_cnt: %-10u  oversize_pkt_cnt: %-10u\n",
					DS_BG_MIB_BASE(k)->no_sop_byte_cnt, DS_BG_MIB_BASE(k)->no_eop_pkt_cnt, DS_BG_MIB_BASE(k)->no_eop_frag_cnt,
					DS_BG_MIB_BASE(k)->no_eop_byte_cnt, DS_BG_MIB_BASE(k)->oversize_pkt_cnt);
			printk( "   oversize_frag_cnt: %-10u oversize_byte_cnt: %-10u overfrag_pkt_cnt: %-10u overfrag_frag_cnt: %-10u overfrag_byte_cnt: %-10u\n\n",
					DS_BG_MIB_BASE(k)->oversize_frag_cnt, DS_BG_MIB_BASE(k)->oversize_byte_cnt, DS_BG_MIB_BASE(k)->overfrag_pkt_cnt,
					DS_BG_MIB_BASE(k)-> overfrag_frag_cnt, DS_BG_MIB_BASE(k)->overfrag_byte_cnt);
		}
	}
	else if ( ifx_strincmp(tokens[0], "ds_link_list", 12) == 0 ) {
		unsigned int i;
		unsigned char buf[8];
		unsigned int total_des_num = 0;

		check_ds_ll_list(DS_BOND_FREE_LL_CTXT_BASE, "Free: ");
		for( i = 0; i < 8; i ++ ) {
			sprintf(buf, "Gif%d: ", i);
			check_ds_ll_list(DS_BOND_GIF_LL_CTXT_BASE(i), buf);
		}

		for(i = 0; i < 4; i ++) {
			sprintf(buf, "BgR%d: ", i);
			check_ds_ll_list( (volatile struct DS_BOND_GIF_LL_CTXT *)
					( (unsigned int *) (DS_BG_CTXT_BASE(i)) + 4 ), buf);
		}

		total_des_num = DS_BOND_FREE_LL_CTXT_BASE->des_num;
		for( i = 0; i < 8; i ++ ) {
			total_des_num += DS_BOND_GIF_LL_CTXT_BASE(i)->des_num;
		}
		for(i = 0; i < 4; i ++) {
			total_des_num += ( (volatile struct DS_BOND_GIF_LL_CTXT *)
					( (unsigned int *) (DS_BG_CTXT_BASE(i)) + 4 )) ->des_num;
		}

		if (total_des_num != 256) {
			printk("[X] ");
		}
		else {
			printk("[Y] ");
		}
		printk("total_des_num = %d\n\n", total_des_num);
	}
	else if ( ifx_strincmp(tokens[0], "rx_bc_cfg", 9) == 0 ) {

		unsigned int bc_s = 0;
		unsigned int bc_e = 1;
		int i = 0;
		unsigned char * ls_txt[] = {
			"Looking",
			"Freewheel Sync False",
			"Synced",
			"Freewheel Sync True"
		};
		unsigned char * rs_txt[] = {
			"Out-of-Sync",
			"Synced"
		};

		if(tokens[0][10] == '\0' && token_num == 1) {
			bc_s = 0;
			bc_e = 1;
		}
		else if ( tokens[0][10] <= '1' && tokens[0][10] >= '0' ) {
			bc_s = bc_e = tokens[0][10] - '0';
		}
		else if ( token_num >= 2 && tokens[1][0] <= '1' && tokens[1][0] >= '0' ) {
			bc_s = bc_e = tokens[1][0] - '0';
		}

		for ( i = bc_s; i <= bc_e; i ++ ) {
			printk("TX_GIF_CFG[%d]\n", i);
			printk("    local_state: %d [ %-20s ] remote_state: %d [ %-12s ]  rx_cw_rdptr:%d \n\n",
					RX_BC_CFG(i)->local_state, ls_txt[RX_BC_CFG(i)->local_state],
					RX_BC_CFG(i)->remote_state, rs_txt[RX_BC_CFG(i)->remote_state],
					RX_BC_CFG(i)->rx_cw_rdptr);
		}

	}
	else if ( ifx_strincmp(tokens[0], "tx_bc_cfg", 9) == 0 ) {
	}
	else if ( ifx_strincmp(tokens[0], "rx_gif_cfg", 10) == 0 ) {
	}
	else if ( ifx_strincmp(tokens[0], "tx_gif_cfg", 10) == 0 ) {
		unsigned int gif_s = 0;
		unsigned int gif_e = 0;
		int i = 0;
		if(tokens[0][10] == '\0' && token_num == 1) {
			gif_s = 0;
			gif_e = 3;
		}
		else if ( tokens[0][10] <= '3' && tokens[0][10] >= '0' ) {
			gif_s = gif_e = tokens[0][10] - '0';
		}
		else if ( token_num >= 2 && tokens[1][0] <= '3' && tokens[1][0] >= '0' ) {
			gif_s = gif_e = tokens[1][0] - '0';
		}

		for ( i = gif_s; i <= gif_e; i ++ ) {
			printk("TX_GIF_CFG[%d]\n", i);
			printk("    curr_qid: %d     fill_pkt_state: %d      post_pkt_state: %d  curr_pdma_context_ptr: 0x%04x\n"
					"     des_qid: %d           des_addr: 0x%04x                     curr_sar_context_ptr: 0x%04x\n"
					"    rem_data: %-4d         rem_crc: %d          rem_fh_len: %d\n"
					"     des_dw0: 0x%08x   des_dw1: 0x%08x  des_bp_dw: 0x%08x\n\n",
					TX_GAMMA_ITF_CFG(i)->curr_qid, TX_GAMMA_ITF_CFG(i)->fill_pkt_state, TX_GAMMA_ITF_CFG(i)->post_pkt_state, TX_GAMMA_ITF_CFG(i)->curr_pdma_context_ptr,
					TX_GAMMA_ITF_CFG(i)->des_qid, TX_GAMMA_ITF_CFG(i)->des_addr, TX_GAMMA_ITF_CFG(i)->curr_sar_context_ptr,
					TX_GAMMA_ITF_CFG(i)->rem_data, TX_GAMMA_ITF_CFG(i)->rem_crc, TX_GAMMA_ITF_CFG(i)->rem_fh_len,
					TX_GAMMA_ITF_CFG(i)->des_dw0, TX_GAMMA_ITF_CFG(i)->des_dw1, TX_GAMMA_ITF_CFG(i)->des_bp_dw);

		}

	}
	else if ( ifx_strincmp(tokens[0], "idlekeep", 8) == 0 ) {
		printk("BC0.idlekp = %d, BC1.idlekp = %d\n",
				(*SFSM_CFG0 >> 15) & 1, (*SFSM_CFG1 >> 15) & 1);
	}
	else if ( ifx_strincmp(tokens[0], "clear", 5) == 0 ) {
		if(token_num >= 2) {
			if(ifx_strincmp(tokens[1], "idlekeep", 8) == 0) {
				if(tokens[1][8] == '\0' || tokens[1][8] == '0')
				*SFSM_CFG0 = ( *SFSM_CFG0 ) & ( ~ ( 1 << 15) );
				if(tokens[1][8] == '\0' || tokens[1][8] == '1')
				*SFSM_CFG1 = ( *SFSM_CFG1 ) & ( ~ ( 1 << 15) );
			}
		}
	}
	else if ( ifx_strincmp(tokens[0], "set", 3) == 0 ) {
	}

	// MOD_DEC_USE_COUNT;
#endif

	return count;
}

#endif

/*------------------------------------------------------------------------------------------*\
 * Complett neu seit UGW 5.1.1 ende
 \*------------------------------------------------------------------------------------------*/

#define RX_TOTAL_PCKNT                  0x1F
#define RX_UNICAST_PCKNT                0x23
#define RX_MULTICAST_PCKNT              0x22
#define RX_CRC_ERR_PCKNT                0x21
#define RX_UNDERSIZE_GOOD_PCKNT         0x1D
#define RX_OVER_SIZE_GOOD_PCKNT         0x1B
#define RX_UNDERSIZE_ERR_PCKNT          0x1E
#define RX_GOOD_PAUSE_PCKNT             0x20
#define RX_OVER_SIZE_ERR_PCKNT          0x1C
#define RX_ALLIGN_ERR_PCKNT             0x1A
#define RX_FILTERED_PCKNT               0x19
#define RX_DISCARD_CONGESTION_PCKNT     0x11

#define RX_SIZE_64B_PCKNT               0x12
#define RX_SIZE_65_127B_PCKNT           0x13
#define RX_SIZE_128_255B_PCKNT          0x14
#define RX_SIZE_256_511B_PCKNT          0x15
#define RX_SIZE_512_1023B_PCKNT         0x16
#define RX_SIZE_1024B_MAX_PCKNT         0x17

#define TX_TOTAL_PCKNT                  0x0C
#define TX_UNICAST_PCKNT                0x06
#define TX_MULTICAST_PCKNT              0x07

#define TX_SINGLE_COLLISION_CNT         0x08
#define TX_MULTIPLE_COLLISION_CNT       0x09
#define TX_LATE_COLLISION_CNT           0x0A
#define TX_EXCESSIVE_COLLISION_CNT      0x0B
#define TX_PAUSE_PCKNT                  0x0D

#define TX_SIZE_64B_PCKNT               0x00
#define TX_SIZE_65_127B_PCKNT           0x01
#define TX_SIZE_128_255B_PCKNT          0x02
#define TX_SIZE_256_511B_PCKNT          0x03
#define TX_SIZE_512_1023B_PCKNT         0x04
#define TX_SIZE_1024B_MAX_PCKNT         0x05

#define TX_DROP_PCKNT                   0x10
#define RX_DROP_PCKNT                   0x18

#define RX_GOOD_BYTE_CNT_LOW            0x24
#define RX_GOOD_BYTE_CNT_HIGH           0x25

#define RX_BAD_BYTE_CNT_LOW             0x26
#define RX_BAD_BYTE_CNT_HIGH            0x27

#define TX_GOOD_BYTE_CNT_LOW            0x0E
#define TX_GOOD_BYTE_CNT_HIGH           0x0F

static INLINE unsigned int sw_get_rmon_counter(int port, int addr) {
	*ETHSW_BM_RAM_ADDR_REG = addr;
	*ETHSW_BM_RAM_CTRL_REG = 0x8000 | port;
	while ((*ETHSW_BM_RAM_CTRL_REG & 0x8000))
		;

	return (*ETHSW_BM_RAM_VAL_1_REG << 16) | (*ETHSW_BM_RAM_VAL_0_REG & 0xFFFF);
}

static INLINE void sw_clr_rmon_counter(int port) {
	int i;

	if (port >= 0 && port < 7) {
		*ETHSW_BM_PCFG_REG(port) = 0;
		*ETHSW_BM_RMON_CTRL_REG(port) = 3;
		for (i = 1000; (*ETHSW_BM_RMON_CTRL_REG(port) & 3) != 0 && i > 0; i--)
			;
		if (i == 0)
			*ETHSW_BM_RMON_CTRL_REG(port) = 0;
		*ETHSW_BM_PCFG_REG(port) = 1;
	}
}

static INLINE int read_port_counters(int port, char *buf) {
	int len = 0;
	unsigned int counter;
	unsigned long long byteCnt;

	len += sprintf(buf + len, "\n\tPort [%d] counters\n\n", port);

	len += sprintf(buf + len, "Rx Total PckCnt     :");
	counter = sw_get_rmon_counter(port, RX_TOTAL_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Rx Unicast PckCnt   :");
	counter = sw_get_rmon_counter(port, RX_UNICAST_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Rx Multicast PckCnt :");
	counter = sw_get_rmon_counter(port, RX_MULTICAST_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Tx Total PckCnt     :");
	counter = sw_get_rmon_counter(port, TX_TOTAL_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Tx Unicase PckCnt   :");
	counter = sw_get_rmon_counter(port, TX_UNICAST_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Tx Multicase PckCnt :");
	counter = sw_get_rmon_counter(port, TX_MULTICAST_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Rx 64B PckCnt       :");
	counter = sw_get_rmon_counter(port, RX_SIZE_64B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Rx [65-127B] PckCnt :");
	counter = sw_get_rmon_counter(port, RX_SIZE_65_127B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Rx [128~255B] PckCnt:");
	counter = sw_get_rmon_counter(port, RX_SIZE_128_255B_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);
	len += sprintf(buf + len, "Rx [256~511B] PckCnt:");
	counter = sw_get_rmon_counter(port, RX_SIZE_256_511B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Rx [512~1023B]PckCnt:");
	counter = sw_get_rmon_counter(port, RX_SIZE_512_1023B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Rx [ >1024B] PckCnt :");
	counter = sw_get_rmon_counter(port, RX_SIZE_1024B_MAX_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Tx [64B] PckCnt     :");
	counter = sw_get_rmon_counter(port, TX_SIZE_64B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Tx [65~127B] PckCnt :");
	counter = sw_get_rmon_counter(port, TX_SIZE_65_127B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Tx [128~255B] PckCnt:");
	counter = sw_get_rmon_counter(port, TX_SIZE_128_255B_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);
	len += sprintf(buf + len, "Tx [256~511B] PckCnt:");
	counter = sw_get_rmon_counter(port, TX_SIZE_256_511B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Tx [512~1023B]PckCnt:");
	counter = sw_get_rmon_counter(port, TX_SIZE_512_1023B_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);
	len += sprintf(buf + len, "Tx [>1024B] PckCnt  :");
	counter = sw_get_rmon_counter(port, TX_SIZE_1024B_MAX_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Rx CRC err PckCnt   :");
	counter = sw_get_rmon_counter(port, RX_CRC_ERR_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Rx Unsize err PCkCnt:");
	counter = sw_get_rmon_counter(port, RX_UNDERSIZE_ERR_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Rx Ovsize err PckCnt:");
	counter = sw_get_rmon_counter(port, RX_OVER_SIZE_ERR_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Rx UnsizeGood PckCnt:");
	counter = sw_get_rmon_counter(port, RX_UNDERSIZE_GOOD_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Rx OvsizeGood PckCnt:");
	counter = sw_get_rmon_counter(port, RX_OVER_SIZE_GOOD_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Rx Good Pause PckCnt:");
	counter = sw_get_rmon_counter(port, RX_GOOD_PAUSE_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Rx Align err PckCnt :");
	counter = sw_get_rmon_counter(port, RX_ALLIGN_ERR_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Rx filterd errPckCnt:");
	counter = sw_get_rmon_counter(port, RX_FILTERED_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Tx Single col Cnt   :");
	counter = sw_get_rmon_counter(port, TX_SINGLE_COLLISION_CNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Tx Multiple col Cnt :");
	counter = sw_get_rmon_counter(port, TX_MULTIPLE_COLLISION_CNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Tx Late col  Cnt    :");
	counter = sw_get_rmon_counter(port, TX_LATE_COLLISION_CNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Tx Excessive col Cnt:");
	counter = sw_get_rmon_counter(port, TX_EXCESSIVE_COLLISION_CNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Tx  Pause Cnt       :");
	counter = sw_get_rmon_counter(port, TX_PAUSE_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Tx Drop PckCnt      :");
	counter = sw_get_rmon_counter(port, TX_DROP_PCKNT);
	len += sprintf(buf + len, "0x%08x\t", counter);

	len += sprintf(buf + len, "Rx Drop PckCnt      :");
	counter = sw_get_rmon_counter(port, RX_DROP_PCKNT);
	len += sprintf(buf + len, "0x%08x\n", counter);

	len += sprintf(buf + len, "Rx Good Byte Cnt    :");
	byteCnt = sw_get_rmon_counter(port, RX_GOOD_BYTE_CNT_HIGH);
	byteCnt <<= 32;
	byteCnt += sw_get_rmon_counter(port, RX_GOOD_BYTE_CNT_LOW);
	len += sprintf(buf + len, "0x%llx \t", byteCnt);

	len += sprintf(buf + len, "Tx Good Byte Cnt    :");
	byteCnt = sw_get_rmon_counter(port, TX_GOOD_BYTE_CNT_HIGH);
	byteCnt <<= 32;
	byteCnt += sw_get_rmon_counter(port, TX_GOOD_BYTE_CNT_LOW);
	len += sprintf(buf + len, "0x%llx \n", byteCnt);

	len += sprintf(buf + len, "Rx Bad Byte Cnt     :");
	byteCnt = sw_get_rmon_counter(port, RX_BAD_BYTE_CNT_HIGH);
	byteCnt <<= 32;
	byteCnt += sw_get_rmon_counter(port, RX_BAD_BYTE_CNT_LOW);
	len += sprintf(buf + len, "0x%llx \t", byteCnt);

	len += sprintf(buf + len, "Rx Discard Cong Cnt :");
	byteCnt = sw_get_rmon_counter(port, RX_DISCARD_CONGESTION_PCKNT);
	len += sprintf(buf + len, "0x%llx \n", byteCnt);

	return len;
}

static int port0_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(0, buf + len);

	*eof = 1;

	return len;
}

static int port1_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(1, buf + len);

	*eof = 1;

	return len;
}

static int port2_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(2, buf + len);

	*eof = 1;

	return len;
}

static int port3_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(3, buf + len);

	*eof = 1;

	return len;
}

static int port4_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(4, buf + len);

	*eof = 1;

	return len;
}

static int port5_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(5, buf + len);

	*eof = 1;

	return len;
}

static int port6_counters_proc_read(char *buf,
		char **start __attribute__ ((unused)),
		off_t offset __attribute__ ((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += read_port_counters(6, buf + len);

	*eof = 1;

	return len;
}

static int port_counters_proc_write(struct file *file, const char *buf,
		unsigned long count, void *data __attribute__ ((unused))) {
	char *p;
	int len, rlen;

	int port;

	len = count < sizeof(common_proc_read_buff) ?
			count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p) {
		return 0;
	}

	if (ifx_stricmp(p, "clean") == 0 || ifx_stricmp(p, "clear") == 0) {
		strcpy(common_proc_read_buff, file->f_dentry->d_iname);
		port = common_proc_read_buff[4] - '0';
		common_proc_read_buff[4] = ' ';
		if (ifx_stricmp(common_proc_read_buff, "port mib") == 0 && port >= 0
				&& port < 7)
			sw_clr_rmon_counter(port);
		else
			printk("unknown entry - %s\n", file->f_dentry->d_iname);
	} else if (ifx_stricmp(p, "clean all") == 0 || ifx_stricmp(p, "clear all") == 0) {
		for (port = 0; port < 7; port++)
			sw_clr_rmon_counter(port);
	}

	return count;
}

static int proc_read_directforwarding(char *page,
		char **start __attribute__ ((unused)), off_t off __attribute__((unused)),
		int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;

	len += sprintf(page + len, "ingress direct forwarding - %s\n",
			g_ingress_direct_forward_enable ? "enable" : "disable");
	len += sprintf(page + len, "egress direct forwarding  - %s\n",
			g_egress_direct_forward_enable ? "enable" : "disable");

#if defined(DEBUG_DMA_RX_CHANNEL_COUNTER)
	for (i = 0; i < MAX_DMA_CHAN; i++){
		len += sprintf(page + len, " dma_rx_chan_count[%d] = %d\n", i , dma_rx_chan_count[i]);
	}
#endif

	*eof = 1;

	return len;
}

static int proc_write_directforwarding(
		struct file *file __attribute__ ((unused)), const char *buf,
		unsigned long count, void *data __attribute__ ((unused))) {
	char *p;
	int len, rlen;

	len = count < sizeof(common_proc_read_buff) ?
			count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p) {
		return 0;
	}

	if (ifx_stricmp(p, "ingress enable") == 0) {
		if (!g_ingress_direct_forward_enable) {
			g_ingress_direct_forward_enable = 1;
			//egress is enabled in eth_xmit
		}
	} else if (ifx_stricmp(p, "egress enable") == 0) {
		if (!g_egress_direct_forward_enable) {
			g_egress_direct_forward_enable = 1;
			//egress is enabled in eth_xmit
		}
	} else if (ifx_stricmp(p, "ingress disable") == 0) {
		if (g_ingress_direct_forward_enable) {
			g_ingress_direct_forward_enable = 0;
		}
	} else if (ifx_stricmp(p, "egress disable") == 0) {
		if (g_egress_direct_forward_enable) {
			g_egress_direct_forward_enable = 0;
			//egress is enabled in eth_xmit
		}
	} else {
		printk(
				"echo <ingress/egress> <enable/disable> > /proc/eth/direct_forwarding\n");
		printk(
				"enable  : pure routing configuration, switch forward packets between port0/1 and cpu port directly w/o learning\n");
		printk(
				"disable : bridging/mix configuration, switch learn MAC address and make decision on which port packet forward to\n");
	}

	return count;
}

static int proc_read_clk(char *page, char **start __attribute__ ((unused)),
		off_t off __attribute__((unused)), int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;
	unsigned int clk;
	int valid = 0;

	switch ((*VR9_CGU_CLKFSR >> 16) & 0x07) {
	case 0:
		clk = 500;
		valid = 1;
		break;
	case 1:
		clk = 432;
		valid = 1;
		break;
	case 2:
		valid = 1;
	default:
		clk = 288;
	}

	if (valid)
		len += sprintf(page + len, "PPE clock - %uM\n", clk);
	else
		len += sprintf(page + len, "PPE clock - %uM (unknown option %u)\n", clk,
				(*VR9_CGU_CLKFSR >> 16) & 0x07);

	*eof = 1;

	return len;
}

static int proc_write_clk(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count,
		void *data __attribute__ ((unused))) {
	char *p;
	int len, rlen;

	len = count < sizeof(common_proc_read_buff) ?
			count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p) {
		return 0;
	}

	if (ifx_stricmp(p, "500M") == 0 || ifx_stricmp(p, "500") == 0) {
		*VR9_CGU_CLKFSR &= ~(0x07 << 16);
	} else if (ifx_stricmp(p, "432M") == 0 || ifx_stricmp(p, "432") == 0) {
		*VR9_CGU_CLKFSR = (*VR9_CGU_CLKFSR & ~(0x07 << 16)) | (0x01 << 16);
	} else if (ifx_stricmp(p, "288M") == 0 || ifx_stricmp(p, "288") == 0) {
		*VR9_CGU_CLKFSR = (*VR9_CGU_CLKFSR & ~(0x07 << 16)) | (0x02 << 16);
	} else {
		printk("echo <500/432/288M> > /proc/eth/clk\n");
		printk("  500M - PPE clock 500MHz\n");
		printk("  432M - PPE clock 432MHz\n");
		printk("  288M - PPE clock 288MHz\n");
	}

	return count;
}

static int proc_read_prio(char *page, char **start __attribute__ ((unused)),
		off_t off __attribute__((unused)), int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;
	unsigned int i, j;

	len += sprintf(page + len, "Priority to Queue Map:\n");
	len += sprintf(page + len, "  prio     :  0  1  2  3  4  5  6  7\n");
	for (i = 0; i < 2; i++) {
		len += sprintf(page + len, "  eth%d     :", i);
		for (j = 0; j < NUM_ENTITY(g_eth_prio_queue_map[i]); j++)
            len += sprintf(page + len," %2d", g_eth_prio_queue_map[i][j]);

		len += sprintf(page + len, "\n");
	}
	if (g_eth_wan_mode == 0) {
		len += sprintf(page + len, "  ptm_vr9     :");
		for (j = 0; j < NUM_ENTITY(g_ptm_prio_queue_map); j++)
			len += sprintf(page + len, "  %2d", g_ptm_prio_queue_map[j]);
		len += sprintf(page + len, "\n");
	}
    if ( (g_wan_itf & 0x03) )
    {
        len += sprintf(page + len,    "Queue to Class Map:\n");
        len += sprintf(page + len,    "  queue    :  0  1  2  3  4  5  6  7\n");
        len += sprintf(page + len,    "  eth%d     :", (g_wan_itf & 0x01) ? 0 : 1);
        for ( j = 0; j < NUM_ENTITY(g_wan_queue_class_map); j++ )
            len += sprintf(page + len," %2d", g_wan_queue_class_map[j]);
        len += sprintf(page + len,    "\n");
    }
	*eof = 1;

	return len;
}

static int proc_write_prio(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count,
		void *data __attribute__ ((unused))) {
	int len;
	char *p1, *p2;
	int colon = 1;
	unsigned int port = ~0u;
	int prio = -1;
	int queue = -1;

	len = sizeof(common_proc_read_buff) < count ?
			sizeof(common_proc_read_buff) - 1 : count;
	len = len - copy_from_user(common_proc_read_buff, buf, len);
	common_proc_read_buff[len] = 0;

	p1 = common_proc_read_buff;
	while (ifx_get_token(&p1, &p2, &len, &colon)) {
		if (ifx_stricmp(p1, "help") == 0) {
			dbg(
					"echo <eth0/eth1/ptm_vr9> prio xx queue xx [prio xx queue xx] > /proc/eth/prio\n");
			break;
		} else if (ifx_stricmp(p1, "eth0") == 0) {
			port = 0;
			prio = queue = -1;
			dbg("port = 0\n");
		} else if (ifx_stricmp(p1, "eth1") == 0) {
			port = 1;
			prio = queue = -1;
			dbg("port = 1\n");
		} else if (ifx_stricmp(p1, "ptm_vr9") == 0) {
			if (g_eth_wan_mode == 0) {
				port = 2;
				prio = queue = -1;
				dbg("port = 2 (PTM0)\n");
			}
		} else if (ifx_stricmp(p1, "pvc") == 0) {
			err("atm is not valid\n");
			prio = queue = -1;
		} else if (port != ~0u) {
			if (ifx_stricmp(p1, "p") == 0 || ifx_stricmp(p1, "prio") == 0) {
				ifx_ignore_space(&p2, &len);
				prio = ifx_get_number(&p2, &len, 0);
				dbg("prio = %d\n", prio);
				/*--- if ( port >= 0 && port <= 1 ) ---*//* port always >= 0 ! */
				if (port <= 1) {
					if (prio
							>= 0 && (unsigned int) prio < NUM_ENTITY(g_eth_prio_queue_map[port])) {if ( queue >= 0 )
					g_eth_prio_queue_map[port][prio] = queue;
				}
				else
				err("prio (%d) is out of range 0 - %d\n", prio, NUM_ENTITY(g_eth_prio_queue_map[port]) - 1);
			}
			else if ( port == 2 )
			{
				if ( prio >= 0 && (unsigned int) prio < NUM_ENTITY(g_ptm_prio_queue_map) )
				{
					if ( queue >= 0 )
					g_ptm_prio_queue_map[prio] = queue;
				}
				else
				err("prio (%d) is out of range 0 - %d\n", prio, NUM_ENTITY(g_ptm_prio_queue_map) - 1);
			}
		}
		else if ( ifx_stricmp(p1, "q") == 0 || ifx_stricmp(p1, "queue") == 0 )
		{
			ifx_ignore_space(&p2, &len);
			queue = ifx_get_number(&p2, &len, 0);
			dbg("queue = %d\n", queue);
			/*--- if ( port >= 0 && port <= 1 ) ---*//* Port always >= 0 ! */
			if ( port <= 1 )
			{
                if ( queue >= 0 && queue < ((g_wan_itf & (1 << port)) && g_wanqos_en ? g_wanqos_en : SWITCH_CLASS_VALUE) )
				{
					if ( prio >= 0 )
					g_eth_prio_queue_map[port][prio] = queue;
				}
				else{
				    err("queue (%d) is out of range 0 - %d", queue, (g_wan_itf & (1 << port)) && g_wanqos_en ? g_wanqos_en - 1 : SWITCH_CLASS_VALUE - 1);
				}
			}
			else if ( port == 2 )
			{
				if ( queue >= 0 && queue < g_wanqos_en )
				{
					if ( prio >= 0 )
					g_ptm_prio_queue_map[prio] = queue;
				}
				else
				err("queue (%d) is out of range 0 - %d\n", queue, g_wanqos_en - 1);
			}
		}
		else
		err("unknown command (%s)\n", p1);
	}
	else
	err("unknown command (%s)\n", p1);

	p1 = p2;
	colon = 1;
}

	return count;
}

static int proc_read_ewan(char *page, char **start __attribute__ ((unused)),
		off_t off __attribute__((unused)), int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;
	int i, j;
	unsigned int bit;

	g_pmac_ewan = *PMAC_EWAN_REG;

	len += sprintf(page + len, "EWAN: ");
	for (i = j = 0, bit = 1; i < 7; i++, bit <<= 1)
		if ((g_pmac_ewan & bit)) {
			len += sprintf(page + len, (const char *) (j ? " | P%d" : "P%d"),
					i);
			j++;
		}
	len += sprintf(page + len, (const char *) (j ? "\n" : "null\n"));

	*eof = 1;

	return len;
}

static int proc_write_ewan(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count,
		void *data __attribute__ ((unused))) {
	int len;
	char *p1, *p2;
	int colon = 1;

	unsigned int got_ports = 0;
	unsigned int ports = 0;
	char *ports_name[] = { "p0", "p1", "p2", "p3", "p4", "p5" };
	unsigned int i;

	len = sizeof(common_proc_read_buff) < count ?
			sizeof(common_proc_read_buff) - 1 : count;
	len = len - copy_from_user(common_proc_read_buff, buf, len);
	common_proc_read_buff[len] = 0;

	p1 = common_proc_read_buff;
	while (ifx_get_token(&p1, &p2, &len, &colon)) {
		if (ifx_stricmp(p1, "help") == 0) {
			printk("echo <p0|p1|p2|p3|p4|p5> > /proc/eth/ewan\n");
			break;
		} else {
			for (i = 0; i < NUM_ENTITY(ports_name); i++)
				if (ifx_stricmp(p1, ports_name[i]) == 0) {
					ports |= 1 << i;
					got_ports++;
					break;
				}
		}

		p1 = p2;
		colon = 1;
	}

	if (got_ports)
		*PMAC_EWAN_REG = g_pmac_ewan = ports;

	return count;
}

#ifdef CONFIG_IFX_PPA_QOS
static int proc_read_qos(char *page, char **start, off_t off, int count, int *eof, void *data __attribute__((unused))) {
    int len = 0;
    char *pstr;
    char str[200];
    int llen;
    int i;
    struct wtx_eg_q_shaping_cfg qos_cfg;
    struct wan_tx_mib_table qos_queue_mib;
    volatile struct tx_qos_cfg tx_qos_cfg = *TX_QOS_CFG_DYNAMIC;
    volatile struct wtx_qos_q_desc_cfg qos_q_desc_cfg;
    unsigned int portid;

    pstr = *start = page;

    __sync();

    llen = sprintf(pstr, "\n  qos         : %s\n  wfq         : %s\n  Rate shaping: %s\n\n",
                    tx_qos_cfg.eth1_qss ?"enabled":"disabled",
                    tx_qos_cfg.wfq_en?"enabled":"disabled",
                    tx_qos_cfg.shape_en ?"enabled":"disabled");
    pstr += llen;
    len += llen;

    llen = sprintf(pstr, "  Ticks  =%u,    overhd    =%u,       qnum=%u  @%p\n", (u32)tx_qos_cfg.time_tick, (u32)tx_qos_cfg.overhd_bytes, (u32)tx_qos_cfg.eth1_eg_qnum, TX_QOS_CFG_DYNAMIC );
    pstr += llen;
    len += llen;

    llen = sprintf(pstr, "  PPE clk=%u MHz, basic tick=%u\n", (u32)cgu_get_pp32_clock() / 1000000, TX_QOS_CFG_DYNAMIC->time_tick / (cgu_get_pp32_clock() / 1000000));
    pstr += llen;
    len += llen;

    if ( tx_qos_cfg.eth1_eg_qnum )
    {
        if ( ifx_ppa_drv_hal_generic_hook != NULL )
        {
            PPE_QOS_RATE_SHAPING_CFG rate_cfg = {0};
            PPE_QOS_WFQ_CFG wfq_cfg = {0};

            if ( g_eth_wan_mode == 0 )
                portid = 7;
            else if ( (g_wan_itf & 0x02) )
                portid = 1;
            else if ( (g_wan_itf & 0x01) )
                portid = 0;
            else
                portid = ~0;

            llen = sprintf(pstr, "\n  Cfg :    T     R     S -->  Bit-rate(kbps)      Weight --> Level       Address       d/w      tick_cnt   b/S\n");
            pstr += llen;
            len += llen;
            for ( i = 0; i < 8; i++ )
            {
  #ifdef CONFIG_IFX_PPA_QOS_RATE_SHAPING
                rate_cfg.portid  = portid;
                rate_cfg.queueid = i;
                ifx_ppa_drv_hal_generic_hook(PPA_GENERIC_HAL_GET_QOS_RATE_SHAPING_CFG, &rate_cfg, 0);
  #endif
  #ifdef CONFIG_IFX_PPA_QOS_WFQ
                wfq_cfg.portid  = portid;
                wfq_cfg.queueid = i;
                ifx_ppa_drv_hal_generic_hook(PPA_GENERIC_HAL_GET_QOS_WFQ_CFG, &rate_cfg, 0);
  #endif
                qos_cfg = *WTX_EG_Q_SHAPING_CFG(i);

                llen = sprintf(str, "\n      %2u:  %03u  %05u  %05u   %07u            %08u   %03u        @0x%p   %08u    %03u     %05u\n", i, qos_cfg.t, qos_cfg.r, qos_cfg.s, rate_cfg.rate_in_kbps, qos_cfg.w, wfq_cfg.weight_level, WTX_EG_Q_SHAPING_CFG(i), qos_cfg.d, qos_cfg.tick_cnt, qos_cfg.b);
                if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
                    goto PROC_READ_QOS_OVERRUN_END;
            }

            if ( portid < 2 )
            {
                //  PTM WAN mode does not have port based rate shaping, only ETH WAN mode has.

  #ifdef CONFIG_IFX_PPA_QOS_RATE_SHAPING
                rate_cfg.portid  = portid;
                rate_cfg.queueid = i;
                ifx_ppa_drv_hal_generic_hook(PPA_GENERIC_HAL_GET_QOS_RATE_SHAPING_CFG, &rate_cfg, 0);
  #endif
                qos_cfg = *WTX_EG_Q_PORT_SHAPING_CFG(0);

                llen = sprintf(str, "\n    port:  %03u  %05u  %05u   %07u                                  @0x%p   %08u    %03u     %05u\n", qos_cfg.t, qos_cfg.r, qos_cfg.s, rate_cfg.rate_in_kbps, WTX_EG_Q_PORT_SHAPING_CFG(0), qos_cfg.d, qos_cfg.tick_cnt, qos_cfg.b);
                if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
                    goto PROC_READ_QOS_OVERRUN_END;
            }
        }

        llen = sprintf(pstr, "\n  MIB : rx_pkt/rx_bytes         tx_pkt/tx_bytes       cpu_small_drop/cpu_drop  fast_small_drop/fast_drop_cnt\n");
        pstr += llen;
        len += llen;
        for ( i = 0; i < 8; i++ )
        {
            qos_queue_mib = *ETH_WAN_TX_MIB_TABLE(i);

            llen = sprintf(str, "    %2u: %010u/%010u  %010u/%010u  %010u/%010u  %010u/%010u  @0x%p\n", i,
                qos_queue_mib.wrx_total_pdu, qos_queue_mib.wrx_total_bytes,
                qos_queue_mib.wtx_total_pdu, qos_queue_mib.wtx_total_bytes,
                qos_queue_mib.wtx_cpu_drop_small_pdu, qos_queue_mib.wtx_cpu_drop_pdu,
                qos_queue_mib.wtx_fast_drop_small_pdu, qos_queue_mib.wtx_fast_drop_pdu,
                ETH_WAN_TX_MIB_TABLE(i));

            if ( proc_buf_copy(&pstr, count, off, &len, str, llen) )
                goto PROC_READ_QOS_OVERRUN_END;
        }

        //QOS queue descriptor
        llen = sprintf(pstr, "\n  Desc: threshold  num    base_addr  rd_ptr   wr_ptr\n");
        pstr += llen;
        len += llen;
        for ( i = 0; i < 8; i++ )
        {
            qos_q_desc_cfg = *WTX_QOS_Q_DESC_CFG(i);

            llen = sprintf(pstr, "    %2u: 0x%02x       0x%02x   0x%04x     0x%04x   0x%04x  @0x%p\n", i,
                qos_q_desc_cfg.threshold,
                qos_q_desc_cfg.length,
                qos_q_desc_cfg.addr,
                qos_q_desc_cfg.rd_ptr,
                qos_q_desc_cfg.wr_ptr,
                WTX_QOS_Q_DESC_CFG(i) );

            pstr += llen;
            len += llen;
        }
    }

    *eof = 1;

PROC_READ_QOS_OVERRUN_END:
    return len - off;
}

#endif

static int proc_read_gamma(char *page, char **start __attribute__ ((unused)),
		off_t off __attribute__((unused)), int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;
	len += sprintf(page + len,
			"PSAVE_CFG     (0x%08X): start_state - %s, sleep_en - %s\n",
			(unsigned int) PSAVE_CFG,
			PSAVE_CFG->start_state ? "partial reset" : "full reset",
			PSAVE_CFG->sleep_en ? "on" : "off");
	len += sprintf(page + len,
			"EG_BWCTRL_CFG (0x%08X): fdesc_wm - %d, class_len - %d\n",
			(unsigned int) EG_BWCTRL_CFG, EG_BWCTRL_CFG->fdesc_wm,
			EG_BWCTRL_CFG->class_len);
	len += sprintf(page + len,
			"TEST_MODE     (0x%08X): mib_clear_mode - %s, test_mode - %s\n",
			(unsigned int) TEST_MODE, TEST_MODE->mib_clear_mode ? "on" : "off",
			TEST_MODE->test_mode ? "on" : "off");
	len += sprintf(page + len, "RX_BC_CFG:\t0x%08x\t0x%08x\n",
			(unsigned int) RX_BC_CFG(0), (unsigned int) RX_BC_CFG(1));
	len += sprintf(page + len, "  local_state:    %8d\t  %8d\n",
			(unsigned int) RX_BC_CFG(0)->local_state,
			(unsigned int) RX_BC_CFG(1)->local_state);
	len += sprintf(page + len, "  remote_state:   %8d\t  %8d\n",
			(unsigned int) RX_BC_CFG(0)->remote_state,
			(unsigned int) RX_BC_CFG(1)->remote_state);
	len += sprintf(page + len, "  to_false_th:    %8d\t  %8d\n",
			(unsigned int) RX_BC_CFG(0)->to_false_th,
			(unsigned int) RX_BC_CFG(1)->to_false_th);
	len += sprintf(page + len, "  to_looking_th:  %8d\t  %8d\n",
			(unsigned int) RX_BC_CFG(0)->to_looking_th,
			(unsigned int) RX_BC_CFG(1)->to_looking_th);
	len += sprintf(page + len, "TX_BC_CFG:\t0x%08x\t0x%08x\n",
			(unsigned int) TX_BC_CFG(0), (unsigned int) TX_BC_CFG(1));
	len += sprintf(page + len, "  fill_wm:\t  %8d\t  %8d\n",
			(unsigned int) TX_BC_CFG(0)->fill_wm,
			(unsigned int) TX_BC_CFG(1)->fill_wm);
	len += sprintf(page + len, "  uflw_wm:\t  %8d\t  %8d\n",
			(unsigned int) TX_BC_CFG(0)->uflw_wm,
			(unsigned int) TX_BC_CFG(1)->uflw_wm);
	len += sprintf(page + len,
			"RX_GAMMA_ITF_CFG:\t0x%08x\t0x%08x\t0x%08x\t0x%08x\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0),
			(unsigned int) RX_GAMMA_ITF_CFG(1),
			(unsigned int) RX_GAMMA_ITF_CFG(2),
			(unsigned int) RX_GAMMA_ITF_CFG(3));
	len += sprintf(page + len,
			"  receive_state: \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->receive_state,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->receive_state,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->receive_state,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->receive_state);
	len += sprintf(page + len,
			"  rx_min_len:    \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_min_len,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_min_len,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_min_len,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_min_len);
	len += sprintf(page + len,
			"  rx_pad_en:     \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_pad_en,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_pad_en,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_pad_en,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_pad_en);
	len += sprintf(page + len,
			"  rx_eth_fcs_ver_dis:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_eth_fcs_ver_dis,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_eth_fcs_ver_dis,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_eth_fcs_ver_dis,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_eth_fcs_ver_dis);
	len += sprintf(page + len,
			"  rx_rm_eth_fcs: \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_rm_eth_fcs,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_rm_eth_fcs,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_rm_eth_fcs,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_rm_eth_fcs);
	len += sprintf(page + len,
			"  rx_tc_crc_ver_dis:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_tc_crc_ver_dis,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_tc_crc_ver_dis,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_tc_crc_ver_dis,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_tc_crc_ver_dis);
	len += sprintf(page + len,
			"  rx_tc_crc_size:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_tc_crc_size,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_tc_crc_size,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_tc_crc_size,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_tc_crc_size);
	len += sprintf(page + len,
			"  rx_eth_fcs_result:\t0x%8X\t0x%8X\t0x%8X\t0x%8X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_eth_fcs_result,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_eth_fcs_result,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_eth_fcs_result,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_eth_fcs_result);
	len += sprintf(page + len,
			"  rx_tc_crc_result:\t0x%8X\t0x%8X\t0x%8X\t0x%8X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_tc_crc_result,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_tc_crc_result,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_tc_crc_result,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_tc_crc_result);
	len += sprintf(page + len,
			"  rx_crc_cfg:    \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_crc_cfg,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_crc_cfg,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_crc_cfg,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_crc_cfg);
	len += sprintf(page + len,
			"  rx_eth_fcs_init_value:0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_eth_fcs_init_value,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_eth_fcs_init_value,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_eth_fcs_init_value,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_eth_fcs_init_value);
	len += sprintf(page + len,
			"  rx_tc_crc_init_value:\t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_tc_crc_init_value,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_tc_crc_init_value,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_tc_crc_init_value,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_tc_crc_init_value);
	len += sprintf(page + len,
			"  rx_max_len_sel:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_max_len_sel,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_max_len_sel,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_max_len_sel,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_max_len_sel);
	len += sprintf(page + len,
			"  rx_edit_num2:  \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_num2,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_num2,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_num2,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_num2);
	len += sprintf(page + len,
			"  rx_edit_pos2:  \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_pos2,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_pos2,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_pos2,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_pos2);
	len += sprintf(page + len,
			"  rx_edit_type2: \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_type2,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_type2,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_type2,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_type2);
	len += sprintf(page + len,
			"  rx_edit_en2:   \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_en2,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_en2,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_en2,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_en2);
	len += sprintf(page + len,
			"  rx_edit_num1:  \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_num1,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_num1,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_num1,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_num1);
	len += sprintf(page + len,
			"  rx_edit_pos1:  \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_pos1,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_pos1,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_pos1,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_pos1);
	len += sprintf(page + len,
			"  rx_edit_type1: \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_type1,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_type1,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_type1,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_type1);
	len += sprintf(page + len,
			"  rx_edit_en1:   \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_edit_en1,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_edit_en1,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_edit_en1,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_edit_en1);
	len += sprintf(page + len,
			"  rx_inserted_bytes_1l:\t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_inserted_bytes_1l,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_inserted_bytes_1l,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_inserted_bytes_1l,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_inserted_bytes_1l);
	len += sprintf(page + len,
			"  rx_inserted_bytes_1h:\t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_inserted_bytes_1h,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_inserted_bytes_1h,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_inserted_bytes_1h,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_inserted_bytes_1h);
	len += sprintf(page + len,
			"  rx_inserted_bytes_2l:\t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_inserted_bytes_2l,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_inserted_bytes_2l,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_inserted_bytes_2l,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_inserted_bytes_2l);
	len += sprintf(page + len,
			"  rx_inserted_bytes_2h:\t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) RX_GAMMA_ITF_CFG(0)->rx_inserted_bytes_2h,
			(unsigned int) RX_GAMMA_ITF_CFG(1)->rx_inserted_bytes_2h,
			(unsigned int) RX_GAMMA_ITF_CFG(2)->rx_inserted_bytes_2h,
			(unsigned int) RX_GAMMA_ITF_CFG(3)->rx_inserted_bytes_2h);
	len += sprintf(page + len,
			"RX_GAMMA_ITF_CFG:\t0x%08x\t0x%08x\t0x%08x\t0x%08x\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0),
			(unsigned int) TX_GAMMA_ITF_CFG(1),
			(unsigned int) TX_GAMMA_ITF_CFG(2),
			(unsigned int) TX_GAMMA_ITF_CFG(3));
	len += sprintf(page + len,
			"  tx_len_adj:    \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_len_adj,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_len_adj,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_len_adj,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_len_adj);
	len += sprintf(page + len,
			"  tx_crc_off_adj:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_crc_off_adj,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_crc_off_adj,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_crc_off_adj,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_crc_off_adj);
	len += sprintf(page + len,
			"  tx_min_len:    \t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_min_len,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_min_len,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_min_len,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_min_len);
	len += sprintf(page + len,
			"  tx_eth_fcs_gen_dis:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_eth_fcs_gen_dis,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_eth_fcs_gen_dis,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_eth_fcs_gen_dis,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_eth_fcs_gen_dis);
	len += sprintf(page + len,
			"  tx_tc_crc_size:\t  %8d\t  %8d\t  %8d\t  %8d\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_tc_crc_size,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_tc_crc_size,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_tc_crc_size,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_tc_crc_size);
	len += sprintf(page + len,
			"  tx_crc_cfg:    \t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_crc_cfg,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_crc_cfg,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_crc_cfg,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_crc_cfg);
	len += sprintf(page + len,
			"  tx_eth_fcs_init_value:0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_eth_fcs_init_value,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_eth_fcs_init_value,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_eth_fcs_init_value,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_eth_fcs_init_value);
	len += sprintf(page + len,
			"  tx_tc_crc_init_value:\t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->tx_tc_crc_init_value,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->tx_tc_crc_init_value,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->tx_tc_crc_init_value,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->tx_tc_crc_init_value);
	len += sprintf(page + len,
			"  queue_mapping: \t0x%08X\t0x%08X\t0x%08X\t0x%08X\n",
			(unsigned int) TX_GAMMA_ITF_CFG(0)->queue_mapping,
			(unsigned int) TX_GAMMA_ITF_CFG(1)->queue_mapping,
			(unsigned int) TX_GAMMA_ITF_CFG(2)->queue_mapping,
			(unsigned int) TX_GAMMA_ITF_CFG(3)->queue_mapping);

	*eof = 1;

	ASSERT(len <= 4096, "proc read buffer overflow");
	return len;
}
#if defined(DEBUG_MIRROR_PROC) && DEBUG_MIRROR_PROC

static int proc_read_mirror(char *page, char **start __attribute__ ((unused)),
		off_t off, int count __attribute__ ((unused)), int *eof,
		void *data __attribute__ ((unused))) {
	int len = 0;
	char devname[IFNAMSIZ + 1] = { 0 };

	if (g_mirror_netdev == NULL)
		strcpy(devname, "NONE");
	else
		strncpy(devname, g_mirror_netdev->name, IFNAMSIZ);

	len += sprintf(page + off + len, "mirror: %s\n", devname);

	*eof = 1;

	return len;
}

static int proc_write_mirror(struct file *file __attribute__ ((unused)),
		const char *buf, unsigned long count,
		void *data __attribute__ ((unused))) {
	char *p;
	int len, rlen;

	len = count < sizeof(common_proc_read_buff) ?
			count : sizeof(common_proc_read_buff) - 1;
	rlen = len - copy_from_user(common_proc_read_buff, buf, len);
	while (rlen && common_proc_read_buff[rlen - 1] <= ' ')
		rlen--;
	common_proc_read_buff[rlen] = 0;
	for (p = common_proc_read_buff; *p && *p <= ' '; p++, rlen--)
		;
	if (!*p)
		return count;

	if (g_mirror_netdev != NULL) {
		dev_put(g_mirror_netdev);
		g_mirror_netdev = NULL;
	}
	if (ifx_stricmp(p, "none") == 0)
		printk("disable mirror\n");
	else {
		g_mirror_netdev = dev_get_by_name(&init_net, p);
		if (g_mirror_netdev != NULL)
			printk("mirror: %s\n", p);
		else
			printk("mirror: none, can't find device \"%s\"\n", p);
	}

	return count;
}

#endif

static int proc_read_class(char *page, char **start __attribute__((unused)), off_t off __attribute__((unused)), int count __attribute__((unused)), int *eof, void *data __attribute__((unused)))
{
    int len = 0;

    len += sprintf(page + len, "CFG_CLASS2QID_MAP (0x%08x): %08x %08x %08x %08x\n", (unsigned int)CFG_CLASS2QID_MAP(0), *CFG_CLASS2QID_MAP(0), *CFG_CLASS2QID_MAP(1), *CFG_CLASS2QID_MAP(2), *CFG_CLASS2QID_MAP(3));
    len += sprintf(page + len, "CFG_QID2CLASS_MAP (0x%08x): %08x %08x %08x %08x\n", (unsigned int)CFG_QID2CLASS_MAP(0), *CFG_QID2CLASS_MAP(0), *CFG_QID2CLASS_MAP(1), *CFG_QID2CLASS_MAP(2), *CFG_QID2CLASS_MAP(3));

    *eof = 1;

    return len;
}

static int proc_write_class(struct file *file __attribute__((unused)), const char *buf, unsigned long count, void *data __attribute__((unused)))
{
    char local_buf[1024];
    int len;
    char *p1, *p2;
    int colon = 1;

    int i, j;

    len = sizeof(local_buf) < count ? sizeof(local_buf) - 1 : count;
    len = len - copy_from_user(local_buf, buf, len);
    local_buf[len] = 0;

    p1 = local_buf;
    while ( ifx_get_token(&p1, &p2, &len, &colon) )
    {
        if ( ifx_stricmp(p1, "CFG_CLASS2QID_MAP") == 0 || ifx_stricmp(p1, "CLASS2QID") == 0 )
        {
            for ( i = 0; i < 4; i++ )
            {
                ifx_ignore_space(&p2, &len);
                if ( !len )
                    break;
                *CFG_CLASS2QID_MAP(i) = ifx_get_number(&p2, &len, 1);
            }
            break;
        }
        else if ( ifx_stricmp(p1, "CFG_QID2CLASS_MAP") == 0 || ifx_stricmp(p1, "QID2CLASS") == 0 )
        {
            for ( i = 0; i < 4; i++ )
            {
                ifx_ignore_space(&p2, &len);
                if ( !len )
                    break;
                *CFG_QID2CLASS_MAP(i) = ifx_get_number(&p2, &len, 1);
                for ( j = 0; j < 8; j++ )
                    g_wan_queue_class_map[i * 8 + j] = (*CFG_QID2CLASS_MAP(i) >> (j * 4)) & 0x0F;
            }
            break;
        }
        else
        {
            printk("echo <CLASS2QID | QID2CLASS> [hex numbers] > /proc/eth/class\n");
            break;
        }

        p1 = p2;
        colon = 1;
    }

    return count;
}

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING

static int proc_read_dev(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
    int len = 0;

    len += sprintf(buf + len, "%s mode\n", g_ppe_base_addr == IFX_PPE_ORG(0) ? "host" : "device");
    len += sprintf(buf + len, "  g_slave_ppe_base_addr  = 0x%08x\n", g_slave_ppe_base_addr);
    len += sprintf(buf + len, "  g_pci_dev_ppe_addr_off = 0x%08x\n", g_pci_dev_ppe_addr_off);
    len += sprintf(buf + len, "  g_ppe_base_addr        = 0x%08x\n", g_ppe_base_addr);
    len += sprintf(buf + len, "  IFX_PMU                = 0x%08x\n", IFX_PMU);
    len += sprintf(buf + len, "  g_bm_addr              = 0x%08x\n", g_bm_addr);
    len += sprintf(buf + len, "  g_pci_host_ddr_addr    = 0x%08x\n", g_pci_host_ddr_addr);

    if ( offset >= len )
    {
        *start = buf;
        *eof = 1;
        return 0;
    }
    *start = buf + offset;
    if ( (len -= offset) > count )
        return count;
    *eof = 1;
    return len;
}

static int proc_write_dev(struct file *file, const char *buf, unsigned long count, void *data)
{
    char str[2048];
    char *p;
    int len, rlen;

    len = count < sizeof(str) ? count : sizeof(str) - 1;
    rlen = len - copy_from_user(str, buf, len);
    while ( rlen && str[rlen - 1] <= ' ' )
        rlen--;
    str[rlen] = 0;
    for ( p = str; *p && *p <= ' '; p++, rlen-- );
    if ( !*p )
        return count;

    if ( ifx_stricmp(p, "host") == 0 )
        g_ppe_base_addr = IFX_PPE_ORG(0);
    else if ( ifx_stricmp(p, "device") == 0 )
        g_ppe_base_addr = IFX_PPE_ORG(1);
    else
        printk("echo <host|device> > /proc/eth/dev");

    return count;
}

#endif

#if defined(DEBUG_BONDING_PROC) && DEBUG_BONDING_PROC
static void init_ds_des_array(void)
{
    if (g_dsl_bonding != 2)
        return;

    {
        SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        memcpy(saved_ds_des_array, (void *)DMA_TX_CH1_DESC_BASE, sizeof(struct rx_descriptor)* DMA_TX_CH1_DESC_NUM);
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    {
        SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        memcpy(&saved_ds_des_array[DMA_TX_CH1_DESC_NUM], (void *)E1_FRAG_RX_DESC_BASE(0), sizeof(struct rx_descriptor)* E1_FRAG_RX_DESC_NUM * 2);
        memcpy(&saved_ds_des_array[DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2], (void *)B1_RX_LINK_LIST_DESC_BASE, sizeof(struct rx_descriptor)* B1_RX_LINK_LIST_DESC_NUM );

        memcpy(&saved_us_des_array, (void *)US_BOND_PKT_DES_BASE(0), sizeof(struct rx_descriptor)* 512 );
        memcpy(&saved_us_des_array[512],  (void *)DMA_RX_CH1_DESC_BASE, sizeof(struct rx_descriptor)* 64 );
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

}

static void check_ds_des_array(void)
{

    int i, j;
    int res;
    int data_ptr_num[DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2 + B1_RX_LINK_LIST_DESC_NUM];

    if (g_dsl_bonding != 2)
        return;

    memset(data_ptr_num, 0, sizeof(data_ptr_num));

    {
        SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        memcpy(des, (void *)DMA_TX_CH1_DESC_BASE, sizeof(struct rx_descriptor)* DMA_TX_CH1_DESC_NUM);
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    {
        SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        memcpy(&des[DMA_TX_CH1_DESC_NUM], (void *)E1_FRAG_RX_DESC_BASE(0), sizeof(struct rx_descriptor)* E1_FRAG_RX_DESC_NUM * 2);
        memcpy(&des[DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2], (void *)B1_RX_LINK_LIST_DESC_BASE, sizeof(struct rx_descriptor)* B1_RX_LINK_LIST_DESC_NUM );
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

    for (i = 0; i < DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2 + B1_RX_LINK_LIST_DESC_NUM; ++ i) {
        unsigned int data_ptr = ((unsigned int *) des)[2*i + 1];
        for(j = 0; j < DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2 + B1_RX_LINK_LIST_DESC_NUM; ++ j) {
            unsigned int save_data_ptr = ((unsigned int *) saved_ds_des_array)[2*j + 1];
            if (save_data_ptr == data_ptr) {
                data_ptr_num[j] ++;
                break;
            }
        }
    }

    res = 0;
    for (i = 0; i < DMA_TX_CH1_DESC_NUM + E1_FRAG_RX_DESC_NUM * 2 + B1_RX_LINK_LIST_DESC_NUM; ++ i) {
        if (data_ptr_num[i] == 0) {
            printk("DS des lost: %08X %08X\n",
               ((unsigned int * ) &saved_ds_des_array[i])[0], ((unsigned int * ) &saved_ds_des_array[i])[1]);
            res ++;
        }
        else if (data_ptr_num[i] > 1) {
            printk("DS des dup : %08X %08X ==> %d\n",
               ((unsigned int * ) &saved_ds_des_array[i])[0], ((unsigned int * ) &saved_ds_des_array[i])[1],
                data_ptr_num[i]);
            res ++;
        }
    }

    if (res == 0) {
        printk("DS descriptor data_ptrs are consistent!\n");
    }
}

static void check_us_des_array(void)
{

    int i, j;
    int res;
    int data_ptr_num[512 + 64];

    if (g_dsl_bonding != 2)
        return;

    memset(data_ptr_num, 0, sizeof(data_ptr_num));

    {
        SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        memcpy(&us_des, (void *)US_BOND_PKT_DES_BASE(0), sizeof(struct rx_descriptor)* 512 );
        memcpy(&us_des[512], (void *)DMA_RX_CH1_DESC_BASE, sizeof(struct rx_descriptor)* 64 );
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

    for (i = 0; i < 512 + 64; ++ i) {
        unsigned int data_ptr = ((unsigned int *) us_des)[2*i + 1];
        for(j = 0; j < 512 + 64; ++ j) {
            unsigned int save_data_ptr = ((unsigned int *) saved_us_des_array)[2*j + 1];
            if (save_data_ptr == data_ptr) {
                data_ptr_num[j] ++;
                break;
            }
        }
    }

    res = 0;
    for (i = 0; i < 512 + 64; ++ i) {
        if (data_ptr_num[i] == 0) {
            printk("US des lost: %08X %08X\n",
               ((unsigned int * ) &saved_us_des_array[i])[0], ((unsigned int * ) &saved_us_des_array[i])[1]);
            res ++;
        }
        else if (data_ptr_num[i] > 1) {
            printk("US des dup : %08X %08X ==> %d\n",
               ((unsigned int * ) &saved_us_des_array[i])[0], ((unsigned int * ) &saved_us_des_array[i])[1],
                data_ptr_num[i]);
            res ++;
        }
    }

    if (res == 0) {
        printk("US descriptor data_ptrs are consistent!\n");
    }
}

static int proc_read_bonding(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    int len = 0;
    unsigned int dw;

    len += sprintf(page + len, "Bonding Config:\n");

    len += sprintf(page + len, "  BOND_CONF [0x2008]       = 0x%08x, max_frag_size=%u, pctrl_cnt= %u, dplus_fp_fcs_en=%u\n"
                                     "                                     bg_num=%u, bond_mode=%u (%s)\n",
        *(volatile u32 *)BOND_CONF, (u32)BOND_CONF->max_frag_size, (u32)BOND_CONF->polling_ctrl_cnt, (u32)BOND_CONF->dplus_fp_fcs_en,
        (u32)BOND_CONF->bg_num, (u32)BOND_CONF->bond_mode, (u32)BOND_CONF->bond_mode ? "L2 Trunking" : "L1 TC");
    len += sprintf(page + len, "                           =             e1_bond_en=%u, d5_acc_dis=%u, d5_b1_en=%u\n",
        (u32)BOND_CONF->e1_bond_en, (u32)BOND_CONF->d5_acc_dis, (u32)BOND_CONF->d5_b1_en);

    dw = *(volatile u32 *)US_BG_QMAP;
    len += sprintf(page + len, "  US_BG_QMAP[0x2009]       = 0x%08x, qmap0=0x%02x, qmap1=0x%02x, qmap2=0x%02x, qmap3=0x%02x\n",
        dw, dw & 0xff, (dw >> 8) & 0xff, (dw >> 16) & 0xff, (dw >> 24) & 0xff );

    dw = *(volatile u32 *)US_BG_GMAP;
    len += sprintf(page + len, "  US_BG_GMAP[0x200A]       = 0x%08x, gmap0=0x%02x, gmap1=0x%02x, gmap2=0x%02x, qmap3=0x%02x\n",
        dw, dw & 0xff, (dw >> 8) & 0xff, (dw >> 16) & 0xff, (dw >> 24) & 0xff );

    dw = *(volatile u32 *)DS_BG_GMAP;
    len += sprintf(page + len, "  DS_BG_GMAP[0x200B]       = 0x%08x, gmap0=0x%02x, gmap1=0x%02x, gmap2=0x%02x, qmap3=0x%02x\n",
        dw, dw & 0xff, (dw >> 8) & 0xff, (dw >> 16) & 0xff, (dw >> 24) & 0xff );

    len += sprintf(page + len, "  CURR_TIME [0x200C]       = 0x%08x\n\n", *(volatile u32 *)TIME_STAMP);

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    len += sprintf(page + len, "Cross Pci Debug Info --- Read                  Write \n");
    len += sprintf(page + len, "    min clock cycle:     %-20u, %u\n", *__READ_MIN_CYCLES, *__WRITE_MIN_CYCLES);
    len += sprintf(page + len, "    max clock cycle:     %-20u, %u\n", *__READ_MAX_CYCLES, *__WRITE_MAX_CYCLES);
    len += sprintf(page + len, "   total access num:     %-20u, %u\n", *__READ_NUM, *__WRITE_NUM);
    len += sprintf(page + len, "     total_cycle_lo:     %-20u, %u\n",
        *__TOTAL_READ_CYCLES_LO,  *__TOTAL_WRITE_CYCLES_LO);
    len += sprintf(page + len, " *total_lo/acc_num*:     %-20u, %u\n",
        *__READ_NUM ?  *__TOTAL_READ_CYCLES_LO / *__READ_NUM   : 0 ,
        *__WRITE_NUM ? *__TOTAL_WRITE_CYCLES_LO / *__WRITE_NUM : 0);
    len += sprintf(page + len, "  total cycle_hi_lo:     0x%08x%08x,   0x%08x%08x\n",
        *__TOTAL_READ_CYCLES_HI, *__TOTAL_READ_CYCLES_LO, * __TOTAL_WRITE_CYCLES_HI, *__TOTAL_WRITE_CYCLES_LO);
#endif

    *eof = 1;

    return len;
}

  #ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING


typedef struct  {
    unsigned int start_test     : 1;
    unsigned int test_type      : 2;
    unsigned int test_cnt       : 29;

} PDMA_PF_TEST_CMD;

typedef struct {

    unsigned int  byte_cnt          : 10;
    unsigned int  int_mem_addr_off  : 11;
    unsigned int  res0              : 6;
    unsigned int  eop               : 1;
    unsigned int  pdma              : 1;
    unsigned int  sar               : 1;
    unsigned int  bc                : 2;

    unsigned int  res1              : 1;
    unsigned int  ext_mem_addr      : 29;
    unsigned int  release           : 1;
    unsigned int  pdma_cmd_type     : 1;

}PDMA_DIRA_CMD;

#define __PF_TEST_CMD                      ((volatile PDMA_PF_TEST_CMD *) SB_BUFFER_BOND(2, 0x51AC))
#define __RW_CYCLES                         SB_BUFFER_BOND(2, 0x51AD)

#define __RW_PERF_TEST_PDMA_CMD0            SB_BUFFER_BOND(2, 0x51AE)
#define __RW_PERF_TEST_PDMA_CMD1            SB_BUFFER_BOND(2, 0x51AF)

#define __RW_PERF_TEST_PDMA_CMD             ((volatile PDMA_DIRA_CMD *) __RW_PERF_TEST_PDMA_CMD0)

#define PFT_READ_ONLY                       2
#define PFT_WRITE_ONLY                      1
#define PFT_READ_WRITE                      (PFT_READ_ONLY | PFT_WRITE_ONLY)


void init_pdma_pftest_pdma_cmd(PDMA_DIRA_CMD * p_cmd, unsigned int offset, unsigned int byte_cnt)
{
    // pdma_cmd0.bc        = 3
    // pdma_cmd0.sar       = 0
    // pdma_cmd0.pdma      = 1
    // pdma_cmd0.eop       = 1
    // pdma_cmd0.reserved  = 0

    p_cmd->byte_cnt          = byte_cnt;
    p_cmd->int_mem_addr_off  = offset;
    p_cmd->res0              = 0;
    p_cmd->eop               = 1;
    p_cmd->pdma              = 1;
    p_cmd->sar               = 0;
    p_cmd->bc                = 3;

    // pdma_cmd1.pdma_cmd_type = 1
    // pdma_cmd1.release       = 0
    // pdma_cmd1.reserve       = 0

    p_cmd->res1              = 0;
    p_cmd->ext_mem_addr      = *B1_DES_PDMA_BAR + offset;
    p_cmd->release           = 0;
    p_cmd->pdma_cmd_type     = 1;
}

void issue_test_cmd(unsigned int test_type, unsigned int offset, unsigned int byte_cnt, unsigned int test_cnt)
{
    int i = 0;
    PDMA_PF_TEST_CMD test_cmd;
    PDMA_DIRA_CMD pdma_cmd;

    // clear idlekeep
    *SFSM_CFG0 = ( *SFSM_CFG0 ) & ( ~ ( 1 << 15) );
    *SFSM_CFG1 = ( *SFSM_CFG1 ) & ( ~ ( 1 << 15) );

    // wait for enough time to release idle cw
    for(i = 0; i < 100000; ++i);

    test_cmd = *__PF_TEST_CMD;
    if (test_cmd.start_test == 1) {
        printk("Error! A test is on-going, please wait \n");
        return;
    }
    if (offset > 63 || byte_cnt > 384 || byte_cnt == 0) {
        printk("Error parameters, offset in [0..63], and byte_cnt in [1..384]\n");
        return;
    }

    init_pdma_pftest_pdma_cmd(&pdma_cmd, offset, byte_cnt);
    * __RW_PERF_TEST_PDMA_CMD = pdma_cmd;

    test_cmd.test_type  = test_type;
    test_cmd.test_cnt   = test_cnt;
    test_cmd.start_test = 1;
    *__PF_TEST_CMD      = test_cmd;
}

void check_pf_test_result(void)
{
    PDMA_PF_TEST_CMD test_cmd;
    PDMA_DIRA_CMD pdma_cmd;
    unsigned int cycles;
    unsigned int rate;

    test_cmd = *__PF_TEST_CMD;
    if (test_cmd.start_test == 1 ) {
        printk("Error! A test is on-going, please wait \n");
        return;
    }

    if(test_cmd.test_cnt == 0 || test_cmd.test_type == 0 ) {
        printk("No test performed\n");
        return;
    }

    pdma_cmd = * __RW_PERF_TEST_PDMA_CMD;
    cycles = *__RW_CYCLES;

    rate = 0;
    if (cycles) {
        // we assume PPE running at 432Mhz
        unsigned int ratio = 1;

        if (test_cmd.test_type == PFT_READ_WRITE)
            ratio = 2;

        rate = test_cmd.test_cnt * pdma_cmd.byte_cnt * ratio * 8 * 432 / cycles ;
    }

    printk("Test Parameters: test_type = %5s, offset = %3d, byte_cnt = %3d, test_cnt = %d\n"
           "Test Results:  ppe_cycle = %10d, Throughput = %10d Mbps\n",
           test_cmd.test_type == PFT_READ_ONLY ? "read" : (test_cmd.test_type == PFT_WRITE_ONLY ?"write" : "R/W"),
           pdma_cmd.int_mem_addr_off, pdma_cmd.byte_cnt, test_cmd.test_cnt,
           cycles, rate);

}

  #endif

void check_ds_ll_list(volatile struct DS_BOND_GIF_LL_CTXT * p_ll_ctxt, unsigned char * leading)
{
    unsigned int sb_curr_des_ptr;
    char c_check_result[16];
    unsigned int des_num;

    des_num = p_ll_ctxt->des_num;
    sb_curr_des_ptr = p_ll_ctxt->head_ptr;

    while(des_num > 1) {
        sb_curr_des_ptr = ((volatile struct DS_BOND_GIF_LL_DES *)SB_BUFFER_BOND(2, sb_curr_des_ptr))->next_des_ptr;
        des_num --;
    }

    if ( (des_num == 1) && (sb_curr_des_ptr != p_ll_ctxt->tail_ptr) ) {
        sprintf( c_check_result, "!=0x%04x", sb_curr_des_ptr);
    }
    else {
        sprintf( c_check_result, "==0x%04x", sb_curr_des_ptr);
    }

    printk("%s", leading);
    printk("head_ptr: 0x%04x  tail_ptr: 0x%04x [%s]  des_num: %-3d  fh_valid: %d  sid:%05d, sop: %d eop: %d\n",
           p_ll_ctxt->head_ptr, p_ll_ctxt->tail_ptr, c_check_result,
           p_ll_ctxt->des_num, p_ll_ctxt->fh_valid, p_ll_ctxt->sid, p_ll_ctxt->sop,  p_ll_ctxt->eop);
}

static int proc_write_bonding(struct file *file, const char *buf, unsigned long count, void *data)
{
    char str[2048];
    char *p;
    char * tokens[5];
    int finished, token_num;

    int len, rlen;

    const int cw_pg_size = 17;

    len = count < sizeof(str) ? count : sizeof(str) - 1;
    rlen = len - copy_from_user(str, buf, len);
    while ( rlen && str[rlen - 1] <= ' ' )
        rlen--;
    str[rlen] = 0;
    for ( p = str; *p && *p <= ' '; p++, rlen-- );
    if ( !*p )
    {
        // MOD_DEC_USE_COUNT;
        return 0;
    }

    token_num = tokenize(p, " \t", tokens, 5, &finished);
    if (token_num <= 0) {
        // MOD_DEC_USE_COUNT;
        return 0;
    }


    if ( strincmp(tokens[0], "help", 4) == 0) {
        printk("echo help > /proc/eth/bonding ==> \n\tprint this help message\n");

        printk("echo clear_rx_cb [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear rx cell/codeword buffer for bc0 and bc1  \n");

        printk("echo clear_rx_cb[0|1] [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear rx cell/codeword buffer for bc0 or bc1  \n");

        printk("echo clear_tx_cb [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear tx cell/codeword buffer for bc0 and bc1  \n");

        printk("echo clear_tx_cb[0|1] [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear tx cell/codeword buffer for bc0 or bc1  \n");

        printk("echo clear_cb[0|1] [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear rx and tx cell/codeword buffer for bc0 or bc1  \n");

        printk("echo clear_eth_dbuf [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear eth rx data buffer  \n");

        printk("echo clear_eth_cbuf [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear eth rx control buffer  \n");

        printk("echo clear_ethbuf [start_pg] [clear_pg_num] > /proc/eth/bonding \n");
        printk("   :clear eth rx data and control buffer  \n");

        printk("echo read_ethbuf [start_pg] [print_pg_num] > /proc/eth/bonding \n");
        printk("   :clear eth rx data and control buffer  \n");

        printk("echo read_rx_cb[0|1] [start_pg] [print_pg_num] > /proc/eth/bonding \n");
        printk("   :read rx cell/codeword buffer for bc0 or bc1  \n");

        printk("echo read_tx_cb[0|1] [start_pg] [print_pg_num] > /proc/eth/bonding \n");
        printk("   :read tx cell/codeword buffer for bc0 or bc1  \n");

        printk("echo us_bg_ctxt > /proc/eth/bonding \n");
        printk("   : US - display us_bg_ctxt\n");

        printk("echo ds_bg_ctxt > /proc/eth/bonding \n");
        printk("   : DS - display ds_bg_ctxt\n");

        printk("echo us_bond_gif_mib > /proc/eth/bonding \n");
        printk("   : US - display us_bond_gif_mib\n");

        printk("echo clear_us_bond_gif_mib > /proc/eth/bonding \n");
        printk("   : US - clear us_bond_gif_mib\n");

        printk("echo ds_bond_gif_mib > /proc/eth/bonding \n");
        printk("   : DS - display ds_bond_gif_mib\n");

        printk("echo clear_ds_bond_gif_mib > /proc/eth/bonding \n");
        printk("   : DS - clear ds_bond_gif_mib\n");

        printk("echo ds_bg_mib > /proc/eth/bonding \n");
        printk("   : DS - display ds_bg_mib\n");

        printk("echo clear_ds_bg_mib > /proc/eth/bonding \n");
        printk("   : DS - clear ds_bg_mib\n");

        printk("echo ds_link_list > /proc/eth/bonding \n");
        printk("   : DS - check link list\n");

        printk("echo rx_bc_cfg > /proc/eth/bonding \n");
        printk("   : read RX bear channel config\n");

        printk("echo tx_bc_cfg > /proc/eth/bonding \n");
        printk("   : read TX bear channel config\n");

        printk("echo rx_gif_cfg [0|1|2|3] > /proc/eth/bonding \n");
        printk("   : read RX gamma interface config\n");

        printk("echo tx_gif_cfg [0|1|2|3] > /proc/eth/bonding \n");
        printk("   : read TX gamma interface config\n");

        printk("echo idlekeep > /proc/eth/bonding \n");
        printk("   : read idle keep of BC0 and BC1\n");

        printk("echo set idlekeep[0|1] > /proc/eth/bonding \n");
        printk("   : set idle keep of BC0 or BC1\n");

        printk("echo clear idlekeep[0|1] > /proc/eth/bonding \n");
        printk("   : clear idle keep of BC0 or BC1\n");

        printk("echo clear_dbg_info > /proc/eth/bonding \n");
        printk("   : clear cross pci debug info\n");

        printk("echo switch_mode <L1|L2> > /proc/eth/bonding \n");
        printk("   : switch to L1 mode (PTM-TC bonding) or L2 (ethernet thunk mode)\n");

        printk("echo check_ds_des > /proc/eth/bonding \n");
        printk("   : check ds descriptor consistency\n");

        printk("echo check_us_des > /proc/eth/bonding \n");
        printk("   : check us descriptor consistency\n");

  #ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
        printk("echo pdma_perf_test <test_type> <offset> <byte_num> [read_times=10000] > /proc/eth/bonding \n");
        printk("   : pdma cross PCI read performance test\n");
        printk("     test_type = [r|w|rw], read, write, or read write \n");
        printk("     offset in range [0..63]; byte_num [1..384], read_times >= 10 \n");

        printk("echo check_pf_result \n");
        printk("   : check pdma cross PCI R/W performance test result\n");
  #endif
    }
  #ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    else if ( strincmp(tokens[0], "pdma_perf_test", 14) == 0 ) {
        //SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        if (token_num < 4) {
            printk("echo pdma_perf_test <test_type> <offset> <byte_num> [read_times=10000] > /proc/eth/bonding \n");
            printk("   : pdma cross PCI read performance test\n");
            printk("     test_type = [r|w|rw], read, write, or read write \n");
            printk("     offset in range [0..63]; byte_num [1..384], read_times >= 10 \n");
        }
        else {
            unsigned int offset, byte_num, test_cnt;
            unsigned int test_type;
            test_cnt = 10000;

            if(strincmp(tokens[1], "rw", 2) == 0)
                test_type = PFT_READ_WRITE;
            else if(strincmp(tokens[1], "r", 1) == 0)
                test_type = PFT_READ_ONLY;
            else if(strincmp(tokens[1], "w", 1) == 0)
                test_type = PFT_WRITE_ONLY;
            else
                test_type = PFT_READ_WRITE;

            offset =  simple_strtoul(tokens[2], NULL, 0);
            byte_num =  simple_strtoul(tokens[3], NULL, 0);
            if(token_num >= 5) {
                test_cnt =  simple_strtoul(tokens[4], NULL, 0);
            }
            if ( test_cnt == 0)
                test_cnt = 10000;
            issue_test_cmd(test_type, offset, byte_num, test_cnt);
        }
        //RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "check_pf_result", 15) == 0 ) {
        //SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);
        check_pf_test_result();
        //RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
  #endif
    else if ( strincmp(tokens[0], "check_ds_des", 12) == 0 ) {
        check_ds_des_array();
    }
    else if ( strincmp(tokens[0], "check_us_des", 12) == 0 ) {
        check_us_des_array();
    }
    else if ( strincmp(tokens[0], "switch_mode", 11) == 0 ) {
        if ( (token_num > 1) &&  strincmp(tokens[1], "L1", 2) == 0 ) {
            if (BOND_CONF->e1_bond_en && BOND_CONF->bond_mode == L2_ETH_TRUNK_MODE) {
                // current we are in L1 ptmtc bonding bonding mode
                switch_to_ptmtc_bonding();
            }
        }
        else if ( (token_num > 1) &&  strincmp(tokens[1], "L2", 2) == 0 ) {
            if (BOND_CONF->e1_bond_en && BOND_CONF->bond_mode == L1_PTM_TC_BOND_MODE) {
                // current we are in L1 ptmtc bonding bonding mode
                switch_to_eth_trunk();
            }
        }
        else {
            printk("usage: 'switch_mode <L1|L2> > /proc/eth/bonding'\n");
            printk("        :switch to L1 mode (PTM-TC bonding) or L2 (ethernet thunk mode)\n");
        }

    }
    else if ( strincmp(tokens[0], "clear_dbg_info", 13) == 0 ) {

        SAVE_AND_SWITCH_PPE_BASE_TO_BM (saved_ppe_base_addr);

        *__READ_MAX_CYCLES  = 0;
        *__WRITE_MAX_CYCLES = 0;
        *__READ_MIN_CYCLES  = 0xFFFFFFFF;
        *__WRITE_MIN_CYCLES = 0xFFFFFFFF;
        *__READ_NUM         = 0;
        *__WRITE_NUM        = 0;
        *__TOTAL_READ_CYCLES_HI  = 0;
        *__TOTAL_READ_CYCLES_LO  = 0;
        *__TOTAL_WRITE_CYCLES_HI = 0;
        *__TOTAL_WRITE_CYCLES_LO = 0;

        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "read_ethbuf", 10) == 0 ) {


        unsigned int start_pg;
        unsigned int print_pg_num;
        unsigned int num;
        unsigned int pg_size;
        volatile unsigned int *dbase, *cbase;
        unsigned int pnum, i;
        volatile unsigned int * cw;
        SAVE_AND_SWITCH_PPE_BASE_TO_SM (saved_ppe_base_addr);

        dbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );
        cbase = (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_cba * )DM_RXCB)->cbase + 0x2000);

        pnum = ( ( struct dmrx_cfg * )DM_RXCFG)->pnum;
        pg_size =  ( ( ( struct  dmrx_cfg * )DM_RXCFG)->psize + 1) * 32 / 4;

        start_pg = 0;
        print_pg_num = pnum;

        if (token_num >= 2)
            start_pg =  simple_strtoul(tokens[1], NULL, 0);
        if (token_num >= 3)
            print_pg_num =  simple_strtoul(tokens[2], NULL, 0);

        start_pg = start_pg % pnum;
        if(print_pg_num > pnum)
            print_pg_num = pnum;

        printk("PTM TX BC 0 CELL data/ctrl buffer:\n\n");
        for(i = start_pg, num = 0 ; num < print_pg_num ; num ++, i = (i + 1) % pnum ) {

            struct ctrl_dmrx_2_fw * p_ctrl_dmrx;
            struct ctrl_fw_2_dsrx * p_ctrl_dsrx;
            int j = 0;

            cw = dbase + i * pg_size;

            p_ctrl_dmrx = (struct ctrl_dmrx_2_fw *) ( &cbase[i]);
            p_ctrl_dsrx = (struct ctrl_fw_2_dsrx *) ( &cbase[i]);


            for(j =0; j < pg_size; j=j+4) {

                if(j==0) {

                    printk("Pg_id %2d: %08x %08x %08x %08x ", i, cw[0], cw[1], cw[2], cw[3]);
                    printk("pg_val: %2d, byte_off: %-3d, cos: %x \n", p_ctrl_dmrx->pg_val, p_ctrl_dmrx->byte_off, p_ctrl_dmrx->cos);
                }


                else if(j==4) {
                    printk("          %08x %08x %08x %08x ",  cw[j], cw[j+1], cw[j+2], cw[j+3]);
                    printk("bytes_cnt: %-3d, eof: %x \n", p_ctrl_dmrx->bytes_cnt, p_ctrl_dmrx->eof);
                }

                else if(j==12) {
                    printk("          %08x %08x %08x %08x ",  cw[j], cw[j+1], cw[j+2], cw[j+3]);
                    printk("pkt_status: %x, flag_len: %x, byte_off: %-3d, acc_sel: %x\n", p_ctrl_dsrx->pkt_status, p_ctrl_dsrx->flag_len, p_ctrl_dsrx->byte_off,p_ctrl_dsrx->acc_sel );
                }

                else if(j==16) {
                    printk("          %08x %08x %08x %08x ",  cw[j], cw[j+1], cw[j+2], cw[j+3]);
                    printk("fcs_en: %x, cos: %x, bytes_cnt: %-3d, eof: %x\n", p_ctrl_dsrx->fcs_en, p_ctrl_dsrx->cos, p_ctrl_dsrx->bytes_cnt,p_ctrl_dsrx->eof );
                }

                else {
                    printk("          %08x %08x %08x %08x \n",  cw[j], cw[j+1], cw[j+2], cw[j+3]);
                }

                }

                printk("\n");

           }

        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

    else if ( strincmp(tokens[0], "read_tx_cb", 10) == 0 ) {

        unsigned int start_pg;
        unsigned int print_pg_num;
        unsigned int num;

        if(tokens[0][10] == '0' || tokens[0][10] == '\0') {

            volatile unsigned int *dbase0;
            unsigned int pnum0, i;
            volatile unsigned int * cw;

            dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000);
            pnum0 = FFSM_CFG(0)->pnum;

            start_pg = 0;
            print_pg_num = pnum0;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                print_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum0;
            if(print_pg_num > pnum0)
                print_pg_num = pnum0;

            printk("PTM TX BC 0 CELL data/ctrl buffer:\n\n");
            for(i = start_pg, num = 0 ; num < print_pg_num ; num ++, i = (i + 1) % pnum0 ) {

                cw = dbase0 + i * cw_pg_size;
                printk("CW %2d: %08x \n"          , i, cw[0]);
                printk("       %08x %08x %08x %08x\n",  cw[1], cw[2], cw[3], cw[4]);
                printk("       %08x %08x %08x %08x\n",  cw[5], cw[6], cw[7], cw[8]);
                printk("       %08x %08x %08x %08x\n",  cw[9], cw[10], cw[11], cw[12]);
                printk("       %08x %08x %08x %08x\n",  cw[13], cw[14], cw[15], cw[16]);
            }

        }

        if (tokens[0][10] == '1' || tokens[0][10] == '\0'){

            volatile unsigned int *dbase1;
            unsigned int pnum1, i;
            volatile unsigned int * cw;

            dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000);
            pnum1 = FFSM_CFG(1)->pnum;

            start_pg = 0;
            print_pg_num = pnum1;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                print_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum1;
            if(print_pg_num > pnum1)
                print_pg_num = pnum1;

            printk("PTM TX BC 1 CELL data/ctrl buffer:\n\n");
            for(i = start_pg, num = 0 ; num < print_pg_num ; num ++, i = (i + 1) % pnum1 ) {

                cw = dbase1 + i * cw_pg_size;
                printk("CW %2d: %08x \n"          , i, cw[0]);
                printk("       %08x %08x %08x %08x\n",  cw[1], cw[2], cw[3], cw[4]);
                printk("       %08x %08x %08x %08x\n",  cw[5], cw[6], cw[7], cw[8]);
                printk("       %08x %08x %08x %08x\n",  cw[9], cw[10], cw[11], cw[12]);
                printk("       %08x %08x %08x %08x\n\n",  cw[13], cw[14], cw[15], cw[16]);
            }

        }

    }

    else if ( strincmp(tokens[0], "read_rx_cb", 10) == 0 ) {

        unsigned int start_pg;
        unsigned int print_pg_num;
        unsigned int num;

        unsigned char * cwid_txt[8] = {
            "All_data",
            "End_of_frame",
            "Start_while_tx",
            "All_idle",
            "Start_while_idle",
            "All_idle_nosync",
            "Error",
            "Res"
         };

        if(tokens[0][10] == '0' || tokens[0][10] == '\0') {

            volatile unsigned int *dbase0, *cbase0;
            unsigned int pnum0, i;
            volatile unsigned int * cw;

            dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000);
            cbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000);

            pnum0 = SFSM_CFG(0)->pnum;

            start_pg = 0;
            print_pg_num = pnum0;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                print_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum0;
            if(print_pg_num > pnum0)
                print_pg_num = pnum0;

            printk("PTM RX BC 0 CELL data/ctrl buffer (pnum0 = %d):\n\n", pnum0);
            for(i = start_pg, num = 0 ; num < print_pg_num ; num ++, i = (i + 1) % pnum0 ) {

                struct PTM_CW_CTRL * p_ctrl;

                cw = dbase0 + i * cw_pg_size;
                p_ctrl = (struct PTM_CW_CTRL *) ( &cbase0[i]);

                printk("CW %2d: %08x                            ", i, cw[0]);
                printk("cwid=%d [%-16s], cwer=%d, preempt=%d, short=%d\n",
                        p_ctrl->cwid, cwid_txt[p_ctrl->cwid], p_ctrl->cwer, p_ctrl->preempt, p_ctrl->shrt);

                printk("       %08x %08x %08x %08x ", cw[1], cw[2], cw[3], cw[4]);
                printk("state=%d, bad=%d, ber=%-3d, spos=%-3d, ffbn=%-3d\n",
                        p_ctrl->state, p_ctrl->bad, p_ctrl->ber, p_ctrl->spos, p_ctrl->ffbn);

                printk("       %08x %08x %08x %08x\n",  cw[5], cw[6], cw[7], cw[8]);
                printk("       %08x %08x %08x %08x\n",  cw[9], cw[10], cw[11], cw[12]);
                printk("       %08x %08x %08x %08x\n",  cw[13], cw[14], cw[15], cw[16]);

            }

        }

        if (tokens[0][10] == '1' || tokens[0][10] == '\0'){

            volatile unsigned int *dbase1, *cbase1;
            unsigned int pnum1, i;
            volatile unsigned int * cw;

            dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000);
            cbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000);

            pnum1 = SFSM_CFG(1)->pnum;

            start_pg = 0;
            print_pg_num = pnum1;


            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                print_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum1;
            if(print_pg_num > pnum1)
                print_pg_num = pnum1;

            printk("PTM RX BC 1 CELL data/ctrl buffer:\n\n");
            for(i = start_pg, num = 0 ; num < print_pg_num ; num ++, i = (i + 1) % pnum1 ) {

                struct PTM_CW_CTRL * p_ctrl;

                cw = dbase1 + i * cw_pg_size;
                p_ctrl = (struct PTM_CW_CTRL *) ( &cbase1[i]);

                printk("CW %2d: %08x                            ", i, cw[0]);
                printk("cwid=%d [%16s], cwer=%d, preempt=%d, short=%d\n",
                        p_ctrl->cwid, cwid_txt[p_ctrl->cwid], p_ctrl->cwer, p_ctrl->preempt, p_ctrl->shrt);

                printk("       %08x %08x %08x %08x ", cw[1], cw[2], cw[3], cw[4]);
                printk("state=%d, bad=%d, ber=%-3d, spos=%-3d, ffbn=%-3d\n",
                        p_ctrl->state, p_ctrl->bad, p_ctrl->ber, p_ctrl->spos, p_ctrl->ffbn);

                printk("       %08x %08x %08x %08x\n",  cw[5], cw[6], cw[7], cw[8]);
                printk("       %08x %08x %08x %08x\n",  cw[9], cw[10], cw[11], cw[12]);
                printk("       %08x %08x %08x %08x\n",  cw[13], cw[14], cw[15], cw[16]);

            }

        }
    }
     else if ( strincmp(tokens[0], "clear_tx_cb", 11) == 0 ) {

        unsigned int start_pg;
        unsigned int clear_pg_num;

        if(tokens[0][11] == '0' || tokens[0][11] == '\0') {
            volatile unsigned int *dbase0;
            unsigned int pnum0;

            dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000);
            pnum0 = FFSM_CFG(0)->pnum;

            start_pg = 0;
            clear_pg_num = pnum0;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum0;
            if(clear_pg_num > pnum0)
                clear_pg_num = pnum0;

            dbase0 = dbase0 + start_pg * cw_pg_size;

            memset((void *)dbase0, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

        }

        if (tokens[0][11] == '1' || tokens[0][11] == '\0'){

            volatile unsigned int *dbase1;
            unsigned int pnum1;

            dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000);
            pnum1 = FFSM_CFG(1)->pnum;

            start_pg = 0;
            clear_pg_num = pnum1;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum1;
            if(clear_pg_num > pnum1)
                clear_pg_num = pnum1;

            dbase1 = dbase1 + start_pg * cw_pg_size;

            memset((void *)dbase1, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

        }
    }

    else if ( strincmp(tokens[0], "clear_rx_cb", 11) == 0 ) {

        unsigned int start_pg;
        unsigned int clear_pg_num;

        if(tokens[0][11] == '0' || tokens[0][11] == '\0'){

            volatile unsigned int *dbase0, *cbase0;
            unsigned int pnum0;

            dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000);
            cbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000);

            pnum0 = SFSM_CFG(0)->pnum;

            start_pg = 0;
            clear_pg_num = pnum0;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum0;
            if(clear_pg_num > pnum0)
                clear_pg_num = pnum0;

            dbase0 = dbase0 + start_pg * cw_pg_size;
            cbase0 = &cbase0[start_pg];

            memset((void *)dbase0, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);
            memset((void *)cbase0, 0,  sizeof(unsigned int ) * clear_pg_num);

        }

        if (tokens[0][11] == '1' || tokens[0][11] == '\0'){

            volatile unsigned int *dbase1, *cbase1;
            unsigned int pnum1;

            dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000);
            cbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000);
            pnum1 = SFSM_CFG(1)->pnum;

            start_pg = 0;
            clear_pg_num = pnum1;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % pnum1;
            if(clear_pg_num > pnum1)
                clear_pg_num = pnum1;

            dbase1 = dbase1 + start_pg * cw_pg_size;
            cbase1 = &cbase1[start_pg];

            memset((void *)dbase1, 0,  cw_pg_size * sizeof(unsigned int )* clear_pg_num);
            memset((void *)cbase1, 0,  sizeof(unsigned int ) * clear_pg_num);
        }
    }

    else if ( strincmp(tokens[0], "clear_cb", 8) == 0 ) {

        unsigned int start_pg;
        unsigned int clear_pg_num;

        if(tokens[0][8] == '0' || tokens[0][8] == '\0'){

            //Clear tx_cb0
            volatile unsigned int *tx_dbase0;
            unsigned int tx_pnum0;
            volatile unsigned int *rx_dbase0, *rx_cbase0;
            unsigned int rx_pnum0;


            tx_dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(0)->dbase + 0x2000);
            tx_pnum0 = FFSM_CFG(0)->pnum;

            start_pg = 0;
            clear_pg_num = tx_pnum0;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % tx_pnum0;

            if(clear_pg_num > tx_pnum0)
                clear_pg_num = tx_pnum0;

            tx_dbase0 = tx_dbase0 + start_pg * cw_pg_size;

            memset((void *)tx_dbase0, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

            //Clear rx_cb0

            rx_dbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(0)->dbase + 0x2000);
            rx_cbase0 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(0)->cbase + 0x2000);

            rx_pnum0 = SFSM_CFG(0)->pnum;

            start_pg = 0;
            clear_pg_num = rx_pnum0;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % rx_pnum0;
            if(clear_pg_num > rx_pnum0)
                clear_pg_num = rx_pnum0;

            rx_dbase0 = rx_dbase0 + start_pg * cw_pg_size;
            rx_cbase0 = &rx_cbase0[start_pg];

            memset((void *)rx_dbase0, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);
            memset((void *)rx_cbase0, 0,  sizeof(unsigned int ) * clear_pg_num);

        }

        if(tokens[0][8] == '1' || tokens[0][8] == '\0'){

            //Clear tx_cb1
            volatile unsigned int *tx_dbase1;
            unsigned int tx_pnum1;
            volatile unsigned int *rx_dbase1, *rx_cbase1;
            unsigned int rx_pnum1;


            tx_dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, FFSM_DBA(1)->dbase + 0x2000);
            tx_pnum1 = FFSM_CFG(1)->pnum;

            start_pg = 0;
            clear_pg_num = tx_pnum1;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % tx_pnum1;

            if(clear_pg_num > tx_pnum1)
                clear_pg_num = tx_pnum1;

            tx_dbase1 = tx_dbase1 + start_pg * cw_pg_size;

            memset((void *)tx_dbase1, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);

            rx_dbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_DBA(1)->dbase + 0x2000);
            rx_cbase1 = (volatile unsigned int *)SB_BUFFER_BOND(2, SFSM_CBA(1)->cbase + 0x2000);
            rx_pnum1 = SFSM_CFG(1)->pnum;

            start_pg = 0;
            clear_pg_num = rx_pnum1;

            if (token_num >= 2)
                start_pg =  simple_strtoul(tokens[1], NULL, 0);
            if (token_num >= 3)
                clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

            start_pg = start_pg % rx_pnum1;
            if(clear_pg_num > rx_pnum1)
                clear_pg_num = rx_pnum1;

            rx_dbase1 = rx_dbase1 + start_pg * cw_pg_size;
            rx_cbase1 = &rx_cbase1[start_pg];

            memset((void *)rx_dbase1, 0,  cw_pg_size * sizeof(unsigned int ) * clear_pg_num);
            memset((void *)rx_cbase1, 0,  sizeof(unsigned int )* clear_pg_num);

        }

    }

    else if ( strincmp(tokens[0], "clear_eth_dbuf", 14) == 0) {
        volatile u32 *p_dbase;
        unsigned int pg_size = 0;
        unsigned int pg_num  = 0;
        unsigned int start_pg;
        unsigned int clear_pg_num;
        SAVE_AND_SWITCH_PPE_BASE_TO_SM(saved_ppe_base_addr);

        pg_num  =  ( ( struct dmrx_cfg * )DM_RXCFG)->pnum;
        pg_size =  ( ( ( struct  dmrx_cfg * )DM_RXCFG)->psize + 1) * 32 / 4;

        p_dbase =  (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase + 0x2000 );

        start_pg = 0;
        clear_pg_num = pg_num;

        if (token_num >= 2)
            start_pg =  simple_strtoul(tokens[1], NULL, 0);
        if (token_num >= 3)
            clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

        start_pg = start_pg % pg_num;
        if(clear_pg_num > pg_num)
            clear_pg_num = pg_num;

        p_dbase = p_dbase + start_pg * pg_size;

        memset((void *)p_dbase, 0, pg_size * sizeof(unsigned int) * clear_pg_num);
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "clear_eth_cbuf", 14) == 0) {
        volatile u32 *p_cbase;
        unsigned int pg_num  = 0;
        unsigned int start_pg;
        unsigned int clear_pg_num;
        SAVE_AND_SWITCH_PPE_BASE_TO_SM(saved_ppe_base_addr);

        pg_num  =  ( ( struct  dmrx_cfg * )DM_RXCFG )->pnum;
        p_cbase =  (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct  dmrx_cba * )DM_RXCB)->cbase + 0x2000);

        start_pg = 0;
        clear_pg_num = pg_num;

        if (token_num >= 2)
            start_pg =  simple_strtoul(tokens[1], NULL, 0);
        if (token_num >= 3)
            clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

        start_pg = start_pg % pg_num;
        if(clear_pg_num > pg_num)
            clear_pg_num = pg_num;

        p_cbase = &p_cbase[start_pg];


        memset((void *)p_cbase, 0, sizeof(unsigned int) * clear_pg_num );
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

    else if ( strincmp(tokens[0], "clear_ethbuf", 12) == 0)
    {
        volatile u32 *p_dbase, *p_cbase;
        unsigned int pg_size = 0;
        unsigned int pg_num  = 0;
        unsigned int start_pg;
        unsigned int clear_pg_num;
        SAVE_AND_SWITCH_PPE_BASE_TO_SM(saved_ppe_base_addr);

        //Clear ethernet data buffer
        pg_num  =  ( ( struct dmrx_cfg * )DM_RXCFG)->pnum;
        pg_size =  ( ( ( struct  dmrx_cfg * )DM_RXCFG)->psize + 1) * 32 / 4;

        p_dbase =  (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct dmrx_dba * )DM_RXDB)->dbase +0x2000);

        start_pg = 0;
        clear_pg_num = pg_num;

        if (token_num >= 2)
            start_pg =  simple_strtoul(tokens[1], NULL, 0);
        if (token_num >= 3)
            clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

        start_pg = start_pg % pg_num;
        if(clear_pg_num > pg_num)
            clear_pg_num = pg_num;

        p_dbase = p_dbase + start_pg * pg_size;

        memset((void *)p_dbase, 0, pg_size * sizeof(unsigned int) * clear_pg_num);

        //Clear ethernet control buffer
        pg_num  =  ( ( struct  dmrx_cfg * )DM_RXCFG )->pnum;
        p_cbase =  (volatile unsigned int *)SB_BUFFER_BOND(2, ( ( struct  dmrx_cba * )DM_RXCB)->cbase + 0x2000 );

        start_pg = 0;
        clear_pg_num = pg_num;

        if (token_num >= 2)
            start_pg =  simple_strtoul(tokens[1], NULL, 0);
        if (token_num >= 3)
            clear_pg_num =  simple_strtoul(tokens[2], NULL, 0);

        start_pg = start_pg % pg_num;
        if(clear_pg_num > pg_num)
            clear_pg_num = pg_num;

        p_cbase = &p_cbase[start_pg];

        memset((void *)p_cbase, 0, sizeof(unsigned int) * clear_pg_num );
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

    else if ( strincmp(tokens[0], "us_bg_ctxt", 10) == 0 )  {
        int k = 0;
        unsigned int pkt_status = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);

        for(; k<4; k++) {
            printk( "US_BG_CTXT[%d]\n", k);
            printk( "         sid: 0x%04x     sop: %x      eop: %x  pkt_status: %x    des_addr: 0x%04x \n",
                US_BG_CTXT_BASE(k)->sid, US_BG_CTXT_BASE(k)->sop, US_BG_CTXT_BASE(k)->eop,
                US_BG_CTXT_BASE(k)->pkt_status, US_BG_CTXT_BASE(k)->des_addr);
            printk( "    data_ptr: 0x%08x qid: %x   gif_id: %x     rem_len: %-4u \n\n",
                US_BG_CTXT_BASE(k)->data_ptr, US_BG_CTXT_BASE(k)->qid, US_BG_CTXT_BASE(k)->gif_id,
                US_BG_CTXT_BASE(k)->rem_len);

            pkt_status = US_BG_CTXT_BASE(k)->pkt_status;

            if(pkt_status == 1) {
                volatile struct US_BOND_PKT_DES * p_des = (volatile struct US_BOND_PKT_DES *) SB_BUFFER_BOND(2, US_BG_CTXT_BASE(k)->des_addr);
                int q_no = (US_BG_CTXT_BASE(k)->des_addr - 0x5c00)/0x80;
                printk("    ==>US_BOND_PKT_DES_BASE[%d]\n", q_no);
                printk("              own: %x         c: %x        sop: %x           eop: %x           byte_off: %d \n",
                    p_des->own,p_des->c,
                    p_des->sop,p_des->eop,
                    p_des->byte_off);
                printk("        last_frag: %x  frag_num: %-2d  data_len: %-4d   data_ptr: 0x%08x\n\n",
                    p_des->last_frag,p_des->frag_num,
                    p_des->data_len,p_des->data_ptr);

           }
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "ds_bg_ctxt", 10) == 0 )   {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        for(; k<4; k++) {
            printk( "DS_BG_CTXT[%d]\n", k);
            printk( "       link_state_chg: %u        expected_sid: 0x%04x     last_sop: %u      last_eop: %u\t  bg_pkt_state: %u\n",
                DS_BG_CTXT_BASE(k)->link_state_chg, DS_BG_CTXT_BASE(k)->expected_sid, DS_BG_CTXT_BASE(k)->last_sop,
                DS_BG_CTXT_BASE(k)->last_eop, DS_BG_CTXT_BASE(k)->bg_pkt_state);
            printk( "        noncosec_flag: %u       oversize_flag: %u       no_eop_flag: %u   no_sop_flag: %u\t   no_err_flag: %u \n",
                DS_BG_CTXT_BASE(k)->noncosec_flag, DS_BG_CTXT_BASE(k)->oversize_flag, DS_BG_CTXT_BASE(k)->no_eop_flag,
                DS_BG_CTXT_BASE(k)->no_sop_flag, DS_BG_CTXT_BASE(k)->no_err_flag);
            printk( "    curr_pkt_frag_cnt: %-3d curr_pkt_byte_cnt: %-4u \n\n",
                DS_BG_CTXT_BASE(k)->curr_pkt_frag_cnt, DS_BG_CTXT_BASE(k)->curr_pkt_byte_cnt);
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "us_bond_gif_mib", 15) == 0 ) {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        printk("US bonding gif frag mib: \n");
        for(k = 0; k < 8; k ++ ) {
            printk("    gif%d: %-u\n", k, * US_BOND_GIF_MIB(k));
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "clear_us_bond_gif_mib", 21) == 0 ) {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        for(k = 0; k < 8; k ++ ) {
            void *p = (void *)US_BOND_GIF_MIB(k);
            memset(p, 0, sizeof(unsigned int));
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "ds_bond_gif_mib", 15) == 0 ) {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        for(; k<8; k++) {
            printk( "DS_BOND_GIF_MIB[%d]\n", k);
            printk( "       total_rx_frag_cnt: %-10u total_rx_byte_cnt: %-10u overflow_frag_cnt: %-10u overflow_byte_cnt: %-10u\n",
                DS_BOND_GIF_MIB_BASE(k)->total_rx_frag_cnt, DS_BOND_GIF_MIB_BASE(k)->total_rx_byte_cnt,
                DS_BOND_GIF_MIB_BASE(k)->overflow_frag_cnt, DS_BOND_GIF_MIB_BASE(k)->overflow_byte_cnt);
            printk( "   out_of_range_frag_cnt: %-10u  missing_frag_cnt: %-10u  timeout_frag_cnt: %-10u\n\n",
                DS_BOND_GIF_MIB_BASE(k)->out_of_range_frag_cnt, DS_BOND_GIF_MIB_BASE(k)->missing_frag_cnt,
                DS_BOND_GIF_MIB_BASE(k)->timeout_frag_cnt);
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "clear_ds_bond_gif_mib", 21) == 0 ) {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        for(; k<8; k++) {
            void *p = (void *)DS_BOND_GIF_MIB_BASE(k);
            memset(p, 0, sizeof(struct DS_BOND_GIF_MIB));
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }

    else if ( strincmp(tokens[0], "ds_bg_mib", 9) == 0 ) {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        for(; k<4; k++) {
            printk( "DS_BG_MIB[%d]\n", k);
            printk( "     conform_pkt_cnt: %-10u  conform_frag_cnt: %-10u conform_byte_cnt: %-10u    no_sop_pkt_cnt: %-10u   no_sop_frag_cnt: %-10u\n",
                DS_BG_MIB_BASE(k)->conform_pkt_cnt, DS_BG_MIB_BASE(k)->conform_frag_cnt,
                DS_BG_MIB_BASE(k)->conform_byte_cnt, DS_BG_MIB_BASE(k)->no_sop_pkt_cnt, DS_BG_MIB_BASE(k)->no_sop_frag_cnt);
            printk( "     no_sop_byte_cnt: %-10u    no_eop_pkt_cnt: %-10u  no_eop_frag_cnt: %-10u   no_eop_byte_cnt: %-10u  oversize_pkt_cnt: %-10u\n",
                DS_BG_MIB_BASE(k)->no_sop_byte_cnt, DS_BG_MIB_BASE(k)->no_eop_pkt_cnt, DS_BG_MIB_BASE(k)->no_eop_frag_cnt,
                DS_BG_MIB_BASE(k)->no_eop_byte_cnt, DS_BG_MIB_BASE(k)->oversize_pkt_cnt);
            printk( "   oversize_frag_cnt: %-10u oversize_byte_cnt: %-10u noncosec_pkt_cnt: %-10u noncosec_frag_cnt: %-10u noncosec_byte_cnt: %-10u\n\n",
                DS_BG_MIB_BASE(k)->oversize_frag_cnt, DS_BG_MIB_BASE(k)->oversize_byte_cnt, DS_BG_MIB_BASE(k)->noncosec_pkt_cnt,
                DS_BG_MIB_BASE(k)-> noncosec_frag_cnt, DS_BG_MIB_BASE(k)->noncosec_byte_cnt);
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "clear_ds_bg_mib", 15) == 0 ) {
        int k = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);
        for(; k<4; k++) {
            void *p = (void *)DS_BG_MIB_BASE(k);
            memset(p, 0, sizeof(struct DS_BG_MIB));
        }
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "ds_link_list", 12) == 0 ) {
        unsigned int i;
        unsigned char buf[8];
        unsigned int total_des_num = 0;
        SAVE_AND_SWITCH_PPE_BASE_TO_BM(saved_ppe_base_addr);

        check_ds_ll_list(DS_BOND_FREE_LL_CTXT_BASE,    "Free: ");
        for( i = 0; i < 8; i ++ ) {
            sprintf(buf, "Gif%d: ", i);
            check_ds_ll_list(DS_BOND_GIF_LL_CTXT_BASE(i), buf);
        }

        for(i = 0; i < 4; i ++) {
            sprintf(buf, "BgR%d: ", i);
            check_ds_ll_list( (volatile struct DS_BOND_GIF_LL_CTXT *)
                                    ( (unsigned int *) (DS_BG_CTXT_BASE(i))  + 4 ), buf);
        }

        total_des_num = DS_BOND_FREE_LL_CTXT_BASE->des_num;
        for( i = 0; i < 8; i ++ ) {
            total_des_num += DS_BOND_GIF_LL_CTXT_BASE(i)->des_num;
        }
        for(i = 0; i < 4; i ++) {
            total_des_num +=   ( (volatile struct DS_BOND_GIF_LL_CTXT *)
                                 ( (unsigned int *) (DS_BG_CTXT_BASE(i))  + 4 )) ->des_num;
        }

        if (total_des_num != 256) {
            printk("[X] ");
        }
        else {
            printk("[Y] ");
        }
        printk("total_des_num = %d\n\n", total_des_num);
        RESTORE_PPE_BASE(saved_ppe_base_addr);
    }
    else if ( strincmp(tokens[0], "rx_bc_cfg", 9) == 0 )  {

        unsigned int bc_s = 0;
        unsigned int bc_e = 1;
        int i = 0;
        unsigned char * ls_txt[] = {
            "Looking",
            "Freewheel Sync False",
            "Synced",
            "Freewheel Sync True"
        };
        unsigned char * rs_txt[] = {
            "Out-of-Sync",
            "Synced"
        };

        if(tokens[0][10] == '\0' && token_num == 1) {
            bc_s = 0;
            bc_e = 1;
        }
        else if ( tokens[0][10] <= '1' && tokens[0][10] >= '0'  ) {
            bc_s = bc_e = tokens[0][10] - '0';
        }
        else if ( token_num >= 2 && tokens[1][0] <= '1' && tokens[1][0] >= '0'  ) {
            bc_s = bc_e = tokens[1][0] - '0';
        }

        for ( i = bc_s; i <= bc_e; i ++ ) {
            printk("TX_GIF_CFG[%d]\n", i);
            printk("    local_state: %d [ %-20s ] remote_state: %d [ %-12s ]  rx_cw_rdptr:%d \n\n",
                   RX_BC_CFG(i)->local_state, ls_txt[RX_BC_CFG(i)->local_state],
                   RX_BC_CFG(i)->remote_state, rs_txt[RX_BC_CFG(i)->remote_state],
                   RX_BC_CFG(i)->rx_cw_rdptr);
        }

    }
    else if ( strincmp(tokens[0], "tx_bc_cfg", 9) == 0 )  {
    }
    else if ( strincmp(tokens[0], "rx_gif_cfg", 10) == 0 )  {
    }
    else if ( strincmp(tokens[0], "tx_gif_cfg", 10) == 0 )  {
        unsigned int gif_s = 0;
        unsigned int gif_e = 0;
        int i = 0;
        if(tokens[0][10] == '\0' && token_num == 1) {
            gif_s = 0;
            gif_e = 3;
        }
        else if ( tokens[0][10] <= '3' && tokens[0][10] >= '0'  ) {
            gif_s = gif_e = tokens[0][10] - '0';
        }
        else if ( token_num >= 2 && tokens[1][0] <= '3' && tokens[1][0] >= '0'  ) {
            gif_s = gif_e = tokens[1][0] - '0';
        }

        for ( i = gif_s; i <= gif_e; i ++ ) {
            printk("TX_GIF_CFG[%d]\n", i);
            printk("    curr_qid: %d     fill_pkt_state: %d      post_pkt_state: %d  curr_pdma_context_ptr: 0x%04x\n"
                   "     des_qid: %d           des_addr: 0x%04x                     curr_sar_context_ptr: 0x%04x\n"
                   "    rem_data: %-4d         rem_crc: %d          rem_fh_len: %d\n"
                   "     des_dw0: 0x%08x   des_dw1: 0x%08x  des_bp_dw: 0x%08x\n\n",
                   TX_GAMMA_ITF_CFG(i)->curr_qid, TX_GAMMA_ITF_CFG(i)->fill_pkt_state, TX_GAMMA_ITF_CFG(i)->post_pkt_state, TX_GAMMA_ITF_CFG(i)->curr_pdma_context_ptr,
                   TX_GAMMA_ITF_CFG(i)->des_qid,  TX_GAMMA_ITF_CFG(i)->des_addr, TX_GAMMA_ITF_CFG(i)->curr_sar_context_ptr,
                   TX_GAMMA_ITF_CFG(i)->rem_data, TX_GAMMA_ITF_CFG(i)->rem_crc, TX_GAMMA_ITF_CFG(i)->rem_fh_len,
                   TX_GAMMA_ITF_CFG(i)->des_dw0, TX_GAMMA_ITF_CFG(i)->des_dw1, TX_GAMMA_ITF_CFG(i)->des_bp_dw);

        }

    }
    else if ( strincmp(tokens[0], "idlekeep", 8) == 0 ) {
        printk("BC0.idlekp = %d, BC1.idlekp = %d\n",
            (*SFSM_CFG0 >> 15) & 1, (*SFSM_CFG1 >> 15) & 1);
    }
    else if ( strincmp(tokens[0], "clear", 5) == 0 ) {
        if(token_num >= 2) {
            if(strincmp(tokens[1], "idlekeep", 8) == 0) {
                if(tokens[1][8] == '\0' || tokens[1][8] == '0')
                    *SFSM_CFG0 = ( *SFSM_CFG0 ) & ( ~ ( 1 << 15) );
                if(tokens[1][8] == '\0' || tokens[1][8] == '1')
                    *SFSM_CFG1 = ( *SFSM_CFG1 ) & ( ~ ( 1 << 15) );
            }
        }
    }
    else if ( strincmp(tokens[0], "set", 3) == 0 ) {
        if(token_num >= 2) {
            if(strincmp(tokens[1], "idlekeep", 8) == 0) {
                if(tokens[1][8] == '\0' || tokens[1][8] == '0')
                    *SFSM_CFG0 =  *SFSM_CFG0 | ( 1 << 15) ;
                if(tokens[1][8] == '\0' || tokens[1][8] == '1')
                    *SFSM_CFG1 =  *SFSM_CFG1 | ( 1 << 15);
            }
        }

    }


    // MOD_DEC_USE_COUNT;

    return count;
}

#endif


#if defined(DEBUG_FIRMWARE_TABLES_PROC) && DEBUG_FIRMWARE_TABLES_PROC
static INLINE int proc_buf_copy(char **pbuf, int size, off_t off, int *ppos,
		const char *str, int len) {
	if (*ppos <= off && *ppos + len > off) {
		my_memcpy(*pbuf, str + off - *ppos, *ppos + len - off);
		*pbuf += *ppos + len - off;
	} else if (*ppos > off) {
		my_memcpy(*pbuf, str, len);
		*pbuf += len;
	}
	*ppos += len;
	return *ppos >= off + size;
}
#endif

#if defined(DEBUG_DUMP_INIT) && DEBUG_DUMP_INIT
static INLINE void dump_init(void)
{
	int i;

	if ( !(g_dbg_datapath & DBG_ENABLE_MASK_DUMP_INIT) )
	return;

	printk("Share Buffer Conf:\n");
	printk("  SB_MST_SEL(%08X) = 0x%08X\n", (u32)SB_MST_SEL, *SB_MST_SEL);

	printk("ETOP:\n");
	printk("  ETOP_MDIO_CFG        = 0x%08X\n", *ETOP_MDIO_CFG);
	printk("  ETOP_MDIO_ACC        = 0x%08X\n", *ETOP_MDIO_ACC);
	printk("  ETOP_CFG             = 0x%08X\n", *ETOP_CFG);
	printk("  ETOP_IG_VLAN_COS     = 0x%08X\n", *ETOP_IG_VLAN_COS);
	printk("  ETOP_IG_DSCP_COSx(0) = 0x%08X\n", *ETOP_IG_DSCP_COSx(0));
	printk("  ETOP_IG_DSCP_COSx(1) = 0x%08X\n", *ETOP_IG_DSCP_COSx(1));
	printk("  ETOP_IG_DSCP_COSx(2) = 0x%08X\n", *ETOP_IG_DSCP_COSx(2));
	printk("  ETOP_IG_DSCP_COSx(3) = 0x%08X\n", *ETOP_IG_DSCP_COSx(3));
	printk("  ETOP_IG_PLEN_CTRL0   = 0x%08X\n", *ETOP_IG_PLEN_CTRL0);
	printk("  ETOP_ISR             = 0x%08X\n", *ETOP_ISR);
	printk("  ETOP_IER             = 0x%08X\n", *ETOP_IER);
	printk("  ETOP_VPID            = 0x%08X\n", *ETOP_VPID);

	for ( i = 0; i < 2; i++ )
	{
		printk("ENET%d:\n", i);
		printk("  ENET_MAC_CFG(%d)      = 0x%08X\n", i, *ENET_MAC_CFG(i));
		printk("  ENETS_DBA(%d)         = 0x%08X\n", i, *ENETS_DBA(i));
		printk("  ENETS_CBA(%d)         = 0x%08X\n", i, *ENETS_CBA(i));
		printk("  ENETS_CFG(%d)         = 0x%08X\n", i, *ENETS_CFG(i));
		printk("  ENETS_PGCNT(%d)       = 0x%08X\n", i, *ENETS_PGCNT(i));
		printk("  ENETS_PKTCNT(%d)      = 0x%08X\n", i, *ENETS_PKTCNT(i));
		printk("  ENETS_BUF_CTRL(%d)    = 0x%08X\n", i, *ENETS_BUF_CTRL(i));
		printk("  ENETS_COS_CFG(%d)     = 0x%08X\n", i, *ENETS_COS_CFG(i));
		printk("  ENETS_IGDROP(%d)      = 0x%08X\n", i, *ENETS_IGDROP(i));
		printk("  ENETS_IGERR(%d)       = 0x%08X\n", i, *ENETS_IGERR(i));
		printk("  ENETS_MAC_DA0(%d)     = 0x%08X\n", i, *ENETS_MAC_DA0(i));
		printk("  ENETS_MAC_DA1(%d)     = 0x%08X\n", i, *ENETS_MAC_DA1(i));
		printk("  ENETF_DBA(%d)         = 0x%08X\n", i, *ENETF_DBA(i));
		printk("  ENETF_CBA(%d)         = 0x%08X\n", i, *ENETF_CBA(i));
		printk("  ENETF_CFG(%d)         = 0x%08X\n", i, *ENETF_CFG(i));
		printk("  ENETF_PGCNT(%d)       = 0x%08X\n", i, *ENETF_PGCNT(i));
		printk("  ENETF_PKTCNT(%d)      = 0x%08X\n", i, *ENETF_PKTCNT(i));
		printk("  ENETF_HFCTRL(%d)      = 0x%08X\n", i, *ENETF_HFCTRL(i));
		printk("  ENETF_TXCTRL(%d)      = 0x%08X\n", i, *ENETF_TXCTRL(i));
		printk("  ENETF_VLCOS0(%d)      = 0x%08X\n", i, *ENETF_VLCOS0(i));
		printk("  ENETF_VLCOS1(%d)      = 0x%08X\n", i, *ENETF_VLCOS1(i));
		printk("  ENETF_VLCOS2(%d)      = 0x%08X\n", i, *ENETF_VLCOS2(i));
		printk("  ENETF_VLCOS3(%d)      = 0x%08X\n", i, *ENETF_VLCOS3(i));
		printk("  ENETF_EGCOL(%d)       = 0x%08X\n", i, *ENETF_EGCOL(i));
		printk("  ENETF_EGDROP(%d)      = 0x%08X\n", i, *ENETF_EGDROP(i));
	}

	printk("DPLUS:\n");
	printk("  DPLUS_TXDB           = 0x%08X\n", *DPLUS_TXDB);
	printk("  DPLUS_TXCB           = 0x%08X\n", *DPLUS_TXCB);
	printk("  DPLUS_TXCFG          = 0x%08X\n", *DPLUS_TXCFG);
	printk("  DPLUS_TXPGCNT        = 0x%08X\n", *DPLUS_TXPGCNT);
	printk("  DPLUS_RXDB           = 0x%08X\n", *DPLUS_RXDB);
	printk("  DPLUS_RXCB           = 0x%08X\n", *DPLUS_RXCB);
	printk("  DPLUS_RXCFG          = 0x%08X\n", *DPLUS_RXCFG);
	printk("  DPLUS_RXPGCNT        = 0x%08X\n", *DPLUS_RXPGCNT);

	printk("Communication:\n")
	printk("  FW_VER_ID(%08X)  = 0x%08X\n", (u32)FW_VER_ID, *(u32*)FW_VER_ID);
}
#endif


static int swap_mac(unsigned char *data __attribute__((unused)))
{
    int ret = 0;

#if defined(ENABLE_DBG_PROC) && ENABLE_DBG_PROC
    if ( (g_dbg_datapath & DBG_ENABLE_MASK_MAC_SWAP) )
    {
        unsigned char tmp[8];
        unsigned char *p = data;

        if ( p[12] == 0x08 && p[13] == 0x06 ) { //  arp
            if ( p[14] == 0x00 && p[15] == 0x01 && p[16] == 0x08 && p[17] == 0x00 && p[20] == 0x00 && p[21] == 0x01 ) {
                //  dest mac
                memcpy(p, p + 6, 6);
                //  src mac
                p[6] = p[7] = 0;
                memcpy(p + 8, p + 38, 4);
                //  arp reply
                p[21] = 0x02;
                //  sender mac
                memcpy(p + 22, p + 6, 6);
                //  sender IP
                memcpy(tmp, p + 28, 4);
                memcpy(p + 28, p + 38, 4);
                //  target mac
                memcpy(p + 32, p, 6);
                //  target IP
                memcpy(p + 38, tmp, 4);

                ret = 42;
            }
        }
        else if ( !(p[0] & 0x01) ) { //  bypass broadcast/multicast
            //  swap MAC
            memcpy(tmp, p, 6);
            memcpy(p, p + 6, 6);
            memcpy(p + 6, tmp, 6);
            p += 12;

            //  bypass VLAN
            while ( p[0] == 0x81 && p[1] == 0x00 )
                p += 4;

            //  IP
            if ( p[0] == 0x08 && p[1] == 0x00 ) {
                p += 14;
                memcpy(tmp, p, 4);
                memcpy(p, p + 4, 4);
                memcpy(p + 4, tmp, 4);
                p += 8;
            }

            ret = (int)((unsigned long)p - (unsigned long)data);
        }
    }
#endif

    return ret;
}

#if defined(ENABLE_DIRECTPATH_TX_QUEUE) && ENABLE_DIRECTPATH_TX_QUEUE
static int send_directpath_tx_queue(void)
{
	uint32_t sys_flag;
	struct sk_buff *skb;
	int len;
	int i;

	for ( i = 0; !g_directpath_dma_full && i < NUM_ENTITY(ppa_pushthrough_data); i++ )
	{
		while ( !g_directpath_dma_full && ppa_pushthrough_data[i].skb_list )
		{
			local_irq_save(sys_flag);

			skb = skb_get(ppa_pushthrough_data[i].skb_list);

			len = skb->len;
			if ( len < ETH_MIN_TX_PACKET_LENGTH + sizeof(struct sw_eg_pkt_header) )
			len = ETH_MIN_TX_PACKET_LENGTH + sizeof(struct sw_eg_pkt_header);

			if ( dma_device_write(g_dma_device_ppe, skb->data, len, skb) == len )
			{
				//  remove skb from list
				if ( skb->prev == skb )
				skb->prev = skb->next = ppa_pushthrough_data[i].skb_list = NULL;
				else
				{
					ppa_pushthrough_data[i].skb_list = skb->next;
					ppa_pushthrough_data[i].skb_list->prev = skb->prev;
					skb->prev->next = skb->next;
					skb->prev = skb->next = NULL;
				}
				ppa_pushthrough_data[i].skb_list_len--;
				ppa_pushthrough_data[i].skb_list_size -= skb->len;
			}
			else
			{
				local_irq_restore(sys_flag);
				dev_kfree_skb_any(skb);
				return -1;
			}

			local_irq_restore(sys_flag);

			dev_kfree_skb_any(skb);

			if ( ppa_pushthrough_data[i].callback.start_tx_fn
#if defined(ENABLE_DIRECTPATH_TX_QUEUE_SIZE)
					&& ppa_pushthrough_data[i].skb_list_size <= (ENABLE_DIRECTPATH_TX_QUEUE_SIZE * 3 / 4)
#else
					&& ppa_pushthrough_data[i].skb_list_len <= (ENABLE_DIRECTPATH_TX_QUEUE_PKTS * 3 / 4)
#endif
			)
			ppa_pushthrough_data[i].callback.start_tx_fn(ppa_pushthrough_data[i].netif);
		}
	}

	return 0;
}
#endif

static INLINE void register_netdev_event_handler(void) {
	g_netdev_event_handler_nb.notifier_call = netdev_event_handler;
	register_netdevice_notifier(&g_netdev_event_handler_nb);
}

static INLINE void unregister_netdev_event_handler(void) {
	unregister_netdevice_notifier(&g_netdev_event_handler_nb);
}

#ifdef AVM_DOESNT_USE_IFX_ETHSW_API
static int netdev_event_handler(struct notifier_block *nb, unsigned long event, void *netdev)
{
	struct net_device *netif = (struct net_device *)netdev;
	IFX_ETHSW_HANDLE handle;
	union ifx_sw_param x;

	if ( netif->type != ARPHRD_ETHER )
	return NOTIFY_DONE;

	if ( (netif->flags & IFF_POINTOPOINT) ) //  ppp interface
	return NOTIFY_DONE;

	switch ( event )
	{
		case NETDEV_UP:
		handle = ifx_ethsw_kopen("/dev/switch/0");
		memset(&x.MAC_tableAdd, 0x00, sizeof(x.MAC_tableAdd));
		x.MAC_tableAdd.nFId = 0;
		x.MAC_tableAdd.nPortId = 6; //  CPU port
		//if ( netif->name[0] == 'p' && netif->name[1] == 't' && netif->name[2] == 'm' )  //  ptm
		//    x.MAC_tableAdd.nPortId = 11;
		//else if ( netif->name[0] == 'n' && netif->name[1] == 'a' && netif->name[2] == 's' ) //  atm
		//    x.MAC_tableAdd.nPortId = 11;
		//else if ( netif->name[0] == 'e' && netif->name[1] == 't' && netif->name[2] == 'h' && netif->name[3] == '1' )    //  eth1
		//    x.MAC_tableAdd.nPortId = ;
		//else
		//{
		//    int i;
		//
		//    for ( i = 0; i < NUM_ENTITY(ppa_pushthrough_data); i++ )
		//    {
		//        if ( netif == ppa_pushthrough_data[i].netif )
		//            x.MAC_tableAdd.nPortId = 7 + i;
		//    }
		//}
		x.MAC_tableAdd.nAgeTimer = 3;
		x.MAC_tableAdd.bStaticEntry = 1;
		x.MAC_tableAdd.nMAC[0] = netif->dev_addr[0];
		x.MAC_tableAdd.nMAC[1] = netif->dev_addr[1];
		x.MAC_tableAdd.nMAC[2] = netif->dev_addr[2];
		x.MAC_tableAdd.nMAC[3] = netif->dev_addr[3];
		x.MAC_tableAdd.nMAC[4] = netif->dev_addr[4];
		x.MAC_tableAdd.nMAC[5] = netif->dev_addr[5];
		printk( KERN_ERR "[%s] setup netif->dev_addr in Switch\n", __FUNCTION__);
		ifx_ethsw_kioctl(handle, IFX_ETHSW_MAC_TABLE_ENTRY_ADD, (unsigned int)&x.MAC_tableAdd);
		ifx_ethsw_kclose(handle);
		return NOTIFY_OK;
		case NETDEV_DOWN:
		handle = ifx_ethsw_kopen("/dev/switch/0");
		memset(&x.MAC_tableRemove, 0x00, sizeof(x.MAC_tableRemove));
		x.MAC_tableRemove.nFId = 0;
		x.MAC_tableRemove.nMAC[0] = netif->dev_addr[0];
		x.MAC_tableRemove.nMAC[1] = netif->dev_addr[1];
		x.MAC_tableRemove.nMAC[2] = netif->dev_addr[2];
		x.MAC_tableRemove.nMAC[3] = netif->dev_addr[3];
		x.MAC_tableRemove.nMAC[4] = netif->dev_addr[4];
		x.MAC_tableRemove.nMAC[5] = netif->dev_addr[5];
		ifx_ethsw_kioctl(handle, IFX_ETHSW_MAC_TABLE_ENTRY_REMOVE, (unsigned int)&x.MAC_tableRemove);
		ifx_ethsw_kclose(handle);
		printk( KERN_ERR "[%s] setup remove addr in switch\n", __FUNCTION__);
		return NOTIFY_OK;
	}

	return NOTIFY_DONE;
}
#else
static int netdev_event_handler(
		struct notifier_block *nb __attribute__ ((unused)),
		unsigned long event __attribute__ ((unused)),
		void *netdev __attribute__ ((unused))) {
	return NOTIFY_DONE;
}
#endif

/*
 * ####################################
 *           Global Function
 * ####################################
 */

static int32_t ppa_datapath_generic_hook(PPA_GENERIC_DATAPATH_HOOK_CMD cmd,
		void *buffer, uint32_t flag __attribute__((unused))) {
	dbg("ppa_datapath_generic_hook cmd 0x%x\n", cmd);
	switch (cmd) {
	case PPA_GENERIC_DATAPATH_GET_PPE_VERION: {
		PPA_VERSION *version = (PPA_VERSION *) buffer;
		if (version->index >= NUM_ENTITY(g_fw_ver))
			return IFX_FAILURE;
		version->family = g_fw_ver[version->index].family;
		version->type = g_fw_ver[version->index].fwtype;
		version->itf = g_fw_ver[version->index].interface;
		version->mode = g_fw_ver[version->index].fwmode;
		version->major = g_fw_ver[version->index].major;
		version->mid = 0;
		version->minor = g_fw_ver[version->index].minor;

		return IFX_SUCCESS;
	}
	case PPA_GENERIC_DATAPATH_ADDR_TO_FPI_ADDR: {
		PPA_FPI_ADDR *addr = (PPA_FPI_ADDR *) buffer;
		addr->addr_fpi = sb_addr_to_fpi_addr_convert(addr->addr_orig);
		return IFX_SUCCESS;
	}
	default:
		dbg("ppa_datapath_generic_hook not support cmd 0x%x\n", cmd);
		return IFX_FAILURE;
	}

	return IFX_FAILURE;
}

int dsl_showtime_enter(struct port_cell_info *port_cell __attribute__((unused)), void *xdata_addr) {
	ASSERT(port_cell != NULL, "port_cell is NULL");
	ASSERT(xdata_addr != NULL, "xdata_addr is NULL");

	//  TODO: ReTX set xdata_addr
	g_xdata_addr = xdata_addr;

	g_showtime = 1;
	wmb();

//	*UTP_CFG = 0x0F;

//#ifdef CONFIG_VR9
//    SW_WRITE_REG32_MASK(1 << 17, 0, FFSM_CFG0);
//#endif

	dbg("enter showtime");

	return IFX_SUCCESS;
}

int dsl_showtime_exit(void) {
	if (!g_showtime)
		return IFX_ERROR;

//#ifdef CONFIG_VR9
//    SW_WRITE_REG32_MASK(0, 1 << 17, FFSM_CFG0);
//#endif

	//*UTP_CFG = 0x00;

	g_showtime = 0;
	wmb();

	//  TODO: ReTX clean state
	g_xdata_addr = NULL;

	dbg("leave showtime");

	return IFX_SUCCESS;
}

#ifdef AVM_DOESNT_USE_IFX_ETHSW_API
int mac_entry_setting(unsigned char *mac, uint32_t fid, uint32_t portid, uint32_t agetime, uint32_t st_entry , uint32_t action)
{
	IFX_ETHSW_HANDLE handle;
	union ifx_sw_param x;
	int ret = IFX_SUCCESS;

	if(!mac)
	return IFX_ERROR;

	dump_stack();
	switch(action)
	{
		case IFX_ETHSW_MAC_TABLE_ENTRY_ADD:
		handle = ifx_ethsw_kopen("/dev/switch/0");
		memset(&x.MAC_tableAdd, 0x00, sizeof(x.MAC_tableAdd));
		x.MAC_tableAdd.nFId = fid;
		x.MAC_tableAdd.nPortId = portid;
		x.MAC_tableAdd.nAgeTimer = agetime;
		x.MAC_tableAdd.bStaticEntry = st_entry;
		x.MAC_tableAdd.nMAC[0] = mac[0];
		x.MAC_tableAdd.nMAC[1] = mac[1];
		x.MAC_tableAdd.nMAC[2] = mac[2];
		x.MAC_tableAdd.nMAC[3] = mac[3];
		x.MAC_tableAdd.nMAC[4] = mac[4];
		x.MAC_tableAdd.nMAC[5] = mac[5];
		ret = ifx_ethsw_kioctl(handle, IFX_ETHSW_MAC_TABLE_ENTRY_ADD, (unsigned int)&x.MAC_tableAdd);
		ifx_ethsw_kclose(handle);
		break;

		case IFX_ETHSW_MAC_TABLE_ENTRY_REMOVE:
		handle = ifx_ethsw_kopen("/dev/switch/0");
		memset(&x.MAC_tableRemove, 0x00, sizeof(x.MAC_tableRemove));
		x.MAC_tableRemove.nFId = fid;
		x.MAC_tableRemove.nMAC[0] = mac[0];
		x.MAC_tableRemove.nMAC[1] = mac[1];
		x.MAC_tableRemove.nMAC[2] = mac[2];
		x.MAC_tableRemove.nMAC[3] = mac[3];
		x.MAC_tableRemove.nMAC[4] = mac[4];
		x.MAC_tableRemove.nMAC[5] = mac[5];
		printk("[%s] REMOVE: nFID =%d\n", __FUNCTION__, fid);
		ifx_ethsw_kioctl(handle, IFX_ETHSW_MAC_TABLE_ENTRY_REMOVE, (unsigned int)&x.MAC_tableRemove);
		ifx_ethsw_kclose(handle);
		break;

		default:
		break;
	}

	return ret;
}
#endif

int get_dslwan_qid_with_vcc(struct atm_vcc *vcc __attribute__ ((unused)), unsigned int prio  __attribute__ ((unused))) {
	return -1;
}

int get_netif_qid_with_pkt(struct sk_buff *skb, void *arg, int is_atm_vcc) {
	if (is_atm_vcc)
		return -1;
	else {
		int port = get_eth_port((struct net_device *) arg);
		if (port >= 0)
            return g_eth_prio_queue_map[port][skb->priority >= NUM_ENTITY(g_eth_prio_queue_map[port]) ? NUM_ENTITY(g_eth_prio_queue_map[port]) - 1 : skb->priority];
		else if (g_eth_wan_mode == 0) {
	        if ( arg == (void *) g_ptm_net_dev[0] )
	            return g_ptm_prio_queue_map[skb->priority >= NUM_ENTITY(g_ptm_prio_queue_map) ? NUM_ENTITY(g_ptm_prio_queue_map) - 1 : skb->priority];
		}
	}

	return 0;
}

int ppe_directpath_send(uint32_t off, struct sk_buff *skb,
		int32_t len __attribute__ ((unused)),
		uint32_t flags __attribute__ ((unused))) {

	uint32_t if_id = PPA_VDEV_ID(off);

//  Careful, no any safety check here.
//  Parameters must be correct.

	dbg_trace_ppa_data("directpath send if:%d", if_id);

#if !defined(ENABLE_DIRECTPATH_TX_QUEUE) || !ENABLE_DIRECTPATH_TX_QUEUE
	if (eth_xmit(skb, if_id /* dummy */, DMA_CHAN_NR_DIRECTPATH_CPU_TO_PPE, if_id /* spid */,
			0 /* lowest priority */)) {
        if ( ppa_virt_rx_devs[ off ] )
            ppa_virt_rx_devs[ off ]->hw_pkt_try_to_acc_dropped++;
		return -1;
	} else
		return 0;
#else
	if ( !g_directpath_dma_full && !ppa_pushthrough_data[if_id - 3].skb_list )
	{
		if ( eth_xmit(skb, if_id /* dummy */, DMA_CHAN_NR_DIRECTPATH_CPU_TO_PPE, if_id /* spid */, 0 /* lowest priority */) )
		{
			ppa_pushthrough_data[off].tx_pkt_dropped++;
			return -1;
		}
		else
		return 0;
	}
	else
	{
		if (
#if defined(ENABLE_DIRECTPATH_TX_QUEUE_SIZE)
				ppa_pushthrough_data[off].skb_list_size + skb->len + sizeof(struct sw_eg_pkt_header) <= ENABLE_DIRECTPATH_TX_QUEUE_SIZE
#else
				ppa_pushthrough_data[off].skb_list_len < ENABLE_DIRECTPATH_TX_QUEUE_PKTS
#endif
		)
		{
			struct sw_eg_pkt_header pkth = {spid: if_id, //  CPU/app0/app1/app2
				dpid: 2,//  eth0/eth1/app0/app1/app2
				qid: 0,//  0 is lowest priority, 3 is highest priority
				direct: 0}; //  disable direct forward
			uint32_t sys_flag;

			if ( skb_headroom(skb) < sizeof(struct sw_eg_pkt_header) )
			{
				struct sk_buff *new_skb;

				new_skb = alloc_skb_tx(skb->len + sizeof(struct sw_eg_pkt_header));
				if ( !new_skb )
				goto PPE_DIRECTPATH_SEND_DROP;
				else
				{
					skb_put(new_skb, skb->len + sizeof(struct sw_eg_pkt_header));
					memcpy(new_skb->data + sizeof(struct sw_eg_pkt_header), skb->data, skb->len);
					dev_kfree_skb_any(skb);
					skb = new_skb;
				}
			}
			else
			skb_push(skb, sizeof(struct sw_eg_pkt_header));
			memcpy(skb->data, &pkth, sizeof(struct sw_eg_pkt_header));

			local_irq_save(sys_flag);
			ppa_pushthrough_data[off].skb_list_len++;
			ppa_pushthrough_data[off].skb_list_size += skb->len;
			if ( ppa_pushthrough_data[off].skb_list != NULL )
			{
				ppa_pushthrough_data[off].skb_list->prev->next = skb;
				skb->next = ppa_pushthrough_data[off].skb_list;
				skb->prev = ppa_pushthrough_data[off].skb_list->prev;
				ppa_pushthrough_data[off].skb_list->prev = skb;
			}
			else
			{
				ppa_pushthrough_data[off].skb_list = skb;
				skb->prev = skb->next = skb;
			}
			local_irq_restore(sys_flag);

			ppa_pushthrough_data[off].tx_pkt_queued++;

			return 0;
		}
		else if ( ppa_pushthrough_data[off].callback.stop_tx_fn )
		ppa_pushthrough_data[off].callback.stop_tx_fn(ppa_pushthrough_data[off].netif);

		PPE_DIRECTPATH_SEND_DROP:
		dev_kfree_skb_any(skb);
		ppa_pushthrough_data[off].tx_pkt_dropped++;
		return -1;
	}
#endif
}

int ppe_directpath_rx_stop(uint32_t if_id __attribute__ ((unused)),
		uint32_t flags __attribute__ ((unused))) {
	return 0;
}

int ppe_directpath_rx_start(uint32_t if_id __attribute__ ((unused)),
		uint32_t flags __attribute__ ((unused))) {
	return 0;
}

int ppe_clk_change(unsigned int arg,
		unsigned int flags __attribute__ ((unused))) {
	unsigned long sys_flag;
	unsigned int ppe_cfg_bit =
			arg == IFX_PPA_PWM_LEVEL_D0 ? (1 << 16) : (2 << 16);

	local_irq_save(sys_flag);
	*VR9_CGU_CLKFSR = (*VR9_CGU_CLKFSR & ~(0x07 << 16)) | ppe_cfg_bit;
	local_irq_restore(sys_flag);

    TX_QOS_CFG_DYNAMIC->time_tick = cgu_get_pp32_clock() / 62500;   //  16 * (cgu_get_pp32_clock() / 1000000)
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_dsl_bonding == 2 )
        TX_QOS_CFG->time_tick = cgu_get_pp32_clock() / 62500;       //  16 * (cgu_get_pp32_clock() / 1000000)
#endif

	return 0;
}

int ppe_pwm_change(unsigned int arg, unsigned int flags)
{
    if ( flags == 1 )   //  clock gating
        PSAVE_CFG->sleep_en = arg ? 1 : 0;

    return 0;
}


void ifx_ppa_setup_ptm_net_dev(struct net_device* dev) {
	static int setup_dev_nr = 0;
	printk(KERN_ERR "[%s] setup_dev_nr = %d\n", __FUNCTION__, setup_dev_nr);
	g_ptm_net_dev[setup_dev_nr++] = dev;
	dev->watchdog_timeo = ETH_WATCHDOG_TIMEOUT;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void avmnet_swi_vr9_e5_register_xmit(void) {
	struct net_device *dev;
	unsigned int i;
	printk(KERN_ERR "[%s] setup new xmit function \n", __FUNCTION__);
	for (i = 0; i < num_registered_eth_dev; i++) {
		struct net_device_ops * ops;
		dev = g_eth_net_dev[i];
		ops = dev->netdev_ops;
		ops->ndo_start_xmit = &ifx_ppa_eth_hard_start_xmit_e5;
		ops->ndo_tx_timeout = &ifx_ppa_eth_tx_timeout_e5;
		ops->ndo_do_ioctl = &ifx_ppa_eth_ioctl_e5;
	}
}

/*------------------------------------------------------------------------------------------*\
 * Modul Init: Init/Cleanup API
 \*------------------------------------------------------------------------------------------*/
static int ppe_e5_init(void) {
	int ret, i;
	char buf[512];
	struct port_cell_info port_cell = { 0 };
	int showtime = 0;
	void *xdata_addr = NULL;
    avmnet_device_t *avm_ptm_dev = NULL;

	if (g_datapath_mod_count){
		printk("E5 ( MII0/1 ) driver already loaded\n");
		return -EBUSY;
	}

	// we might have to wait for offload ioctrls to finish
	rtnl_offload_write_lock();

	g_datapath_mod_count++;

	push_through_devs_supported_by_datapath = 4;

	printk("Loading E5 (MII0/1) driver ...... ");
	g_stop_datapath = 1;
	wmb();

	// (1) block all incoming switch ports
    // tell driver to suspend packet flow on all devices
    avmnet_swi_7port_drop_everything();
    if(avmnet_hw_config_entry->config->suspend){
        avmnet_hw_config_entry->config->suspend(avmnet_hw_config_entry->config, NULL);
    }else{
        printk(KERN_ERR "[%s] Config error, no suspend function found!\n", __func__);
        BUG();
    }


	if (avm_ata_dev)
	    printk("avm_ata_dev: '%s' \n" , avm_ata_dev);

    // stop NAPI, empty skb backlog queues
    for_each_possible_cpu(i){
        struct softnet_data *queue;
        struct sk_buff *skb, *tmp;

        queue = &per_cpu(g_7port_eth_queue, i);

        napi_disable(&queue->backlog);
        queue->backlog.poll = NULL;

        skb_queue_walk_safe(&(queue->input_pkt_queue), skb, tmp){
            __skb_unlink(skb, &(queue->input_pkt_queue));
            dev_kfree_skb_any(skb);
        }
    }

	stop_7port_dma_gracefully();
  	dma_device_unregister(g_dma_device_ppe);
	dma_device_release(g_dma_device_ppe);
	g_dma_device_ppe = NULL;

#if IFX_PPA_DP_DBG_PARAM_ENABLE
	if (ifx_ppa_drv_dp_dbg_param_enable == 1) {
		ethwan = ifx_ppa_drv_dp_dbg_param_ethwan;
		wanitf = ifx_ppa_drv_dp_dbg_param_wanitf;
		ipv6_acc_en = ifx_ppa_drv_dp_dbg_param_ipv6_acc_en;
		wanqos_en = ifx_ppa_drv_dp_dbg_param_wanqos_en;
	}
#endif  //IFX_PPA_DP_DBG_PARAM_ENABLE

#if !defined(RESET_AR10_STYLE)
	reset_ppe();
#endif

	init_local_variables();

	init_hw();
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_dsl_bonding == 2 )   //  off chip bonding
        init_bonding_hw();
#endif

	if (g_eth_wan_mode == 0 /* DSL WAN */){
		init_ptm_tables();
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
        if ( g_dsl_bonding == 2 )
        {
            g_ppe_base_addr = IFX_PPE_ORG(1);
            init_ptm_tables();
            g_ppe_base_addr = IFX_PPE_ORG(0);
        }
#endif
        if ( g_dsl_bonding )
            init_ptm_bonding_tables();
	}

	init_communication_data_structures(g_fwcode);

	ret = alloc_dma();
	if (ret)
		goto ALLOC_DMA_FAIL;

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_dsl_bonding == 2 )   //  off chip bonding
        alloc_bonding_dma();
#endif

	dbg(KERN_ERR "[%s] line %d\n", __FUNCTION__, __LINE__);

	ret = request_irq(PPE_MAILBOX_IGU0_INT, mailbox0_irq_handler, IRQF_DISABLED,
			"e5_mailbox0_isr", NULL);
	if (ret) {
		if (ret == -EBUSY)
			err( "IRQ may be occupied by other PPE driver, please reconfig to disable it.\n");
		goto REQUEST_IGU0_IRQ_FAIL;
	}
	ret = request_irq(PPE_MAILBOX_IGU1_INT, mailbox1_irq_handler, IRQF_DISABLED,
			"e5_mailbox_isr", NULL);
	if (ret) {
		if (ret == -EBUSY)
			err(
					"IRQ may be occupied by other PPE driver, please reconfig to disable it.\n");
		goto REQUEST_IRQ_FAIL;
	}

	//don't clear pushthtrough data in Datapath-Init in AVM Setup -> we use common pushthrough_data for all datapath modules
	// memset(ppa_pushthrough_data, 0, sizeof(ppa_pushthrough_data));

	dump_init();

	if (g_eth_wan_mode == 0)
	{
		ret = pp32_start(0, 1);
		if (ret)
			goto PP32_START_E1_FAIL;
	}

	ret = pp32_start(1, 0);
	if (ret)
		goto PP32_START_D5_FAIL;

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_eth_wan_mode == 0 && g_dsl_bonding == 2 )
    {
        g_mailbox_signal_mask &= ~WAN_RX_SIG(0);
        g_dma_device->tx_chan[1]->open(g_dma_device->tx_chan[1]);
    }
#endif

	*MBOX_IGU0_IER = g_eth_wan_mode ? DMA_TX_CH0_SIG | DMA_TX_CH1_SIG : DMA_TX_CH0_SIG;
	*MBOX_IGU1_IER = g_mailbox_signal_mask;

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
    if ( g_dsl_bonding == 2 )
    {
        g_ppe_base_addr = IFX_PPE_ORG(1);
        pp32_start(0, 1);
        pp32_start(1, 0);
        *MBOX_IGU1_IER = 0;
        g_ppe_base_addr = IFX_PPE_ORG(0);
    }
#endif
	start_cpu_port();

	//  turn on DMA TX channels (0, 1) for fast path
	g_dma_device_ppe->tx_chan[0]->open(g_dma_device_ppe->tx_chan[0]);  // ppa -> switch
	g_dma_device_ppe->rx_chan[2]->open(g_dma_device_ppe->rx_chan[2]);

	// g_dma_device_ppe->tx_chan[1]->open(g_dma_device_ppe->tx_chan[1]);  // dsl -> switch(port6)

	// shut the fuck up :-)
	// g_dma_device_ppe->tx_chan[1]->close(g_dma_device_ppe->tx_chan[1]);
	// g_dma_device_ppe->rx_chan[1]->close(g_dma_device_ppe->rx_chan[1]);

	if (g_eth_wan_mode == 0 /* DSL WAN */) {

		avm_wan_mode = AVM_WAN_MODE_PTM;
        avmnet_swi_7port_set_ethwan_dev_by_name( NULL );

		if (!IS_ERR(&ifx_mei_atm_showtime_check) && &ifx_mei_atm_showtime_check)
			ifx_mei_atm_showtime_check(&showtime, &port_cell, &xdata_addr);
		else
			printk("Weak-Symbol nicht gefunden: ifx_mei_atm_showtime_check\n");
		if (showtime)
			dsl_showtime_enter(&port_cell, xdata_addr);

#if defined(ENABLE_LED_FRAMEWORK) && ENABLE_LED_FRAMEWORK
		ifx_led_trigger_register("dsl_data", &g_data_led_trigger);
#endif
	}

	if ( g_eth_wan_mode == 3 /* ETH WAN */ ) {
    	avm_wan_mode = AVM_WAN_MODE_ETH;
    }

	register_netdev_event_handler();

	/*  create proc file    */

	dsl_file_create();

	proc_file_create();

	if (g_eth_wan_mode == 0 /* DSL WAN */) {
		if (!IS_ERR(&ifx_mei_atm_showtime_enter) && &ifx_mei_atm_showtime_enter)
			ifx_mei_atm_showtime_enter = dsl_showtime_enter;
		else
			err("Weak-Symbol nicht gefunden: ifx_mei_atm_showtime_enter\n");

		if (!IS_ERR(&ifx_mei_atm_showtime_exit) && &ifx_mei_atm_showtime_exit)
			ifx_mei_atm_showtime_exit = dsl_showtime_exit;
		else
			err("Weak-Symbol nicht gefunden: ifx_mei_atm_showtime_exit\n");

	}
	//Fix warning message ---start
	ifx_ppa_drv_get_dslwan_qid_with_vcc_hook = get_dslwan_qid_with_vcc;
	ifx_ppa_drv_get_netif_qid_with_pkt_hook = get_netif_qid_with_pkt;

	ifx_ppa_drv_ppe_clk_change_hook = ppe_clk_change;
    ifx_ppa_drv_ppe_pwm_change_hook = ppe_pwm_change;

	in_avm_ata_mode_hook = e5_in_avm_ata_mode;

	ppa_pushthrough_hook = ppe_directpath_send;

#ifdef CONFIG_IFX_PPA_API_DIRECTPATH
	ifx_ppa_drv_g_ppe_directpath_data = ppa_pushthrough_data;
	ifx_ppa_drv_directpath_send_hook = ppe_directpath_send;
	ifx_ppa_drv_directpath_rx_start_hook = ppe_directpath_rx_start;
	ifx_ppa_drv_directpath_rx_stop_hook = ppe_directpath_rx_stop;
#endif //CONFIG_IFX_PPA_API_DIRECTPATH
	ifx_ppa_drv_datapath_generic_hook = ppa_datapath_generic_hook;
#ifdef AVM_DOESNT_USE_IFX_ETHSW_API
	ifx_ppa_drv_datapath_mac_entry_setting = mac_entry_setting;
#else
	ifx_ppa_drv_datapath_mac_entry_setting = NULL;
#endif
	//Fix warning message ---end

	/* ptm device funktionen setzen und das device vom avmnet-Treiber alloziieren lassen */
	avm_ptm_dev = get_avmdev_by_name("ptm_vr9");
	if (avm_ptm_dev) {
		printk("got a fancy ptm_device: do function setup\n");
		avm_ptm_dev->device_ops.ndo_init = ifx_ppa_ptm_init;
		avm_ptm_dev->device_ops.ndo_open = ifx_ppa_ptm_open;
		avm_ptm_dev->device_ops.ndo_stop = ifx_ppa_ptm_stop;
		avm_ptm_dev->device_ops.ndo_start_xmit = g_dsl_bonding ? ifx_ppa_ptm_hard_start_xmit_b : ifx_ppa_ptm_hard_start_xmit;
		avm_ptm_dev->device_ops.ndo_do_ioctl = ifx_ppa_ptm_ioctl;
		avm_ptm_dev->device_ops.ndo_tx_timeout = ifx_ppa_ptm_tx_timeout;

		avm_ptm_dev->device_setup = ifx_ppa_setup_ptm_net_dev;
		avm_ptm_dev->flags &= ~AVMNET_DEVICE_FLAG_WAIT_FOR_MODULE_FUNCTIONS;
		avmnet_create_netdevices();
	}

	// replace common NAPI poll function with our one
    for_each_possible_cpu(i) {
        struct softnet_data *queue;

        queue = &per_cpu(g_7port_eth_queue, i);

        skb_queue_head_init(&queue->input_pkt_queue);
        queue->completion_queue = NULL;
        INIT_LIST_HEAD(&queue->poll_list);

        queue->backlog.poll = process_backlog_e5;
        napi_enable(&queue->backlog);
    }

#if 0 //DMA lauft immer
	/* check if some interfaces have been opened by common driver before */
	for (i = 0; i < NUM_ETH_INF; i++)
	if (g_eth_net_dev[i] && netif_running(g_eth_net_dev[i])) {
		printk(KERN_ERR "[%s] dev=%d is already up and running -> reenable dma\n", __FUNCTION__, i);
		turn_on_dma_rx(1);
		break;
	}
#else
	turn_on_dma_rx(1);
#endif

	/* (re-)aktivieren der DMA-Ports */
	show_fdma_sdma();
	enable_fdma_sdma();
	show_fdma_sdma();

	/* (re-)aktivieren der MACs */
    // TKL: wird jetzt am Schluss durch resume() gemacht
	//avmnet_swi_7port_reinit_macs(avmnet_hw_config_entry->config);

	/* neue (e5 spezifische) xmit-Funktionen registrieren */
	avmnet_swi_vr9_e5_register_xmit();

	/* disable learning, flush mac table*/
	avmnet_swi_7port_disable_learning(avmnet_hw_config_entry->config);

	if (( g_eth_wan_mode == 3 ) && avm_ata_dev) /* ETH WAN */
        avmnet_swi_7port_set_ethwan_dev_by_name( avm_ata_dev );


    // tell driver to resume packet flow on all devices
    if(avmnet_hw_config_entry->config->resume){
        avmnet_hw_config_entry->config->resume(avmnet_hw_config_entry->config, NULL);
    }else{
        printk(KERN_ERR "[%s] Config error, no resume function found!\n", __func__);
        BUG();
    }

    /* disable drop xmit packets*/
	wmb();
	g_stop_datapath = 0;

	printk("Succeeded!\n");

	dbg(KERN_ERR "[%s] showtime_exit = %pF, showtime_enter =%pF\n",
			__FUNCTION__, dsl_showtime_exit, dsl_showtime_enter);
	print_driver_ver(buf, sizeof(buf), "PPE datapath driver info", VER_FAMILY,
			VER_DRTYPE, VER_INTERFACE, VER_ACCMODE, VER_MAJOR, VER_MID,
			VER_MINOR);
	printk(buf);
	print_fw_ver(buf, sizeof(buf));
	printk(buf);

#if defined( AVM_NET_TRACE_DUMP ) && AVM_NET_TRACE_DUMP && defined(CONFIG_AVM_NET_TRACE)
	register_avm_net_trace_device ( &ifx_ant_e5 );
	printk(KERN_ERR "[%s] minor:%d", ifx_ant_e5.minor);
#endif

	// dump_dplus("e5 initialized");

    g_dsl_ig_fast_brg_bytes     = ITF_MIB_TBL(7)->ig_fast_brg_bytes;
    g_dsl_ig_fast_rt_ipv4_bytes = ITF_MIB_TBL(7)->ig_fast_rt_ipv4_bytes;
    g_dsl_ig_fast_rt_ipv6_bytes = ITF_MIB_TBL(7)->ig_fast_rt_ipv6_bytes;
    g_dsl_ig_cpu_bytes          = ITF_MIB_TBL(7)->ig_cpu_bytes;
    g_dsl_eg_fast_pkts          = ITF_MIB_TBL(7)->eg_fast_pkts;

	rtnl_offload_write_unlock();
	return 0;

PP32_START_D5_FAIL:
	printk(KERN_ERR "[%s] start d5 failed\n", __FUNCTION__);

	if (g_eth_wan_mode == 0 /* DSL WAN */)
		pp32_stop(0);
	PP32_START_E1_FAIL: printk(KERN_ERR "[%s] start e1 failed\n", __FUNCTION__);
	free_irq(PPE_MAILBOX_IGU1_INT, NULL);
	REQUEST_IRQ_FAIL: printk(KERN_ERR "[%s] irq req failed\n", __FUNCTION__);
	free_irq(PPE_MAILBOX_IGU0_INT, NULL);
	REQUEST_IGU0_IRQ_FAIL: printk(KERN_ERR "[%s] igu0 irq fail\n",
			__FUNCTION__);
	// ifx_eth_fw_unregister_netdev(g_ptm_net_dev[0], 1); //TODO uregister_netdev realisieren....
	// ifx_eth_fw_free_netdev(g_ptm_net_dev[0], 1); //TODO free_netdev realisieren....
	g_ptm_net_dev[0] = NULL;

	stop_7port_dma_gracefully();

	free_dma();

	ALLOC_DMA_FAIL: printk(KERN_ERR "[%s] dma alloc failed\n", __FUNCTION__);
	clear_local_variables();

	g_datapath_mod_count--;
   	avm_wan_mode = AVM_WAN_MODE_NOT_CONFIGURED;

	rtnl_offload_write_unlock();
	return ret;
}

static void __exit ppe_e5_exit(void) {
	int i;
    avmnet_device_t *avm_ptm_dev = NULL;

	// we might have to wait for offload ioctrls to finish
	rtnl_offload_write_lock();
    g_stop_datapath = 1;
    wmb();

    avmnet_swi_7port_drop_everything();
    // (1) block all incoming switch ports
    // tell driver to suspend packet flow on all devices
    if(avmnet_hw_config_entry->config->suspend){
        avmnet_hw_config_entry->config->suspend(avmnet_hw_config_entry->config, NULL);
    }else{
        printk(KERN_ERR "[%s] Config error, no suspend function found!\n", __func__);
        BUG();
    }

	// stop NAPI, empty skb backlog queues
    for_each_possible_cpu(i){
        struct softnet_data *queue;
        struct sk_buff *skb, *tmp;

        queue = &per_cpu(g_7port_eth_queue, i);

        napi_disable(&queue->backlog);
        queue->backlog.poll = NULL;

        skb_queue_walk_safe(&(queue->input_pkt_queue), skb, tmp){
            __skb_unlink(skb, &(queue->input_pkt_queue));
            dev_kfree_skb_any(skb);
        }
    }

	if (g_eth_wan_mode == 0 /* DSL WAN */) {
		if (!IS_ERR(&ifx_mei_atm_showtime_enter) && &ifx_mei_atm_showtime_enter)
			ifx_mei_atm_showtime_enter = NULL;
		else
			printk("Weak-Symbol nicht gefunden: ifx_mei_atm_showtime_enter\n");

		if (!IS_ERR(&ifx_mei_atm_showtime_exit) && &ifx_mei_atm_showtime_exit)
			ifx_mei_atm_showtime_exit = NULL;
		else
			printk("Weak-Symbol nicht gefunden: ifx_mei_atm_showtime_exit\n");
	}

	proc_file_delete();

    dsl_file_delete();

	unregister_netdev_event_handler();

	in_avm_ata_mode_hook = NULL;

	ppa_pushthrough_hook = NULL;

    ifx_ppa_drv_get_dslwan_qid_with_vcc_hook = NULL;
    ifx_ppa_drv_get_netif_qid_with_pkt_hook = NULL;
    ifx_ppa_drv_ppe_clk_change_hook = NULL;
    ifx_ppa_drv_ppe_pwm_change_hook = NULL;

#ifdef CONFIG_IFX_PPA_API_DIRECTPATH
    ifx_ppa_drv_g_ppe_directpath_data = NULL;
    ifx_ppa_drv_directpath_send_hook = NULL;
    ifx_ppa_drv_directpath_rx_start_hook = NULL;
    ifx_ppa_drv_directpath_rx_stop_hook = NULL;
#endif //CONFIG_IFX_PPA_API_DIRECTPATH
    ifx_ppa_drv_datapath_generic_hook = NULL;

#if defined(ENABLE_LED_FRAMEWORK) && ENABLE_LED_FRAMEWORK
	if ( g_eth_wan_mode == 0 /* DSL WAN */)
	{
		ifx_led_trigger_deregister(g_data_led_trigger);
		g_data_led_trigger = NULL;
	}
#endif

	// stop_datapath();
#if 1

	stop_7port_dma_gracefully();
#else
	dbg(  "[%s] unregister g_dma_device_ppe\n", __FUNCTION__);
	dma_device_unregister(g_dma_device_ppe);
	dbg(  "[%s] unregister g_dma_device_ppe\n", __FUNCTION__);
	dma_device_release(g_dma_device_ppe);
#endif

	if (g_eth_wan_mode == 0 /* DSL WAN */)
		pp32_stop(0);
	pp32_stop(1);

	free_irq(PPE_MAILBOX_IGU0_INT, NULL);

	free_irq(PPE_MAILBOX_IGU1_INT, NULL);

	avm_ptm_dev = get_avmdev_by_name("ptm_vr9");
	if (avm_ptm_dev) {
	    avmnet_destroy_netdevice(avm_ptm_dev);
		g_ptm_net_dev[0] = NULL;
	}

#if 0
	for (i = 0; i < NUM_ENTITY(g_eth_net_dev); i++) {
		unregister_netdev(g_eth_net_dev[i]);
		free_netdev(g_eth_net_dev[i]);
		g_eth_net_dev[i] = NULL;
	}
#endif

	free_dma();

	clear_share_buffer();

	clear_local_variables();

	reinit_7port_common_eth();

    // tell driver to resume packet flow on all devices
    if(avmnet_hw_config_entry->config->resume){
        avmnet_hw_config_entry->config->resume(avmnet_hw_config_entry->config, NULL);
    }else{
        printk(KERN_ERR "[%s] Config error, no resume function found!\n", __func__);
        BUG();
    }

	wmb();
    g_stop_datapath = 0;

	g_datapath_mod_count--;
   	avm_wan_mode = AVM_WAN_MODE_NOT_CONFIGURED;

	rtnl_offload_write_unlock();

}


#if defined(CONFIG_IFX_PPA_DATAPATH)

static int __init wan_mode_setup(char *line)
{
	if ( strcmp(line, "1") == 0 )
	ethwan = 1;
	else if ( strcmp(line, "2") == 0 )
	ethwan = 2;

	dbg(KERN_ERR "[%s] ethwan=%#x\n", __FUNCTION__, ethwan);
	dbg(KERN_ERR "[%s] wanitf=%#x\n", __FUNCTION__, wanitf);
	return 0;
}

static int __init wanitf_setup(char *line)
{
	wanitf = simple_strtoul(line, NULL, 0);

	if ( wanitf > 0xFF )
	wanitf = ~0;
	dbg("[%s] wanitf=%#x\n", __FUNCTION__, wanitf);

	return 0;
}
static int __init ipv6_acc_en_setup(char *line)
{
	if ( strcmp(line, "0") == 0 )
	ipv6_acc_en = 0;
	else
	ipv6_acc_en = 1;

	return 0;
}

static int __init wanqos_en_setup(char *line)
{
	wanqos_en = simple_strtoul(line, NULL, 0);

	if ( wanqos_en < 0 || wanqos_en > 8 )
		wanqos_en = 0;

	return 0;
}

static int __init dsl_bonding_setup(char *line) {
    int tmp_wanqos_en = simple_strtoul(line, NULL, 0);

    if ( isdigit(*line) && tmp_wanqos_en >= 0 && tmp_wanqos_en <= 8 )
        wanqos_en = tmp_wanqos_en;

    return 0;
}

#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING

static int __init pci_ppe_addr_setup(char *line)
{
    pci_ppe_addr = simple_strtoul(line, NULL, 0);

    return 0;
}

static int __init bm_addr_setup(char *line)
{
    bm_addr = simple_strtoul(line, NULL, 0);

    return 0;
}

static int __init pci_ddr_addr_setup(char *line)
{
    pci_ppe_addr = simple_strtoul(line, NULL, 0);

    return 0;
}

static int __init pci_bm_ppe_addr_setup(char *line)
{
    pci_bm_ppe_addr = simple_strtoul(line, NULL, 0);

    return 0;
}

#endif

static int __init queue_gamma_map_setup(char *line)
{
	char *p;
	int i;

	for ( i = 0, p = line; i < NUM_ENTITY(queue_gamma_map) && isxdigit(*p); i++ )
	{
		queue_gamma_map[i] = simple_strtoul(p, &p, 0);
		if ( *p == ',' || *p == ';' || *p == ':' )
		p++;
	}

	return 0;
}

#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_SIZE)
static int __init directpath_tx_queue_size_setup(char *line)
{
	unsigned long i;

	i = simple_strtoul(line, NULL, 0);
	directpath_tx_queue_size = (int)i;

	return 0;
}
#endif

#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_PKTS)
static int __init directpath_tx_queue_pkts_setup(char *line)
{
	unsigned long i;

	i = simple_strtoul(line, NULL, 0);
	directpath_tx_queue_pkts = (int)i;

	return 0;
}
#endif
#endif

module_init(ppe_e5_init);
module_exit(ppe_e5_exit);
#ifdef CONFIG_IFX_PPA_DATAPATH
__setup("ethaddr=", eth0addr_setup);
__setup("eth1addr=", eth1addr_setup);
__setup("ethwan=", wan_mode_setup);
__setup("wanitf=", wanitf_setup);
__setup("ipv6_acc_en=", ipv6_acc_en_setup);
__setup("wanqos_en=", wanqos_en_setup);
__setup("dsl_bonding=", dsl_bonding_setup);
#ifdef CONFIG_IFX_PPE_E5_OFFCHIP_BONDING
__setup("pci_ppe_addr=", pci_ppe_addr_setup);
__setup("bm_addr=", bm_addr_setup);
__setup("pci_ddr_addr=", pci_ddr_addr_setup);
__setup("pci_bm_ppe_addr=", pci_bm_ppe_addr_setup);
#endif
__setup("queue_gamma_map=", queue_gamma_map_setup);
#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_SIZE)
__setup("directpath_tx_queue_size=", directpath_tx_queue_size_setup);
#endif
#if defined(CONFIG_IFX_PPA_DIRECTPATH_TX_QUEUE_PKTS)
__setup("directpath_tx_queue_pkts=", directpath_tx_queue_pkts_setup);
#endif
#endif

// EXPORT_SYMBOL(dsl_showtime_enter);
// EXPORT_SYMBOL(dsl_showtime_exit);
// EXPORT_SYMBOL(ppa_pushthrough_data);
// EXPORT_SYMBOL(get_dslwan_qid_with_vcc);
// EXPORT_SYMBOL(get_netif_qid_with_pkt);
// EXPORT_SYMBOL(ppe_directpath_send);
// EXPORT_SYMBOL(ppe_directpath_rx_stop);
// EXPORT_SYMBOL(ppe_directpath_rx_start);
// EXPORT_SYMBOL(ppe_clk_change);
// EXPORT_SYMBOL(ppe_drv_get_firmware_id);

// EXPORT_SYMBOL(ifx_ppa_eth_init_e5);
// EXPORT_SYMBOL(ifx_ppa_eth_open);
// EXPORT_SYMBOL(ifx_ppa_eth_stop);

// EXPORT_SYMBOL(ifx_ppa_eth_qos_hard_start_xmit);
MODULE_LICENSE("GPL");

/*------------------------------------------------------------------------------------------*\
 * Not Merged so far:

 static void dma_activate_poll(struct dma_device_info *dma_dev)
 {
 ifx_eth_fw_poll_schedule(g_eth_net_dev[0]);
 ifx_eth_fw_poll_schedule(g_eth_net_dev[1]);
 }

 static void dma_inactivate_poll(struct dma_device_info *dma_dev)
 {
 ifx_eth_fw_poll_complete(g_eth_net_dev[0]);
 ifx_eth_fw_poll_complete(g_eth_net_dev[1]);
 }

 static INLINE char *get_wanitf(int iswan)

 was ist directpath_data ?!

 \*------------------------------------------------------------------------------------------*/

